!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2005 - 2014 Pedro Jimenez-Delgado
! 
! This file is part of Chromopolis.
! 
! Chromopolis is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
! 
! Chromopolis is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with Chromopolis.  If not, see <http://www.gnu.org/licenses/>.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine chisq(oflag,nused,chi2)
  use database
  implicit none
  integer::i,oflag,nused,sused(nsets)=0
  double precision::chi2,schi2(nsets)=0d0
  11 format(a8,i4,a7,i5,a37,i4,a2,f6.4,a1)
  12 format(17x,a30,i4,a2,i4,a3,i5,a2,f6.4,a1)
  lmcount=0
  do i = 1,nsets
     if (touse(i)) call chi2set(i,sused(i),schi2(i))
  end do
  chi2 = sum(schi2)
  nused = sum(sused)
  if ((nused/=sum(merge(1,0,used))).or.(nlm/=lmcount)) then
     write(6,*) "CHISQ: data error, counters of used points:",nused,sum(merge(1,0,used))
     write(6,*) "CHISQ: data error, nlm, lmcount:",nlm,lmcount
    stop
  end if
  if (oflag==0) then
     return
  else  if (oflag==1) then
     write(6,11) " CHISQ: ",nused," (from ",ntotal,") points used, chi2 (chi2/#points) = ",nint(chi2)," (",chi2/dble(nused),")"
     do i = 1,nsets
        if (touse(i)) write(6,12) text(i),sused(i)," (",npoints(i),"), ",nint(schi2(i))," (",schi2(i)/dble(sused(i)),")"
     end do
  else
     write(6,*) "CHISQ: Bad value for oflag = ",oflag
     stop
  end if
 return
end subroutine chisq

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine chi2set(set,nused,chi2)
 use database
 implicit none
 integer::set,nused,i,j,k,IR(-ncorra(set):ncorrm(set)),IFAIL
 double precision::chi2,chi2i,error(ncumul(set-1)+1:ncumul(set))
 double precision::A(-ncorra(set):ncorrm(set),-ncorra(set):ncorrm(set)),B(-ncorra(set):ncorrm(set)),r(-ncorra(set):ncorrm(set))
 B=0d0
 A=0d0
 do j=-ncorra(set),ncorrm(set)
    A(j,j) = 1d0
 end do
 do i=ncumul(set-1)+1,ncumul(set)
    if (.not.used(i)) cycle
    error(i) = dsqrt(unc(-1,i)**2d0 + unc(0,i)**2d0 + (unc(1,i)*rescale(i))**2d0)
    do j=-ncorra(set),ncorrm(set)
       if (j<0) then
          B(j) = B(j) + corr(j,i)*(data(i)-theory(i))/error(i)**2d0
       else
          B(j) = B(j) + corr(j,i)*rescale(i)*(data(i)-theory(i))/error(i)**2d0
       end if
       do k=-ncorra(set),ncorrm(set)
          if ((j<0).and.(k<0)) then
             A(j,k) = A(j,k) + corr(j,i)*corr(k,i)/error(i)**2d0
          else if (((j>=0).and.(k<0)).or.((j<0).and.(k>=0))) then
             A(j,k) = A(j,k) + corr(j,i)*corr(k,i)*rescale(i)/error(i)**2d0
          else
             A(j,k) = A(j,k) + corr(j,i)*rescale(i)*corr(k,i)*rescale(i)/error(i)**2d0
          end if
       end do
    end do
 end do
 call deqn(ncorra(set)+1+ncorrm(set),A,ncorra(set)+1+ncorrm(set),IR,IFAIL,1,B)
 if (IFAIL/=0) then
    write(6,*) "CHI2SET: A is singular!"
    stop
 end if
 r = - B
 nused = 0
 chi2 = 0d0
 do i=ncumul(set-1)+1,ncumul(set)
    if (used(i)) then
       nused = nused + 1
    else
       cycle
    end if
    chi2i = data(i) - theory(i)
    do j=-ncorra(set),ncorrm(set)
       if (j<0) then
          chi2i = chi2i + r(j)*corr(j,i)
       else
          chi2i = chi2i + r(j)*corr(j,i)*rescale(i)
       end if
    end do
    chi2i = chi2i/error(i)
    lmcount=lmcount+1
    lmfvec(lmcount)=chi2i
    chi2i = chi2i**2d0
    chi2 = chi2 + chi2i
 end do
 do j = -ncorra(set),ncorrm(set)
    chi2 = chi2 + r(j)**2d0
    lmcount=lmcount+1
    lmfvec(lmcount)=r(j)
 end do
 return
end subroutine chi2set
