!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!  2-Dimensional Linear Interpolation Routine:
!!!    Calculates F(X0,Y0) given array F(X,Y) by two successive
!!!    interpolations, first in X and then in Y
!!!
!!!    Assumes uniform spacing of array in X and Y
!!!
!!!    (NX,NY) = number of points in (X,Y) arrays.
!!!    (DELX,DELY) = size of (evenly spaced) intervals in (X,Y) arrays
!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        DOUBLE PRECISION FUNCTION DINTERP2D(X,Y,F,X0,Y0,NX,NY,DELX,DELY)
        IMPLICIT NONE
        INTEGER NX,NY
        INTEGER I_EXTRAP,I,J
        REAL*8  X(NX),Y(NY),F(NX,NY)
        REAL*8  X0,Y0,DELX,DELY
        REAL*8  RINTX1,RINTX2

        I_EXTRAP = 0
        I = DINT((X0-X(1))/DELX) + 1
        J = DINT((Y0-Y(1))/DELY) + 1
        IF((I+1.GT.NX).OR.(I.LT.1).OR.(J+1.GT.NY).OR.(J.LT.1))THEN
          I_EXTRAP = I_EXTRAP + 1
          DINTERP2D = 0.D0
          RETURN
        ENDIF

        RINTX1 = ((X0-X(I))/(X(I+1)-X(I)))*(F(I+1,J)-F(I,J))
     &                  + F(I,J)
        RINTX2 = ((X0-X(I))/(X(I+1)-X(I)))*(F(I+1,J+1)-F(I,J+1))
     &                  + F(I,J+1)
        DINTERP2D = ((Y0-Y(J))/(Y(J+1)-Y(J)))*(RINTX2-RINTX1) + RINTX1

        RETURN
        END

