!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2005 - 2014 Pedro Jimenez-Delgado
! 
! This file is part of Chromopolis.
! 
! Chromopolis is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
! 
! Chromopolis is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with Chromopolis.  If not, see <http://www.gnu.org/licenses/>.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!! MELLIN SPACE, INVERSE MELLIN TRANSFORMATION  AND SOME ADDITIONAL FUNCTIONS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! It uses a fixed array of points in Mellin space suitable for a (non-adaptative) 8-points Gauss quadrature.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!! USAGE:
!! MSPACE: Initialization, calculation of support points (Mellin space) and weights for the integration.
!! MINVERSION(x,fn,xfx):
!!   x == x-variable.
!!   fn(136) == array containing the n-moments of the quantity to be inverted, f(n).
!!   xfx == result, inverted quantity (times x), x*f(x).
!! Some additional functions acting on the whole set of support points are appended.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine mspace
 implicit none
 integer i,j,k
 double precision c,phi,co,si,sum,diff,z,zs(8),wz(8),down(17),up(17),wn(136)
 double complex cc
 double complex,dimension(0:6,136)::n,ns,nc,nfo,nfi,nsi,nse,nei,nni,np1,np1s,np1c,nm1,nm1s,np2,np2s,np2c,np3,np3s,np4,&
                                    S1,S1s,S2,S3,S4,SPp2,SPm2,SPp3,SPm3,STp,STm,S1m,S1ms,S2m,S3m,S1p,S1ps,S2p,S1p2
 common /contour/ c,cc,wn
 common /support/ n
 common /mspacec/ ns,nc,nfo,nfi,nsi,nse,nei,nni,np1,np1s,np1c,nm1,nm1s,np2,np2s,np2c,np3,np3s,np4,&
                  S1,S1s,S2,S3,S4,SPp2,SPm2,SPp3,SPm3,STp,STm,S1m,S1ms,S2m,S3m,S1p,S1ps,S2p,S1p2
 interface
    function S(index,n)
     implicit none
     integer,intent(in)::index
     double complex,intent(in)::n(0:6,136)
     double complex S(0:6,136)
    end function S
    function SP(eta,index,n)
     implicit none
     integer,intent(in)::eta,index
     double complex,intent(in)::n(0:6,136)
     double complex SP(0:6,136)
    end function SP
   function ST(eta,n)
    implicit none
    integer,intent(in)::eta
    double complex,intent(in)::n(0:6,136)
    double complex ST(0:6,136)
    end function ST
 end interface
!Gauss quadrature and integration contour parameters.
 data wz /0.101228536290376d0,0.222381034453374d0,&
          0.313706645877887d0,0.362683783378362d0,&
          0.362683783378362d0,0.313706645877887d0,&
          0.222381034453374d0,0.101228536290376d0/
 data zs /-0.960289856497536d0,-0.796666477413627d0,&
          -0.525532409916329d0,-0.183434642495650d0,&
           0.183434642495650d0,0.525532409916329d0,&
           0.796666477413627d0,0.960289856497536d0/
 data down /0d0,0.1d0,0.3d0,0.6d0,1.0d0,1.6d0,2.4d0,3.5d0,&
             5d0,7d0,10d0,14d0,19d0,25d0,32d0,40d0,50d0/
 data up  /0.1d0,0.3d0,0.6d0,1.0d0,1.6d0,2.4d0,3.5d0,&
             5d0,7d0,10d0,14d0,19d0,25d0,32d0,40d0,50d0,63d0/
 c = 0.9d0
 phi = 3d0/4d0*3.141592653589793238d0
 co = dcos(phi)
 si = dsin(phi)
 cc = dcmplx(co,si)
!Calculation of support points and weights.
 n = dcmplx(0d0,0d0)
 i = 0
 do j = 1, 17
    sum  = up(j) + down(j)
    diff = up(j) - down(j)
    do k = 1, 8
       i = i + 1
       z = 0.5d0*(sum + diff*zs(k))
       wn(i) = diff/2d0*wz(k)
       n(0,i) = dcmplx(c+co*z+1d0,si*z)
    end do
 end do
 n(2,:) = n(0,:) + 2d0
 n(4,:) = n(0,:) + 4d0
 n(6,:) = n(0,:) + 6d0
!Precalculations for MFACTOR.
 ns = n*n
 nc = n*ns
 nfo = n*nc
 nfi = n*nfo
 nsi = n*nfi
 nse = n*nsi
 nei = n*nse
 nni = n*nei
 np1 = n + 1d0
 np1s = np1*np1
 np1c = np1*np1s
 nm1 = n - 1d0
 nm1s = nm1*nm1
 np2 = n + 2d0
 np2s = np2*np2
 np2c = np2*np2s
 np3 = n + 3d0
 np3s = np3*np3
 np4 = n + 4d0
 S1 = S(1,n)
 S1s = S1*S1
 S2 = S(2,n)
 S3 = S(3,n)
 S4 = S(4,n)
 SPp2 = SP(1,2,n/2d0)
 SPm2 = SP(-1,2,n/2d0)
 SPp3 = SP(1,3,n/2d0)
 SPm3 = SP(-1,3,n/2d0)
 STp = ST(1,n)
 STm = ST(-1,n)
 S1m = S(1,nm1)
 S1ms = S1m*S1m
 S2m = S(2,nm1)
 S3m = S(3,nm1)
 S1p = S(1,np1)
 S1ps = S1p*S1p
 S2p = S(2,np1)
 S1p2 = S(1,np2)
 return
end subroutine mspace

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine minversion(x,fn,xfx)
 implicit none
 integer i
 double precision c,wn(136),x,ax,ex,fz,xfx
 double complex cc,xnm,cex,fn(136)
 double complex n(0:6,136)
 common /contour/ c,cc,wn
 common /support/ n
!x-dependent quantities:
 ax = log(x)
 ex = exp(-c*ax)
!Integration:
 xfx = 0d0
 do i = 1, 136
    xnm = (c-n(0,i)+1d0)*ax
    cex = exp(xnm)/3.141592653589793116d0*cc
    fz = aimag(fn(i)*cex)
    xfx = xfx + wn(i)*fz
 end do
 xfx = xfx*ex
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
function psis(index,n)
 implicit none
 integer,intent(in)::index
 integer i,j
 double complex wpsipg
 double complex,intent(in)::n(0:6,136)
 double complex psis(0:6,136)
 do i = 0,6,2
    do j = 1,136
       psis(i,j) = wpsipg(n(i,j),index)
    end do
 end do
 return
end function psis

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
function S(index,n)
 implicit none
 integer,intent(in)::index
 double complex,intent(in)::n(0:6,136)
 double complex S(0:6,136)
 interface
   function psis(index,n)
    integer,intent(in)::index
    double complex,intent(in)::n(0:6,136)
    double complex psis(0:6,136)
   end function psis
 end interface
select case(index)
 case(1)
    S = 0.5772156649015328606d0 + psis(0,n+1)
 case(2)
    S = 1.6449340668482264365d0 - psis(1,n+1)
 case(3)
    S = 1.2020569031595942854d0 + 0.5*psis(2,n+1)
 case(4)
    S = 1.0823232337111381916d0 + 1d0/3d0*psis(3,n+1)
 case default
    write(*,*) "S: Bad value for index!", index
    stop
 end select
return
end function S

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
function SP(eta,index,n)
 implicit none
 integer,intent(in)::eta,index
 double complex,intent(in)::n(0:6,136)
 double complex SP(0:6,136)
 interface
   function S(index,n)
    implicit none
    integer,intent(in)::index
    double complex,intent(in)::n(0:6,136)
    double complex S(0:6,136)
   end function S
 end interface
 select case(eta)
  case(-1)
   SP = S(index,n-0.5d0)
  case(1)
   SP = S(index,n)
  case default
     write(*,*) "SP: Bad value for eta!", eta
     stop
 end select
 return
end function SP

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
function ST(eta,n)
 implicit none
 integer,intent(in)::eta
 double complex,intent(in)::n(0:6,136)
 double complex ST(0:6,136)
 interface
   function psis(index,n)
    implicit none
    integer,intent(in)::index
    double complex,intent(in)::n(0:6,136)
    double complex psis(0:6,136)
   end function psis
   function S(index,n)
    implicit none
    integer,intent(in)::index
    double complex,intent(in)::n(0:6,136)
    double complex S(0:6,136)
   end function S
 end interface
 if ((eta/=-1).and.(eta/=1)) then
     write(*,*) "ST: Bad value for eta!", eta
     stop
 else
   ST = -5d0/8d0*1.202057d0 + eta*(S(1,n)/n/n - 0.5d0*1.644934d0*(psis(0,(n+1d0)/2d0) - psis(0,n/2d0)) &
                                    + 1.01d0/(n+1d0) - 0.846d0/(n+2d0) + 1.155d0/(n+3d0) - 1.074d0/(n+4d0) +0.55d0/(n+5d0))
 end if
 return
end function ST
