!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C ALL for pi^0 production in pp collisions
C Original codes by M. Hirai et al., PRD74(2006)014015
C Adapted for Chromopolis by P. Jimenez-Delgado, March 2014
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C ---------------------------------------------------------------------
      double precision function ALLpi0(pT,etamin,etamax,sqrts)
C ---------------------------------------------------------------------
      implicit none
      integer NRSTEP,NVWSTEP
      double precision pT,etamin,etamax,sqrts
      double precision PI,SRS,FUNRap
      double precision pol,unpol
      logical polarized 
      COMMON /NOSTEPC/NRSTEP,NVWSTEP
      COMMON /PIC/PI
      COMMON /SRSC/SRS
      common /polarizationc/ polarized

      DATA NRSTEP,NVWSTEP/10,64/ ! Number of steps for int_eta, int_vw
      PI=4D0*DATAN(1D0)
      SRS = sqrts

      CALL GAUSSLEGZ(NVWSTEP)    ! for GAULEG:Gauss integral routine

      polarized = .true.
      CALL PUTAPDF_POL(pT)       ! PDF into SPLINE array
      pol = FUNRap(pT,etamin,etamax)

      polarized = .false.
      CALL PUTAPDF_UNPOL(pT)     ! PDF into SPLINE array
      unpol = FUNRap(pT,etamin,etamax)
      unpol = 1.5d0*unpol

      ALLpi0 = pol/unpol
      
      return
      END
C ---------------------------------------------------------------------
      REAL*8 FUNCTION FUNRap(Pt,etamin,etamax)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)

      DIMENSION X(512),W(512)
      COMMON /NOSTEPC/NRSTEP,NVWSTEP

      NSTEP=NRSTEP

      Xmin=etamin
      Xmax=etamax

      DX=(Xmax-Xmin)/DFLOAT(NSTEP)
      FUNB=FUNZ(Pt,Xmin)
      SUM=0.D0
      DO I=1,NSTEP-1,2
        XE=Xmin+DFLOAT(I)*DX
        XO=XE+DX
        FUNA=FUNZ(Pt,XO)
       SUM=SUM+(FUNB+4.D0*FUNZ(Pt,XE)+FUNA)*DX/3.D0
        FUNB=FUNA
      ENDDO
      FUNRap=SUM

      RETURN
      END
C ---------------------------------------------------------------------
      REAL*8 FUNCTION FUNZ(Pt,Rap)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(512),W(512)
      COMMON /SRSC/SRS
      COMMON /NOSTEPC/NRSTEP,NVWSTEP

      NSTEP=NVWSTEP

      Xt=2.D0*Pt/SRS
      Xmin=Xt*(Dexp(Rap)+Dexp(-Rap))/2.D0
      Xmax=1.D0-1.D-5

      CALL GAULEG(NSTEP,Xmin,Xmax,X,W) ! Gauss-Legendre Integral
      SUMV=0.D0
      DO I=1,NSTEP
        SUMV=SUMV+W(I)*FUNV(Pt,Rap,X(I))
      ENDDO
      FUNZ=SUMV

      RETURN
      END
C ---------------------------------------------------------------------
      REAL*8 FUNCTION FUNV(Pt,Rap,Zc)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION uff(2), dff(2), sff(2), cff(2), bff(2)
     +         ,X(512),W(512)
     +         ,FFHKNS(-5:5),GRAD(-5:5,17)
      COMMON /SRSC/SRS
      COMMON /NOSTEPC/NRSTEP,NVWSTEP
      COMMON /FRAGF/FF(0:10)

      IF(Zc.GT.1.D0) THEN 
        FUNV=0.D0
        RETURN
      END IF

C FF of hkns06 
      CALL HKNSFF(Pt*Pt,Zc,1,3,FFHKNS,GRAD)
      FF(0)=FFHKNS(0)
      FF(1)=FFHKNS(1)
      FF(2)=FFHKNS(-1)
      FF(3)=FFHKNS(2)
      FF(4)=FFHKNS(-2)
      FF(5)=FFHKNS(3)
      FF(6)=FFHKNS(-3)

C FF are KKP Fragmentaion Func: 0:g, 1:u, 2:ub, 3:d, 4:db, 5:s, 6:sb
!       ISET=5  ! 4:p+pb, 5:pi0, 7:h+h-
!       CALL kkp(ISET,0,Zc,Pt,FF)

C FF are Kretzer Fragmentaion Func, 
c      ISET=1     ! ISET=1:LO Pion, 3:LO Kion 
c      ICHARGE=3  ! ICHARGE=1(+), 2(-)
c      CALL PKHFF(ISET,ICHARGE,Zc,Pt*Pt,uff,dff,sff,cff,bff,gff)
c      FF(0)=gff
c      FF(1)=uff(1)
c      FF(2)=uff(2)
c      FF(3)=dff(1)
c      FF(4)=dff(2)
c      FF(5)=sff(1)
c      FF(6)=sff(2)
c      FF=FF/2.D0     ! pi^0, ICHARGE=3[(+)+(-)] needs factor 1/2

      NSTEP=NVWSTEP

      Xt=2.D0*Pt/SRS
      Xmin=Xt*Dexp(Rap)/(2.D0*Zc)
      Xmax=1.D0-Xt*Dexp(-1.D0*Rap)/(2.D0*Zc)

      CALL GAULEG(NSTEP,Xmin,Xmax,X,W) ! Gauss-Legendre Integral
      SUMW=0.D0
      DO I=1,NSTEP
        SUMW=SUMW+W(I)*FUNW(Pt,Rap,Zc,X(I))
      ENDDO
      FUNV=SUMW

      RETURN
      END
C ---------------------------------------------------------------------
      REAL*8 FUNCTION FUNW(Pt,Rap,Zc,V)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON /SRSC/SRS
      COMMON /NOSTEPC/NRSTEP,NVWSTEP
      COMMON /ALP/ALPHAS

      COMMON /NXSTEP/NXSTEP
      COMMON /PBW1/PB(7),PAW1(7)
      COMMON /APDF/X(5000), UPDF(5000,10)
      COMMON /SPDF/BP(5000,10),CP(5000,10),DP(5000,10)

      logical polarized 
      common /polarizationc/ polarized

      NSTEP=NVWSTEP

      Xt=2.D0*Pt/SRS
      Vmin=Xt*Dexp(Rap)/(2.D0*Zc)
      Vmax=1.D0-Xt*Dexp(-1.D0*Rap)/(2.D0*Zc)

      Wmin=Vmin/V
      Xmin=Wmin
      Xmax=1.D0

C PB are PDF(I): 1:u, 2:d, 3:ub, 4:db, 5:s, 6:g 
      Xaw1=Vmin/V
      Xb=(1.D0-Vmax)/(1.D0-V) 
      DO IN=1,6
        PAW1(IN)=SEVAL(NXSTEP+1,Xaw1,X,UPDF,BP,CP,DP,IN)/Xaw1
        PB(IN)=SEVALB(NXSTEP+1,Xb,X,UPDF,BP,CP,DP,IN)/Xb
      ENDDO

      IF(DABS(Xmin-Xmax).LT.1.D-9) THEN  ! Wmin = Wmax 
        FUNW=0.D0
      ELSE                               ! Wmin < Wmax 
        if (polarized) FUNVLO=FUNSUML_POL(Pt,Rap,Zc,V,Xmax)         ! delta(1-W) for LO Xsec
        if (.not.polarized) FUNVLO=FUNSUML_UNPOL(Pt,Rap,Zc,V,Xmax)  ! delta(1-W) for LO Xsec
        FUNW=ALPHAS*ALPHAS*FUNVLO
      ENDIF

      RETURN
      END
C ---------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!! comblib.f !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C ---------------------------------------------------------------------
      SUBROUTINE GETALP(Q2,ALPHA)
C ---------------------------------------------------------------------
C RUNNING COUPLING CONSTANTS.
      IMPLICIT REAL*8(A-H,O-Z)
      external as
      COMMON /PIC/PI

      ALPHA = 4d0*PI*AS(Q2)       

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE SSPLINE(N,X,Y,B,C,D,IN)
C ----
C CALCULATE THE COEFFICIENTS B,C,D IN A CUBIC SPLINE INTERPOLATION.
C INTERPOLATION SUBROUTINES ARE TAKEN FROM 
C G.E. FORSYTHE, M.A. MALCOLM AND C.B. MOLER,
C COMPUTER METHODS FOR MATHEMATICAL COMPUTATIONS (PRENTICE-HALL, 1977).
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(5000),Y(5000,10),B(5000,10),C(5000,10),D(5000,10)

      NM1=N-1
      IF(N.LT.2) RETURN
      IF(N.LT.3) GO TO 250

      D(1,IN)=X(2)-X(1)
      C(2,IN)=(Y(2,IN)-Y(1,IN))/D(1,IN)
      DO I=2,NM1
        D(I,IN)=X(I+1)-X(I)
        B(I,IN)=2.0D0*(D(I-1,IN)+D(I,IN))
        C(I+1,IN)=(Y(I+1,IN)-Y(I,IN))/D(I,IN)
        C(I,IN)=C(I+1,IN)-C(I,IN)
      END DO

      B(1,IN)=-D(1,IN)
      B(N,IN)=-D(N-1,IN)
      C(1,IN)=0.0D0
      C(N,IN)=0.0D0

      IF(N.EQ.3) GO TO 215
      C(1,IN)=C(3,IN)/(X(4)-X(2))-C(2,IN)/(X(3)-X(1))
      C(1,IN)=C(1,IN)*D(1,IN)**2.0D0/(X(4)-X(1))
      C(N,IN)=C(N-1,IN)/(X(N)-X(N-2))-C(N-2,IN)/(X(N-1)-X(N-3))
      C(N,IN)=-C(N,IN)*D(N-1,IN)**2.0D0/(X(N)-X(N-3))
  215 CONTINUE

      DO I=2,N
        T=D(I-1,IN)/B(I-1,IN)
        B(I,IN)=B(I,IN)-T*D(I-1,IN)
        C(I,IN)=C(I,IN)-T*C(I-1,IN)
      END DO
      C(N,IN)=C(N,IN)/B(N,IN)
      DO IB=1,NM1
        I=N-IB
        C(I,IN)=(C(I,IN)-D(I,IN)*C(I+1,IN))/B(I,IN)
      END DO

      B(N,IN)=(Y(N,IN)-Y(NM1,IN))/D(NM1,IN)
     +        +D(NM1,IN)*(C(NM1,IN)+2.0D0*C(N,IN))
      DO I=1,NM1
        B(I,IN)=(Y(I+1,IN)-Y(I,IN))/D(I,IN)
     +          -D(I,IN)*(C(I+1,IN)+2.0D0*C(I,IN))
        D(I,IN)=(C(I+1,IN)-C(I,IN))/D(I,IN)
        C(I,IN)=3.0D0*C(I,IN)
      END DO

      C(N,IN)=3.0D0*C(N,IN)
      D(N,IN)=D(N-1,IN)
      RETURN

  250 CONTINUE
      B(1,IN)=(Y(2,IN)-Y(1,IN))/(X(2)-X(1))
      C(1,IN)=0.0D0
      D(1,IN)=0.0D0
      B(2,IN)=B(1,IN)
      C(2,IN)=0.0D0
      D(2,IN)=0.0D0

      RETURN
      END
C ---------------------------------------------------------------------
      REAL*8 FUNCTION SEVAL(N,XX,X,Y,B,C,D,IN)
C ----
C CALCULATE THE DISTRIBUTION AT XX BY CUBIC SPLINE INTERPOLATION.
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z) 
      DIMENSION X(5000),Y(5000,10),B(5000,10),C(5000,10),D(5000,10)
      DATA I/1/
      
      IF(I.GE.N) I=1
      IF(XX.LT.X(I)) GO TO 310
      IF(XX.LE.X(I+1)) GO TO 330

  310 CONTINUE

      I=1
      J=N+1

  320 CONTINUE
      K=(I+J)/2
      IF(XX.LT.X(K)) J=K
      IF(XX.GE.X(K)) I=K
      IF(J.GT.I+1) GO TO 320

  330 CONTINUE
      DX=XX-X(I)
      SEVAL=Y(I,IN)+DX*(B(I,IN)+DX*(C(I,IN)+DX*D(I,IN)))

      RETURN
      END
C ---------------------------------------------------------------------
      REAL*8 FUNCTION SEVALB(N,XX,X,Y,B,C,D,IN)
C ----
C CALCULATE THE DISTRIBUTION AT XX BY CUBIC SPLINE INTERPOLATION.
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z) 
      DIMENSION X(5000),Y(5000,10),B(5000,10),C(5000,10),D(5000,10)
      DATA I/1/

      IF(I.GE.N) I=1
      IF(XX.LT.X(I)) GO TO 310
      IF(XX.LE.X(I+1)) GO TO 330

  310 CONTINUE
      I=1
      J=N+1

  320 CONTINUE
      K=(I+J)/2
      IF(XX.LT.X(K)) J=K
      IF(XX.GE.X(K)) I=K
      IF(J.GT.I+1) GO TO 320

  330 CONTINUE
      DX=XX-X(I)
      SEVALB=Y(I,IN)+DX*(B(I,IN)+DX*(C(I,IN)+DX*D(I,IN)))

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE GAUSSLEGZ(N)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      DATA EPS/1.D-14/
      COMMON/GAUS2L/GLZ(1024),GLP2(1024)

      m=(n+1)/2
      DO i=1,m
        z=cos(3.141592654d0*(i-0.25d0)/(n+0.5d0))
1       continue
          p1=1.d0
          p2=0.d0
          DO j=1,n
            p3=p2
            p2=p1
            p1=((2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
          ENDDO
          pp=n*(z*p1-p2)/(z*z-1.d0)
          z1=z
          z=z1-p1/pp
        if(abs(z-z1).gt.EPS) goto 1
        GLZ(I)=z
        GLP2(I)=pp
      ENDDO

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE GAULEG(N,x1,x2,X,W)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      DATA EPS/1.D-14/
      COMMON/GAUS2L/GLZ(1024),GLP2(1024)
      REAL*8 X(N),W(N)

      m=(n+1)/2
      xm=0.5d0*(x2+x1)
      xl=0.5d0*(x2-x1)
      DO i=1,m
        z=GLZ(i)
        pp=GLP2(i)
        x(i)=xm-xl*z
        x(n+1-i)=xm+xl*z
        w(i)=2.d0*xl/((1.d0-z*z)*pp*pp)
        w(n+1-i)=w(i)
      ENDDO

      RETURN
      END
C ---------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!! subxsec_pol !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C ---------------------------------------------------------------------
      REAL*8 FUNCTION FUNSUML_POL(Pt,Rap,Zc,V,W)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION PA(7)

      COMMON /PIC/PI
      COMMON /PBW1/PB(7),PAW1(7)
      COMMON /FRAGF/FF(0:10)

C PA, PB are PDF(I): 1:u, 2:d, 3:ub, 4:db, 5:s, 6:g
      DO IN=1,6
        PA(IN)=PAW1(IN)
      ENDDO

C FF are KKP Fragmentaion Func: 0:g, 1:u, 2:ub, 3:d, 4:db, 5:s, 6:sb

C q q' -> q q' (AB<->BA)
      pxqQqQAB=4.D0/9.D0*((1.D0-V*V*W*W)/(V-1.D0)**2.D0)
      SUMqQAB=(PA(1)*(PB(2)+PB(5))*FF(1))
     +       +(PA(2)*(PB(1)+PB(5))*FF(3))
     +       +(PA(5)*(PB(1)+PB(2))*FF(5))
     +       +(PA(3)*(PB(4)+PB(5))*FF(2))
     +       +(PA(4)*(PB(3)+PB(5))*FF(4))
     +       +(PA(5)*(PB(3)+PB(4))*FF(6))

      pxqQqQBA=4.D0/9.D0*(1.D0-(V-1.D0)**2.D0)/(V*V*W*W)
      SUMqQBA=(PB(1)*(PA(2)+PA(5))*FF(1))
     +       +(PB(2)*(PA(1)+PA(5))*FF(3))
     +       +(PB(5)*(PA(1)+PA(2))*FF(5))
     +       +(PB(3)*(PA(4)+PA(5))*FF(2))
     +       +(PB(4)*(PA(3)+PA(5))*FF(4))
     +       +(PB(5)*(PA(3)+PA(4))*FF(6))

      FUNqQq=SUMqQAB*pxqQqQAB+SUMqQBA*pxqQqQBA


C q qb' -> q qb' (AB<->BA) 
      SUMqQbAB=(PA(1)*(PB(4)+PB(5))*FF(1))
     +        +(PA(2)*(PB(3)+PB(5))*FF(3))
     +        +(PA(5)*(PB(3)+PB(4))*FF(5))
     +        +(PA(3)*(PB(2)+PB(5))*FF(2))
     +        +(PA(4)*(PB(1)+PB(5))*FF(4))
     +        +(PA(5)*(PB(1)+PB(2))*FF(6))

      SUMqQbBA=(PB(1)*(PA(4)+PA(5))*FF(1))
     +        +(PB(2)*(PA(3)+PA(5))*FF(3))
     +        +(PB(5)*(PA(3)+PA(4))*FF(5))
     +        +(PB(3)*(PA(2)+PA(5))*FF(2))
     +        +(PB(4)*(PA(1)+PA(5))*FF(4))
     +        +(PB(5)*(PA(1)+PA(2))*FF(6))

      FUNqQbq=SUMqQbAB*pxqQqQAB+SUMqQbBA*pxqQqQBA

C q qb -> q' qb'
      pxqqbQQb=-4.D0/9.D0*(V*V*W*W+(V-1.D0)**2.D0)
      SUMqqb=(PA(1)*PB(3)+PA(3)*PB(1))*(FF(3)+FF(4)+FF(5)+FF(6))
     +      +(PA(2)*PB(4)+PA(4)*PB(2))*(FF(1)+FF(2)+FF(5)+FF(6))
     +      +(PA(5)*PB(5)+PA(5)*PB(5))*(FF(1)+FF(2)+FF(3)+FF(4))
      FUNqqbQQb=SUMqqb*pxqqbQQb

C q q -> q q (qb qb -> qb qb)
      pxqq2=4.D0/9.D0*((1.D0-V*V*W*W)/(V-1.D0)**2.D0
     +                +(1.D0-(V-1.D0)**2.D0)/(V*V*W*W)
     +                +2.D0/(3.D0*V*W*(V-1.D0)))
      SUMqq2=PA(1)*PB(1)*FF(1)
     +      +PA(2)*PB(2)*FF(3)
     +      +PA(5)*PB(5)*FF(5)
     +      +PA(3)*PB(3)*FF(2)
     +      +PA(4)*PB(4)*FF(4)
     +      +PA(5)*PB(5)*FF(6)
      FUNqq2=SUMqq2*pxqq2

C q qb -> q qb (AB<->BA)
      pxqqb2AB=4.D0/9.D0*((1.D0-V*V*W*W)/(V-1.D0)**2.D0
     +                   -(V*V*W*W)-(V-1.D0)**2.D0
     +                   +2.D0*V*V*W*W/(3.D0*(V-1.D0)))
      SUMqqb2AB=PA(1)*PB(3)*FF(1)
     +         +PA(2)*PB(4)*FF(3)
     +         +PA(5)*PB(5)*FF(5)
     +         +PA(3)*PB(1)*FF(2)
     +         +PA(4)*PB(2)*FF(4)
     +         +PA(5)*PB(5)*FF(6)

      pxqqb2BA=4.D0/9.D0*((1.D0-(V-1.D0)**2.D0)/(V*V*W*W)
     +                   -(V-1.D0)**2.D0-(V*V*W*W)
     +                   -2.D0*(V-1.D0)**2.D0/(3.D0*V*W))
      SUMqqb2BA=PB(1)*PA(3)*FF(1)
     +         +PB(2)*PA(4)*FF(3)
     +         +PB(5)*PA(5)*FF(5)
     +         +PB(3)*PA(1)*FF(2)
     +         +PB(4)*PA(2)*FF(4)
     +         +PB(5)*PA(5)*FF(6)

      FUNqqb2=SUMqqb2AB*pxqqb2AB+SUMqqb2BA*pxqqb2BA


C q qb -> gg
      pxqqbgg=8.D0/3.D0
     +      *(4.D0/9.D0*(V*V*W*W+(V-1.D0)**2.D0)/(V*W*(V-1.D0))
     +        +V*V*W*W+(V-1.D0)**2.D0)
      SUMqqbgg=(PA(1)*PB(3)+PA(3)*PB(1)
     +         +PA(2)*PB(4)+PA(4)*PB(2)
     +         +PA(5)*PB(5)+PA(5)*PB(5))*FF(0)
      FUNqqbgg=SUMqqbgg*pxqqbgg


C q g -> q g (AB<->BA)
      pxqg2AB=(1.D0-V*V*W*W)*(1.D0/(V-1.D0)**2.D0+4.D0/(9.D0*V*W))
      SUMqg2AB=(PA(1)*FF(1)                     ! q_A*g_B*FF_q
     +        + PA(2)*FF(3)
     +        + PA(5)*FF(5)
     +        + PA(3)*FF(2)
     +        + PA(4)*FF(4)
     +        + PA(5)*FF(6))*PB(6)
     +        +(PB(1)+PB(2)+PB(5)
     +        + PB(3)+PB(4)+PB(5))*PA(6)*FF(0)  ! q_B*g_A*FF_g

      pxqg2BA=(1.D0-(V-1.D0)**2.D0)
     +        *(1.D0/(V*V*W*W)-4.D0/(9.D0*(V-1.D0)))
      SUMqg2BA=(PB(1)*FF(1)                    ! q_B*g_A*FF_q
     +        + PB(2)*FF(3)
     +        + PB(5)*FF(5)
     +        + PB(3)*FF(2)
     +        + PB(4)*FF(4)
     +        + PB(5)*FF(6))*PA(6)
     +        +(PA(1)+PA(2)+PA(5)
     +        + PA(3)+PA(4)+PA(5))*PB(6)*FF(0)  ! q_A*g_B*FF_g

      FUNqg2=SUMqg2AB*pxqg2AB+SUMqg2BA*pxqg2BA

C g g  -> g g
      pxgg2=-9.D0/2.D0*(3.D0+2.D0/(V*W*(V-1.D0))+V*W*(V-1.D0))

      SUMgg2=PA(6)*PB(6)*FF(0)
      FUNgg2=SUMgg2*pxgg2

C g g  -> q qb
      pxggqqb=3.D0/8.D0
     +        *(4.D0/9.D0*(V*V*W*W+(V-1.D0)**2.D0)/(V*W*(V-1.D0))
     +          +V*V*W*W+(V-1.D0)**2.D0)
      SUMggqqb=PA(6)*PB(6)*(FF(1)+FF(2)+FF(3)+FF(4)+FF(5)+FF(6))
      FUNggqqb=SUMggqqb*pxggqqb

C --- LO
      FUNSUML_POL=0.D0
     +       +FUNqQq
     +       +FUNqQbq
     +       +FUNqqbQQb
     +       +FUNqq2
     +       +FUNqqb2
     +       +FUNqqbgg
     +       +FUNqg2
     +       +FUNgg2
     +       +FUNggqqb

      FUNSUML_POL=PI*FUNSUML_POL

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE PUTAPDF_POL(Pt)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z) 
      PARAMETER(Xmin=1.D-3)
      DIMENSION POLPDF(-3:3), GRAD(-3:3,11), xpa(-6:6)

      COMMON /ALP/ALPHAS

      COMMON /NXSTEP/NXSTEP
      COMMON /APDF/X(5000), UPDF(5000,10)
      COMMON /SPDF/BP(5000,10),CP(5000,10),DP(5000,10)

      NXSTEP=1000
      Q2=Pt*Pt

      CALL GETALP(Q2,ALPHAS)

      DO I=1,NXSTEP+1      ! INITIALIZING THE ARRAYS X & UPDF
        X(I)=0.D0
        DO IN=1,10
          UPDF(I,IN)=0.D0
        ENDDO
      ENDDO

      ! Log scale
      DX=-1.D0*DLOG10(Xmin)/NXSTEP
      DO I=1,NXSTEP
        DLOGX=DLOG10(Xmin)+DFLOAT(I-1)*DX
        X(I)=10.D0**(DLOGX)

      ! Linear scale
c      DX=(1.D0-Xmin)/NXSTEP
c      DO I=1,NXSTEP
c        X(I)=Xmin+DFLOAT(I-1)*DX

C ---- Definition of the pol-PDF (multiplied by x: xPDF) ----

        call ppdfs(X(I),Q2,xpa)
        UPDF(I,1)= xpa(1) + xpa(-1)
        UPDF(I,2)= xpa(2) + xpa(-2)
        UPDF(I,3)= xpa(-1)
        UPDF(I,4)= xpa(-2)
        UPDF(I,5)= xpa(-3)
        UPDF(I,6)= xpa(0)

C Polarized PDFs AAC
c        ISET=2 ! ISET [1:positive, 2:node]
c        CALL AAC08PDF(Q2,X(I),ISET,POLPDF,GRAD)
c        UPDF(I,1)=POLPDF(1)
c        UPDF(I,2)=POLPDF(2)
c        UPDF(I,3)=POLPDF(-1)
c        UPDF(I,4)=POLPDF(-2)
c        UPDF(I,5)=POLPDF(3)
c        UPDF(I,6)=POLPDF(0)

C Polarized PDF's GRSV01
c        CALL PARPOL (ISET,X(I),Q2,U,D,UB,DB,ST,GL,G1P,G1N)
c        UPDF(I,1)=U
c        UPDF(I,2)=D
c        UPDF(I,3)=UB
c        UPDF(I,4)=DB
c        UPDF(I,5)=ST
c        UPDF(I,6)=GL

      ENDDO

      DO IN=1,6
        CALL SSPLINE(NXSTEP+1,X,UPDF,BP,CP,DP,IN)
      END DO

      RETURN
      END
C ---------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!! subxsec_unpol !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C ---------------------------------------------------------------------
      REAL*8 FUNCTION FUNSUML_UNPOL(Pt,Rap,Zc,V,W)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION PA(7)

      COMMON /PIC/PI
      COMMON /PBW1/PB(7),PAW1(7)
      COMMON /FRAGF/FF(0:10)

C PA, PB are PDF(I): 1:u, 2:d, 3:ub, 4:db, 5:s, 6:g
      DO IN=1,6
        PA(IN)=PAW1(IN)
      ENDDO

C FF are KKP Fragmentaion Func: 0:g, 1:u, 2:ub, 3:d, 4:db, 5:s, 6:sb

C q q' -> q q' (AB<->BA)
      pxqQqQAB=4.D0/9.D0*((1.D0+V*V*W*W)/(V-1.D0)**2.D0)
      SUMqQAB=(PA(1)*(PB(2)+PB(5))*FF(1))
     +       +(PA(2)*(PB(1)+PB(5))*FF(3))
     +       +(PA(5)*(PB(1)+PB(2))*FF(5))
     +       +(PA(3)*(PB(4)+PB(5))*FF(2))
     +       +(PA(4)*(PB(3)+PB(5))*FF(4))
     +       +(PA(5)*(PB(3)+PB(4))*FF(6))

      pxqQqQBA=4.D0/9.D0*(1.D0+(V-1.D0)**2.D0)/(V*V*W*W)
      SUMqQBA=(PB(1)*(PA(2)+PA(5))*FF(1))
     +       +(PB(2)*(PA(1)+PA(5))*FF(3))
     +       +(PB(5)*(PA(1)+PA(2))*FF(5))
     +       +(PB(3)*(PA(4)+PA(5))*FF(2))
     +       +(PB(4)*(PA(3)+PA(5))*FF(4))
     +       +(PB(5)*(PA(3)+PA(4))*FF(6))

      FUNqQq=SUMqQAB*pxqQqQAB+SUMqQBA*pxqQqQBA


C q qb' -> q qb' (AB<->BA) 
      SUMqQbAB=(PA(1)*(PB(4)+PB(5))*FF(1))
     +        +(PA(2)*(PB(3)+PB(5))*FF(3))
     +        +(PA(5)*(PB(3)+PB(4))*FF(5))
     +        +(PA(3)*(PB(2)+PB(5))*FF(2))
     +        +(PA(4)*(PB(1)+PB(5))*FF(4))
     +        +(PA(5)*(PB(1)+PB(2))*FF(6))

      SUMqQbBA=(PB(1)*(PA(4)+PA(5))*FF(1))
     +        +(PB(2)*(PA(3)+PA(5))*FF(3))
     +        +(PB(5)*(PA(3)+PA(4))*FF(5))
     +        +(PB(3)*(PA(2)+PA(5))*FF(2))
     +        +(PB(4)*(PA(1)+PA(5))*FF(4))
     +        +(PB(5)*(PA(1)+PA(2))*FF(6))

      FUNqQbq=SUMqQbAB*pxqQqQAB+SUMqQbBA*pxqQqQBA

C q qb -> q' qb'
      pxqqbQQb= 4.D0/9.D0*(V*V*W*W+(V-1.D0)**2.D0)
      SUMqqb=(PA(1)*PB(3)+PA(3)*PB(1))*(FF(3)+FF(4)+FF(5)+FF(6))
     +      +(PA(2)*PB(4)+PA(4)*PB(2))*(FF(1)+FF(2)+FF(5)+FF(6))
     +      +(PA(5)*PB(5)+PA(5)*PB(5))*(FF(1)+FF(2)+FF(3)+FF(4))
      FUNqqbQQb=SUMqqb*pxqqbQQb

C q q -> q q (qb qb -> qb qb)
      pxqq2=4.D0/9.D0*((1.D0+V*V*W*W)/(V-1.D0)**2.D0
     +                +(1.D0+(V-1.D0)**2.D0)/(V*V*W*W)
     +                +2.D0/(3.D0*V*W*(V-1.D0)))
      SUMqq2=PA(1)*PB(1)*FF(1)
     +      +PA(2)*PB(2)*FF(3)
     +      +PA(5)*PB(5)*FF(5)
     +      +PA(3)*PB(3)*FF(2)
     +      +PA(4)*PB(4)*FF(4)
     +      +PA(5)*PB(5)*FF(6)
      FUNqq2=SUMqq2*pxqq2

C q qb -> q qb (AB<->BA)
      pxqqb2AB=4.D0/9.D0*((1.D0+V*V*W*W)/(V-1.D0)**2.D0
     +                   +(V*V*W*W)+(V-1.D0)**2.D0
     +                   -2.D0*V*V*W*W/(3.D0*(V-1.D0)))
      SUMqqb2AB=PA(1)*PB(3)*FF(1)
     +         +PA(2)*PB(4)*FF(3)
     +         +PA(5)*PB(5)*FF(5)
     +         +PA(3)*PB(1)*FF(2)
     +         +PA(4)*PB(2)*FF(4)
     +         +PA(5)*PB(5)*FF(6)

      pxqqb2BA=4.D0/9.D0*((1.D0+(V-1.D0)**2.D0)/(V*V*W*W)
     +                   +(V-1.D0)**2.D0+(V*V*W*W)
     +                   +2.D0*(V-1.D0)**2.D0/(3.D0*V*W))
      SUMqqb2BA=PB(1)*PA(3)*FF(1)
     +         +PB(2)*PA(4)*FF(3)
     +         +PB(5)*PA(5)*FF(5)
     +         +PB(3)*PA(1)*FF(2)
     +         +PB(4)*PA(2)*FF(4)
     +         +PB(5)*PA(5)*FF(6)

      FUNqqb2=SUMqqb2AB*pxqqb2AB+SUMqqb2BA*pxqqb2BA


C q qb -> gg
      pxqqbgg=-8.D0/3.D0
     +      *(4.D0/9.D0*(V*V*W*W+(V-1.D0)**2.D0)/(V*W*(V-1.D0))
     +        +V*V*W*W+(V-1.D0)**2.D0)
      SUMqqbgg=(PA(1)*PB(3)+PA(3)*PB(1)
     +         +PA(2)*PB(4)+PA(4)*PB(2)
     +         +PA(5)*PB(5)+PA(5)*PB(5))*FF(0)
      FUNqqbgg=SUMqqbgg*pxqqbgg


C q g -> q g (AB<->BA)
      pxqg2AB=(V*V*W*W+1.D0)*(1.D0/(V-1.D0)**2.D0+4.D0/(9.D0*V*W))
      SUMqg2AB=(PA(1)*FF(1)                     ! q_A*g_B*FF_q
     +        + PA(2)*FF(3)
     +        + PA(5)*FF(5)
     +        + PA(3)*FF(2)
     +        + PA(4)*FF(4)
     +        + PA(5)*FF(6))*PB(6)
     +        +(PB(1)+PB(2)+PB(5)
     +        + PB(3)+PB(4)+PB(5))*PA(6)*FF(0)  ! q_B*g_A*FF_g

      pxqg2BA=((V-1.D0)**2.D0+1.D0)
     +        *(1.D0/(V*V*W*W)-4.D0/(9.D0*(V-1.D0)))
      SUMqg2BA=(PB(1)*FF(1)                    ! q_B*g_A*FF_q
     +        + PB(2)*FF(3)
     +        + PB(5)*FF(5)
     +        + PB(3)*FF(2)
     +        + PB(4)*FF(4)
     +        + PB(5)*FF(6))*PA(6)
     +        +(PA(1)+PA(2)+PA(5)
     +        + PA(3)+PA(4)+PA(5))*PB(6)*FF(0)  ! q_A*g_B*FF_g

      FUNqg2=SUMqg2AB*pxqg2AB+SUMqg2BA*pxqg2BA

C g g  -> g g
      pxgg2=9.D0/2.D0*(3.D0+V*W*(V-1.D0)+V*W/(V-1.D0)**2.D0
     +                -(V-1.D0)/(V*V*W*W))
      SUMgg2=PA(6)*PB(6)*FF(0)
      FUNgg2=SUMgg2*pxgg2

C g g  -> q qb
      pxggqqb=-3.D0/8.D0
     +        *(4.D0/9.D0*(V*V*W*W+(V-1.D0)**2.D0)/(V*W*(V-1.D0))
     +          +V*V*W*W+(V-1.D0)**2.D0)
      SUMggqqb=PA(6)*PB(6)*(FF(1)+FF(2)+FF(3)+FF(4)+FF(5)+FF(6))
      FUNggqqb=SUMggqqb*pxggqqb

C --- LO
      FUNSUML_UNPOL=0.D0
     +       +FUNqQq
     +       +FUNqQbq
     +       +FUNqqbQQb
     +       +FUNqq2
     +       +FUNqqb2
     +       +FUNqqbgg
     +       +FUNqg2
     +       +FUNgg2
     +       +FUNggqqb

      FUNSUML_UNPOL=PI*FUNSUML_UNPOL

      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE PUTAPDF_UNPOL(Pt)
C ---------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z) 
      PARAMETER(Xmin=1.D-3)
      DIMENSION xpa(-6:6)

      COMMON /ALP/ALPHAS

      COMMON /NXSTEP/NXSTEP
      COMMON /APDF/X(5000),UPDF(5000,10)
      COMMON /SPDF/BP(5000,10),CP(5000,10),DP(5000,10)

      NXSTEP=1000
      Q2=Pt*Pt

      CALL GETALP(Q2,ALPHAS)

      DO I=1,NXSTEP+1      ! INITIALIZING THE ARRAYS X & UPDF
        X(I)=0.D0
        DO IN=1,10
          UPDF(I,IN)=0.D0
        ENDDO
      ENDDO
      
  ! Log scale
      DX=-1.D0*DLOG10(Xmin)/NXSTEP
      DO I=1,NXSTEP
        DLOGX=DLOG10(Xmin)+DFLOAT(I-1)*DX
        X(I)=10.D0**(DLOGX)
        
  ! Linear scale
c      DX=(1.D0-Xmin)/NXSTEP
c      DO I=1,NXSTEP
c        X(I)=Xmin+DFLOAT(I-1)*DX

        call pdfs(X(I),Q2,xpa)
        UPDF(I,1)= xpa(1) + xpa(-1)
        UPDF(I,2)= xpa(2) + xpa(-2)
        UPDF(I,3)= xpa(-1)
        UPDF(I,4)= xpa(-2)
        UPDF(I,5)= xpa(-3)
        UPDF(I,6)= xpa(0)

C unpolarized PDFs : GRV98
c        CALL GRV98PA (1,X(I),Q2,UV,DV,US,DS,SS,GL)
c        UPDF(I,1)=UV+US
c        UPDF(I,2)=DV+DS
c        UPDF(I,3)=US
c        UPDF(I,4)=DS
c        UPDF(I,5)=SS
c        UPDF(I,6)=GL

C unpolarized PDFs : CTEQ6
C        Call SetCtq6(ISET)
C        UPDF(I,1)=Ctq6Pdf(1,X(I),Pt)*X(I)
C        UPDF(I,2)=Ctq6Pdf(2,X(I),Pt)*X(I)
C        UPDF(I,3)=Ctq6Pdf(-1,X(I),Pt)*X(I)
C        UPDF(I,4)=Ctq6Pdf(-2,X(I),Pt)*X(I)
C        UPDF(I,5)=Ctq6Pdf(3,X(I),Pt)*X(I)
C        UPDF(I,6)=Ctq6Pdf(0,X(I),Pt)*X(I)

      ENDDO
      
      DO IN=1,6
        CALL SSPLINE(NXSTEP+1,X,UPDF,BP,CP,DP,IN)
      END DO

      RETURN
      END
C ---------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
