!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!! FFNS CHARM CONTRIBUTIONS TO CC DIS STRUCTURE FUNCTIONS !!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! It uses the exact coefficient functions up to NLO (Kretzer et al.)
!!!  In addition it includes the MSbar running mass definition as
!!!  implemented in Alekhin, Moch, PLB699(2011)345
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!   nfunction: 1(FT=2xF1), 2(F2), 3(F3)
!!!   hadron == 1(proton), 50 (steel of NuTeV and CCFR)
!!!   boson == 1(W+), -1(W-)
!!!   order == Order of pQCD calculation = 0(LO), 1(NLO) 
!!!   mcharm == mass (GeV) of the heavy quark 
!!!   x == Bjorken-x 
!!!   Q2 == Q**2 (GeV**2)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      real*8 function FCCcharm(nfunction,hadron,boson,order,mcharm,x,Q2)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      integer nfunction,hadron,ihadron,boson,input,order
      double precision mcharm,x,Q2
      common /QCDPARCC/ cf
      common /CONSTANTSCC/ pi,Vcd,Vcs
      common /MASSESCC/ mc,mb
      common /FORALPSRENORMCC/ rscale,kordhq
      common /forpdfsetCC/ kfnscheme
      common /forschemedefCC/ msbarm
      logical msbarm
      external FTCCcharmi,F2CCcharmi,F3CCcharmi
      common /forfheavyCC/ xb0,q2ss,rm2,rmu2,an,rq,nb0,nq0
      common /hadroncCC/ ihadron      
      common /inputc/ input 
      data pi /3.1415927d0/
      data cf /1.33333d0/
      data Vcd,Vcs /0.2252d0,0.97345d0/
      ihadron = hadron
      if (abs(boson)/=1) then
         write(*,*) "FCCCHARM: Bad value for boson = ",boson
         stop
      end if
      kordhq=order
      !Use MSbar running mass definition (true=yes, false=not)
      msbarm=.true.
      !Flavor number scheme of alphas (negative=variable, 0=3-flavour, 1=4-flavour, 2=5-flavour)
      !It controls the number of flavors in the QCD beta-function for the MSbar running mass
      !definition (see nfloopsCC; not in the decoupling coefficients)
      kfnscheme=-1
      !Charm mass and charge (for nfloopsCC only)
      mc=mcharm
!      if (input==3) msbarm=.false.
      !Bottom mass and charge (for nfloopsCC only)
      mb=4.2d0
      !Charm mass for the rest
      rm2=mcharm**2d0
      !The factorization scale
      hqscale1=1d0
      hqscale2=1d0
      rmu2=Q2*hqscale1+rm2*hqscale2
      !The number of fermions in the strong coupling constant.
      nfe=nfloopsCC(Q2,kfnscheme)
      !Set to ren.scale/fac.scale used
      rscale=1d0    
      alr=-log(rscale)
      bet0=(11-2./3.*nfe)*alr
      rfac0=1-an*bet0
      !Strong coupling constant
      an=xqgCC(0,0.1d0,rmu2)/4./pi*rfac0     
      !Kinematics
      xb=x
      rq=Q2/(Q2+rm2)
      xi=min(xb/rq,1d0)
      xb0=xb
      q2ss=Q2
      !Beam or vector boson
      nb=boson
      nb0=nb
      !The decoupling constants for nf=3,4
      data d1dec /1.33333333d0/
      ! Default number of intervals for the integrations 
      data nfthq, nf2hq, nf3hq /10, 10, 10/ !Precision for total chi2: 1=10-3(35s),5=10-4(47s),10=10-5(60s)
      ! Step for calculation of the PDF derivatives (appears in MSbar running mass definition)
      data delder /0.01d0/
      select case(nfunction)
      case(1)
         !The LO term
         FCCcharm=2*xb*f2cqqpmCC(nb,xi,rmu2)/xi
         !The NLO contribution
         if (kordhq.ge.1.and.xi.lt.1d0) then
          CALL DGAUSS(FTCCcharmi,xi,1d0,nfthq,f2c,EPS)
          fac=2*xb*2*an
          ak=1./rq*(1-rq)*log(1-rq)
          dc=cf*(-4-0.5/rq-pi**2/3.-(1+3*rq)/2./rq*ak
     +    +(xi+xi**2/2.+2*log(1-xi))*log((Q2+rm2)/rmu2)
     +    +0.5*(rq*xi-rq**2*xi+log(1-rq*xi)*(1-rq*xi))/rq**2/(1-rq*xi)
     -    -2*(-(log(1-xi))**2+ddilog((1-rq*xi)/(1-rq))-ddilog(1./(1-rq))
     +    +log(1-rq*xi)*log(rq*(1-xi)/(1-rq)))
     -    -2*log(1-xi))*f2cqqpmCC(nb,xi,rmu2)/xi
          FCCcharm=FCCcharm+(f2c+dc)*fac
          !The term appearing for the MS-bar definition of the heavy-quark mass
          if(msbarm) then
             xip=xi*(1+delder)
             xim=xi*(1-delder)
             ftxp=f2cqqpmCC(nb0,xip,rmu2)/xip
             ftxm=f2cqqpmCC(nb0,xim,rmu2)/xim
             dftx=(ftxp-ftxm)/2./delder/xi
             FCCcharm=FCCcharm + 4*d1dec*dftx*(xi-xb)*fac
          end if
        end if
        if (kordhq.ge.2) then
           write(*,*) "F2CCCHARM: Bad value for order = ",kordhq
           stop
        end if
      case(2)
        !The LO term
        FCCcharm=2*f2cqqpmCC(nb,xi,rmu2) 
        !The NLO contribution
        if (kordhq.ge.1.and.xi.lt.1d0) then              
          CALL DGAUSS(F2CCcharmi,xi,1d0,nf2hq,f2c,EPS)
          fac=2*xi*2*an
          ak=1./rq*(1-rq)*log(1-rq)
          dc=cf*(-4-0.5/rq-pi**2/3.-(1+3*rq)/2./rq*ak + ak
     +    +(xi+xi**2/2.+2*log(1-xi))*log((Q2+rm2)/rmu2)
     +    +0.5*(rq*xi-rq**2*xi+log(1-rq*xi)*(1-rq*xi))/rq**2/(1-rq*xi)
     -    -2*(-(log(1-xi))**2+ddilog((1-rq*xi)/(1-rq))-ddilog(1./(1-rq))
     +    +log(1-rq*xi)*log(rq*(1-xi)/(1-rq)))
     -    -2*log(1-rq*xi)/rq
     +    +2*(1-rq)/rq*log(1-rq))*f2cqqpmCC(nb,xi,rmu2)/xi
          FCCcharm=FCCcharm+(f2c+dc)*fac
          !The term appearing for the MS-bar definition of the heavy-quark mass
          if(msbarm) then
            xip=xi*(1+delder) 
            xim=xi*(1-delder)
            f2x=f2cqqpmCC(nb0,xi,rmu2)/xi
            f2xp=f2cqqpmCC(nb0,xip,rmu2)/xip
            f2xm=f2cqqpmCC(nb0,xim,rmu2)/xim
            df2x=(f2xp-f2xm)/2./delder/xi
            FCCcharm=FCCcharm + 4*d1dec*(f2x+xi*df2x)*(1-xb/xi)*fac
          end if
        end if
        if (kordhq.ge.2) then
           write(*,*) "F2CCCHARM: Bad value for order = ",kordhq
           stop
        end if
      case(3)
        !The LO term
        FCCcharm=2*f3cqqpm(nb,xi,rmu2)/xi  
        !The NLO contribution
        if (kordhq.ge.1.and.xi.lt.1d0) then
          CALL DGAUSS(F3CCcharmi,xi,1d0,nf3hq,f3c,EPS)
          fac=2*2*an
          ak=1./rq*(1-rq)*log(1-rq)
          dc=cf*(-4-0.5/rq-pi**2/3.-(1+3*rq)/2./rq*ak
     +    +(xi+xi**2/2.+2*log(1-xi))*log((Q2+rm2)/rmu2)
     -    -2*(-(log(1-xi))**2+ddilog((1-rq*xi)/(1-rq))-ddilog(1./(1-rq))
     +    +log(1-rq*xi)*log(rq*(1-xi)/(1-rq)))
     +    +0.5*(rq*xi-rq**2*xi+log(1-rq*xi)*(1-rq*xi))/rq**2/(1-rq*xi)
     -    -2*log(1-xi))*f3cqqpm(nb,xi,rmu2)/xi
          FCCcharm=FCCcharm+(f3c+dc)*fac
          !The term appearing for the MS-bar definition of the heavy-quark mass
          if(msbarm) then
            xip=xi*(1+delder)
            xim=xi*(1-delder)
            f3xp=f3cqqpm(nb0,xip,rmu2)/xip
            f3xm=f3cqqpm(nb0,xim,rmu2)/xim
            df3x=(f3xp-f3xm)/2./delder/xi
            FCCcharm=FCCcharm + 4*d1dec*df3x*(xi-xb)*fac
          end if
        end if
        if (kordhq.ge.2) then
           write(*,*) "F2CCCHARM: Bad value for order = ",kordhq
           stop
        end if
      case default
         write(*,*) "FCCCHARM: Bad value for nfunction = ",nfunction
         stop
      end select
      return
      end
!------------------
      real*8 function FTCCcharmi(y)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      common /QCDPARCC/ cf
      common /forfheavyCC/ xb0,q2ss,rm2,rmu2,an,rq,nb0,nq0
      xi=xb0/rq
      z=xi/y
      rl=log((1-rq*y)/(1-rq)/y)
      !The gluon-initiated contribution
      pqg0=(y**2+(1-y)**2)/2.
      ftg=pqg0*(rl+log((q2ss+rm2)/rmu2))
      ftg=ftg+pqg0*(2*log(1-y)-log(1-rq*y)-log(y))
      ftg=ftg+y*(1-y)*(4-4*(1-rq))
      ftg=ftg+(1-rq)*y/(1-rq*y)-1
      ftg=ftg+(1-rq)*y*rl*(2+rq*y*(-4))
      ftg=ftg*xqgCC(1,z,rmu2)
      !The quark-initiated contribution
      ftz=f2cqqpmCC(nb0,z,rmu2)
      ftx=f2cqqpmCC(nb0,xi,rmu2)
      ftq=-(1+y**2)*log(y)/(1-y)
      ftq=cf*ftq*ftz
      ftq0=(1+y**2)/(1-y)*log((q2ss+rm2)/rmu2)*(ftz-ftx)
      ftq0=ftq0+(2*log(1-y)-log(1-rq*y))/(1-y)*((1+y**2)*ftz-2*ftx)
      ftq0=ftq0+1./(1-y)*((1-4*y+y**2)*ftz+2*ftx)
      ftq0=ftq0+1./(1-rq*y)*(y-y**2)*ftz
      ftq0=ftq0+0.5*(1-y)/(1-rq*y)**2*(ftz-ftx)
      ftq0=cf*ftq0
      !The total contribution
      FTCCcharmi=(ftq0+ftq+ftg)/xi
      return
      end
!------------------
      real*8 function F2CCcharmi(y)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      common /QCDPARCC/ cf
      common /forfheavyCC/ xb0,q2ss,rm2,rmu2,an,rq,nb0,nq0
      xi=xb0/rq
      z=xi/y
      rl=log((1-rq*y)/(1-rq)/y)
      !The gluon-initiated contribution
      pqg0=(y**2+(1-y)**2)/2.
      f2g=pqg0*(rl+log((q2ss+rm2)/rmu2))
      f2g=f2g+pqg0*(2*log(1-y)-log(1-rq*y)-log(y))
      f2g=f2g+y*(1-y)*(8-18*(1-rq)+12*(1-rq)**2)
      f2g=f2g+(1-rq)/(1-rq*y)-1
      f2g=f2g+(1-rq)*y*rl*(6*rq+rq*y*(-12*rq))
      f2g=f2g*xqgCC(1,z,rmu2)
      !The quark-initiated contribution
      f2z=f2cqqpmCC(nb0,z,rmu2)
      f2x=f2cqqpmCC(nb0,xi,rmu2)
      f2q=-(1+y**2)*log(y)/(1-y)
      f2q=cf*f2q*f2z
      f2q0=(1+y**2)/(1-y)*log((q2ss+rm2)/rmu2)*(f2z-f2x)
      f2q0=f2q0+(2*log(1-y)-log(1-rq*y))/(1-y)*((1+y**2)*f2z-2*f2x)
      f2q0=f2q0+2.*(1+y)*f2z
      f2q0=f2q0+1./(1-rq*y)*((-1-y)*f2z+2*f2x)
      f2q0=f2q0+0.5*(1-y)/(1-rq*y)**2*(f2z-f2x)
      f2q0=f2q0+2*(rq-1)/(1-y)/(1-rq*y)*(f2z-f2x)
      f2q0=cf*f2q0
      !The total contribution
      F2CCcharmi=(f2q0+f2q+f2g)/xi
      return
      end
!------------------
      real*8 function F3CCcharmi(y)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      common /QCDPARCC/ cf
      common /CONSTANTSCC/ pi,Vcd,Vcs
      common /forfheavyCC/ xb0,q2ss,rm2,rmu2,an,rq,nb0,nq0
      xi=xb0/rq
      z=xi/y
      rl=log((1-rq*y)/(1-rq)/y)
      !The gluon-initiated contribution
      pqg0=(y**2+(1-y)**2)/2.
      f3g=pqg0*(-rl+log((q2ss+rm2)/rmu2))
      f3g=f3g+pqg0*(2*log(1-y)-log(1-rq*y)-log(y))
      f3g=f3g+y*(1-y)*2*(1-rq)
      f3g=f3g+(1-rq)*y*rl*(-2*(1-y)+rq*y*2)
      f3g=f3g*xqgCC(1,z,rmu2)
      !The quark-initiated contribution
      f3z=f3cqqpm(nb0,z,rmu2)
      f3x=f3cqqpm(nb0,xi,rmu2)
      f3q=-(1+y**2)*log(y)/(1-y)
      f3q=cf*f3q*f3z
      f3q0=(1+y**2)/(1-y)*log((q2ss+rm2)/rmu2)*(f3z-f3x)
      f3q0=f3q0+(2*log(1-y)-log(1-rq*y))/(1-y)*((1+y**2)*f3z-2*f3x)
      f3q0=f3q0+1./(1-y)*((-1-y**2)*f3z+2*f3x)
      f3q0=f3q0+1./(1-rq*y)*(1-y)*f3z
      f3q0=f3q0+0.5*(1-y)/(1-rq*y)**2*(f3z-f3x)
      f3q0=cf*f3q0
      !The total contribution
      if (nb0.eq.1) F3CCcharmi=(f3q0+f3q+f3g)/xi
      if (nb0.eq.-1) F3CCcharmi=(f3q0+f3q-f3g)/xi
      return
      end
!------------------
      double precision function xqgCC(iq,xx,Q2)
      implicit none
      integer iq,ihadron
      double precision,parameter::Znucl=23.403d0,Nnucl=26.215d0
      double precision xx,Q2,pi,Vcd,Vcs,as,xpa(-6:6)
      double precision RUV,RDV,RUB,RDB,RS,RG
      external as
      common /hadroncCC/ ihadron
      common /CONSTANTSCC/ pi,Vcd,Vcs
      call pdfs(xx,Q2,xpa)
      xqgCC=0d0
      select case(ihadron)
       case(1)
          select case(iq)
           case(0)!alphas
              xqgCC=4d0*pi*as(Q2)
           case(1)!gluon
              xqgCC=xpa(0)
           case(2)!d
              xqgCC=xpa(2)+xpa(-2)
           case(3)!dbar
              xqgCC=xpa(-2)
           case(4)!u
              xqgCC=xpa(1)+xpa(-1)
           case(5)!ub
              xqgCC=xpa(-1)
           case(6)!s
              xqgCC=xpa(3)+xpa(-3)
           case(7)!sbar
              xqgCC=xpa(-3)
           case(8)!c
              xqgCC=xpa(4)+xpa(-4)
           case(9)!cbar
              xqgCC=xpa(-4)
           case(10)!b
              xqgCC=xpa(5)+xpa(-5)
           case(11)!bbar
              xqgCC=xpa(-5)
           case default
              write(*,*) "xqgCC: Bad value for iq = ",iq
          end select
       case(50)
        call nDS(2,xx,dsqrt(Q2),49.618d0,RUV,RDV,RUB,RDB,RS,RG)
        select case(iq)
           case(0)!alphas
              xqgCC=4d0*pi*as(Q2)
           case(1)!gluon
              xqgCC=RG*xpa(0)
           case(2)!d
              xqgCC = xqgCC + Znucl*(RDV*xpa(2)+RDB*xpa(-2))
              xqgCC = xqgCC + Nnucl*(RUV*xpa(1)+RUB*xpa(-1))
              xqgCC = xqgCC/(Znucl+Nnucl)
           case(3)!dbar
              xqgCC = xqgCC + Znucl*RDB*xpa(-2)
              xqgCC = xqgCC + Nnucl*RUB*xpa(-1)
              xqgCC = xqgCC/(Znucl+Nnucl)
           case(4)!u
              xqgCC = xqgCC + Znucl*(RUV*xpa(1)+RUB*xpa(-1))
              xqgCC = xqgCC + Nnucl*(RDV*xpa(2)+RDB*xpa(-2))
              xqgCC = xqgCC/(Znucl+Nnucl)
           case(5)!ub
              xqgCC = xqgCC + Znucl*RUB*xpa(-1)
              xqgCC = xqgCC + Nnucl*RDB*xpa(-2)
              xqgCC = xqgCC/(Znucl+Nnucl)
           case(6)!s
              xqgCC=RS*(xpa(3)+xpa(-3))
           case(7)!sbar
              xqgCC=RS*xpa(-3)
           case(8)!c
              xqgCC=xpa(4)+xpa(-4)
           case(9)!cbar
              xqgCC=xpa(-4)
           case(10)!b
              xqgCC=xpa(5)+xpa(-5)
           case(11)!bbar
              xqgCC=xpa(-5)
           case default
              write(*,*) "xqgCC: Bad value for iq = ",iq
          end select
       case default
          write(*,*) "xqgCC: Bad value for ihadron = ",ihadron
          stop
       end select
      return
      end
!------------------
      !The number of fermions in the loops of the strong coupling evolution 
      !For negative kpdf values it changes with a scale, otherwise it is kpdf+3
      integer FUNCTION nfloopsCC(q2,kpdf)
      implicit double precision (a-h,o-z)
      common /MASSESCC/ mc,mb
      if (kpdf.ge.0) then 
        nfloopsCC=kpdf+3
      else
      !  the scales below the c-quark mass: 3 fermions 
        if (q2.le.mc**2) nfloopsCC=3
      !  the scales above the c-quark mass and below the c-quark mass: 4 fermions
        if (q2.gt.mc**2.and.q2.le.mb**2) nfloopsCC=4
      !  the scales above the b-quark mass: 5 fermions
        if (q2.gt.mb**2) nfloopsCC=5
      end if
      return
      end
!------------------
      real*8 function f2cqqpmCC(nb,xb,q2)
      implicit double precision (a-h,o-z)
      common /CONSTANTSCC/ pi,Vcd,Vcs
      f2cqqpmCC=0.
      if(nb.eq.1) then
        f2cqqpmCC=f2cqqpmCC+xqgCC(2,xb,q2)*Vcd**2d0
        f2cqqpmCC=f2cqqpmCC+xqgCC(6,xb,q2)*Vcs**2d0
      else if(nb.eq.-1) then
        f2cqqpmCC=f2cqqpmCC+xqgCC(3,xb,q2)*Vcd**2d0
        f2cqqpmCC=f2cqqpmCC+xqgCC(7,xb,q2)*Vcs**2d0
      else
        write(*,*) "F2QQPMCC: Bad value for nb = ",nb
        stop
      end if
      return
      end
!------------------
      real*8 function f3cqqpm(nb,xb,q2)
      implicit double precision (a-h,o-z)
      f3cqqpm=0.
      if(nb.eq.1) then
        f3cqqpm=f2cqqpmCC(nb,xb,q2)
      else if(nb.eq.-1) then
        f3cqqpm=-f2cqqpmCC(nb,xb,q2)
      else
        write(*,*) "F3QQPMCC: Bad value for nb = ",nb
        stop
      end if
      return
      end
      