!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2005 - 2014 Pedro Jimenez-Delgado
! 
! This file is part of Chromopolis.
! 
! Chromopolis is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
! 
! Chromopolis is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with Chromopolis.  If not, see <http://www.gnu.org/licenses/>.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! STRONG COUPLING CONSTANT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! AMATCH(order,mth,mZ,alphasmZ2): Initialization, it uses alphas(mZ2) as boundary condition.
!!!   order == 0(LO), 1(NLO), 2(NNLO).
!!!   mth == Heavy quark masses (GeV).
!!!   mZ == Mass of the Z boson (GeV).
!!!   alphasmZ2 == alpha_s(mZ**2d0).
!!! AS(Q2): alpha_s/4/pi.
!!!   Q2 == Renormalization scale (GeV**2d0).
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Initialization routine !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine amatch(order,mth,mZ,alphasmZ2)
 implicit none
 integer order,orderl,nfr
 double precision pi,mZ,mZ2,a,asmZ2,alphasmZ2,asimNLO,asimNNLO,deltap,deltam
 double precision,dimension(3:6)::bt0,bt1,bt2,b1,b2,lambda
 double precision,dimension(4:6)::mth,mth2,mth2l
! logical,external::isexception
 common /alphasc/ lambda,mth2l,bt0,bt1,bt2,b1,b2,orderl
 lambda=0d0
 pi=dacos(-1d0)
 orderl=order
 mth2=mth*mth
 mth2l=mth2
 mZ2=mZ*mZ
 asmZ2=alphasmZ2/4d0/pi
 do nfr = 3,6
    bt0(nfr) = 11d0 - 2d0/3d0*nfr
    bt1(nfr) = 102d0 - 38d0/3d0*nfr
    b1(nfr) = bt1(nfr)/bt0(nfr)
    bt2(nfr) = 2857d0/2d0 - 5033d0/18d0*nfr + 325d0/54d0*nfr*nfr
    b2(nfr) = bt2(nfr)/bt0(nfr)
 end do
 select case(order)
  case(0)
     lambda(5) = dexp(0.5d0*(dlog(mZ2)-1d0/bt0(5)/asmZ2))
     lambda(4) = dexp(0.5d0*(dlog(lambda(5)**2d0)+(1d0-bt0(5)/bt0(4))*dlog(mth2(5)/lambda(5)**2d0)))
     lambda(3) = dexp(0.5d0*(dlog(lambda(4)**2d0)+(1d0-bt0(4)/bt0(3))*dlog(mth2(4)/lambda(4)**2d0)))
     lambda(6) = dexp(0.5d0*(dlog(lambda(5)**2d0)+(1d0-bt0(5)/bt0(6))*dlog(mth2(6)/lambda(5)**2d0)))
  case(1)
     lambda(5) = dexp(0.5d0*(dlog(mZ2)-1d0/bt0(5)/asmZ2 + bt1(5)/bt0(5)/bt0(5)*dlog(1d0/bt0(5)/asmZ2 + bt1(5)/bt0(5)/bt0(5))))
     a = asimNLO(mth2(5),lambda(5),bt0(5),bt1(5))
     lambda(4) = dexp(0.5d0*(dlog(mth2(5))-1d0/bt0(4)/a + bt1(4)/bt0(4)/bt0(4)*dlog(1d0/bt0(4)/a + bt1(4)/bt0(4)/bt0(4))))
     a = asimNLO(mth2(4),lambda(4),bt0(4),bt1(4))
     lambda(3) = dexp(0.5d0*(dlog(mth2(4))-1d0/bt0(3)/a + bt1(3)/bt0(3)/bt0(3)*dlog(1d0/bt0(3)/a + bt1(3)/bt0(3)/bt0(3))))
     a = asimNLO(mth2(6),lambda(5),bt0(5),bt1(5))
     lambda(6) = dexp(0.5d0*(dlog(mth2(6))-1d0/bt0(6)/a + bt1(6)/bt0(6)/bt0(6)*dlog(1d0/bt0(6)/a + bt1(6)/bt0(6)/bt0(6))))
  case(2)
     lambda(5) = dexp(0.5d0*(dlog(mZ2) - 1d0/bt0(5)/asmZ2 + bt1(5)/2d0/bt0(5)/bt0(5)*&
               dlog(bt2(5)/bt0(5)/bt0(5)/bt0(5) + bt1(5)/bt0(5)/bt0(5)/bt0(5)/asmZ2 + 1d0/bt0(5)/bt0(5)/asmZ2/asmZ2)&
               - (2d0*bt2(5)*bt0(5) - bt1(5)*bt1(5))/bt0(5)/bt0(5)/dsqrt(4d0*bt2(5)*bt0(5) - bt1(5)*bt1(5))*&
               datan((2d0*bt2(5)*asmZ2 + bt1(5))/dsqrt(4d0*bt2(5)*bt0(5) - bt1(5)*bt1(5)))))
     a = asimNNLO(mth2(5),lambda(5),bt0(5),bt1(5),bt2(5))
     a = a + 22d0/9d0*a**3d0 !MS-bar masses
!     a = a - 14d0/3d0*a**3d0 !Pole masses

     lambda(4) = dexp(0.5d0*(dlog(mth2(5)) - 1d0/bt0(4)/a + bt1(4)/2d0/bt0(4)/bt0(4)*&
               dlog(bt2(4)/bt0(4)/bt0(4)/bt0(4) + bt1(4)/bt0(4)/bt0(4)/bt0(4)/a + 1d0/bt0(4)/bt0(4)/a/a)&
               - (2d0*bt2(4)*bt0(4) - bt1(4)*bt1(4))/bt0(4)/bt0(4)/dsqrt(4d0*bt2(4)*bt0(4) - bt1(4)*bt1(4))*&
               datan((2d0*bt2(4)*a + bt1(4))/dsqrt(4d0*bt2(4)*bt0(4) - bt1(4)*bt1(4)))))
     a = asimNNLO(mth2(4),lambda(4),bt0(4),bt1(4),bt2(4))
     a = a + 22d0/9d0*a**3d0 !MS-bar masses
!     a = a - 14d0/3d0*a**3d0 !Pole masses

     lambda(3) = dexp(0.5d0*(dlog(mth2(4)) -1d0/bt0(3)/a  + bt1(3)/2d0/bt0(3)/bt0(3)*&
               dlog(bt2(3)/bt0(3)/bt0(3)/bt0(3) + bt1(3)/bt0(3)/bt0(3)/bt0(3)/a + 1d0/bt0(3)/bt0(3)/a/a)&
               - (2d0*bt2(3)*bt0(3) - bt1(3)*bt1(3))/bt0(3)/bt0(3)/dsqrt(4d0*bt2(3)*bt0(3) - bt1(3)*bt1(3))*&
               datan((2d0*bt2(3)*a + bt1(3))/dsqrt(4d0*bt2(3)*bt0(3) - bt1(3)*bt1(3)))))
     a = asimNNLO(mth2(6),lambda(5),bt0(5),bt1(5),bt2(5))
     a = a - 22d0/9d0*a**3d0 !MS-bar masses
!     a = a + 14d0/3d0*a**3d0 !Pole masses
     deltap = bt1(6)/2d0/bt2(6) + dsqrt(bt1(6)*bt1(6)/4d0/bt2(6)/bt2(6)-bt0(6)/bt2(6))
     deltam = bt1(6)/2d0/bt2(6) - dsqrt(bt1(6)*bt1(6)/4d0/bt2(6)/bt2(6)-bt0(6)/bt2(6))
     lambda(6)= dexp(0.5d0*(dlog(mth2(6)) - bt1(6)/bt0(6)/bt0(6)*dlog(a) - 1d0/bt0(6)/a &
                -1d0*(-bt1(6)*bt1(6)+bt0(6)*bt2(6)+bt1(6)*bt2(6)*deltap)&
                    /(bt0(6)*bt0(6)*bt2(6)*deltap-bt0(6)*bt0(6)*bt2(6)*deltam)*dlog(abs(a+deltap))&
                -1d0*(-bt1(6)*bt1(6)+bt0(6)*bt2(6)+bt1(6)*bt2(6)*deltam)&
                    /(bt0(6)*bt0(6)*bt2(6)*deltam-bt0(6)*bt0(6)*bt2(6)*deltap)*dlog(abs(a+deltam))))
  case default
     write(6,*) 'AMATCH: Bad value of order = ',order
     stop
 end select
! if (isexception(lambda(3)).or.isexception(lambda(4)).or.isexception(lambda(5)).or.isexception(lambda(6))) then
!    write(6,*) "AMATCH: Exception detected!",sngl(lambda(3)),sngl(lambda(4)),sngl(lambda(5)),sngl(lambda(6))
!    return
! end if
 return
end subroutine amatch

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Same for the approximated solution !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine amatcha(order,mth,mZ,alphasmZ2)
 implicit none
 integer order,orderl,nfr
 double precision pi,mZ,mZ2,a,asmZ2,alphasmZ2,asapNLO,lambdaaNLO
 double precision,dimension(3:6)::bt0,bt1,bt2,b1,b2,lambda
 double precision,dimension(4:6)::mth,mth2,mth2l
 common /alphasc/ lambda,mth2l,bt0,bt1,bt2,b1,b2,orderl
 lambda=0d0
 pi=dacos(-1d0)
 orderl=order
 mth2=mth*mth
 mth2l=mth2
 mZ2=mZ*mZ
 asmZ2=alphasmZ2/4d0/pi
 do nfr = 3,6
    bt0(nfr) = 11d0 - 2d0/3d0*nfr
    bt1(nfr) = 102d0 - 38d0/3d0*nfr
    b1(nfr) = bt1(nfr)/bt0(nfr)
    bt2(nfr) = 2857d0/2d0 - 5033d0/18d0*nfr + 325d0/54d0*nfr*nfr
    b2(nfr) = bt2(nfr)/bt0(nfr)
 end do
 select case(order)
  case(0)
     lambda(5) = dexp(0.5d0*(dlog(mZ2)-1d0/bt0(5)/asmZ2))
     lambda(4) = dexp(0.5d0*(dlog(lambda(5)**2d0)+(1d0-bt0(5)/bt0(4))*dlog(mth2(5)/lambda(5)**2d0)))
     lambda(3) = dexp(0.5d0*(dlog(lambda(4)**2d0)+(1d0-bt0(4)/bt0(3))*dlog(mth2(4)/lambda(4)**2d0)))
     lambda(6) = dexp(0.5d0*(dlog(lambda(5)**2d0)+(1d0-bt0(5)/bt0(6))*dlog(mth2(6)/lambda(5)**2d0)))
  case(1)
     lambda(5) = lambdaaNLO(asmZ2,mZ2,bt0(5),bt1(5))
     a = asapNLO(mth2(5),lambda(5),bt0(5),bt1(5))
     lambda(4) = lambdaaNLO(a,mth2(5),bt0(4),bt1(4))
     lambda(3) = 0d0
     lambda(3) = lambda(3)/0d0
     a = asapNLO(mth2(6),lambda(5),bt0(5),bt1(5))
     lambda(6) = lambdaaNLO(a,mth2(6),bt0(6),bt1(6))
  case(2)
     write(6,*) "AMATCHA: Not ready to match the approximated solution at NNLO."
     stop
     case default
     write(6,*) 'AMATCHA: Bad value of order = ',order
     stop
 end select
 return
end subroutine amatcha

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Strong coupling !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function as(Q2)
 implicit none
 logical revolve
 integer order,i,nfr,errors
 double precision,dimension(3:6)::bt0,bt1,bt2,b1,b2
 double precision Q2,Q2old,asold,asimNLO,asimNNLO,asimNNLO6
 double precision,dimension(3:6)::lambda(3:6),mth2(4:6)
 save Q2old,asold
 common /errorsc/ errors
 common /revolvec/ revolve
 common /alphasc/ lambda,mth2,bt0,bt1,bt2,b1,b2,order
 as=0d0
 nfr=3
 do i = 4,6
    if (Q2 > mth2(i)) then
       nfr = i
    else
       exit
    end if
 end do
 if ((Q2/=Q2old).or.(revolve).or.(errors/=0)) then
    select case(order)
     case(0)
        as = 1d0/bt0(nfr)/dlog(Q2/lambda(nfr)/lambda(nfr))
     case(1)
        as = asimNLO(Q2,lambda(nfr),bt0(nfr),bt1(nfr))
     case(2)
        if(nfr<=5) then
           as = asimNNLO(Q2,lambda(nfr),bt0(nfr),bt1(nfr),bt2(nfr))
        else if(nfr==6) then
           as = asimNNLO6(Q2,lambda(nfr),bt0(nfr),bt1(nfr),bt2(nfr))
        else
           write(*,*) "AS: Bad number of flavors",nfr
           stop
        end if
     case default
        write(6,*) "AS: Bad value of order = ",order
        stop
    end select
    Q2old=Q2
    asold = as
 end if
 as = asold
 return
end function

!!!!!!!!!!!!!!!!!!!!!!!!!!! Solution of the NLO approximated expression !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function asapNLO(Q2,lambda,bt0,bt1)
 implicit none
 double precision Q2,lambda,lQ2lm2,bt0lQ2lm2
 double precision bt0,bt1,bt100,ibt0
 lQ2lm2 = dlog(Q2/lambda/lambda)
 bt0lQ2lm2 = bt0*lQ2lm2
 bt100 = bt1/bt0/bt0
 ibt0 = 1d0/bt0
 asapNLO = 1d0/bt0lQ2lm2 - bt1/bt0*dlog(lQ2lm2)/bt0lQ2lm2/bt0lQ2lm2
 return
end function asapNLO

!!!!!!!!!!!!!!!!!!!!!!!!!! Lambda for the approximated NLO solution using the Newton-Rhapson method !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function lambdaaNLO(as,Q2,bt0,bt1)
 implicit none
 double precision as,Q2,lQ2lm2,bt0lQ2lm2
 double precision bt0,bt1,bt100,ibt0
 double precision lambda,lambdac,lambdam,lambdap,y,ym,yp,dy
 bt100 = bt1/bt0/bt0
 ibt0 = 1d0/bt0
 lambda = dexp(0.5d0*(dlog(Q2)-ibt0/as + bt100*dlog(ibt0/as + bt100)))
 lambdac = 0d0
 do while(abs((lambda-lambdac)/lambda)>1d-10)
    lambdac = lambda
    lQ2lm2 = dlog(Q2/lambda/lambda)
    bt0lQ2lm2 = bt0*lQ2lm2
    y = as - 1d0/bt0lQ2lm2 + bt1/bt0*dlog(lQ2lm2)/bt0lQ2lm2/bt0lQ2lm2
    lambdam = 0.99d0*lambda
    lQ2lm2 = dlog(Q2/lambdam/lambdam)
    bt0lQ2lm2 = bt0*lQ2lm2
    ym = as - 1d0/bt0lQ2lm2 + bt1/bt0*dlog(lQ2lm2)/bt0lQ2lm2/bt0lQ2lm2
    lambdap = 1.01d0*lambda
    lQ2lm2 = dlog(Q2/lambdap/lambdap)
    bt0lQ2lm2 = bt0*lQ2lm2
    yp = as - 1d0/bt0lQ2lm2 + bt1/bt0*dlog(lQ2lm2)/bt0lQ2lm2/bt0lQ2lm2
    dy = (yp-ym)/0.02d0/lambda
    lambda = lambda - y/dy
 end do
 lambdaaNLO = lambda
 return
end function lambdaaNLO

!!!!!!!!!!!!!!!!!!!!!!!!!!! Solution of the NLO implicit expression using the Newton-Rhapson method !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function asimNLO(Q2,lambda,bt0,bt1)
 implicit none
 double precision Q2,lambda,lQ2lm2,bt0lQ2lm2
 double precision bt0,bt1,bt100,ibt0
 double precision a,ac,am,ap,y,ym,yp,dy
 lQ2lm2 = dlog(Q2/lambda/lambda)
 bt0lQ2lm2 = bt0*lQ2lm2
 bt100 = bt1/bt0/bt0
 ibt0 = 1d0/bt0
 a = 1d0/bt0lQ2lm2 - bt1/bt0*dlog(lQ2lm2)/bt0lQ2lm2/bt0lQ2lm2
 ac = 0
 do while(abs((a-ac)/a)>1d-10)
    ac = a
    y = lQ2lm2-ibt0/a+bt100*dlog(ibt0/a+bt100)
    am = 0.99d0*a
    ym = lQ2lm2-ibt0/am+bt100*dlog(ibt0/am+bt100)
    ap = 1.01d0*a
    yp = lQ2lm2-ibt0/ap+bt100*dlog(ibt0/ap+bt100)
    dy = (yp-ym)/0.02d0/a
    a = a - y/dy
 end do
 asimNLO = a
 return
end function asimNLO

!!!!!!!!!!!!!!!!!!!! Solution of the NNLO(nf<=5) implicit expression using the Newton-Rhapson method !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function asimNNLO(Q2,lambda,bt0,bt1,bt2)
 implicit none
 double precision Q2,lambda,lQ2lm2,bt0lQ2lm2
 double precision bt0,bt1,bt2,ibt0,ibt00,bt10,bt1d00,bt1000,bt1s,dbt2,bt2000,root,pref
 double precision a,ac,am,ap,y,ym,yp,dy
 lQ2lm2 = dlog(Q2/lambda/lambda)
 bt0lQ2lm2 = bt0*lQ2lm2
 ibt0 = 1d0/bt0
 ibt00 = ibt0/bt0
 bt10 = bt1/bt0
 bt1d00 = bt10/2d0/bt0
 bt1000 = bt10*ibt00
 bt1s = bt1*bt1
 dbt2 = 2d0*bt2
 bt2000 = bt2/bt0*ibt00
 root = dsqrt(4d0*bt2*bt0 - bt1s)
 pref = (dbt2*bt0 - bt1s)*ibt00/root
 a = 1d0/bt0lQ2lm2 - bt10*dlog(lQ2lm2)/bt0lQ2lm2/bt0lQ2lm2 &
                   + (bt10*bt10*(dlog(lQ2lm2)**2d0 - dlog(lQ2lm2) - 1d0) + bt2/bt0)/bt0lQ2lm2/bt0lQ2lm2/bt0lQ2lm2
 ac = 0
 do while(abs((a-ac)/a)>1d-10)
    ac = a
    y = lQ2lm2 -ibt0/a + bt1d00*dlog(bt2000 + bt1000/a + ibt00/a/a) - pref*atan((dbt2*a + bt1)/root)
    am = 0.99d0*a
    ym = lQ2lm2 -ibt0/am + bt1d00*dlog(bt2000 + bt1000/am + ibt00/am/am) - pref*atan((dbt2*am + bt1)/root)
    ap = 1.01d0*a
    yp =lQ2lm2 -ibt0/ap + bt1d00*dlog(bt2000 + bt1000/ap + ibt00/ap/ap) - pref*atan((dbt2*ap + bt1)/root)
    dy = (yp-ym)/0.02d0/a
    a = a - y/dy
 end do
 asimNNLO = a
 return
end function asimNNLO

!!!!!!!!!!!!!!!!!!!! Solution of the NNLO(nf=6) implicit expression using the Newton-Rhapson method !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function asimNNLO6(Q2,lambda,bt0,bt1,bt2)
 implicit none
 double precision Q2,lambda,lQ2lm2,bt0lQ2lm2
 double precision bt0,bt1,bt2,bt0s,ibt0,ibt00,bt10,bt100,bt1d00,bt1000,bt1s,dbt2,bt2s,bt2000
 double precision root,deltap,deltam,fracp,fracm
 double precision a,ac,am,ap,y,ym,yp,dy
 lQ2lm2 = dlog(Q2/lambda/lambda)
 bt0lQ2lm2 = bt0*lQ2lm2
 bt0s = bt0*bt0
 ibt0 = 1d0/bt0
 ibt00 = ibt0/bt0
 bt10 = bt1/bt0
 bt100 = bt10/bt0
 bt1d00 = bt10/2d0/bt0
 bt1000 = bt10*ibt00
 bt1s = bt1*bt1
 dbt2 = 2d0*bt2
 bt2s = bt2*bt2
 bt2000 = bt2/bt0*ibt00
 root = dsqrt(bt1s/4d0/bt2s-bt0/bt2)
 deltap = bt1/dbt2 + root
 deltam = bt1/dbt2 - root
 fracp = (-bt1s+bt0*bt2+bt1*bt2*deltap)/(bt0s*bt2*deltap-bt0s*bt2*deltam)
 fracm = (-bt1s+bt0*bt2+bt1*bt2*deltam)/(bt0s*bt2*deltam-bt0s*bt2*deltap)
 a = 1d0/bt0lQ2lm2 - bt10*dlog(lQ2lm2)/bt0lQ2lm2/bt0lQ2lm2 &
                   + (bt10*bt10*(dlog(lQ2lm2)**2d0 - dlog(lQ2lm2) - 1d0) + bt2/bt0)/bt0lQ2lm2/bt0lQ2lm2/bt0lQ2lm2
 ac = 0d0
 do while(abs((a-ac)/a)>1d-10)
    ac = a
    y = lQ2lm2 - bt100*dlog(a) - ibt0/a - fracp*dlog(abs(a+deltap)) - fracm*dlog(abs(a+deltam))
    am = 0.99d0*a
    ym = lQ2lm2 - bt100*dlog(am) - ibt0/am - fracp*dlog(abs(am+deltap)) - fracm*dlog(abs(am+deltam))
    ap = 1.01d0*a
    yp = lQ2lm2 - bt100*dlog(ap) - ibt0/ap - fracp*dlog(abs(ap+deltap)) - fracm*dlog(abs(ap+deltam))
    dy = (yp-ym)/0.02d0/a
    a = a - y/dy
 end do
 asimNNLO6 = a
 return
end function asimNNLO6

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

