!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2005 - 2014 Pedro Jimenez-Delgado
! 
! This file is part of Chromopolis.
! 
! Chromopolis is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
! 
! Chromopolis is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with Chromopolis.  If not, see <http://www.gnu.org/licenses/>.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Parton densities in x-Space !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! x == Bjorken-x
!!! Q2 == Q**2 (GeV**2) == Factorization scale
!!! xpa:-6(x*tb),-5(x*bb),-4(x*db),-3(x*sb),-2(x*db),-1(x*ub),0(x*g),1(x*uv),2(x*dv),3(x*(s-sb)),4(x*(c-cb)),5(x*(b-bb)),6(x*(t-tb))
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine pdfs(x,Q2,xpa)
 implicit none
 logical resetPDFsx
 integer::i,pdfscheme=0,fnscheme,fnsfixed,fnschemeold,errors
 double precision x,Q2,Q2c,xpa(-6:6),xpaold(-6:6),as,asQ2,asQ2s,xold,Q2old
 double complex pdfn(-6:6,136),qns(-5:5,136),qs(2,136)
 common /fnschemec/ fnscheme,fnsfixed
 common /errorsc/ errors
 common /resetPDFsxc/ resetPDFsx
 common /transfer/ Q2c,asQ2,asQ2s
 external as
 save xold,Q2old,fnschemeold,pdfn,xpaold
 if ((x<0d0).or.((x>1d0).and.((x-1d0)>1d-2))) then
    write(*,*) "PDFSEC: x out of range",x
    stop
 end if
 if ((x==xold).and.(Q2==Q2old).and.(fnscheme==fnschemeold).and.(.not.resetPDFsx).and.(errors==0)) then
    xpa = xpaold
    return
 end if
 xold=x
 if ((Q2/=Q2old).or.(fnscheme/=fnschemeold).or.(resetPDFsx).or.(errors/=0)) then
    Q2c = Q2
    asQ2 = as(Q2)
    asQ2s = asQ2*asQ2
    call npdfs(pdfscheme,0,pdfn,qns,qs)
    Q2old=Q2
    fnschemeold=fnscheme
    resetPDFsx = .false.
 end if
 xpa=0d0
 do i = -6,6
   call minversion(x,pdfn(i,:),xpa(i))
 end do
 xpaold = xpa
 return
end

subroutine npdfso
 implicit none
 integer isup,i,pdfsoform
 double precision dbeta,p(8:44)
 double complex wbeta,cp(8:44),n(0:6,136),pdfon(7,0:6,136)
 external dbeta,wbeta
 common /support/ n
 common /npdfsoc/ pdfon
 common /pdfsoc/ p,pdfsoform
 select case(pdfsoform)
  case(0)!traditional form, extended valence, strangeness asymmetry
     p(11)= 2d0/(        dbeta(p(12)    ,p(13)+1d0) + p(14)*dbeta(p(12)+0.5d0,p(13)+1d0)&
                 + p(15)*dbeta(p(12)+1d0,p(13)+1d0) +  p(8)*dbeta(p(12)+2d0  ,p(13)+1d0))
     p(16)= 1d0/(        dbeta(p(17)    ,p(18)+1d0) + p(19)*dbeta(p(17)+0.5d0,p(18)+1d0)&
                 + p(20)*dbeta(p(17)+1d0,p(18)+1d0) +  p(9)*dbeta(p(17)+2d0  ,p(18)+1d0))
     p(21)= (1d0 - p(11)*(       dbeta(p(12)+1d0,p(13)+1d0)+p(14)*dbeta(p(12)+1.5d0,p(13)+1d0)&
                          +p(15)*dbeta(p(12)+2d0,p(13)+1d0)+ p(8)*dbeta(p(12)  +3d0,p(13)+1d0))&
                 - p(16)*(       dbeta(p(17)+1d0,p(18)+1d0)+p(19)*dbeta(p(17)+1.5d0,p(18)+1d0)&
                          +p(20)*dbeta(p(17)+2d0,p(18)+1d0)+ p(9)*dbeta(p(17)  +3d0,p(18)+1d0))&
                 - 2d0*p(26)*(dbeta(p(27)+1d0,p(28)+1d0)+p(29)*dbeta(p(27)+1.5d0,p(28)+1d0)+p(30)*dbeta(p(27)+2d0,p(28)+1d0))&
                 - 2d0*p(36)*(dbeta(p(37)+1d0,p(38)+1d0)+p(39)*dbeta(p(37)+1.5d0,p(38)+1d0)+p(40)*dbeta(p(37)+2d0,p(38)+1d0))&
             )/(dbeta(p(22)+1d0,p(23)+1d0)+p(24)*dbeta(p(22)+1.5d0,p(23)+1d0)+p(25)*dbeta(p(22)+2d0,p(23)+1d0))
     p(44) = dbeta(p(42)+1d0,p(43)+1d0)/dbeta(p(42),p(43)+1d0)
     cp = dcmplx(p(8:44))
     do isup = 0,6,2
        do i = 1,136
           pdfon(1,isup,i) = cp(11)*(        wbeta(n(isup,i)+cp(12)-1d0  ,cp(13)+1d0)&
                                     +cp(14)*wbeta(n(isup,i)+cp(12)-0.5d0,cp(13)+1d0)&
                                     +cp(15)*wbeta(n(isup,i)+cp(12)      ,cp(13)+1d0)&
                                      +cp(8)*wbeta(n(isup,i)+cp(12)+1d0  ,cp(13)+1d0))!uv
           pdfon(2,isup,i) = cp(16)*(        wbeta(n(isup,i)+cp(17)-1d0  ,cp(18)+1d0)&
                                     +cp(19)*wbeta(n(isup,i)+cp(17)-0.5d0,cp(18)+1d0)&
                                     +cp(20)*wbeta(n(isup,i)+cp(17)      ,cp(18)+1d0)&
                                      +cp(9)*wbeta(n(isup,i)+cp(17)+1d0  ,cp(18)+1d0))!dv
           pdfon(3,isup,i) = cp(21)*(        wbeta(n(isup,i)+cp(22)-1d0  ,cp(23)+1d0)&
                                     +cp(24)*wbeta(n(isup,i)+cp(22)-0.5d0,cp(23)+1d0)&
                                     +cp(25)*wbeta(n(isup,i)+cp(22)      ,cp(23)+1d0))!g
           pdfon(4,isup,i) = cp(26)*(wbeta(n(isup,i)+cp(27)-1d0,cp(28)+1d0)&
                             +cp(29)*wbeta(n(isup,i)+cp(27)-0.5d0,cp(28)+1d0)&
                             +cp(30)*wbeta(n(isup,i)+cp(27),cp(28)+1d0))!Sigma
           pdfon(5,isup,i) = cp(31)*(wbeta(n(isup,i)+cp(32)-1d0,cp(33)+1d0)&
                             +cp(34)*wbeta(n(isup,i)+cp(32)-0.5d0,cp(33)+1d0)&
                             +cp(35)*wbeta(n(isup,i)+cp(32),cp(33)+1d0))!Delta
           pdfon(6,isup,i) = cp(36)*(wbeta(n(isup,i)+cp(37)-1d0,cp(38)+1d0)&
                             +cp(39)*wbeta(n(isup,i)+cp(37)-0.5d0,cp(38)+1d0)&
                             +cp(40)*wbeta(n(isup,i)+cp(37),cp(38)+1d0))!sp/2
           pdfon(7,isup,i) = cp(41)*(wbeta(n(isup,i)+cp(42)-1d0,cp(43)+1d0)-wbeta(n(isup,i)+cp(42),cp(43)+1d0)/cp(44))!sm
        end do
     end do
  case(1)!"alternative" form, strangeness asymmetry
     p(11)= 2d0/(dbeta(p(12),p(13)+1d0)+p(14)*dbeta(p(12)+p(15),p(13)+1d0))
     p(16)= 1d0/(dbeta(p(17),p(18)+1d0)+p(19)*dbeta(p(17)+p(20),p(18)+1d0))
     p(44) = dbeta(p(42)+1d0,p(43)+1d0)/dbeta(p(42),p(43)+1d0)
     p(21)= (1d0 - p(11)*(dbeta(p(12)+1d0,p(13)+1d0)+p(14)*dbeta(p(12)+p(15)+1d0,p(13)+1d0))&
                 - p(16)*(dbeta(p(17)+1d0,p(18)+1d0)+p(19)*dbeta(p(17)+p(20)+1d0,p(18)+1d0))&
             - 2d0*p(26)*(dbeta(p(27)+1d0,p(28)+1d0)+p(29)*dbeta(p(27)+p(30)+1d0,p(28)+1d0))&
             - 2d0*p(36)*(dbeta(p(37)+1d0,p(38)+1d0)+p(39)*dbeta(p(37)+p(40)+1d0,p(38)+1d0))&
                       )/(dbeta(p(22)+1d0,p(23)+1d0)+p(24)*dbeta(p(22)+p(25)+1d0,p(23)+1d0))
     cp = dcmplx(p(8:44))
     do isup = 0,6,2
        do i = 1,136
           pdfon(1,isup,i) = cp(11)*(wbeta(n(isup,i)+cp(12)-1d0,cp(13)+1d0)&
                             +cp(14)*wbeta(n(isup,i)+cp(12)+cp(15)-1d0,cp(13)+1d0))!uv
           pdfon(2,isup,i) = cp(16)*(wbeta(n(isup,i)+cp(17)-1d0,cp(18)+1d0)&
                             +cp(19)*wbeta(n(isup,i)+cp(17)+cp(20)-1d0,cp(18)+1d0))!dv
           pdfon(3,isup,i) = cp(21)*(wbeta(n(isup,i)+cp(22)-1d0,cp(23)+1d0)&
                             +cp(24)*wbeta(n(isup,i)+cp(22)+cp(25)-1d0,cp(23)+1d0))!g
           pdfon(4,isup,i) = cp(26)*(wbeta(n(isup,i)+cp(27)-1d0,cp(28)+1d0)&
                             +cp(29)*wbeta(n(isup,i)+cp(27)+cp(30)-1d0,cp(28)+1d0))!Sigma
           pdfon(5,isup,i) = cp(31)*(wbeta(n(isup,i)+cp(32)-1d0,cp(33)+1d0)&
                             +cp(34)*wbeta(n(isup,i)+cp(32)+cp(35)-1d0,cp(33)+1d0))!Delta
           pdfon(6,isup,i) = cp(36)*(wbeta(n(isup,i)+cp(37)-1d0,cp(38)+1d0)&
                             +cp(39)*wbeta(n(isup,i)+cp(37)+cp(40)-1d0,cp(38)+1d0))!sp/2
           pdfon(7,isup,i) = cp(41)*(wbeta(n(isup,i)+cp(42)-1d0,cp(43)+1d0)-wbeta(n(isup,i)+cp(42),cp(43)+1d0)/cp(44))!sm
        end do
     end do
 end select
 return
end subroutine npdfso

subroutine npdfs(pdfscheme,isup,pdfn,qns_out,qs_out)
 use mfactor
 implicit none
 logical resetPDFs,forwards
 integer,parameter::saving=12000
 integer,save::nsave=0,pdfscheme_save(saving)=-1,isup_save(saving)=-1,fnscheme_save(saving)=-1
 integer order,evolscheme,fnscheme,fnsfixed,pdfscheme,nfr,nff,nfl,nfsf,isup,isupc,i
 double precision Q2o,mth2(4:6),aso,asth(4:6),Q2,asQ2,asQ2s,ast,asl,asth2
 double precision,save::Q2_save(saving)=0d0
 double complex pdfon(7,0:6,136),pdfn(-6:6,136)
 double complex qns(-5:5,136),qns_msb(-5:5,136),qns_dis(-5:5,136),qns_out(-5:5,136)
 double complex qs(2,136),qs_msb(2,136),qs_dis(2,136),qs_out(2,136)
 double complex qs1(136),qs2(136)
 double complex,save::pdfn_save(saving,-6:6,136),qns_save(saving,-5:5,136),qs_save(saving,2,136)
 interface
    function pro(A,q)
     double complex,dimension(2)::pro,q
     double complex,dimension(2,2)::A
    end function pro
 end interface
 common /orderc/ order
 common /fnschemec/ fnscheme,fnsfixed
 common /evolschemec/ evolscheme
 common /transfer/ Q2,asQ2,asQ2s
 common /scalesc/ Q2o,mth2
 common /couplingsc/ aso,asth
 common /npdfsoc/ pdfon
 common /DGLAPc/ qns,qs,ast,asl,isupc,nfr,nff
 common /disf2/ qns_dis,qs_dis
 common /nfsfc/ nfsf
 common /resetPDFsc/ resetPDFs
 if (resetPDFs) then
    nsave=0
    resetPDFs = .false.
 end if
 do i=1,nsave
    if (dabs(Q2-Q2_save(i))<1d-5) then
       if ((pdfscheme==pdfscheme_save(i)).and.(isup==isup_save(i)).and.(fnscheme==fnscheme_save(i))) then
          pdfn =  pdfn_save(i,:,:)
          qns_out =  qns_save(i,:,:)
          qs_out = qs_save(i,:,:)
         return
       end if
    end if
 end do
 isupc=isup
 !Flavor composition of the input (three flavors with all current evolution paths)
 qns(-5,:) = pdfon(1,isup,:) + pdfon(2,isup,:) + pdfon(7,isup,:)!vm35 = uv + dv + sm + cm + bm - 5tm
 qns(-4,:) = pdfon(1,isup,:) + pdfon(2,isup,:) + pdfon(7,isup,:)!vm24 = uv + dv + sm + cm -4bm
 qns(-3,:) = pdfon(1,isup,:) + pdfon(2,isup,:) + pdfon(7,isup,:)!vm15 = uv + dv + sm -3cm
 qns(-2,:) = pdfon(1,isup,:) + pdfon(2,isup,:) - 2d0*pdfon(7,isup,:)!vm8 = uv + dv - 2sm
 qns(-1,:) = pdfon(1,isup,:) - pdfon(2,isup,:)!vm3 = uv - dv
 qns(0,:) = pdfon(1,isup,:) + pdfon(2,isup,:) + pdfon(7,isup,:)!qnsv = uv + dv + sm + cm + bm + tm
 qns(1,:) = pdfon(1,isup,:) - pdfon(2,isup,:) - 2d0*pdfon(5,isup,:)!vp3 = uv - dv - 2Delta
 qns(2,:) = pdfon(1,isup,:) + pdfon(2,isup,:) + 2d0*pdfon(4,isup,:) - 4d0*pdfon(6,isup,:)!vp8 = uv + dv + 2Sigma - 2sp
 qns(3,:) = pdfon(1,isup,:) + pdfon(2,isup,:) + 2d0*pdfon(4,isup,:) + 2d0*pdfon(6,isup,:)!vp15 = uv + dv + 2Sigma + sp - 3cp
 qns(4,:) = pdfon(1,isup,:) + pdfon(2,isup,:) + 2d0*pdfon(4,isup,:) + 2d0*pdfon(6,isup,:)!vp24 = uv + dv + 2Sigma + sp + cp - 4bp
 qns(5,:) = pdfon(1,isup,:) + pdfon(2,isup,:) + 2d0*pdfon(4,isup,:) + 2d0*pdfon(6,isup,:)!vp35 = uv + dv + 2Sigma + sp + cp + bp - 5tp
 qs(1,:) = pdfon(1,isup,:) + pdfon(2,isup,:) + 2d0*pdfon(4,isup,:) + 2d0*pdfon(6,isup,:)!qs = uv + dv + 2Sigma + sp + cp + bp  + tp
 qs(2,:) = pdfon(3,isup,:)!g
 !Number of active flavors at the starting scale
 if (Q2o > mth2(6)) then
    nfl = 6
 else if (Q2o > mth2(5)) then
    nfl = 5
 else if (Q2o > mth2(4)) then
    nfl = 4
 else if (Q2o > 0d0) then
    nfl = 3
 else
    write(*,*) "NPDFS: Bad value for input scale = ",Q2o
    stop
 end if
 !Backwards evolution
 forwards=.false.
 if ((Q2 < Q2o).or.((fnscheme>3).and.(Q2o>mth2(4)))) then
    if (nfl==6) then
       ast = aso
       nfr = nfl
       nff = 3
       if ((Q2 < mth2(6)).or.(fnscheme>3)) then
          asl = asth(6)
          if(order==2) asl = asth(6)-22d0/9d0*asth(6)**3d0 !MS-bar masses
!          if(order==2) asl = asth(6)+14d0/3d0*asth(6)**3d0 !Pole masses
          nfl = 5
       else
          asl = asQ2
       end if
       call DGLAP
    end if
    if (nfl==5) then
       if (Q2o >= mth2(6)) then
          ast = asth(6)
       else
          ast = aso
       end if
       nfr = nfl
       nff = 3
       if ((Q2 < mth2(5)).or.(fnscheme>3)) then
          asl = asth(5)
          if(order==2) asl = asth(5)-22d0/9d0*asth(5)**3d0 !MS-bar masses
!          if(order==2) asl = asth(5)+14d0/3d0*asth(5)**3d0 !Pole masses
          nfl = 4
       else
          asl = asQ2
       end if
       call DGLAP
    end if
    if (nfl==4) then
       if (Q2o >= mth2(5)) then
          ast = asth(5)
       else
          ast = aso
       end if
       nfr = nfl
       nff = 3
       if ((Q2 < mth2(4)).or.(fnscheme>3)) then
          asl = asth(4)
          if(order==2) asl = asth(4)-22d0/9d0*asth(4)**3d0 !MS-bar masses
!          if(order==2) asl = asth(4)+14d0/3d0*asth(4)**3d0 !Pole masses
          nfl = 3
       else
          asl = asQ2
       end if
       call DGLAP
    end if
    if (nfl==3) then
       if (Q2 <= mth2(4)) then
          if (Q2o >= mth2(4)) then
             ast = asth(4)
          else
             ast = aso
          end if
          nfr = nfl
          nff = 3
          asl = asQ2
          call DGLAP
       else
          nfl = 4
          forwards=.true.
       end if
    end if
 end if
 !Forwards evolution
 if ((Q2 >= Q2o).or.(forwards)) then
    if (nfl==3) then
       ast = aso
       select case(fnscheme)
        case(3:6)
           nfr = nfl
           nff = 3
        case default
           write(*,*) "NPDFS: Bad value for fnscheme = ",fnscheme
           stop
       end select
       if (Q2 <= mth2(4)) then
          asl = asQ2
       else
          asl = asth(4)
          nfl = 4
       end if
       call DGLAP
    end if
    if (nfl==4) then
       if ((Q2o <= mth2(4)).or.(fnscheme>3)) then
          ast = asth(4)
          if(order==2) ast = asth(4)-22d0/9d0*asth(4)**3d0 !MS-bar masses
!          if(order==2) ast = asth(4)+14d0/3d0*asth(4)**3d0 !Pole masses
       else
          ast = aso
       end if
       select case(fnscheme)
        case(3)
           nfr = nfl
           nff = 3
        case(4:6)
           nfr = nfl
           nff = nfl
           qns(-3,:) = qns(0,:)
           qns(3,:) = qs(1,:)
           if (order>=2) then
              asth2 = (asth(4)-22d0/9d0*asth(4)**3d0)**2d0 !MS-bar masses
!              asth2 = (asth(4)+14d0/3d0*asth(4)**3d0)**2d0 !Pole masses
              qs1 = qs(1,:)
              qs2 = qs(2,:)
              do i=-3,3
                 qns(i,:) = qns(i,:) + asth2*A2NSQQ(isup,:)*qns(i,:)
              end do
              qns(3,:) = qns(3,:) - 3d0*asth2*(A2PSHQ(isup,:)*qs1 + A2SHG(isup,:)*qs2)
              qs(1,:) = qs1 + asth2*((A2NSQQ(isup,:)+A2PSHQ(isup,:))*qs1 + A2SHG(isup,:)*qs2)
              qs(2,:) = qs2 + asth2*(A2SGQ(isup,:)*qs1 + A2SGG(isup,:)*qs2)
           end if
        case default
           write(*,*) "NPDFS: Bad value for fnscheme = ",fnscheme
           stop
       end select
       if (Q2 <= mth2(5)) then
          asl = asQ2
       else
          asl = asth(5)
          nfl = 5
       end if
       call DGLAP
    end if
    if (nfl==5) then
       if ((Q2o <= mth2(5)).or.(fnscheme>3)) then
          ast = asth(5)
          if(order==2) ast = asth(5)-22d0/9d0*asth(5)**3d0 !MS-bar masses
!          if(order==2) ast = asth(5)+14d0/3d0*asth(5)**3d0 !Pole masses
       else
          ast = aso
       end if
       select case(fnscheme)
        case(3)
           nfr = nfl
           nff = 3
        case(4)
           nfr = nfl
           nff = 4
        case(5:6)
           nfr = nfl
           nff = nfl
           qns(-4,:) = qns(0,:)
           qns(4,:) = qs(1,:)
           if (order>=2) then
              asth2 = (asth(5)-22d0/9d0*asth(5)**3d0)**2d0 !MS-bar masses
!              asth2 = (asth(5)+14d0/3d0*asth(5)**3d0)**2d0 !Pole masses
              qs1 = qs(1,:)
              qs2 = qs(2,:)
              do i=-4,4
                 qns(i,:) = qns(i,:) + asth2*A2NSQQ(isup,:)*qns(i,:)
              end do
              qns(4,:) = qns(4,:) - 4d0*asth2*(A2PSHQ(isup,:)*qs1 + A2SHG(isup,:)*qs2)
              qs(1,:) = qs1 + asth2*((A2NSQQ(isup,:)+A2PSHQ(isup,:))*qs1 + A2SHG(isup,:)*qs2)
              qs(2,:) = qs2 + asth2*(A2SGQ(isup,:)*qs1 + A2SGG(isup,:)*qs2)
           end if
        case default
           write(*,*) "NPDFS: Bad value for fnscheme = ",fnscheme
           stop
       end select
       if (Q2 <= mth2(6)) then
          asl = asQ2
       else
          asl = asth(6)
          nfl = 6
       end if
       call DGLAP
    end if
    if (nfl==6) then
       if ((Q2o <= mth2(6)).or.(fnscheme>3)) then
          ast = asth(6)
          if(order==2) ast = asth(6)-22d0/9d0*asth(6)**3d0 !MS-bar masses
!          if(order==2) ast = asth(6)+14d0/3d0*asth(6)**3d0 !Pole masses
       else
          ast = aso
       end if
        select case(fnscheme)
         case(3)
            nfr = nfl
            nff = 3
         case(4)
            nfr = nfl
            nff = 4
         case(5)
            nfr = nfl
            nff = 5
         case(6)
            nfr = nfl
            nff = nfl
            qns(-5,:) = qns(0,:)
            qns(5,:) = qs(1,:)
            if (order>=2) then
               asth2 = (asth(6)-22d0/9d0*asth(6)**3d0)**2d0 !MS-bar masses
!               asth2 = (asth(6)+14d0/3d0*asth(6)**3d0)**2d0 !Pole masses
               qs1 = qs(1,:)
               qs2 = qs(2,:)
               do i=-5,5
                  qns(i,:) = qns(i,:) + asth2*A2NSQQ(isup,:)*qns(i,:)
               end do
               qns(5,:) = qns(5,:) - 5d0*asth2*(A2PSHQ(isup,:)*qs1 + A2SHG(isup,:)*qs2)
               qs(1,:) = qs1 + asth2*((A2NSQQ(isup,:)+A2PSHQ(isup,:))*qs1 + A2SHG(isup,:)*qs2)
               qs(2,:) = qs2 + asth2*(A2SGQ(isup,:)*qs1 + A2SGG(isup,:)*qs2)
            end if
         case default
            write(*,*) "NPDFS: Bad value for fnscheme = ",fnscheme
            stop
        end select
        asl = asQ2
        call DGLAP
    end if
 end if
 !Factorization scheme transformations
 select case(evolscheme)
  case(0)
     qns_msb = qns
     qs_msb = qs
     select case(pdfscheme)
      case(0)
      case(1)!Transformations MSB --> DIS
          if (order >= 0) then!Leading order
          qns_dis = qns_msb
          qs_dis = qs_msb
          else
          write(6,*) 'NPDFS: Bad value of order = ',order
          stop
          end if
          if (order >= 1) then
             do i=-5,5
                qns_dis(i,:) = qns_dis(i,:) + asQ2*C2Q1(nff,isup,:)*qns_msb(i,:)
             end do
             do i = 1,136
                qs_dis(:,i) = qs_dis(:,i) + asQ2*pro(CS1(:,:,nff,isup,i),qs_msb(:,i))
             end do
          end if
          if (order >= 2) then
             do i=-5,5
                qns_dis(i,:) = qns_dis(i,:) + asQ2*asQ2*C2NSp2(nff,isup,:)*qns_msb(i,:)
             end do
             do i = 1,136
                qs_dis(:,i) = qs_dis(:,i) + asQ2*asQ2*pro(CS2(:,:,nff,isup,i),qs_msb(:,i))
             end do
          end if
          if (order > 2) then
             write(6,*) 'NPDFS: Bad value of order = ',order
             stop
          end if
      case default
         write(6,*) "NPDFS: Bad value for pdfscheme = ",pdfscheme
         stop
     end select
  case(1)
     qns_dis = qns
     qs_dis = qs
     select case(pdfscheme)
      case(0)!Transformations DIS --> MSB
          if (order >= 0) then
          qns_msb = qns_dis
          qs_msb = qs_dis
          else
          write(6,*) 'NPDFS: Bad value of order = ',order
          stop
          end if
          if (order >= 1) then
             do i=-5,5
                qns_msb(i,:) = qns_msb(i,:) - asQ2*C2Q1(nff,isup,:)*qns_dis(i,:)
             end do
             do i = 1,136
                qs_msb(:,i) = qs_msb(:,i) - asQ2*pro(CS1(:,:,nff,isup,i),qs_dis(:,i))
             end do
          end if
          if (order >= 2) then
             do i=-5,5
                qns_msb(i,:) = qns_msb(i,:) - asQ2*asQ2*C2NSp2(nff,isup,:)*qns_dis(i,:)
             end do
             do i = 1,136
                qs_msb(:,i) = qs_msb(:,i) - asQ2*asQ2*pro(CS2(:,:,nff,isup,i),qs_dis(:,i))
             end do
          end if
          if (order > 2) then
             write(6,*) 'NPDFS: Bad value of order = ',order
             stop
          end if
      case(1)
      case default
         write(6,*) "NPDFS: Bad value for pdfscheme = ",pdfscheme
         stop
     end select
  case default
     write(6,*) "NPDFS: Bad value for evolscheme = ",evolscheme
     stop
 end select
 !Output selection
 select case(pdfscheme)
  case(0)
     qns = qns_msb
     qs = qs_msb
  case(1)
     qns = qns_dis
     qs = qs_dis
  case default
     write(6,*) "NPDFS: Bad value for pdfscheme = ",pdfscheme
     stop
 end select
 !Flavor decomposition of the output
 select case(nff)
  case(3)
    qns(-5,:) = 0d0!vm35 = uv + dv + sm + cm + bm - 5tm
    qns(-4,:) = 0d0!vm24 = uv + dv + sm + cm -4bm
    qns(-3,:) = 0d0!vm15 = uv + dv + sm -3cm
    pdfn(6,:) = 0d0!tm
    pdfn(5,:) = 0d0!bm
    pdfn(4,:) = 0d0!cm
    pdfn(3,:) = qns(0,:)/3d0 - qns(-2,:)/3d0
        !sm = qnsv/3       - vm8/3
    pdfn(2,:) = qns(0,:)/3d0 - qns(-1,:)/2d0 + qns(-2,:)/6d0
          !dv = qnsv/3       - vm3/2         + vm8/6
    pdfn(1,:) = qns(0,:)/3d0 + qns(-1,:)/2d0 + qns(-2,:)/6d0
          !uv = qnsv/3       + vm3/2         + vm8/6
    pdfn(0,:) = qs(2,:)!g
    pdfn(-1,:) = (qs(1,:)/3d0 + qns(1,:)/2d0 + qns(2,:)/6d0 - pdfn(1,:))/2d0
           !ub = (qs/3        + vp3/2        + vp8/6        - uv)/2
    pdfn(-2,:) = (qs(1,:)/3d0 - qns(1,:)/2d0 + qns(2,:)/6d0 - pdfn(2,:))/2d0
           !db = (qs/3        - vp3/2        + vp8/6        - dv)/2
    pdfn(-3,:) = (qs(1,:)/3d0 - qns(2,:)/3d0 - pdfn(3,:))/2d0
           !sb = (qs/3        - vp8/3        - sm)/2
    pdfn(-4,:) = 0d0!cb
    pdfn(-5,:) = 0d0!bb
    pdfn(-6,:) = 0d0!tb
    qns(3,:) = 0d0!vp15 = uv + dv + 2Sigma + sp - 3cp
    qns(4,:) = 0d0!vp24 = uv + dv + 2Sigma + sp + cp - 4bp
    qns(5,:) = 0d0!vp35 = uv + dv + 2Sigma + sp + cp + bp - 5tp
  case(4)
    qns(-5,:) = 0d0!vm35 = uv + dv + sm + cm + bm - 5tm
    qns(-4,:) = 0d0!vm24 = uv + dv + sm + cm -4bm
    pdfn(6,:) = 0d0!tm
    pdfn(5,:) = 0d0!bm
    pdfn(4,:) = qns(0,:)/4d0 - qns(-3,:)/4d0
        !cm = qnsv/4       - vm15/4
    pdfn(3,:) = qns(0,:)/4d0 - qns(-2,:)/3d0 + qns(-3,:)/12d0
        !sm = qnsv/4       - vm8/3         + vm15/12
    pdfn(2,:) = qns(0,:)/4d0 - qns(-1,:)/2d0 + qns(-2,:)/6d0 + qns(-3,:)/12d0
          !dv = qnsv/4       - vm3/2         + vm8/6         + vm15/12
    pdfn(1,:) = qns(0,:)/4d0 + qns(-1,:)/2d0 + qns(-2,:)/6d0 + qns(-3,:)/12d0
          !uv = qnsv/4       + vm3/2         + vm8/6         + vm15/12
    pdfn(0,:) = qs(2,:)!g
    pdfn(-1,:) = (qs(1,:)/4d0 + qns(1,:)/2d0 + qns(2,:)/6d0 + qns(3,:)/12d0  - pdfn(1,:))/2d0
           !ub = (qs/4        + vp3/2        + vp8/6        + vp15/12        - uv)/2
    pdfn(-2,:) = (qs(1,:)/4d0 - qns(1,:)/2d0 + qns(2,:)/6d0 + qns(3,:)/12d0 - pdfn(2,:))/2d0
           !db = (qs/4        - vp3/2        + vp8/6        + vp15/12       - dv)/2
    pdfn(-3,:) = (qs(1,:)/4d0 - qns(2,:)/3d0 + qns(3,:)/12d0 - pdfn(3,:))/2d0
           !sb = (qs/4        - vp8/3        + vp15/12       - sm)/2
    pdfn(-4,:) = (qs(1,:)/4d0 - qns(3,:)/4d0 - pdfn(4,:))/2d0
           !cb = (qs/4        - vp15/4       - cm)/2
    pdfn(-5,:) = 0d0!bb
    pdfn(-6,:) = 0d0!tb
    qns(4,:) = 0d0!vp24 = uv + dv + 2Sigma + sp + cp - 4bp
    qns(5,:) = 0d0!vp35 = uv + dv + 2Sigma + sp + cp + bp - 5tp
  case(5)
    qns(-5,:) = 0d0!vm35 = uv + dv + sm + cm + bm - 5tm
    pdfn(6,:) = 0d0!tm
    pdfn(5,:) = qns(0,:)/5d0 - qns(-4,:)/5d0
        !bm = qnsv/5       - vm24/5
    pdfn(4,:) = qns(0,:)/5d0 - qns(-3,:)/4d0 + qns(-4,:)/20d0
        !cm = qnsv/5       - vm15/4        + vm24/20
    pdfn(3,:) = qns(0,:)/5d0 - qns(-2,:)/3d0 + qns(-3,:)/12d0 + qns(-4,:)/20d0
        !sm = qnsv/5       - vm8/3         + vm15/12        + vm24/20
    pdfn(2,:) = qns(0,:)/5d0 - qns(-1,:)/2d0 + qns(-2,:)/6d0 + qns(-3,:)/12d0 + qns(-4,:)/20d0
          !dv = qnsv/5       - vm3/2         + vm8/6         + vm15/12        + vm24/20
    pdfn(1,:) = qns(0,:)/5d0 + qns(-1,:)/2d0 + qns(-2,:)/6d0 + qns(-3,:)/12d0 + qns(-4,:)/20d0
          !uv = qnsv/5       + vm3/2         + vm8/6         + vm15/12        + vm24/20
    pdfn(0,:) = qs(2,:)!g
    pdfn(-1,:) = (qs(1,:)/5d0 + qns(1,:)/2d0 + qns(2,:)/6d0 + qns(3,:)/12d0 + qns(4,:)/20d0 - pdfn(1,:))/2d0
           !ub = (qs/5        + vp3/2        + vp8/6        + vp15/12       + vp24/20       - uv)/2
    pdfn(-2,:) = (qs(1,:)/5d0 - qns(1,:)/2d0 + qns(2,:)/6d0 + qns(3,:)/12d0 + qns(4,:)/20d0 - pdfn(2,:))/2d0
           !db = (qs/5        - vp3/2        + vp8/6        + vp15/12       + vp24/20       - dv)/2
    pdfn(-3,:) = (qs(1,:)/5d0 - qns(2,:)/3d0 + qns(3,:)/12d0 + qns(4,:)/20d0 - pdfn(3,:))/2d0
           !sb = (qs/5        - vp8/3        + vp15/12       + vp24/20       - sm)/2
    pdfn(-4,:) = (qs(1,:)/5d0 - qns(3,:)/4d0 + qns(4,:)/20d0 - pdfn(4,:))/2d0
           !cb = (qs/5        - vp15/4       + vp24/20       - cm)/2
    pdfn(-5,:) = (qs(1,:)/5d0 - qns(4,:)/5d0 - pdfn(5,:))/2d0
           !bb = (qs/5        - vp24/5       - bm)/2
    pdfn(-6,:) = 0d0!tb
    qns(5,:) = 0d0!vp35 = uv + dv + 2Sigma + sp + cp + bp - 5tp
  case(6)
    pdfn(6,:) = qns(0,:)/6d0 - qns(-5,:)/6d0
        !tm = qnsv/6       - vm35/6
    pdfn(5,:) = qns(0,:)/6d0 - qns(-4,:)/5d0 + qns(-5,:)/30d0
        !bm = qnsv/6       - vm24/5        + vm35/30
    pdfn(4,:) = qns(0,:)/6d0 - qns(-3,:)/4d0 + qns(-4,:)/20d0 + qns(-5,:)/30d0
        !cm = qnsv/6       - vm15/4        + vm24/20        + vm35/30
    pdfn(3,:) = qns(0,:)/6d0 - qns(-2,:)/3d0 + qns(-3,:)/12d0 + qns(-4,:)/20d0 + qns(-5,:)/30d0
        !sm = qnsv/6       - vm8/3         + vm15/12        + vm24/20        + vm35/30
    pdfn(2,:) = qns(0,:)/6d0 - qns(-1,:)/2d0 + qns(-2,:)/6d0 + qns(-3,:)/12d0 + qns(-4,:)/20d0 + qns(-5,:)/30d0
          !dv = qnsv/6       - vm3/2         + vm8/6         + vm15/12        + vm24/20        + vm35/30
    pdfn(1,:) = qns(0,:)/6d0 + qns(-1,:)/2d0 + qns(-2,:)/6d0 + qns(-3,:)/12d0 + qns(-4,:)/20d0 + qns(-5,:)/30d0
          !uv = qnsv/6       + vm3/2         + vm8/6         + vm15/12        + vm24/20        + vm35/30
    pdfn(0,:) = qs(2,:)!g
    pdfn(-1,:) = (qs(1,:)/6d0 + qns(1,:)/2d0 + qns(2,:)/6d0 + qns(3,:)/12d0 + qns(4,:)/20d0 + qns(5,:)/30d0 - pdfn(1,:))/2d0
           !ub = (qs/6        + vp3/2        + vp8/6        + vp15/12       + vp24/20       + vp35/30 - uv)/2
    pdfn(-2,:) = (qs(1,:)/6d0 - qns(1,:)/2d0 + qns(2,:)/6d0 + qns(3,:)/12d0 + qns(4,:)/20d0 + qns(5,:)/30d0 - pdfn(2,:))/2d0
           !db = (qs/6        - vp3/2        + vp8/6        + vp15/12       + vp24/20       + vp35/30       - dv)/2
    pdfn(-3,:) = (qs(1,:)/6d0 - qns(2,:)/3d0 + qns(3,:)/12d0 + qns(4,:)/20d0 + qns(5,:)/30d0 - pdfn(3,:))/2d0
           !sb = (qs/6        - vp8/3        + vp15/12       + vp24/20       + vp35/30       - sm)/2
    pdfn(-4,:) = (qs(1,:)/6d0 - qns(3,:)/4d0 + qns(4,:)/20d0 + qns(5,:)/30d0 - pdfn(4,:))/2d0
           !cb = (qs/6        - vp15/4       + vp24/20       + vp35/30       - cm)/2
    pdfn(-5,:) = (qs(1,:)/6d0 - qns(4,:)/5d0 + qns(5,:)/30d0 - pdfn(5,:))/2d0
           !bb = (qs/6        - vp24/5       + vp35/30       - bm)/2
    pdfn(-6,:) = (qs(1,:)/6d0 - qns(5,:)/6d0 - pdfn(6,:))/2d0
           !tb = (qs/6        - vp35/6       - tm)/2
 case default
    write(6,*) 'NPDFS: Bad factorization flavour number = ',nff
    stop
 end select
 !Copies
 qns_out=qns
 qs_out=qs
 nfsf = nff
 !Saving
 if(nsave==saving) then
    write(*,*) "NPDFS: Saving limit reached!"
    nsave = 0
 end if
 nsave=nsave+1
 Q2_save(nsave) = Q2
 pdfscheme_save(nsave) = pdfscheme
 isup_save(nsave) = isup
 fnscheme_save(nsave) = fnscheme
 pdfn_save(nsave,:,:) = pdfn
 qns_save(nsave,:,:) = qns
 qs_save(nsave,:,:) = qs
 return
end subroutine npdfs

subroutine DGLAP
 use mfactor
 implicit none
 integer i,isup,nfr,nff,order
 double precision ast,asl,atl
 double complex LNS(-5:5,3:6,3:6,0:6,136),DGLAPNS(-5:5,3:6,3:6,0:6,136),qns(-5:5,136)
 double complex LS(2,2,3:6,3:6,0:6,136),DGLAPS(2,2,3:6,3:6,0:6,136),qs(2,136)
 interface
    function prod(a,b)
     double complex,intent(in)::a(2,2),b(2,2)
     double complex::prod(2,2)
    end function prod
    function pro(A,q)
     double complex,dimension(2)::pro,q
     double complex,dimension(2,2)::A
    end function pro
 end interface
 common /orderc/ order
 common /DGLAPc/ qns,qs,ast,asl,isup,nfr,nff
 atl = ast/asl
 do i = 1, 136
    !LO
    if (order >= 0) then
       LNS(:,nfr,nff,isup,i) = atl**RONS(:,nfr,nff,isup,i)
       LS(:,:,nfr,nff,isup,i) = atl**ep(nfr,nff,isup,i)*Rp(:,:,nfr,nff,isup,i) + atl**em(nfr,nff,isup,i)*Rm(:,:,nfr,nff,isup,i)
       DGLAPNS(:,nfr,nff,isup,i) = LNS(:,nfr,nff,isup,i)
       DGLAPS(:,:,nfr,nff,isup,i) = LS(:,:,nfr,nff,isup,i)
    else
       write(6,*) 'DGLAP: Bad value of order = ',order
       stop
    end if
    !NLO
    if (order >= 1) then
       DGLAPNS(:,nfr,nff,isup,i) = DGLAPNS(:,nfr,nff,isup,i) + (asl-ast)*U1NS(:,nfr,nff,isup,i)*LNS(:,nfr,nff,isup,i)
       DGLAPS(:,:,nfr,nff,isup,i) = DGLAPS(:,:,nfr,nff,isup,i) + asl*prod(U1S(:,:,nfr,nff,isup,i),LS(:,:,nfr,nff,isup,i))&
                                                       - ast*prod(LS(:,:,nfr,nff,isup,i),U1S(:,:,nfr,nff,isup,i))
    end if
    !NNLO
    if (order >= 2) then
       DGLAPNS(:,nfr,nff,isup,i) = DGLAPNS(:,nfr,nff,isup,i) + asl*asl*U2NS(:,nfr,nff,isup,i)*LNS(:,nfr,nff,isup,i)&
                                                     - asl*ast*U1NS(:,nfr,nff,isup,i)*LNS(:,nfr,nff,isup,i)*U1NS(:,nfr,nff,isup,i)&
                      + ast*ast*LNS(:,nfr,nff,isup,i)*(U1NS(:,nfr,nff,isup,i)*U1NS(:,nfr,nff,isup,i) - U2NS(:,nfr,nff,isup,i))
       DGLAPS(:,:,nfr,nff,isup,i) = DGLAPS(:,:,nfr,nff,isup,i) + asl*asl*prod(U2S(:,:,nfr,nff,isup,i),LS(:,:,nfr,nff,isup,i))&
                          - asl*ast*prod(U1S(:,:,nfr,nff,isup,i),prod(LS(:,:,nfr,nff,isup,i),U1S(:,:,nfr,nff,isup,i)))&
           + ast*ast*prod(LS(:,:,nfr,nff,isup,i),prod(U1S(:,:,nfr,nff,isup,i),U1S(:,:,nfr,nff,isup,i)) - U2S(:,:,nfr,nff,isup,i))
    end if
    if (order > 2) then
       write(6,*) 'DGLAP: Bad value of order = ',order
       stop
    end if
    qns(:,i) = DGLAPNS(:,nfr,nff,isup,i)*qns(:,i)
    qs(:,i) = pro(DGLAPS(:,:,nfr,nff,isup,i),qs(:,i))
 end do
end subroutine DGLAP

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
