!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2005 - 2014 Pedro Jimenez-Delgado
! 
! This file is part of Chromopolis.
! 
! Chromopolis is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
! 
! Chromopolis is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with Chromopolis.  If not, see <http://www.gnu.org/licenses/>.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!! MELLIN MOMENTS (ON MSPACE) OF Q2-INDEPENDENT FACTORS FOR PQCD EVOLUTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! DIS Coefficient functions in the MSB factorization scheme and Splitting functions in MSB and DIS factorization schemes
!!! The number of active flavours in the renormalization (nfr) varies between 3 and 6
!!! The number of active flavours in the factorization (nff) varies between 3 and 6
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
module mfactor
 implicit none
 double precision,dimension(3:6)::bt0,bt1,bt2,b1,b2
 double complex,dimension(0:6,136)::A2PSHQ,A2SHG,A2NSQQ,A2SGQ,A2SGG
 double complex,dimension(3:6,0:6,136)::C2Q1,C2G1,C2NSp2,C2NSm2,C2PS2,C2G2
 double complex,dimension(3:6,0:6,136)::PC1Q1,PC1G1
 double complex,dimension(3:6,0:6,136)::CLQ1,CLG1,CLNSp2,CLNSm2,CLPS2,CLG2,CLNSp3,CLNSm3,CLPS3,CLG3
 double complex,dimension(3:6,0:6,136)::C3Q1,C3NSm2,C3NSp2
 double complex,dimension(2,2,3:6,0:6,136)::CS1,CS2
 double complex,dimension(3:6,3:6,0:6,136)::ep,em,pep,pem
 double complex,dimension(-5:5,3:6,3:6,0:6,136)::RONS,PRONS,U1NS,PU1NS,U2NS
 double complex,dimension(2,2,3:6,3:6,0:6,136)::Rp,Rm,PRp,PRm,U1S,PU1S,U2S
end module mfactor

subroutine mfactors
 use mfactor
 implicit none
 integer evolscheme,nfr,nff,i,j,k
 double complex,dimension(0:6,136)::n,&
                                    ns,nc,nfo,nfi,nsi,nse,nei,nni,np1,np1s,np1c,nm1,nm1s,np2,np2s,np2c,np3,np3s,np4,&
                                    S1,S1s,S2,S3,S4,SPp2,SPm2,SPp3,SPm3,STp,STm,S1m,S1ms,S2m,S3m,S1p,S1ps,S2p,S1p2
 double complex,dimension(3:6,0:6,136)::P0QQ,P0QG,P0GQ,P0GG,P1NSp,P1NSm,P1QQ,P1QG,P1GQ,P1GG,P2NSv,P2NSp,P2NSm,P2PS,P2QG,P2GQ,P2GG
 double complex,dimension(3:6,0:6,136)::PP0QQ,PP0QG,PP0GQ,PP0GG,PP1NSp,PP1NSm,PP1QQ,PP1QG,PP1GQ,PP1GG
 double complex,dimension(3:6,0:6,136)::deltaCL3A,deltaCL3B
 double complex,dimension(-5:5,3:6,3:6,0:6,136)::P0NS,P1NS,P2NS,R1NS,R2NS,R2strNS
 double complex,dimension(-5:5,3:6,3:6,0:6,136)::PP0NS,PP1NS,PR1NS
 double complex,dimension(2,2,3:6,3:6,0:6,136)::P0S,P1S,P2S,ROS,R1S,R2S,R2strS
 double complex,dimension(2,2,3:6,3:6,0:6,136)::PP0S,PP1S,PROS,PR1S
 interface
    function prod(a,b)
     double complex,intent(in)::a(2,2),b(2,2)
     double complex prod(2,2)
    end function prod
    function comm(a,b)
     double complex,intent(in)::a(2,2),b(2,2)
     double complex comm(2,2)
    end function comm
 end interface
 common /evolschemec/ evolscheme
 common /support/ n
 common /mspacec/ ns,nc,nfo,nfi,nsi,nse,nei,nni,np1,np1s,np1c,nm1,nm1s,np2,np2s,np2c,np3,np3s,np4,&
                  S1,S1s,S2,S3,S4,SPp2,SPm2,SPp3,SPm3,STp,STm,S1m,S1ms,S2m,S3m,S1p,S1ps,S2p,S1p2
!Beta Coefficients
 do nfr = 3,6
    bt0(nfr) = 11d0 - 2d0/3d0*nfr
    bt1(nfr) = 102d0 - 38d0/3d0*nfr
    b1(nfr) = bt1(nfr)/bt0(nfr)
    bt2(nfr) = 2857d0/2d0 - 5033d0/18d0*nfr + 325d0/54d0*nfr*nfr
    b2(nfr) = bt2(nfr)/bt0(nfr)
 end do
!DIS Coefficient functions in the MSB factorization scheme
 do nff = 3,6
    C2Q1(nff,:,:) = 4d0/3d0*(2d0*S1s - 2d0*S2 + 3d0*S1 - 2d0*S1/n/np1 + 3d0/n + 4d0/np1 + 2d0/ns - 9d0)
    C2G1(nff,:,:) = -2d0*nff*(S1*(ns + n + 2d0)/n/np1/np2 + 1d0/n - 1d0/ns - 6d0/np1 + 6d0/np2)
    CLQ1(nff,:,:) = 16d0/3d0/np1
    CLG1(nff,:,:) = 8d0*nff/np1/np2
    C3Q1(nff,:,:) = C2Q1(nff,:,:) - 8d0/3d0*(1d0/n + 1d0/np1)
    PC1Q1(nff,:,:) = 4d0/3d0*(2d0*S1s - 2d0*S2 +(3d0-2d0/n/np1)*S1 + 2d0/ns + 1d0/n + 2d0/np1 - 9d0) 
    PC1G1(nff,:,:) = 2d0*nff*(-1d0*nm1/n/np1*(S1+1d0) - 1d0/ns + 2d0/n/np1)   
    !NUPH,B724(2005)3
    do i = 0,6
       do j = 1,136
         call c22p(n(i,j),nff,C2NSp2(nff,i,j),C2PS2(nff,i,j),C2G2(nff,i,j))
         call cl2p(n(i,j),nff,CLNSp2(nff,i,j),CLPS2(nff,i,j),CLG2(nff,i,j))
         call cl3p(n(i,j),nff,CLNSp3(nff,i,j),CLPS3(nff,i,j),CLG3(nff,i,j))
       end do
    end do
    !NUPH,B568(2000)263
    C2NSm2(nff,:,:) = 3.55555d0*(6d0*S4 + 8d0*S3*S1 + 3d0*S2**2d0 + 6d0*S2*S1s + S1s*S1s) &
                    + 20.4444d0*(2d0*S3 + 3d0*S2*S1 + S1s*S1) - 15.5525d0*(S2 + S1s) - 188.64d0*S1 &
                    + 229.916d0/n*S3 + (31.58d0/ns + 9.7467d0/n)*S2 + 393.703d0/n*S2*S1 + 2.9678d0/n*S1s*S1 &
                    + (192.4d0/ns + 9.7467d0/n)*S1s - (160.82d0/nc - 61.1321d0/n)*S1 &
                    + 22.488d0/nfo - 39.12d0/nc + 265.774d0/ns - 164.777d0/n - 1010d0/np1 - 337.992d0 &
                    + nff*(- 0.59259d0*(2d0*S3 + 3d0*S2*S1 + S1s*S1) - 4.2963d0*(S2 + S1s) &
                    - 6.3489d0*S1 - 6.072d0/n*S3 - (6.072d0/ns - 18.0408d0/n)*S2 &
                    - (6.072d0/nc - 17.97d0/ns + 14.3574d0/n)*S1 + 0.07078d0/n*S1s + 4.488d0/nc & 
                    + 4.21808d0/ns - 21.6028d0/n - 37.91d0/np1 + 46.8406d0)   
    CLNSm2(nff,:,:) = - 128.4d0/n*S2 + 13.30d0/n*S1s + (59.12d0/n - 141.7d0/ns)*S1 - 0.086d0/nc &
                      + 22.21d0/ns + 180.818d0/n + 46.58d0/np1c + 100.8d0/np1 - 0.150d0 &
                      + nff*16d0/27d0*(-6d0/np1*S1 + 6d0/n + 6d0/np1s - 25d0/np1)
    C3NSm2(nff,:,:) =  3.55555d0*(6d0*S4 + 8d0*S3*S1 + 3d0*S2**2d0 + 6d0*S1s*S2 + S1s*S1s) &
                      + 20.4444d0*(2d0*S3 + 3d0*S2*S1 + S1s*S1) - 15.5525d0*(S2 + S1s) - 188.64d0*S1 &
                      + 297.756d0/n*S3 + (147.9d0/ns + 33.2767d0/n)*S2 + 298.733d0/n*S2*S1 +0.9778d0/n*S1s*S1 &
                      + (147.9d0/ns + 33.2767d0/n)*S1s**2 - 45.8683d0/n*S1 &
                      + 23.532d0/nfo - 66.62d0/nc + 67.6d0/ns &
                      - 373.029d0/n - 576.8d0/np1 - 338.625d0 + nff*(-0.59259d0*(2d0*S3 + 3d0*S2*S1 + S1s*S1) &
                      - 4.2963d0*(S2 + S1s) - 6.3489d0*S1 -0.042d0/n*(2d0*S3 + 3d0*S2*S1 + S1s*S1) &
                      + 0.96978d0/n*S1s + ( 9.684d0/ns - 16.4074d0/n)*S1 + 10.6538d0/n*S2 + 4.414d0/nc &
                      -8.683d0/ns - 15.9177d0/n - 14.97d0/np1 + 46.856d0)
    C3NSp2(nff,:,:) = 3.55555d0*(6d0*S4 + 8d0*S3*S1 + 3d0*S2*S2 + 6d0*S1s*S2 + S1s*S1s) &
                     + 20.4444d0*(2d0*S3 + 3d0*S2*S1 + S1s*S1) - 15.5525d0*(S2 + S1s) - 188.64d0*S1 &
                     + 186.816d0/n*S3 + (92.43d0/ns + 33.2767d0/n)*S2  + 187.793d0/n*S2*S1 + 0.9778d0/n*S1s*S1 &
                     + (92.43d0/ns + 33.2767d0/n)*S1s + 123.121d0/n*S1 &
                     + 18.294d0/nfo - 60.28d0/nc + 79.14d0/ns - 276.473d0/n - 467.2d0/np1 - 338.681d0 &
                     + nff*( -0.59259d0*(2d0*S3 + 3d0*S2*S1 + S1s*S1) - 4.2963d0*(S2 + S1s) - 6.3489d0*S1 &
                     - 0.042d0/n*(2d0*S3 + 3d0*S2*S1 + S1s*S1)  + 0.96978d0/n*S1s + (9.684d0/ns - 16.4074d0/n)*S1  &
                     +  10.6538d0/n*S2 + 4.414d0/nc - 8.683d0/ns - 15.9177d0/n - 14.97d0/np1 + 46.856d0)
    !NUPH,B568(2000)263
    deltaCL3A(nff,:,:) = - 495.49d0*(1d0/np2-2d0/np3+1d0/np4)+ 906.86d0/n/np1 - 983.23d0*(-2d0*n-3d0)/np1s/np2s &
                         + 53.706d0*2d0/nc - 5.3059d0*6d0/nfo + nff*( 29.95d0*(1d0/np3 -2d0/np4 + 1d0/(n+5d0)) &
                         - 59.087d0*(1d0/np2-2d0/np3+1d0/np4)+ 379.91d0/n/np1 - 273.042d0*(-1d0/ns+1d0/np1s) &
                         + 71.482d0*(-1d0/ns + 1d0/np1s))
    deltaCL3B(nff,:,:) = 78.306d0*(-S1/n + 2d0*S1p/np1 - S1p2/np2) + 6.3838d0*(1d0/np1 - 2d0/np2 + 1d0/np3) &
                         - 20.809d0*(2d0*n+3d0)/np1s/np2s - 114.47d0*2d0/nc + 22.222d0*6d0/nfo &
                         + nff*( 12.532d0*(-S1/n + 2d0*S1p/np1 - S1p2/np2) + 141.99d0*(1d0/np2-2d0/np3+1d0/np4) &
                         - 250.62d0*(1d0/np1 - 2d0/np2 + 1d0/np3) + 153.586d0*(2d0*n+3d0)/np1s/np2s &
                         - 0.6569d0*(-1d0/ns + 1d0/np1s))
    CLNSm3(nff,:,:) = CLNSp3(nff,:,:) - 0.5d0*(deltaCL3A(nff,:,:) + deltaCL3B(nff,:,:))
 end do
!Singlet coefficient function matrix for F2 for the factorization scheme transformations
 CS1(1,1,:,:,:) = C2Q1
 CS1(1,2,:,:,:) = C2G1
 CS1(2,1,:,:,:) = -CS1(1,1,:,:,:)
 CS1(2,2,:,:,:) = -CS1(1,2,:,:,:)
 CS2(1,1,:,:,:) = C2NSp2 + C2PS2
 CS2(1,2,:,:,:) = C2G2(:,:,:)
 CS2(2,1,:,:,:) = -CS2(1,1,:,:,:)
 CS2(2,2,:,:,:) = -CS2(1,2,:,:,:)
!Splitting functions in the MSB factorization scheme
 do nff = 3,6
   P0QQ(nff,:,:) = 4d0/3d0*(3d0 + 2d0/n/np1 - 4d0*S1)
   P0QG(nff,:,:) = 2d0*nff*(ns + n + 2d0)/n/np1/np2
   P0GQ(nff,:,:) = 8d0/3d0*(ns + n + 2d0)/nm1/n/np1
   P0GG(nff,:,:) = 3d0*(11d0/3d0 + 4d0/n/nm1 + 4d0/np1/np2 - 4d0*S1) - 2d0/3d0*nff
   PP0QQ(nff,:,:) = P0QQ(nff,:,:)
   PP0QG(nff,:,:) = 2d0*nff*nm1/n/np1
   PP0GQ(nff,:,:) = 8d0/3d0*np2/n/np1
   PP0GG(nff,:,:) = 3d0*(11d0/3d0 + 8d0/n/np1 - 4d0*S1) - 2d0/3d0*nff
   P1NSp(nff,:,:) = -16d0/9d0*(8d0*S1*(2d0*n + 1d0)/ns/np1s + 8d0*(2d0*S1 -1d0/n/np1)*(S2 - SPp2) &
                                                        + 12d0*S2 + 32d0*STp - 4d0*SPp3 - 3d0/2d0 &
                 -4d0*(3d0*nc + ns - 1d0)/nc/np1c - 8d0*(2d0*ns + 2d0*n + 1d0)/nc/np1c) &
          -4d0*(268d0/9d0*S1 - 4d0*(2d0*S1 - 1d0/n/np1)*(2d0*S2 - SPp2) -44d0/3d0*S2 - 16d0*STp + 2d0*SPp3 - 17d0/6d0 &
              - 2d0/9d0*(151d0*nfo + 236d0*nc + 88d0*ns + 3d0*n + 18d0)/nc/np1c + 4d0*(2d0*ns + 2d0*n + 1d0)/nc/np1c) &
           -2d0/3d0*nff*(-80d0/9d0*S1 + 16d0/3d0*S2 + 2d0/3d0 + 8d0/9d0*(11d0*ns + 5d0*n - 3d0)/ns/np1s)
   P1NSm(nff,:,:) = -16d0/9d0*(8d0*S1*(2d0*n + 1d0)/ns/np1s + 8d0*(2d0*S1 -1d0/n/np1)*(S2 - SPm2)&
                                                       + 12d0*S2 + 32d0*STm - 4d0*SPm3 - 3d0/2d0 &
                    -4d0*(3d0*nc + ns - 1d0)/nc/np1c + 8d0*(2d0*ns + 2d0*n + 1d0)/nc/np1c) &
          -4d0*(268d0/9d0*S1 - 4d0*(2d0*S1 - 1d0/n/np1)*(2d0*S2 - SPm2) -44d0/3d0*S2 - 16d0*STm + 2d0*SPm3 - 17d0/6d0 &
              - 2d0/9d0*(151d0*nfo + 236d0*nc + 88d0*ns + 3d0*n + 18d0)/nc/np1c - 4d0*(2d0*ns + 2d0*n + 1d0)/nc/np1c) &
          -2d0/3d0*nff*(-80d0/9d0*S1 + 16d0/3d0*S2 + 2d0/3d0 + 8d0/9d0*(11d0*ns + 5d0*n - 3d0)/ns/np1s)
   P1QQ(nff,:,:) = P1NSp(nff,:,:) + 16d0/3d0*nff*(5d0*nfi + 32d0*nfo + 49d0*nc + 38d0*ns + 28d0*n + 8d0)/nm1/nc/np1c/np2s
   P1QG(nff,:,:) = 6d0*nff*(2d0*(S2 - SPp2 - S1s)*(ns + n + 2d0)/n/np1/np2 + 8d0*S1*(2d0*n + 3d0)/np1s/np2s &
         + 2d0*(nni + 6d0*nei + 15d0*nse + 25d0*nsi + 36d0*nfi + 85d0*nfo + 128d0*nc + 104d0*ns + 64d0*n + 16d0)/nm1/nc/np1c/np2c) &
           +8d0/3d0*nff*((2d0*S1s - 2d0*S2 + 5d0)*(ns + n + 2d0)/n/np1/np2 &
           - 4d0*S1/ns + (11d0*nfo + 26d0*nc + 15d0*ns + 8d0*n + 4d0)/nc/np1c/np2)
   P1GQ(nff,:,:) = 32d0/9d0*((-2d0*S1s + 10d0*S1 - 2d0*S2)*(ns + n + 2d0)/nm1/n/np1 - 4d0*S1/np1s &
                            - (12d0*nsi + 30d0*nfi + 43d0*nfo + 28d0*nc - ns - 12d0*n -4d0)/nm1/nc/np1c) &
         +16d0*((S1s + S2 - SPp2)*(ns + n + 2d0)/nm1/n/np1 - S1*(17d0*nfo + 41d0*ns - 22d0*n - 12d0)/3d0/nm1s/ns/np1 &
             + (109d0*nni + 621d0*nei + 1400d0*nse + 1678d0*nsi + 695d0*nfi - 1031d0*nfo - 1304d0*nc - 152d0*ns + 432d0*n + 144d0) &
               /9d0/nm1s/nc/np1c/np2s) &
        +32d0/9d0*nff*((S1 - 8d0/3d0)*(ns + n + 2d0)/nm1/n/np1 + 1d0/np1s)
   P1GG(nff,:,:) = -3d0/2d0*nff*(-80d0/9d0*S1 + 16d0/3d0 + 8d0/9d0*(38d0*nfo + 76d0*nc + 94d0*ns + 56d0*n + 12d0)/nm1/ns/np1s/np2)&
         -2d0/3d0*nff*(4d0 + 8d0*(2d0*nsi + 4d0*nfi + nfo - 10d0*nc - 5d0*ns -4d0*n - 4d0)/nm1/nc/np1c/np2) &
         -9d0*(+268d0/9d0*S1 + 32d0*S1*(2d0*nfi + 5d0*nfo + 8d0*nc + 7d0*ns - 2d0*n - 2d0)/nm1s/ns/np1s/np2s - 32d0/3d0 &
                    + 16d0*SPp2*(ns + n + 1d0)/nm1/n/np1/np2 - 8d0*S1*SPp2 + 16d0*STp - 2d0*SPp3 - 2d0/9d0*( &
            457d0*nni + 2742d0*nei + 6040d0*nse + 6098d0*nsi + 1567d0*nfi - 2344d0*nfo - 1632d0*nc + 560d0*ns + 1488d0*n + 576d0) &
             /nm1s/nc/np1c/np2c)
   PP1NSp(nff,:,:) = P1NSm(nff,:,:)
   PP1NSm(nff,:,:) = P1NSp(nff,:,:)
   PP1QQ(nff,:,:) = PP1NSp(nff,:,:) - 8d0*4d0/3d0*nff/2d0*(nfo + 2d0*nc + 2d0*ns + 5d0*n + 2d0)/nc/np1c
   PP1QG(nff,:,:) = -4d0*3d0*nff*(nm1/n/np1*(-S2 + SPm2 +S1s) -4d0/n/np1s*S1 -1d0*(nfi +nfo -4d0*nc +3d0*ns -7d0*n -2d0)/nc/np1c)&
                 -8d0/3d0*nff*(2d0*nm1/n/np1*(S2-S1s) + 4d0*nm1/ns/np1*S1 -1d0*(5d0*nfi + 5d0*nfo -10d0*nc -ns +3d0*n -2d0)/nc/np1c)
   PP1GQ(nff,:,:) = -64d0/9d0*(np2/n/np1*(S2+S1s) -1d0*(3d0*ns+7d0*n+2d0)/n/np1s*S1 &
                               +(9d0*nfi+30d0*nfo+24d0*nc-7d0*ns-16d0*n-4d0)/2d0/nc/np1c)&
                    -16d0*(np2/n/np1*(-S2+SPm2-S1s) + (11d0*ns+22d0*n+12d0)/3d0/ns/np1*S1 &
                           -1d0*(76d0*nfi+271d0*nfo+254d0*nc+41d0*ns+72d0*n+36d0)/9d0/nc/np1c)&
                    -32d0/3d0*nff*(-1d0*np2/3d0/n/np1*S1 + (5d0*ns+12d0*n+4d0)/9d0/n/np1s)
   PP1GG(nff,:,:) = -24d0*nff*(-5d0/9d0*S1+(3d0*nfo+6d0*nc+16d0*ns+13d0*n-3d0)/9d0/ns/np1s) &
                    -8d0/3d0*nff*(nsi+3d0*nfi+5d0*nfo+nc-8d0*ns+2d0*n+4d0)/nc/np1c &
                   -36d0*(-0.5d0*SPm3-2d0*S1*SPm2+4d0*STm+4d0/n/np1*SPm2+(67d0*nfo+134d0*nc+67d0*ns+144d0*n+72d0)/9d0/ns/np1s*S1 &
                           -1d0*(48d0*nsi+144d0*nfi+469d0*nfo+698d0*nc+7d0*ns+258d0*n+144d0)/18d0/nc/np1c)
    do i = 0,6
       do j = 1,136
         call p2nsn(P2NSp(nff,i,j),P2NSm(nff,i,j),P2NSv(nff,i,j),n(i,j),nff)
         call p2ijn(P2PS(nff,i,j),P2QG(nff,i,j),P2GQ(nff,i,j),P2GG(nff,i,j),n(i,j),nff)
       end do
    end do
 end do
!Flavor structure of the splitting functions in the MSB factorization scheme (In the MSB scheme they are nfr-independent, however nfr may eventually enter beyond LO through factorization scheme transformations, i.e. in the DIS scheme they deppend on both nfr and nff)
 do nfr = 3,6
    !Leading order
    P0NS(-5,nfr,:,:,:) = P0QQ
    P0NS(-4,nfr,:,:,:) = P0QQ
    P0NS(-3,nfr,:,:,:) = P0QQ
    P0NS(-2,nfr,:,:,:) = P0QQ
    P0NS(-1,nfr,:,:,:) = P0QQ
    P0NS(0,nfr,:,:,:) = P0QQ
    P0NS(1,nfr,:,:,:) = P0QQ
    P0NS(2,nfr,:,:,:) = P0QQ
    P0NS(3,nfr,:,:,:) = P0QQ
    P0NS(4,nfr,:,:,:) = P0QQ
    P0NS(5,nfr,:,:,:) = P0QQ
    P0S(1,1,nfr,:,:,:) = P0QQ
    P0S(1,2,nfr,:,:,:) = P0QG
    P0S(2,1,nfr,:,:,:) = P0GQ
    P0S(2,2,nfr,:,:,:) = P0GG
    PP0NS(-5,nfr,:,:,:) = PP0QQ
    PP0NS(-4,nfr,:,:,:) = PP0QQ
    PP0NS(-3,nfr,:,:,:) = PP0QQ
    PP0NS(-2,nfr,:,:,:) = PP0QQ
    PP0NS(-1,nfr,:,:,:) = PP0QQ
    PP0NS(0,nfr,:,:,:) = PP0QQ
    PP0NS(1,nfr,:,:,:) = PP0QQ
    PP0NS(2,nfr,:,:,:) = PP0QQ
    PP0NS(3,nfr,:,:,:) = PP0QQ
    PP0NS(4,nfr,:,:,:) = PP0QQ
    PP0NS(5,nfr,:,:,:) = PP0QQ
    PP0S(1,1,nfr,:,:,:) = PP0QQ
    PP0S(1,2,nfr,:,:,:) = PP0QG
    PP0S(2,1,nfr,:,:,:) = PP0GQ
    PP0S(2,2,nfr,:,:,:) = PP0GG
    !NLO
    P1NS(-5,nfr,:,:,:) = P1NSm
    P1NS(-4,nfr,:,:,:) = P1NSm
    P1NS(-3,nfr,:,:,:) = P1NSm
    P1NS(-2,nfr,:,:,:) = P1NSm
    P1NS(-1,nfr,:,:,:) = P1NSm
    P1NS(0,nfr,:,:,:) = P1NSm
    P1NS(1,nfr,:,:,:) = P1NSp
    P1NS(2,nfr,:,:,:) = P1NSp
    P1NS(3,nfr,:,:,:) = P1NSp
    P1NS(4,nfr,:,:,:) = P1NSp
    P1NS(5,nfr,:,:,:) = P1NSp
    P1S(1,1,nfr,:,:,:) = P1QQ
    P1S(1,2,nfr,:,:,:) = P1QG
    P1S(2,1,nfr,:,:,:) = P1GQ
    P1S(2,2,nfr,:,:,:) = P1GG
    PP1NS(-5,nfr,:,:,:) = PP1NSm
    PP1NS(-4,nfr,:,:,:) = PP1NSm
    PP1NS(-3,nfr,:,:,:) = PP1NSm
    PP1NS(-2,nfr,:,:,:) = PP1NSm
    PP1NS(-1,nfr,:,:,:) = PP1NSm
    PP1NS(0,nfr,:,:,:) = PP1NSm
    PP1NS(1,nfr,:,:,:) = PP1NSp
    PP1NS(2,nfr,:,:,:) = PP1NSp
    PP1NS(3,nfr,:,:,:) = PP1NSp
    PP1NS(4,nfr,:,:,:) = PP1NSp
    PP1NS(5,nfr,:,:,:) = PP1NSp
    PP1S(1,1,nfr,:,:,:) = PP1QQ
    PP1S(1,2,nfr,:,:,:) = PP1QG
    PP1S(2,1,nfr,:,:,:) = PP1GQ
    PP1S(2,2,nfr,:,:,:) = PP1GG
    !NNLO
    P2NS(-5,nfr,:,:,:) = P2NSm
    P2NS(-4,nfr,:,:,:) = P2NSm
    P2NS(-3,nfr,:,:,:) = P2NSm
    P2NS(-2,nfr,:,:,:) = P2NSm
    P2NS(-1,nfr,:,:,:) = P2NSm
    P2NS(0,nfr,:,:,:) = P2NSv
    P2NS(1,nfr,:,:,:) = P2NSp
    P2NS(2,nfr,:,:,:) = P2NSp
    P2NS(3,nfr,:,:,:) = P2NSp
    P2NS(4,nfr,:,:,:) = P2NSp
    P2NS(5,nfr,:,:,:) = P2NSp
    P2S(1,1,nfr,:,:,:) = P2NSp + P2PS
    P2S(1,2,nfr,:,:,:) = P2QG
    P2S(2,1,nfr,:,:,:) = P2GQ
    P2S(2,2,nfr,:,:,:) = P2GG
 end do
!Factorization scheme transformations of the splitting functions
 select case(evolscheme)
  case(0)
  case(1)
    do nfr=3,6
       do i=-5,5
           P1NS(i,nfr,:,:,:) = P1NS(i,nfr,:,:,:) - bt0(nfr)*C2Q1
           P2NS(i,nfr,:,:,:) = P2NS(i,nfr,:,:,:) + bt0(nfr)*C2Q1*C2Q1 - 2d0*bt0(nfr)*C2NSp2 - bt1(nfr)*C2Q1
       end do
       do nff=3,6
          do j = 0,4
             do k = 1,136
             P1S(:,:,nfr,nff,j,k) = P1S(:,:,nfr,nff,j,k) - bt0(nfr)*CS1(:,:,nff,j,k) + comm(CS1(:,:,nff,j,k),P0S(:,:,nfr,nff,j,k))
             P2S(:,:,nfr,nff,j,k) = P2S(:,:,nfr,nff,j,k) + bt0(nfr)*prod(CS1(:,:,nff,j,k),CS1(:,:,nff,j,k)) &
                                                        - 2*bt0(nfr)*CS2(:,:,nff,j,k) - bt1(nfr)*CS1(:,:,nff,j,k) &
                                                  - prod(comm(CS1(:,:,nff,j,k),P0S(:,:,nfr,nff,j,k)),CS1(:,:,nff,j,k)) &
                                        + comm(CS1(:,:,nff,j,k),P1S(:,:,nfr,nff,j,k)) + comm(CS2(:,:,nff,j,k),P0S(:,:,nfr,nff,j,k))
             end do
          end do
       end do
    end do
  case default
     write(6,*) "MFACTOR: bad value for evolscheme = ",evolscheme
     stop
 end select
!R-operators
 do nfr = 3,6
    RONS(:,nfr,:,:,:)= P0NS(:,nfr,:,:,:)/bt0(nfr)
    PRONS(:,nfr,:,:,:)= PP0NS(:,nfr,:,:,:)/bt0(nfr)
    ROS(:,:,nfr,:,:,:) = P0S(:,:,nfr,:,:,:)/bt0(nfr)
    PROS(:,:,nfr,:,:,:) = PP0S(:,:,nfr,:,:,:)/bt0(nfr)
    R1NS(:,nfr,:,:,:)= P1NS(:,nfr,:,:,:)/bt0(nfr) - b1(nfr)*RONS(:,nfr,:,:,:)
    PR1NS(:,nfr,:,:,:)= PP1NS(:,nfr,:,:,:)/bt0(nfr) - b1(nfr)*PRONS(:,nfr,:,:,:)
    R1S(:,:,nfr,:,:,:) = P1S(:,:,nfr,:,:,:)/bt0(nfr) - b1(nfr)*ROS(:,:,nfr,:,:,:)
    PR1S(:,:,nfr,:,:,:) = PP1S(:,:,nfr,:,:,:)/bt0(nfr) - b1(nfr)*PROS(:,:,nfr,:,:,:)
    R2NS(:,nfr,:,:,:)= P2NS(:,nfr,:,:,:)/bt0(nfr) - b1(nfr)*R1NS(:,nfr,:,:,:) - b2(nfr)*RONS(:,nfr,:,:,:)
    R2S(:,:,nfr,:,:,:) = P2S(:,:,nfr,:,:,:)/bt0(nfr) - b1(nfr)*R1S(:,:,nfr,:,:,:) - b2(nfr)*ROS(:,:,nfr,:,:,:)
 end do
!Eigenvalues of ROS and proyecting matrices
 ep = 0.5d0*(ROS(1,1,:,:,:,:) + ROS(2,2,:,:,:,:) &
             + sqrt((ROS(1,1,:,:,:,:) - ROS(2,2,:,:,:,:))**2d0 + 4d0*ROS(1,2,:,:,:,:)*ROS(2,1,:,:,:,:)))
 em = 0.5d0*(ROS(1,1,:,:,:,:) + ROS(2,2,:,:,:,:) &
             - sqrt((ROS(1,1,:,:,:,:) - ROS(2,2,:,:,:,:))**2d0 + 4d0*ROS(1,2,:,:,:,:)*ROS(2,1,:,:,:,:)))
 pep = 0.5d0*(PROS(1,1,:,:,:,:) + PROS(2,2,:,:,:,:) &
             + sqrt((PROS(1,1,:,:,:,:) - PROS(2,2,:,:,:,:))**2d0 + 4d0*PROS(1,2,:,:,:,:)*PROS(2,1,:,:,:,:)))
 pem = 0.5d0*(PROS(1,1,:,:,:,:) + PROS(2,2,:,:,:,:) &
             - sqrt((PROS(1,1,:,:,:,:) - PROS(2,2,:,:,:,:))**2d0 + 4d0*PROS(1,2,:,:,:,:)*PROS(2,1,:,:,:,:)))
!Proyecting matrices (the nfr deppendence drops in the ratios)
 Rp(1,1,:,:,:,:) = (ROS(1,1,:,:,:,:) - em)/(ep - em)
 Rp(1,2,:,:,:,:) = ROS(1,2,:,:,:,:)/(ep - em)
 Rp(2,1,:,:,:,:) = ROS(2,1,:,:,:,:)/(ep - em)
 Rp(2,2,:,:,:,:) = 1 - Rp(1,1,:,:,:,:)
 Rm(1,1,:,:,:,:) = Rp(2,2,:,:,:,:)
 Rm(1,2,:,:,:,:) = -Rp(1,2,:,:,:,:)
 Rm(2,1,:,:,:,:) = -Rp(2,1,:,:,:,:)
 Rm(2,2,:,:,:,:) = Rp(1,1,:,:,:,:)
 PRp(1,1,:,:,:,:) = (PROS(1,1,:,:,:,:) - pem)/(pep - pem)
 PRp(1,2,:,:,:,:) = PROS(1,2,:,:,:,:)/(pep - pem)
 PRp(2,1,:,:,:,:) = PROS(2,1,:,:,:,:)/(pep - pem)
 PRp(2,2,:,:,:,:) = 1 - PRp(1,1,:,:,:,:)
 PRm(1,1,:,:,:,:) = PRp(2,2,:,:,:,:)
 PRm(1,2,:,:,:,:) = -PRp(1,2,:,:,:,:)
 PRm(2,1,:,:,:,:) = -PRp(2,1,:,:,:,:)
 PRm(2,2,:,:,:,:) = PRp(1,1,:,:,:,:)
!Evolution operators
 do nff = 3,6
 do nfr = 3,6
 do j = 0,6
 do k = 1,136
   U1NS(:,nfr,nff,j,k) = - R1NS(:,nfr,nff,j,k)
   U1S(:,:,nfr,nff,j,k) = -(  prod(Rp(:,:,nfr,nff,j,k),prod(R1S(:,:,nfr,nff,j,k),Rp(:,:,nfr,nff,j,k)))&
                      + prod(Rm(:,:,nfr,nff,j,k),prod(R1S(:,:,nfr,nff,j,k),Rm(:,:,nfr,nff,j,k))))&
       + prod(Rp(:,:,nfr,nff,j,k),prod(R1S(:,:,nfr,nff,j,k),Rm(:,:,nfr,nff,j,k)))/(em(nfr,nff,j,k) - ep(nfr,nff,j,k) - 1d0)&
       + prod(Rm(:,:,nfr,nff,j,k),prod(R1S(:,:,nfr,nff,j,k),Rp(:,:,nfr,nff,j,k)))/(ep(nfr,nff,j,k) - em(nfr,nff,j,k) - 1d0)
   PU1NS(:,nfr,nff,j,k) = - PR1NS(:,nfr,nff,j,k)
   PU1S(:,:,nfr,nff,j,k) = -(  prod(PRp(:,:,nfr,nff,j,k),prod(PR1S(:,:,nfr,nff,j,k),PRp(:,:,nfr,nff,j,k)))&
                      + prod(PRm(:,:,nfr,nff,j,k),prod(PR1S(:,:,nfr,nff,j,k),PRm(:,:,nfr,nff,j,k))))&
       + prod(PRp(:,:,nfr,nff,j,k),prod(PR1S(:,:,nfr,nff,j,k),PRm(:,:,nfr,nff,j,k)))/(pem(nfr,nff,j,k) - pep(nfr,nff,j,k) - 1d0)&
       + prod(PRm(:,:,nfr,nff,j,k),prod(PR1S(:,:,nfr,nff,j,k),PRp(:,:,nfr,nff,j,k)))/(pep(nfr,nff,j,k) - pem(nfr,nff,j,k) - 1d0)
   R2strNS(:,nfr,nff,j,k) = R1NS(:,nfr,nff,j,k)*U1NS(:,nfr,nff,j,k) + R2NS(:,nfr,nff,j,k)
   R2strS(:,:,nfr,nff,j,k) = prod(R1S(:,:,nfr,nff,j,k),U1S(:,:,nfr,nff,j,k)) + R2S(:,:,nfr,nff,j,k)
   U2NS(:,nfr,nff,j,k) = - 0.5d0*R2strNS(:,nfr,nff,j,k)
   U2S(:,:,nfr,nff,j,k) = -0.5d0*(  prod(Rp(:,:,nfr,nff,j,k),prod(R2strS(:,:,nfr,nff,j,k),Rp(:,:,nfr,nff,j,k)))&
                          + prod(Rm(:,:,nfr,nff,j,k),prod(R2strS(:,:,nfr,nff,j,k),Rm(:,:,nfr,nff,j,k))))&
    + prod(Rp(:,:,nfr,nff,j,k),prod(R2strS(:,:,nfr,nff,j,k),Rm(:,:,nfr,nff,j,k)))/(em(nfr,nff,j,k) - ep(nfr,nff,j,k) - 2d0)&
    + prod(Rm(:,:,nfr,nff,j,k),prod(R2strS(:,:,nfr,nff,j,k),Rp(:,:,nfr,nff,j,k)))/(ep(nfr,nff,j,k) - em(nfr,nff,j,k) - 2d0)
 end do
 end do
 end do
 end do
!NNLO Matching coefficients for VFNS evolution
 A2PSHQ = 2d0/3d0*(-8d0*(nfo+2d0*nc+5d0*ns+4d0*n+4d0)/nm1/ns/np1s/np2*S2m - 448d0/27d0/nm1 - 44d0/n +48d0/ns &
             -4d0/nc +24d0/nfo -12d0/np1 + 56d0/np1s&
             +28d0/np1c +24d0/np1c/np1 + 1960d0/27d0/np2 + 448d0/9d0/np2s + 64d0/3d0/np2c)
 A2SHG = +1.111d0*S1s*S1/n -0.4d0*S1s/n + (2.77d0/n + 293.6d0/nc)*S1 +3.333d0/n*S1*S2&
              -(0.4d0/n -293.6d0/ns)*S2 + 295.822d0*S3/n - 0.006d0 -24.89d0/nm1-(187.8d0+293.6d0*1.2020569031595942854d0)/n&
              +(93.68d0-293.6d0*1.6449340668482264365d0)/ns -6.584d0/nc + 9.336d0/nfo +249.6d0/np1
 A2NSQQ = 2d0/3d0*(-224d0/27d0*S1m + 40d0/9d0*S2m - 8d0/3d0*S3m +73d0/18d0 + 44d0/27d0/n -4d0/9d0/ns -268d0/27d0/np1&
             +44d0/9d0/np1s - 4d0/3d0/nc -4d0/3d0/np1c)
 A2SGQ = 2d0/3d0*(8d0/3d0/nm1*(S1ms-10d0/3d0*S1m+S2m+56d0/9d0)&
                  -8d0/3d0/n*(S1s-10d0/3d0*S1+S2+56d0/9d0)&
                  +4d0/3d0/np1*(S1ps-16d0/3d0*S1p+S2p+86d0/9d0))
 A2SGG = 4d0*2d0/3d0*(-15d0/4d0 -2d0/nm1+20d0/n-8d0/ns+3d0/nc-2d0/nfo-12d0/np1-12d0/np1s+5d0/np1c-2d0/np1c/np1-6d0/np2) &
         +4d0*3d0/2d0*(-56d0/27d0*S1m+S1p/3d0/np1+5d0/18d0+139d0/27d0/nm1-157d0/27d0/n-13d0/9d0/ns+2d0/3d0/nc+137d0/27d0/np1&
                       -22d0/9d0/np1s+2d0/3d0/np1c-175d0/27d0/np2)
 return
end subroutine mfactors
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
