!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2005 - 2014 Pedro Jimenez-Delgado
! 
! This file is part of Chromopolis.
! 
! Chromopolis is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
! 
! Chromopolis is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with Chromopolis.  If not, see <http://www.gnu.org/licenses/>.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!! NUCLEAR CORRECTIONS FOR DEUTERON AND HELIUM STRUCTURE FUNCTIONS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine deuteronF2(x,Q2,F2d)
 implicit none
 integer i
 double precision x,Q2,F2d,gamma,y,ymin,ymax,F2N,f22
 double precision,external::F2de,int2d_f22
 double precision XI(96),WI(96),XX(97)
 double precision,parameter::mN=0.9389185d0
 double precision Doffsh,Dshad
 double precision,external::Doffsh_KPMDL,Dshad_MT
 common /GAUS96/ XX,XI,WI
 gamma = dsqrt(1d0 + 4d0*x**2d0*mN**2d0/Q2)
 !Nucleonic variables x,y in [0,2]
 ymax = 2d0
 ymin = x
 F2d = 0d0
 do i = 1,96
    y = 0.5d0*(ymax-ymin)*XI(i)+0.5d0*(ymax+ymin)
    f22 = int2d_f22(y,gamma)
    if (x/y < 0.99d0) then
    F2N = F2de(x/y,Q2)
    else
    F2N = 0d0
    end if
    F2d = F2d + 0.5d0*(ymax-ymin)*WI(i)*f22*F2N
 end do
 !Off-shell correction
 Doffsh = Doffsh_KPMDL(x,0)
 F2d = F2d/(1d0-Doffsh)
 !Shadowing correction
 Dshad = Dshad_MT(x,Q2)
 F2d = F2d/(1d0+Dshad)
 return
end subroutine deuteronF2

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine deuteronFL(x,Q2,FLd)
 implicit none
 integer i
 double precision x,Q2,FLd,gamma,y,ymin,ymax,F2N,FLN,gammaxy
 double precision f22,fLL,fL2,fTT,fT2
 double precision,external::F2de,FLde,int2d_f22,int2d_fTT,int2d_fT2
 double precision XI(96),WI(96),XX(97)
 double precision,parameter::mN=0.9389185d0
 common /GAUS96/ XX,XI,WI
 gamma = dsqrt(1d0 + 4d0*x**2d0*mN**2d0/Q2)
 !Nucleonic variables x,y in [0,2]
 ymax = 2d0
 ymin = x
 FLd = 0d0
 do i = 1,96
    y = 0.5d0*(ymax-ymin)*XI(i)+0.5d0*(ymax+ymin)
    f22 = int2d_f22(y,gamma)
    fTT = int2d_fTT(y,gamma)
    fT2 = int2d_fT2(y,gamma)
    gammaxy = dsqrt(1d0 + 4d0*(x/y)**2d0*mN**2d0/Q2)
    fLL = fTT
    fL2 = gamma**2d0*f22 - gammaxy**2d0*fTT - fT2
    if (x/y < 0.99d0) then
       FLN = FLde(x/y,Q2)
       F2N = F2de(x/y,Q2)
    else
       FLN = 0d0
       F2N = 0d0
    end if
    FLd = FLd + 0.5d0*(ymax-ymin)*WI(i)*(fLL*FLN + fL2*F2N)
 end do
 return
end subroutine deuteronFL

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine deuteronPSF(x,Q2,gd)
 implicit none
 integer i
 double precision x,Q2,gd(1:2),gamma,y,ymin,ymax
 double precision g11,g12,g21,g22,xg1N,xg2N,g(-2:2)
 double precision,external::int2d_g11,int2d_g12
 double precision,external::int2d_g21,int2d_g22
 double precision,external::T3g1d,T4g1d,T3g2d
 double precision XI(96),WI(96),XX(97)
 double precision,parameter::mN=0.9389185d0
 common /GAUS96/ XX,XI,WI
 gamma = dsqrt(1d0 + 4d0*x**2d0*mN**2d0/Q2)
 !Nucleonic variables x,y in [0,2]
 ymax = 2d0
 ymin = x
 gd = 0d0
 do i = 1,96
    y = 0.5d0*(ymax-ymin)*XI(i)+0.5d0*(ymax+ymin)
    g11 = int2d_g11(y,gamma)
    g12 = int2d_g12(y,gamma)
    g21 = int2d_g21(y,gamma)
    g22 = int2d_g22(y,gamma)
    if (x/y < 0.99d0) then
       call pstructurefs(x/y,Q2,g)
       xg1N = x/y*(g(1) + g(-1) + T3g1d(x/y,Q2) +  T4g1d(x/y)/Q2)
       xg2N = x/y*(g(2) + g(-2) + T3g2d(x/y))
    else
       xg1N = 0d0
       xg2N = 0d0
    end if
    gd(1) = gd(1) + 0.5d0*(ymax-ymin)*WI(i)*(g11*xg1N + g12*xg2N)/x
    gd(2) = gd(2) + 0.5d0*(ymax-ymin)*WI(i)*(g21*xg1N + g22*xg2N)/x
 end do
 return
end subroutine deuteronPSF

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine heliumSF(x,Q2,FHe)
 implicit none
 integer i
 double precision x,Q2,FHe(1:2),gamma,y,ymin,ymax,gammaxy
 double precision f11p,f12p,f22p,f11n,f12n,f22n
 double precision,external::int2d_f11p,int2d_f12p,int2d_f22p
 double precision,external::int2d_f11n,int2d_f12n,int2d_f22n
 double precision F(-3:3),xF1(-1:1),xF1He
 double precision,external::T4F2p,T6F2p,T4FLp,T6FLp
 double precision,external::T4F2n,T6F2n,T4FLn,T6FLn
 double precision XI(96),WI(96),XX(97)
 double precision,parameter::mN=0.9389185d0
 common /GAUS96/ XX,XI,WI
 gamma = dsqrt(1d0 + 4d0*x**2d0*mN**2d0/Q2)
 !Nucleonic variables x,y in [0,3]
 ymax = 3d0
 ymin = x
 xF1He = 0d0
 FHe = 0d0
 do i = 1,96
    y = 0.5d0*(ymax-ymin)*XI(i)+0.5d0*(ymax+ymin)
    f11p = int2d_f11p(y,gamma)
    f12p = int2d_f12p(y,gamma)
    f22p = int2d_f22p(y,gamma)
    f11n = int2d_f11n(y,gamma)
    f12n = int2d_f12n(y,gamma)
    f22n = int2d_f22n(y,gamma)
    if (x/y < 0.99d0) then
       call structurefs(x/y,Q2,F)
    else
       F = 0d0
    end if
    F(2) = F(2) + T4F2p(x/y)/Q2 + T6F2p(x/y)/Q2/Q2
    F(-2) = F(-2) + T4F2n(x/y)/Q2 + T6F2n(x/y)/Q2/Q2
    F(1) = F(1) + T4FLp(x/y)/Q2 + T6FLp(x/y)/Q2/Q2
    F(-1) = F(-1) + T4FLn(x/y)/Q2 + T6FLn(x/y)/Q2/Q2
    gammaxy = dsqrt(1d0 + 4d0*(x/y)**2d0*mN**2d0/Q2)
    xF1(1) = (gammaxy*gammaxy*F(2)-F(1))/2d0
    xF1(-1) = (gammaxy*gammaxy*F(-2)-F(-1))/2d0
    xF1He = xF1He + 0.5d0*(ymax-ymin)*WI(i)*(2d0*f11p*xF1(1) + 2d0*f12p*F(2) + f11n*xF1(-1) + f12n*F(-2))
    FHe(2) = FHe(2) + 0.5d0*(ymax-ymin)*WI(i)*(2d0*f22p*F(2) + f22n*F(-2))
 end do
    FHe(1) = gamma*gamma*FHe(2) - 2d0*xF1He
    FHe = FHe/3d0
 return
end subroutine heliumSF

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine heliumPSF(x,Q2,gHe)
 implicit none
 integer i
 double precision x,Q2,gHe(1:2),gamma,y,ymin,ymax,g(-2:2)
 double precision g11p,g12p,g21p,g22p
 double precision g11n,g12n,g21n,g22n
 double precision,external::int2d_g11p,int2d_g12p
 double precision,external::int2d_g21p,int2d_g22p
 double precision,external::int2d_g11n,int2d_g12n
 double precision,external::int2d_g21n,int2d_g22n
 double precision,external::T3g1p,T4g1p,T3g2p
 double precision,external::T3g1n,T4g1n,T3g2n
 double precision XI(96),WI(96),XX(97)
 double precision,parameter::mN=0.9389185d0
 common /GAUS96/ XX,XI,WI
 gamma = dsqrt(1d0 + 4d0*x**2d0*mN**2d0/Q2)
 !Convolution uses nucleonic variables x,y in [0,3]
 ymax = 3d0
 ymin = x
 gHe = 0d0
 do i = 1,96
    y = 0.5d0*(ymax-ymin)*XI(i)+0.5d0*(ymax+ymin)
    g11p = int2d_g11p(y,gamma)
    g12p = int2d_g12p(y,gamma)
    g21p = int2d_g21p(y,gamma)
    g22p = int2d_g22p(y,gamma)
    g11n = int2d_g11n(y,gamma)
    g12n = int2d_g12n(y,gamma)
    g21n = int2d_g21n(y,gamma)
    g22n = int2d_g22n(y,gamma)
    if (x/y < 0.99d0) then
       call pstructurefs(x/y,Q2,g)
       g(1) = x/y*(g(1) + T3g1p(x/y,Q2) +  T4g1p(x/y)/Q2)
       g(2) = x/y*(g(2) + T3g2p(x/y))
       g(-1) = x/y*(g(-1) + T3g1n(x/y,Q2) +  T4g1n(x/y)/Q2)
       g(-2) = x/y*(g(-2) + T3g2n(x/y))
    else
       g = 0d0
    end if
    gHe(1) = gHe(1) + 0.5d0*(ymax-ymin)*WI(i)*(g11p*g(1) + g12p*g(2) + g11n*g(-1) + g12n*g(-2))/x
    gHe(2) = gHe(2) + 0.5d0*(ymax-ymin)*WI(i)*(g21p*g(1) + g22p*g(2) + g21n*g(-1) + g22n*g(-2))/x
 end do
 return
end subroutine heliumPSF

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_f22(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=26,ny=1001
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/f2.WBARELparis")
    do igam=1,ngam
  do iy=1,ny
     read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
  end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_f22 = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.2d0,0.002d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_fTT(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=26,ny=1001
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/fTT.WBARELparis")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_fTT = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.2d0,0.002d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_fT2(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=26,ny=1001
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/fT2.WBARELparis")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_fT2 = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.2d0,0.002d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g11(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=26,ny=1001
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g11.WBARELparis")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g11 = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.2d0,0.002d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g12(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=26,ny=1001
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g12.WBARELparis")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g12 = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.2d0,0.002d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g21(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=26,ny=1001
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g21.WBARELparis")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g21 = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.2d0,0.002d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 double precision function int2d_g22(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=26,ny=1001
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g22.WBARELparis")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g22 = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.2d0,0.002d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_f11p(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_f11p = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/f11p.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_f11p = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_f12p(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_f12p = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/f12p.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_f12p = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_f22p(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_f22p = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/f22p.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_f22p = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_f11n(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_f11n = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/f11n.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_f11n = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_f12n(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_f12n = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/f12n.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_f12n = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_f22n(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_f22n = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/f22n.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_f22n = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g11p(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_g11p = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g11p.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g11p = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g12p(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_g12p = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g12p.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g12p = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g21p(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_g21p = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g21p.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g21p = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g22p(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_g22p = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g22p.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g22p = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g11n(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_g11n = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g11n.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g11n = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g12n(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_g12n = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g12n.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g12n = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g21n(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_g21n = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g21n.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g21n = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function int2d_g22n(y,gamma)
 implicit none
 logical::saved=.false.
 integer,parameter::ngam=11,ny=200
 integer igam,iy
 double precision y,gamma
 double precision g_array(ngam),y_array(ny),f_array(ngam,ny)
 double precision,external::dinterp2d
 save g_array,y_array,f_array
 if (y>2d0) then
    int2d_g22n = 0d0
    return
 end if
 if (.not.saved) then
    open(124,file="../../code/nuclearSF/g22n.K")
    do igam=1,ngam
       do iy=1,ny
          read(124,*) g_array(igam),y_array(iy),f_array(igam,iy)
       end do
    end do
    close(124)
    saved = .true.
 end if
 int2d_g22n = dinterp2d(g_array,y_array,f_array,gamma,y,ngam,ny,0.1d0,0.01d0)
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! KPMDL nucleon off-shell correction from
!! Kulagin, Petti, NPA765 (2006) 126
!!
!! Defined such that F2d(corrected) = F2d + Doffsh*F2d(corrected)
!!         = F2d/(1d0-Doffsh)
!! imode = -1,0,1 --> lowest(least negative), central, highest(most negative)
!!
!! Parameters defined for nucleonic variable x in [0,2]
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision function Doffsh_KPMDL(x,imode)
 implicit none
 integer imode
 double precision x, poff(10),pmin(10),pmax(10)
 data poff /0.00917324d0,-4.87488d0,-0.129925d0,24.1748d0,0.249259d0,-4.24231d0,-0.0353244d0,-59.7096d0,-0.214936d0,46.3766d0/
 data pmin /0.00606295d0,-3.97072d0,-0.0551534d0,10.5675d0,-0.245408d0,-5.09068d0,0.699754d0,-18.0466d0,-0.426636d0,24.6164d0/
 data pmax /0.00964982d0,-6.84144d0,-0.0985649d0,43.0597d0,-0.0181488d0,-23.9927d0,0.616283d0,-75.3024d0,-0.711494d0,65.4527d0/
 Doffsh_KPMDL = 0d0
 if (imode.eq.-1) then! Lower limit (least negative)
    Doffsh_KPMDL = (pmin(1) + pmin(3)*x + pmin(5)*x**2 + pmin(7)*x**3 + pmin(9)*x**4)* &
                   (1d0     + pmin(2)*x + pmin(4)*x**2 + pmin(6)*x**3 + pmin(8)*x**4 + pmin(10)*x**5)
 else if (imode.eq.0) then! Central
    Doffsh_KPMDL = (poff(1) + poff(3)*x + poff(5)*x**2 + poff(7)*x**3 + poff(9)*x**4)* &
                   (1d0     + poff(2)*x + poff(4)*x**2 + poff(6)*x**3 + poff(8)*x**4 + poff(10)*x**5)
 else if (imode.eq.1) then! Upper limit (most negative)
    Doffsh_KPMDL = (pmax(1) + pmax(3)*x + pmax(5)*x**2 + pmax(7)*x**3 + pmax(9)*x**4)* &
                   (1d0     + pmax(2)*x + pmax(4)*x**2 + pmax(6)*x**3 + pmax(8)*x**4 + pmax(10)*x**5)
 end if
 return
end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Nuclear shadowing correction
!! Melnitchouk, Thomas, PRD47 (1993) 3783
!! includes VMD, Pomeron and meson (antishadowing) exchange
!!
!! Defined such that F2d(corrected) = F2d - Dshad*F2d(corrected)
!!         = F2d/(1d0+DShad)
!!
!! Parameters defined for nucleonic variable x in [0,2]
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
double precision FUNCTION Dshad_MT(x,Q2)
 implicit none
 double precision x,Q2,Q02,DEL_V,DEL_P,DEL_M
 double precision A(0:4),B(0:4),C(0:4)
 data A /-0.038d0,-0.04d0,1.8d0,-3.4d0,0.9d0/
 data B /-0.003d0,-0.13d0,5.d0, -2.2d0,0.4d0/
 data C / 0.002d0, 0.03d0,6.d0,    0d0,  0d0/
 Dshad_MT = 0d0
 if (x.gt.0.3d0) return !parametrization valid for x <~ 0.3
 !VMD (Q2 in GeV^2, Q02 = 0.7 GeV^2)
 Q02 = 0.7d0
 DEL_V = Q2/(1 + Q2/Q02)**2 * A(0) * x**A(1) * (1.d0-x)**A(2) * (1.d0 + A(3)*x**A(4))
 !Pomeron-exchange
 DEL_P = B(0) * x**B(1) * (1.d0-x)**B(2) * (1.d0 + B(3)*x**B(4))
 !Meson-exchange (antishadowing)
 DEL_M = C(0) * x**C(1) * (1.d0-x)**C(2) * (1.d0 + C(3)*x**C(4))
 Dshad_MT = DEL_V + DEL_P + DEL_M
 return
end
