!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2005 - 2014 Pedro Jimenez-Delgado
! 
! This file is part of Chromopolis.
! 
! Chromopolis is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
! 
! Chromopolis is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with Chromopolis.  If not, see <http://www.gnu.org/licenses/>.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!! POLARIZED DEEP INELASTIC SCATTERING NUCLEON STRUCTURE FUNCTIONS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Polarized DIS nucleon structure functions !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!    x = Bjorken-x
!!    Q2 = Q**2 (GeV**2) == DIS Momentum transfer squared.
!!    g = Output = 1(g1p),2(g2p)
!!                -1(g1n),-2(g2n)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 subroutine pstructurefs(x,Q2,g)
 implicit none
 logical resetPSFsx
 integer::i,tmcflag,errors,tmcflagold
 double precision,parameter::mp=0.938272d0,mp2=mp*mp,mn=0.939565d0,mn2=mn*mn
 double precision::x,xold,Q2,Q2old,Q2c,g(-2:2),gold(-2:2),as,asQ2,asQ2s,m2Q2(-1:1)=0d0,m4Q4(-1:1)=0d0,m6Q6(-1:1)=0d0
 double complex,dimension(0:6,136)::n,ns,nc,nfo,nfi,nsi,nse,nei,nni,np1,np1s,np1c,nm1,nm1s,np2,np2s,np2c,np3,np3s,np4,&
                                     S1,S1s,S2,SPp2,SPm2,SPp3,SPm3,STp,STm,S1m,S1ms,S2m,S3m,S1p,S1ps,S2p,S1p2
 double complex::sfn(-2:2,0:6,136)
 external as
 common /tmcc/ tmcflag
 common /transfer/ Q2c,asQ2,asQ2s
 common /support/ n
 common /mspacec/ ns,nc,nfo,nfi,nsi,nse,nei,nni,np1,np1s,np1c,nm1,nm1s,np2,np2s,np2c,np3,np3s,np4,&
                  S1,S1s,S2,SPp2,SPm2,SPp3,SPm3,STp,STm,S1m,S1ms,S2m,S3m,S1p,S1ps,S2p,S1p2
 common /errorsc/ errors
 common /resetPSFsxc/ resetPSFsx
 save xold,Q2old,sfn,gold,tmcflagold
 if ((x<0d0).or.((x>1d0).and.((x-1d0)>1d-10))) then
    write(*,*) "PSTRUCTUREFS: x out of range",x
    stop
 end if
 if ((x==xold).and.(Q2==Q2old).and.(tmcflag==tmcflagold).and.(.not.resetPSFsx).and.(errors==0)) then
    g = gold
    return
 end if
 xold=x
 if ((Q2/=Q2old).or.(tmcflag/=tmcflagold).or.(resetPSFsx).or.(errors/=0)) then
    if (tmcflag >= 0) then
       Q2c = Q2
       asQ2 = as(Q2)
       asQ2s = asQ2*asQ2
       call pnstructurefs(0,sfn(-1:1,0,:))
    else
       write(6,*) 'PSTRUCTUREFS: Bad value of tmcflag = ',tmcflag
       stop
    end if
    if (tmcflag >= 1) then
       m2Q2(-1) = mn2/Q2
       m2Q2(1) = mp2/Q2
       call pnstructurefs(2,sfn(-1:1,2,:))
       do i = -1,1,2
          sfn(isign(1,i),0,:) = sfn(isign(1,i),0,:) + ns(0,:)*np1(0,:)/np2s(0,:)*m2Q2(isign(1,i))*sfn(isign(1,i),2,:)
       end do
    end if
    if (tmcflag >= 2) then
       m4Q4 = m2Q2*m2Q2
       call pnstructurefs(4,sfn(-1:1,4,:))
       do i = -1,1,2
          sfn(isign(1,i),0,:) = sfn(isign(1,i),0,:) +&
                                               ns(0,:)*np1(0,:)*np2(0,:)/2d0/np4(0,:)/np4(0,:)*m4Q4(isign(1,i))*sfn(isign(1,i),4,:)
       end do
    end if
    if (tmcflag >= 3) then
       m6Q6 = m4Q4*m2Q2
       call pnstructurefs(6,sfn(-1:1,6,:))
       do i = -1,1,2
          sfn(isign(1,i),0,:) = sfn(isign(1,i),0,:) +&
                                      ns(0,:)*np1(0,:)*np2(0,:)*np3(0,:)/6d0/np4(2,:)/np4(2,:)*m6Q6(isign(1,i))*sfn(isign(1,i),6,:)
       end do
    end if
    if (tmcflag > 3) then
       write(6,*) 'STRUCTUREFS: Bad value of tmcflag = ',tmcflag
       stop
    end if
    Q2old=Q2
    tmcflagold=tmcflag
    resetPSFsx=.false.
 end if
 do i = -1,1,2
    sfn(isign(2,i),0,:) = -nm1(0,:)/n(0,:)*sfn(isign(1,i),0,:)
 end do
 !Mellin inversion and output
 g=0d0
 g=0d0/g
 do i = -2,2
    if (i == 0) cycle
    call minversion(x,sfn(i,0,:),g(i))
 end do
 g = g/2d0/x
 gold = g
 return
 end subroutine pstructurefs

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Mellin moments of polarized DIS Nucleon Structure Functions !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 subroutine pnstructurefs(isup,sfn)
 use mfactor
 implicit none
 logical resetPSFs
 integer,parameter::saving=10000
 integer,save::nsave=0,evolscheme_save(saving)=-1,isup_save(saving)=-1,nff_save(saving)=-1
 integer::isup,order,nff,evolscheme,i
 double precision,parameter::eu=0.666666d0,ed=-0.333333d0
 double precision a2Lu,a2Ld,a2Lvp3,a2Lvp8,a2Lvp15,a2Lvp24,a2Lvp35,a2Lqs
 double precision::Q2,asQ2,asQ2s
 double precision,save::Q2_save(saving)=0d0
 double complex::pdfn(-6:6,136),qns(-5:5,136),qs(2,136),sfn(-1:1,136),q2Lns(136),q2Lns_neut(136)
 double complex::qns_dis(-5:5,136),qs_dis(2,136)
 double complex,save::sfn_save(saving,-1:1,136)
 common /orderc/ order
 common /nfsfc/ nff
 common /transfer/ Q2,asQ2,asQ2s
 common /evolschemec/ evolscheme
 common /disf2/ qns_dis,qs_dis
 common /resetPSFsc/ resetPSFs
 if(resetPSFs) then
    nsave=0
    resetPSFs = .false.
 end if
 do i=1,nsave
    if (dabs(Q2-Q2_save(i))<1d-5) then
       if ((evolscheme==evolscheme_save(i)).and.(isup==isup_save(i)).and.(nff==nff_save(i))) then
          sfn =  sfn_save(i,:,:)
          return
       end if
    end if
 end do
 !Getting pdfs and nff
 sfn = dcmplx(0d0,0d0)
 call nppdfs(0,isup,pdfn,qns,qs)
 !Couplings
 a2Lu = eu*eu
 a2Ld = ed*ed
 a2Lvp3 = (a2Lu - a2Ld)/2d0
 a2Lvp8  = (a2Lu - a2Ld)/6d0
 a2Lvp15 = (a2Ld - a2Lu)/6d0
 a2Lvp24  = (a2Lu - a2Ld)/10d0
 a2Lvp35 = (a2Ld - a2Lu)/10d0
 select case(nff)
  case(3)
     a2Lqs = (a2Lu + 2d0*a2Ld)/3d0
  case(4)
     a2Lqs = (2d0*a2Lu + 2d0*a2Ld)/4d0
  case(5)
     a2Lqs = (2d0*a2Lu + 3d0*a2Ld)/5d0
  case(6)
     a2Lqs = (3d0*a2Lu + 3d0*a2Ld)/6d0
  case default
     write(6,*) 'NPDFS: Bad factorization flavour number = ',nff
     stop
  end select
 !Appropriate combinations of non-singlet parton distributions
 q2Lns = a2Lqs*qs(1,:) + a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:) + a2Lvp15*qns(3,:) + a2Lvp24*qns(4,:) + a2Lvp35*qns(5,:)
 q2Lns_neut = a2Lqs*qs(1,:) - a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:) + a2Lvp15*qns(3,:) + a2Lvp24*qns(4,:) + a2Lvp35*qns(5,:)
 !Moments of Structure Functions
 do i = 1, 136
    if (order >= 0) then!Leading order
    sfn(-1,i) = q2Lns_neut(i)
    sfn(1,i) = q2Lns(i)
    else
       write(6,*) 'NPDFS: Bad value of order = ',order
       stop
    end if
    if (order >= 1) then!Addition of the NLO contribution
    sfn(-1,i) = sfn(-1,i) + asQ2*(PC1Q1(nff,isup,i)*q2Lns_neut(i) + a2Lqs*PC1G1(nff,isup,i)*qs(2,i))
    sfn(1,i) = sfn(1,i) + asQ2*(PC1Q1(nff,isup,i)*q2Lns(i) + a2Lqs*PC1G1(nff,isup,i)*qs(2,i))
    end if
    if (order > 1) then
       write(6,*) 'NPDFS: Bad value of order = ',order
       stop
    end if
 end do
 !Saving
 nsave=nsave+1
 Q2_save(nsave) = Q2
 evolscheme_save(nsave) = evolscheme
 isup_save(nsave) = isup
 nff_save(nsave) = nff
 sfn_save(nsave,:,:) = sfn
 return
 end subroutine pnstructurefs
