!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2005 - 2014 Pedro Jimenez-Delgado
! 
! This file is part of Chromopolis.
! 
! Chromopolis is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
! 
! Chromopolis is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with Chromopolis.  If not, see <http://www.gnu.org/licenses/>.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!! DEEP INELASTIC SCATTERING NUCLEON STRUCTURE FUNCTIONS!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! DIS Neutral current nucleon structure functions !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!    x = Bjorken-x
!!    Q2 = Q**2 (GeV**2) == DIS Momentum transfer squared.
!!    F = Output = 1(FLp), 2(F2p), 3(xF3p)
!!                -1(FLn),-2(F2n),-3(xF3n)
!!    tmcflag = external common block = Order used in target mass corrections for F2 = 0 (no TMC), 1(mn**2), 2(mn**4), 3(mn**6).
!!    nccflag = external common block = structure functions for photon exchange only(0) or the whole neutral current(1).
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 subroutine structurefs(x,Q2,F)
 implicit none
 logical resetSFsx
 integer::i,tmcflag,errors,tmcflagold
 double precision,parameter::mp=0.938272d0,mp2=mp*mp,mn=0.939565d0,mn2=mn*mn
 double precision::x,xold,Q2,Q2old,Q2c,F(-3:3),Fold(-3:3),as,asQ2,asQ2s,m2Q2(-1:1)=0d0,m4Q4(-1:1)=0d0,m6Q6(-1:1)=0d0
 double complex,dimension(0:6,136)::n,ns,nc,nfo,nfi,nsi,nse,nei,nni,np1,np1s,np1c,nm1,nm1s,np2,np2s,np2c,np3,np3s,np4,&
                                     S1,S1s,S2,SPp2,SPm2,SPp3,SPm3,STp,STm,S1m,S1ms,S2m,S3m,S1p,S1ps,S2p,S1p2
 double complex::sfn(-3:3,0:6,136)
 external as
 common /tmcc/ tmcflag
 common /transfer/ Q2c,asQ2,asQ2s
 common /support/ n
 common /mspacec/ ns,nc,nfo,nfi,nsi,nse,nei,nni,np1,np1s,np1c,nm1,nm1s,np2,np2s,np2c,np3,np3s,np4,&
                  S1,S1s,S2,SPp2,SPm2,SPp3,SPm3,STp,STm,S1m,S1ms,S2m,S3m,S1p,S1ps,S2p,S1p2
 common /errorsc/ errors
 common /resetSFsxc/ resetSFsx
 save xold,Q2old,sfn,Fold,tmcflagold
 if ((x<0d0).or.((x>1d0).and.((x-1d0)>1d-10))) then
    write(*,*) "STRUCTUREFS: x out of range",x
    stop
 end if
 if ((x==xold).and.(Q2==Q2old).and.(tmcflag==tmcflagold).and.(.not.resetSFsx).and.(errors==0)) then
    F = Fold
    return
 end if
 xold=x
 if ((Q2/=Q2old).or.(tmcflag/=tmcflagold).or.(resetSFsx).or.(errors/=0)) then
    if (tmcflag >= 0) then
       Q2c = Q2
       asQ2 = as(Q2)
       asQ2s = asQ2*asQ2
       call nstructurefs(0,sfn(:,0,:))
    else
       write(6,*) 'STRUCTUREFS: Bad value of tmcflag = ',tmcflag
       stop
    end if
    if (tmcflag >= 1) then
       m2Q2(-1) = mn2/Q2
       m2Q2(1) = mp2/Q2
       call nstructurefs(2,sfn(:,2,:))
       do i = -1,1,2
          sfn(isign(2,i),0,:) = sfn(isign(2,i),0,:) + n(0,:)*nm1(0,:)/np2(0,:)*m2Q2(isign(1,i))*sfn(isign(2,i),2,:)
          sfn(isign(1,i),0,:) = sfn(isign(1,i),0,:) + np1(0,:)*m2Q2(isign(1,i))*sfn(isign(1,i),2,:) &
                                                    + 4d0/np2(0,:)*m2Q2(isign(1,i))*sfn(isign(2,i),2,:)
       end do
    end if
    if (tmcflag >= 2) then
       m4Q4 = m2Q2*m2Q2
       call nstructurefs(4,sfn(:,4,:))
       do i = -1,1,2
          sfn(isign(2,i),0,:) = sfn(isign(2,i),0,:)&
                                + np2(0,:)*np1(0,:)*n(0,:)*nm1(0,:)/2d0/np4(0,:)/np3(0,:)*m4Q4(isign(1,i))*sfn(isign(2,i),4,:)
          sfn(isign(1,i),0,:) = sfn(isign(1,i),0,:) + np2(0,:)*np1(0,:)/2d0*m4Q4(isign(1,i))*sfn(isign(1,i),4,:) &
                                                    + 4d0*np2(0,:)*np1(0,:)/np4(0,:)/np3(0,:)*m4Q4(isign(1,i))*sfn(isign(2,i),4,:)
       end do
    end if
    if (tmcflag >= 3) then
       m6Q6 = m4Q4*m2Q2
       call nstructurefs(6,sfn(:,6,:))
       do i = -1,1,2
          sfn(isign(2,i),0,:) = sfn(isign(2,i),0,:) &
                             + np3(0,:)*np2(0,:)*np1(0,:)*n(0,:)*nm1(0,:)/6d0/np4(2,:)/np3(2,:)*m6Q6(isign(1,i))*sfn(isign(2,i),6,:)
          sfn(isign(1,i),0,:) = sfn(isign(1,i),0,:) + np3(0,:)*np2(0,:)*np1(0,:)/6d0*m6Q6(isign(1,i))*sfn(isign(1,i),6,:) &
                                             + 2d0*np3(0,:)*np2(0,:)*np1(0,:)/np4(2,:)/np3(2,:)*m6Q6(isign(1,i))*sfn(isign(2,i),6,:)
       end do
    end if
    if (tmcflag > 3) then
       write(6,*) 'STRUCTUREFS: Bad value of tmcflag = ',tmcflag
       stop
    end if
    Q2old=Q2
    tmcflagold=tmcflag
 end if
 !Mellin inversion and output
 F=0d0
 F=0d0/F
 do i = -3,3
    if (i == 0) cycle
    call minversion(x,sfn(i,0,:),F(i))
 end do
 Fold = F
 return
 end subroutine structurefs

!!!!!!!!!!!!!!!!!!!!!!!!!!! Mellin moments of DIS Neutral Current Nucleon Structure Functions !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 subroutine nstructurefs(isup,sfn)
 use mfactor
 implicit none
 logical resetSFs
 integer,parameter::saving=10000
 integer,save::nsave=0,nccflag_save(saving)=-1,evolscheme_save(saving)=-1,isup_save(saving)=-1,nff_save(saving)=-1
 integer::isup,order,nff,evolscheme,i,nccflag
 double precision,parameter::sin2w=0.23122d0,MZ=91.1876d0,MZ2=MZ*MZ,Kinf=1.3971d0, &
                             Ae=-0.5d0,Ve=-0.5d0+2d0*sin2w,Ve2pAe2=Ae*Ae+Ve*Ve, &
                             eu=0.666666d0,Au=0.5d0,Vu=0.5d0-2d0*eu*sin2w,Vu2pAu2=Au*Au+Vu*Vu, &
                             ed=-0.333333d0,Ad=-0.5d0,Vd=-0.5d0-2d0*ed*sin2w,Vd2pAd2=Ad*Ad+Vd*Vd
 double precision::Q2,asQ2,asQ2s,KQ2=0d0,KQ2s,a2Lu,a2Ld,a2Lvp3,a2Lvp8,a2Lvp15,a2Lvp24,a2Lvp35,a2Lqs,&
                                              a3u,a3d,a3vm3,a3vm8,a3vm15,a3vm24,a3vm35,a3qnsv
 double precision,save::Q2_save(saving)=0d0
 double complex::pdfn(-6:6,136),qns(-5:5,136),qs(2,136),sfn(-3:3,136),q2Lns(136),q3ns(136),q2Lns_neut(136),q3ns_neut(136)
 double complex::qns_dis(-5:5,136),qs_dis(2,136)
 double complex,save::sfn_save(saving,-3:3,136)
 common /orderc/ order
 common /nfsfc/ nff
 common /transfer/ Q2,asQ2,asQ2s
 common /evolschemec/ evolscheme
 common /disf2/ qns_dis,qs_dis
 common /nccflagc/ nccflag
 common /resetSFsc/ resetSFs
 if(resetSFs) then
    nsave=0
    resetSFs = .false.
 end if
 do i=1,nsave
    if (dabs(Q2-Q2_save(i))<1d-5) then
       if ((nccflag==nccflag_save(i)).and.(evolscheme==evolscheme_save(i)).and.(isup==isup_save(i)).and.(nff==nff_save(i))) then
          sfn =  sfn_save(i,:,:)
          return
       end if
    end if
 end do
 !Getting pdfs and nff
 sfn = dcmplx(0d0,0d0)
 call npdfs(0,isup,pdfn,qns,qs)
 !Neutral current couplings
 select case(nccflag)
 case(0)
    KQ2 = 0d0
 case(1)
    KQ2 = Kinf*Q2/(Q2+MZ2)
 case default
    write(6,*) 'NSTRUCTUREFS: Bad value of nccflag = ',nccflag
    stop
 end select
 KQ2s = KQ2*KQ2
 a2Lu = eu*eu -2d0*eu*Ve*Vu*KQ2 + Ve2pAe2*Vu2pAu2*KQ2s
 a2Ld = ed*ed -2d0*ed*Ve*Vd*KQ2 + Ve2pAe2*Vd2pAd2*KQ2s
 a2Lvp3 = (a2Lu - a2Ld)/2d0
 a2Lvp8  = (a2Lu - a2Ld)/6d0
 a2Lvp15 = (a2Ld - a2Lu)/6d0
 a2Lvp24  = (a2Lu - a2Ld)/10d0
 a2Lvp35 = (a2Ld - a2Lu)/10d0
 a3u = -2d0*eu*Ae*Au*KQ2 + 4d0*Ve*Ae*Vu*Au*KQ2s
 a3d = -2d0*ed*Ae*Ad*KQ2 + 4d0*Ve*Ae*Vd*Ad*KQ2s
 a3vm3 = (a3u - a3d)/2d0
 a3vm8  = (a3u - a3d)/6d0
 a3vm15 = (a3d - a3u)/6d0
 a3vm24  = (a3u - a3d)/10d0
 a3vm35 = (a3d - a3u)/10d0
 select case(nff)
  case(3)
     a2Lqs = (a2Lu + 2d0*a2Ld)/3d0
     a3qnsv = (a3u + 2d0*a3d)/3d0
  case(4)
     a2Lqs = (2d0*a2Lu + 2d0*a2Ld)/4d0
     a3qnsv = (2d0*a3u + 2d0*a3d)/4d0
  case(5)
     a2Lqs = (2d0*a2Lu + 3d0*a2Ld)/5d0
     a3qnsv = (2d0*a3u + 3d0*a3d)/5d0
  case(6)
     a2Lqs = (3d0*a2Lu + 3d0*a2Ld)/6d0
     a3qnsv = (3d0*a3u + 3d0*a3d)/6d0
  case default
     write(6,*) 'NSTRUCTUREFS: Bad factorization flavour number = ',nff
     stop
  end select
 !Appropriate combinations of non-singlet parton distributions
 select case(nff)
  case(3)
    q2Lns = a2Lqs*qs(1,:) + a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:)
    q3ns = a3qnsv*qns(0,:) + a3vm3*qns(-1,:) + a3vm8*qns(-2,:)
    q2Lns_neut = a2Lqs*qs(1,:) - a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:)
    q3ns_neut = a3qnsv*qns(0,:) - a3vm3*qns(-1,:) + a3vm8*qns(-2,:)
  case(4)
    q2Lns = a2Lqs*qs(1,:) + a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:) + a2Lvp15*qns(3,:)
    q3ns = a3qnsv*qns(0,:) + a3vm3*qns(-1,:) + a3vm8*qns(-2,:) + a3vm15*qns(-3,:)
    q2Lns_neut = a2Lqs*qs(1,:) - a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:) + a2Lvp15*qns(3,:)
    q3ns_neut = a3qnsv*qns(0,:) - a3vm3*qns(-1,:) + a3vm8*qns(-2,:) + a3vm15*qns(-3,:)
  case(5)
    q2Lns = a2Lqs*qs(1,:) + a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:) + a2Lvp15*qns(3,:) + a2Lvp24*qns(4,:)
    q3ns = a3qnsv*qns(0,:) + a3vm3*qns(-1,:) + a3vm8*qns(-2,:) + a3vm15*qns(-3,:) + a3vm24*qns(-4,:)
    q2Lns_neut = a2Lqs*qs(1,:) - a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:) + a2Lvp15*qns(3,:) + a2Lvp24*qns(4,:)
    q3ns_neut = a3qnsv*qns(0,:) - a3vm3*qns(-1,:) + a3vm8*qns(-2,:) + a3vm15*qns(-3,:) + a3vm24*qns(-4,:)
  case(6)
    q2Lns = a2Lqs*qs(1,:) + a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:) + a2Lvp15*qns(3,:) + a2Lvp24*qns(4,:) + a2Lvp35*qns(5,:)
    q3ns = a3qnsv*qns(0,:) + a3vm3*qns(-1,:) + a3vm8*qns(-2,:) + a3vm15*qns(-3,:) + a3vm24*qns(-4,:) + a3vm35*qns(-5,:)
    q2Lns_neut = a2Lqs*qs(1,:) - a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:) + a2Lvp15*qns(3,:) + a2Lvp24*qns(4,:) + a2Lvp35*qns(5,:)
    q3ns_neut = a3qnsv*qns(0,:) - a3vm3*qns(-1,:) + a3vm8*qns(-2,:) + a3vm15*qns(-3,:) + a3vm24*qns(-4,:) + a3vm35*qns(-5,:)
  case default
     write(6,*) 'NSTRUCTUREFS: Bad factorization flavour number = ',nff
     stop
 end select
 !Moments of Structure Functions
 do i = 1, 136
    if (order >= 0) then
    sfn(-3,i) = q3ns_neut(i)
    sfn(-2,i) = q2Lns_neut(i)
    sfn(-1,i) = asQ2*(CLQ1(nff,isup,i)*q2Lns_neut(i) + a2Lqs*CLG1(nff,isup,i)*qs(2,i))
    sfn(1,i) = asQ2*(CLQ1(nff,isup,i)*q2Lns(i) + a2Lqs*CLG1(nff,isup,i)*qs(2,i))
    sfn(2,i) = q2Lns(i)
    sfn(3,i) = q3ns(i)
    else
       write(6,*) 'NSTRUCTUREFS: Bad value of order = ',order
       stop
    end if
    if (order >= 1) then
    sfn(-3,i) = sfn(-3,i) + asQ2*C3Q1(nff,isup,i)*q3ns_neut(i)
    sfn(-2,i) = sfn(-2,i) + asQ2*(C2Q1(nff,isup,i)*q2Lns_neut(i) + a2Lqs*C2G1(nff,isup,i)*qs(2,i))
    sfn(-1,i) = sfn(-1,i) + asQ2s*(CLNSp2(nff,isup,i)*q2Lns_neut(i) + a2Lqs*(CLPS2(nff,isup,i)*qs(1,i) + CLG2(nff,isup,i)*qs(2,i)))
    sfn(1,i) = sfn(1,i) + asQ2s*(CLNSp2(nff,isup,i)*q2Lns(i) + a2Lqs*(CLPS2(nff,isup,i)*qs(1,i) + CLG2(nff,isup,i)*qs(2,i)))
    sfn(2,i) = sfn(2,i) + asQ2*(C2Q1(nff,isup,i)*q2Lns(i) + a2Lqs*C2G1(nff,isup,i)*qs(2,i))
    sfn(3,i) = sfn(3,i) + asQ2*C3Q1(nff,isup,i)*q3ns(i)
    end if
    if (order >= 2) then
    sfn(-3,i) = sfn(-3,i) + asQ2s*C3NSm2(nff,isup,i)*q3ns_neut(i)
    sfn(-2,i) = sfn(-2,i) + asQ2s*(C2NSp2(nff,isup,i)*q2Lns_neut(i) + a2Lqs*(C2PS2(nff,isup,i)*qs(1,i) + C2G2(nff,isup,i)*qs(2,i)))
    sfn(-1,i) = sfn(-1,i) + asQ2*asQ2s*(CLNSp3(nff,isup,i)*q2Lns_neut(i) &
                          + a2Lqs*(CLPS3(nff,isup,i)*qs(1,i) + CLG3(nff,isup,i)*qs(2,i)))
    sfn(1,i) = sfn(1,i) + asQ2*asQ2s*(CLNSp3(nff,isup,i)*q2Lns(i) + a2Lqs*(CLPS3(nff,isup,i)*qs(1,i) + CLG3(nff,isup,i)*qs(2,i)))
    sfn(2,i) = sfn(2,i) + asQ2s*(C2NSp2(nff,isup,i)*q2Lns(i) + a2Lqs*(C2PS2(nff,isup,i)*qs(1,i) + C2G2(nff,isup,i)*qs(2,i)))
    sfn(3,i) = sfn(3,i) + asQ2s*C3NSm2(nff,isup,i)*q3ns(i)
    end if
    if (order > 2) then
       write(6,*) 'NSTRUCTUREFS: Bad value of order = ',order
       stop
    end if
    !Overwriting F2 with the DIS factorization scheme expression
    if (evolscheme==1) then
       q2Lns(i) = a2Lqs*qs_dis(1,i) + a2Lvp3*qns_dis(1,i) + a2Lvp8*qns_dis(2,i) &
                                    + a2Lvp15*qns_dis(3,i) + a2Lvp24*qns_dis(4,i) + a2Lvp35*qns_dis(5,i)
       q2Lns_neut(i) = a2Lqs*qs_dis(1,i) - a2Lvp3*qns_dis(1,i) + a2Lvp8*qns_dis(2,i) &
                                         + a2Lvp15*qns_dis(3,i) + a2Lvp24*qns_dis(4,i) + a2Lvp35*qns_dis(5,i)
       sfn(-2,i) = q2Lns_neut(i)
       sfn(2,i) = q2Lns(i)
    end if
 end do
 !Saving
 nsave=nsave+1
 Q2_save(nsave) = Q2
 nccflag_save(nsave) = nccflag
 evolscheme_save(nsave) = evolscheme
 isup_save(nsave) = isup
 nff_save(nsave) = nff
 sfn_save(nsave,:,:) = sfn
 return
 end subroutine nstructurefs

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Light-quark contributions to CC DIS Structure Functions !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!    x = Bjorken-x
!!    Q2 = Q**2 (GeV**2) == DIS Momentum transfer squared.
!!    F = Output = 1(FLp), 2(F2p), 3(xF3p) for W+ (e+p)
!!                -1(FLp),-2(F2p),-3(xF3p) for W- (e-p)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 subroutine structurefscc(x,Q2,F)
 implicit none
 logical resetSFCCsx
 integer::i,errors
 double precision::x,xold,Q2,Q2old,Q2c,F(-3:3),Fold(-3:3),as,asQ2,asQ2s
 double complex::sfn(-3:3,0:6,136)
 external as
 common /transfer/ Q2c,asQ2,asQ2s
 common /errorsc/ errors
 save xold,Q2old,sfn,Fold
 common /resetSFCCsxc/ resetSFCCsx
 if ((x<0d0).or.((x>1d0).and.((x-1d0)>1d-10))) then
    write(*,*) "STRUCTUREFS: x out of range",x
    stop
 end if
 if ((x==xold).and.(Q2==Q2old).and.(.not.resetSFCCsx).and.(errors==0)) then
    F = Fold
    return
 end if
 xold=x
 if ((Q2/=Q2old).or.(resetSFCCsx).or.(errors/=0)) then
    Q2c = Q2
    asQ2 = as(Q2)
    asQ2s = asQ2*asQ2
    call nstructurefscc(0,sfn(:,0,:))
    resetSFCCsx = .false.
 end if
 !Mellin inversion and output.
 F=0d0
 F=0d0/F
 do i = -3,3
    if (i == 0) cycle
    call minversion(x,sfn(i,0,:),F(i))
 end do
 return
 end subroutine structurefscc

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Mellin moments of CC DIS Charm Production Structure Functions !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine nstructurefscc(isup,sfn)
 use mfactor
 implicit none
 logical resetSFCCs
 integer,parameter::saving=10000
 integer::isup,order,nff,i
 integer,save::nsave=0,isup_save(saving)=-1,nff_save(saving)=-1
 double precision::Q2,asQ2,asQ2s
 double precision,save::Q2_save(saving)=0d0
 double precision,parameter::Vud=0.97428d0,Vud2=Vud*Vud,Vus=0.2253d0,Vus2=Vus*Vus
 double precision,parameter::a2L=1d0/3d0*(Vud2+Vus2),a3=1d0/3d0*(Vud2+Vus2)
 double precision,parameter::a2Lvp3=Vus2/4d0,a2Lvm8=Vus2/4d0,a3vp8=Vus2/4d0,a3vm3=Vus2/4d0
 double precision,parameter::a2Lvp8=(2d0*Vud2-Vus2)/12d0,a3vm8=(2d0*Vud2-Vus2)/12d0
 double precision,parameter::a2Lvm3=(2d0*Vud2+Vus2)/4d0,a3vp3=(2d0*Vud2+Vus2)/4d0
 double complex::pdfn(-6:6,136),qns(-5:5,136),qs(2,136),sfn(-3:3,136),q2LNSp(136),q2LNSm(136),q3NSp(136),q3NSm(136)
 double complex,save::sfn_save(saving,-3:3,136)
 common /orderc/ order
 common /nfsfc/ nff
 common /transfer/ Q2,asQ2,asQ2s
 common /resetSFCCsc/ resetSFCCs
 if(resetSFCCs) then
    nsave=0
    resetSFCCs = .false.
 end if
 do i=1,nsave
    if (dabs(Q2-Q2_save(i))<1d-5) then
       if ((isup==isup_save(i)).and.(nff==nff_save(i))) then
          sfn =  sfn_save(i,:,:)
          return
       end if
    end if
 end do
 !Getting pdfs and nff
 sfn = dcmplx(0d0,0d0)
 call npdfs(0,isup,pdfn,qns,qs)
 if (nff/=3) then
    write(*,*) "NSTRUCTUREFSCC: Bad value for nff: ",nff
    stop
 end if
 !Appropriate combinations of non-singlet parton distributions
 q2LNSp =  a2L*qs(1,:) + a2Lvp3*qns(1,:) + a2Lvp8*qns(2,:)
 q2LNSm =  a2Lvm3*qns(-1,:) + a2Lvm8*qns(-2,:)
 q3NSp = a3vp3*qns(1,:) + a3vp8*qns(2,:)
 q3NSm = a3*qns(0,:) + a3vm3*qns(-1,:) + a3vm8*qns(-2,:)
 !Moments of Structure Functions
 do i = 1, 136
    if (order >= 0) then
    sfn(-3,i) =  q3NSp(i) + q3NSm(i)
    sfn(-2,i) = q2LNSp(i) + q2LNSm(i)
    sfn(-1,i) = asQ2*(CLQ1(nff,isup,i)*(q2LNSp(i) + q2LNSm(i)) + a2L*CLG1(nff,isup,i)*qs(2,i))
    sfn(1,i)  = asQ2*(CLQ1(nff,isup,i)*(q2LNSp(i) - q2LNSm(i)) + a2L*CLG1(nff,isup,i)*qs(2,i))
    sfn(2,i)  = q2LNSp(i) - q2LNSm(i)
    sfn(3,i)  = -q3NSp(i) + q3NSm(i)
    else
       write(6,*) 'NSTRUCTUREFS: Bad value of order = ',order
       stop
    end if
    if (order >= 1) then
    sfn(-3,i) = sfn(-3,i) + asQ2*(C3Q1(nff,isup,i)*(q3NSp(i) + q3NSm(i)))
    sfn(-2,i) = sfn(-2,i) + asQ2*(C2Q1(nff,isup,i)*(q2LNSp(i) + q2LNSm(i)) + a2L*C2G1(nff,isup,i)*qs(2,i))
    sfn(-1,i) = sfn(-1,i) + asQ2s*(CLNSp2(nff,isup,i)*q2LNSp(i) + CLNSm2(nff,isup,i)*q2LNSm(i) &
                                   + a2L*CLPS2(nff,isup,i)*qs(1,i) + a2L*CLG2(nff,isup,i)*qs(2,i))
    sfn(1,i)  = sfn(1,i)  + asQ2s*(CLNSp2(nff,isup,i)*q2LNSp(i) - CLNSm2(nff,isup,i)*q2LNSm(i) &
                                   + a2L*CLPS2(nff,isup,i)*qs(1,i) + a2L*CLG2(nff,isup,i)*qs(2,i))
    sfn(2,i) = sfn(2,i) + asQ2*(C2Q1(nff,isup,i)*(q2LNSp(i) - q2LNSm(i)) + a2L*C2G1(nff,isup,i)*qs(2,i))
    sfn(3,i) = sfn(3,i) + asQ2*(C3Q1(nff,isup,i)*(-q3NSp(i) + q3NSm(i)))
    end if
    if (order >= 2) then
    sfn(-3,i) = sfn(-3,i) + asQ2s*(C3NSp2(nff,isup,i)*q3NSp(i) + C3NSm2(nff,isup,i)*q3NSm(i))
    sfn(-2,i) = sfn(-2,i) + asQ2s*(C2NSp2(nff,isup,i)*q2LNSp(i) + C2NSm2(nff,isup,i)*q2LNSm(i) &
                                   + a2L*C2PS2(nff,isup,i)*qs(1,i) + a2L*C2G2(nff,isup,i)*qs(2,i))
    sfn(-1,i) = sfn(-1,i) + asQ2*asQ2s*(CLNSp3(nff,isup,i)*q2LNSp(i) + CLNSm3(nff,isup,i)*q2LNSm(i) &
                                   + a2L*CLPS3(nff,isup,i)*qs(1,i) + a2L*CLG3(nff,isup,i)*qs(2,i))
    sfn(1,i)  = sfn(1,i)  + asQ2*asQ2s*(CLNSp3(nff,isup,i)*q2LNSp(i) - CLNSm3(nff,isup,i)*q2LNSm(i) &
                                   + a2L*CLPS3(nff,isup,i)*qs(1,i) + a2L*CLG3(nff,isup,i)*qs(2,i))
    sfn(2,i) = sfn(2,i)  + asQ2s*(C2NSp2(nff,isup,i)*q2LNSp(i) - C2NSm2(nff,isup,i)*q2LNSm(i) &
                                  + a2L*C2PS2(nff,isup,i)*qs(1,i) + a2L*C2G2(nff,isup,i)*qs(2,i))
    sfn(3,i) = sfn(3,i) + asQ2s*(-C3Nsp2(nff,isup,i)*q3NSp(i) + C3NSm2(nff,isup,i)*q3NSm(i))
    end if
    if (order > 2) then
       write(6,*) 'NSTRUCTUREFS: Bad value of order = ',order
       stop
    end if
 end do
 sfn = 2d0*sfn
 !Saving
 nsave=nsave+1
 Q2_save(nsave) = Q2
 isup_save(nsave) = isup
 nff_save(nsave) = nff
 sfn_save(nsave,:,:) = sfn
 return
 end subroutine nstructurefscc

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ZMVFNS HEAVY QUARK CONTRIBUTIONS TO NUCLEON STRUCTURE FUNCTIONS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  x == Bjorken-x.
!!  Q2 = Q**2 (GeV**2) == DIS Momentum transfer squared.
!!  hF = Output:
!!      nfunction: 1(xFL), 2(xF2)
!!      nq: Heavy quark: 4(c), 5(b).
!!      nccflag = external common block = structure functions for photon exchange only(0) or the whole neutral current(1).
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine heavfs(xl,Q2l,hF)
 implicit none
 integer i,j,evolscheme,fnscheme,fnsfixed
 double precision xl,Q2l,Q2c,hF(2,4:5),as,asQ2,asQ2s,Q2o,mth2(4:6)
 double complex hsfn(2,4:5,136)
 common /transfer/ Q2c,asQ2,asQ2s
 common /evolschemec/ evolscheme
 common /fnschemec/ fnscheme,fnsfixed
 common /scalesc/ Q2o,mth2
!Checking  Flavor number scheme
 if (fnscheme/=5) then
     write(6,*) "HEAVFS: Bad value for fnscheme = ",fnscheme
     stop
 end if
 !Checking Factorization scheme
 if (evolscheme/=0) then
     write(6,*) "HEAVFS: Bad value for evolscheme = ",evolscheme
     stop
 end if
 !x- and Q2-dependent quantities
 Q2c = Q2l
 asQ2 = as(Q2l)
 asQ2s = asQ2*asQ2
 !Moment space solution.
 call nheavfs(hsfn)
 !Mellin inversion and output
 hF=0d0
 hF=0d0/hF
 do i = 1,2
    do j = 4,5
       select case(j)
        case(4)
           if ((Q2l >= mth2(4)).and.(Q2l*(1d0/xl-1d0) > 4d0*mth2(4))) then
              call minversion(xl,hsfn(i,j,:),hF(i,j))
!              if (hF(i,4)<0d0) hF(i,4)=0d0
           else
              hF(i,4)=0d0
              hF(i,4)=0d0/hF(i,4)
           end if
        case(5)
           if ((Q2l >= mth2(5)).and.(Q2l*(1d0/xl-1d0) > 4d0*mth2(5))) then
              call minversion(xl,hsfn(i,j,:),hF(i,j))
!              if (hF(i,5)<0d0) hF(i,5)=0d0
           else
              hF(i,5)=0d0
              hF(i,5)=0d0/hF(i,5)
           end if
       end select
    end do
 end do
 return
end subroutine heavfs

!!!!!!!!!!!!!!!!!!!!! Mellin moments of DIS NC ZMVFNS heavy quark contributions to nucleon structure functions !!!!!!!!!!!!!!!!!!!!!
 subroutine nheavfs(hsfn)
 use mfactor
 implicit none
 integer::order,nff,i,nccflag
 double precision,parameter::sin2w=0.23122d0,MZ=91.1876d0,MZ2=MZ*MZ,Kinf=1.3971d0, &
                             Ae=-0.5d0,Ve=-0.5d0+2d0*sin2w,Ve2pAe2=Ae*Ae+Ve*Ve, &
                             ec=0.666666d0,Ac=0.5d0,Vc=0.5d0-2d0*ec*sin2w,Vc2pAc2=Ac*Ac+Vc*Vc, &
                             eb=-0.333333d0,Ab=-0.5d0,Vb=-0.5d0-2d0*eb*sin2w,Vb2pAb2=Ab*Ab+Vb*Vb
 double precision::Q2,asQ2,asQ2s,KQ2=0d0,KQ2s,a2Lc,a2Lb
 double complex::pdfn(-6:6,136),qns(-5:5,136),qs(2,136),hsfn(2,4:5,136)
 common /orderc/ order
 common /nfsfc/ nff
 common /transfer/ Q2,asQ2,asQ2s
 common /nccflagc/ nccflag
 !Precalculations.
 hsfn = dcmplx(0d0,0d0)
 call npdfs(0,0,pdfn,qns,qs)
 !Neutral current couplings.
 select case(nccflag)
 case(0)
    KQ2 = 0d0
 case(1)
    KQ2 = Kinf*Q2/(Q2+MZ2)
 case default
    write(6,*) 'NSTRUCTUREFS: Bad value of nccflag = ',nccflag
    stop
 end select
 KQ2s = KQ2*KQ2
 a2Lc = ec*ec -2d0*ec*Ve*Vc*KQ2 + Ve2pAe2*Vc2pAc2*KQ2s
 a2Lb = eb*eb -2d0*eb*Ve*Vb*KQ2 + Ve2pAe2*Vb2pAb2*KQ2s
!Moments of Structure Functions.
 do i = 1, 136
    if (order >= 0) then
    hsfn(1,4,i) = a2Lc*asQ2*(CLQ1(nff,0,i)*(pdfn(4,i) + 2d0*pdfn(-4,i)) + CLG1(nff,0,i)/nff*qs(2,i))
    hsfn(1,5,i) = a2Lb*asQ2*(CLQ1(nff,0,i)*(pdfn(5,i) + 2d0*pdfn(-5,i)) + CLG1(nff,0,i)/nff*qs(2,i))
    hsfn(2,4,i) = a2Lc*(pdfn(4,i) + 2d0*pdfn(-4,i))
    hsfn(2,5,i) = a2Lb*(pdfn(5,i) + 2d0*pdfn(-5,i))
    else
       write(6,*) 'NSTRUCTUREFS: Bad value of order = ',order
       stop
    end if
    if (order >= 1) then
    hsfn(1,4,i) = hsfn(1,4,i) + a2Lc*asQ2s*(CLNSp2(nff,0,i)*(pdfn(4,i) + 2d0*pdfn(-4,i)) + CLPS2(nff,0,i)/nff*qs(1,i) &
                                                                                        + CLG2(nff,0,i)/nff*qs(2,i))
    hsfn(1,5,i) = hsfn(1,5,i) + a2Lb*asQ2s*(CLNSp2(nff,0,i)*(pdfn(5,i) + 2d0*pdfn(-5,i)) + CLPS2(nff,0,i)/nff*qs(1,i) &
                                                                                        + CLG2(nff,0,i)/nff*qs(2,i))
    hsfn(2,4,i) = hsfn(2,4,i) + a2Lc*asQ2*(C2Q1(nff,0,i)*(pdfn(4,i) + 2d0*pdfn(-4,i)) + C2G1(nff,0,i)/nff*qs(2,i))
    hsfn(2,5,i) = hsfn(2,5,i) + a2Lb*asQ2*(C2Q1(nff,0,i)*(pdfn(5,i) + 2d0*pdfn(-5,i)) + C2G1(nff,0,i)/nff*qs(2,i))
    end if
    if (order >= 2) then
    hsfn(1,4,i) = hsfn(1,4,i) + a2Lc*asQ2s*asQ2*(CLNSp3(nff,0,i)*(pdfn(4,i) + 2d0*pdfn(-4,i)) + CLPS3(nff,0,i)/nff*qs(1,i) &
                                                                                             + CLG3(nff,0,i)/nff*qs(2,i))
    hsfn(1,5,i) = hsfn(1,5,i) + a2Lb*asQ2s*asQ2*(CLNSp3(nff,0,i)*(pdfn(5,i) + 2d0*pdfn(-5,i)) + CLPS3(nff,0,i)/nff*qs(1,i) &
                                                                                             + CLG3(nff,0,i)/nff*qs(2,i))
    hsfn(2,4,i) = hsfn(2,4,i) + a2Lc*asQ2s*(C2NSp2(nff,0,i)*(pdfn(4,i) + 2d0*pdfn(-4,i)) + C2PS2(nff,0,i)/nff*qs(1,i) &
                                                                                        + C2G2(nff,0,i)/nff*qs(2,i))
    hsfn(2,5,i) = hsfn(2,5,i) + a2Lb*asQ2s*(C2NSp2(nff,0,i)*(pdfn(5,i) + 2d0*pdfn(-5,i)) + C2PS2(nff,0,i)/nff*qs(1,i) &
                                                                                        + C2G2(nff,0,i)/nff*qs(2,i))
    end if
    if (order > 2) then
       write(6,*) 'NSTRUCTUREFS: Bad value of order = ',order
       stop
    end if
 end do
 return
 end subroutine nheavfs
