!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! DOUBLE DIFFERENTIAL DRELL-YAN CROSS-SECTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!   DYdMdxF == M**3 dsigma/dM dxF (GeV^2 nb) (calculated by rescaling with LO kinematics: dsigma/dMdxF = dsigma/dMdy/(x1+x2))
!!!   DYdMdy == M**3 dsigma/dM dy (GeV^2 nb)
!!!   target == 1(proton-proton), 2(proton-neutron), 3(proton-deuterium), 4(proton-copper)
!!!   order == Order of pQCD calculation = 0(LO), 1(NLO), 2(NNLO)
!!!   xi == Bjorken-x of the ith parton
!!!   m2 == m**2 (GeV**2) Invariant mass of the dilepton pair
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      double precision function DYdMdxF(target,order,x1,x2,m2)
       implicit none
       integer target,order
       double precision x1,x2,m2,DYdMdy
       external DYdMdy
       DYdMdxF = DYdMdy(target,order,x1,x2,m2)/(x1+x2)
       return
      end function DYdMdxF

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      double precision function DYdMdy(target,order,x1,x2,m2)
       implicit none
       integer target,order
       double precision x1,x2,m2,sqrts,m,y,kine,gev2nb,dydsig_dM_dy
       sqrts=dsqrt(m2/x1/x2)
       m=dsqrt(m2)
       y=0.5d0*dlog(x1/x2)
       if (y>0d0) then
          DYdMdy = dydsig_dM_dy(target,1,order+1,sqrts,m,-y)/2d0
       else if (y<=0d0) then
          DYdMdy = dydsig_dM_dy(1,target,order+1,sqrts,m,y)/2d0
       else
          write(6,*) 'DY: Bad value of y',y
          stop
       end if
       kine = 8d0*3.141592653589793116d0/137d0/137d0/9d0
       gev2nb = 389379.292d0!GeV**2*nb
       DYdMdy = gev2nb*kine*DYdMdy
       return
      end function DYdMdy

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      double precision function alphasDY(Q2)
       implicit none
       double precision Q2,as
       external as
       alphasDY = as(Q2)*4d0*3.141592653589793116d0
       return
      end function alphasDY

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      subroutine pdfsDYn(x,Q2,xpa)
       implicit none
       double precision x,Q2,xpa(-6:6),xpal(-6:6)
       call pdfsMGDY(x,Q2,xpal)
       xpa(-6) = xpal(-6)
       xpa(-5) = xpal(-5)
       xpa(-4) = xpal(-4)
       xpa(-3) = xpal(-3)
       xpa(-2) = xpal(-1)
       xpa(-1) = xpal(-2)
       xpa(0) = xpal(0)
       xpa(1) = xpal(2)
       xpa(2) = xpal(1)
       xpa(3) = xpal(3)
       xpa(4) = xpal(4)
       xpa(5) = xpal(5)
       xpa(6) = xpal(6)
       return
      end subroutine pdfsDYn

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      subroutine pdfsDYd(x,Q2,xpa)
       implicit none
       double precision x,Q2,xpa(-6:6),xpal(-6:6)
       call pdfsMGDY(x,Q2,xpal)
       xpa(-6) = xpal(-6)
       xpa(-5) = xpal(-5)
       xpa(-4) = xpal(-4)
       xpa(-3) = xpal(-3)
       xpa(-2) = 0.5d0*(xpal(-2)+xpal(-1))
       xpa(-1) = 0.5d0*(xpal(-1)+xpal(-2))
       xpa(0) = xpal(0)
       xpa(1) = 0.5d0*(xpal(1)+xpal(2))
       xpa(2) = 0.5d0*(xpal(2)+xpal(1))
       xpa(3) = xpal(3)
       xpa(4) = xpal(4)
       xpa(5) = xpal(5)
       xpa(6) = xpal(6)
       return
      end subroutine pdfsDYd

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      subroutine pdfsDYCu(x,Q2,xpa)
       implicit none
       integer input,order,evolscheme,fnscheme,fnsfixed
       double precision x,Q2,xpa(-6:6),xpal(-6:6),RUV,RDV,RUB,RDB,RS,RG
       double precision::Znucl=29d0,Nnucl=34.034d0,uv,dv,ub,db
       common /fnschemec/ fnscheme,fnsfixed
       common /inputc/ input 
       common /orderc/ order
       common /evolschemec/ evolscheme
       if ((fnscheme /= 3).and.(fnscheme /=4)) then
          write(*,*) "PDFSDYCU: Bad value for fnscheme =",fnscheme
          stop
       end if
       if ((order /= 1).and.(order /=2)) then
          write(*,*) "PDFSDYCU: Bad value for order =",order
          stop
       end if
       call nDS(2,x,dsqrt(Q2),63.54d0,RUV,RDV,RUB,RDB,RS,RG)
!        if (input==3) then
!           RUV = 1d0
!           RDV = 1d0
!           RUB = 1d0
!           RDB = 1d0
!           RS = 1d0
!           RG = 1d0
!         end if
       call pdfsMGDY(x,Q2,xpal)
       xpa(-6) = xpal(-6)
       xpa(-5) = xpal(-5)
       xpa(-4) = xpal(-4)
       xpa(-3) = RS*xpal(-3)
       db = RDB*xpal(-2)
       ub = RUB*xpal(-1)
       xpa(0) = RG*xpal(0)
       uv = RUV*xpal(1)
       dv = RDV*xpal(2)
       xpa(3) = RS*xpal(3)
       xpa(4) = xpal(4)
       xpa(5) = xpal(5)
       xpa(6) = xpal(6)
       xpa(-2) = (Znucl*db + Nnucl*ub)/(Znucl+Nnucl)
       xpa(-1) = (Znucl*ub + Nnucl*db)/(Znucl+Nnucl)
       xpa(1)  = (Znucl*uv + Nnucl*dv)/(Znucl+Nnucl)
       xpa(2)  = (Znucl*dv + Nnucl*uv)/(Znucl+Nnucl)
       return
      end subroutine pdfsDYCu

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Original Code !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      real*8 function dydsig_dm_dy(nb,nt,no,ecm,rm,y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 Etot,mtop,Lt
      REAL*8 var,mw,mz,pi,xw,Gf,conv,alpha,dat,t1,t2,t3,spp,mll
      REAL*8 smm,smGam,smg,esmgv,esmga,usmgv,usmga,dsmgv,dsmga
      REAL*8 smtot,g,m,Gam,egv,ega,ugv,uga,dgv,dga,kktot,gamdat
      dimension var(9),dat(13,7,3),smm(2),smGam(2),smg(2),esmgv(2),
     +     esmga(2),usmgv(2),usmga(2),dsmgv(2),dsmga(2),g(4),m(4),
     +     Gam(4),egv(4),ega(4),ugv(4),uga(4),dgv(4),dga(4),
     +     kktot(4,121),gamdat(7),smtot(121)

      external dyi,dynloi,dynnloi,dynlor

CCC   These define parameters for the vegas integration
      dimension xl(9),xu(9)

CC    Input parameters
      pi=3.14159265358979D0
      Etot=ecm
      buff=0.0001D0

CC    Quark charges
      eu=2.D0/3.D0
      ed=1.D0/3.D0
      es=1.D0/3.D0
      ec=2.D0/3.D0
      eb=1.D0/3.D0

C     Factorizaton and renormalization scales
      mF=rm
      facscale = mF
      renscale = facscale
      q = renscale

C     Strong coupling constant
      alphas=pi*SSC(renscale)
      alphaspi=alphas/pi

C     Basic x ( *hadronic x *)
      x = mF**2/Etot**2

c     mode1 -  determines the projectile
c     mode2 - determines the target
      mode1 = nb
      mode2 = nt

C     order = 1 (LO), 2(NLO), 3(NNLO)
      morder = no

C     Basic *u* variable
      u=dexp(-2D0*Y)

      crsLO = sigmaLO(x,u)

      if (morder.eq.1) then
        dydsig_dm_dy = crsLO
      end if

      relpredic = 1d-2 !Precision for total chi2: 1d0=1d-3(5s),1d-1=10-4(15s),1d-2=10-4(30s),1d-3=10-5(1.5m),1d-4=10-5(5m),1d-5=10-5(30m)
      precdy = relpredic*crsLO !precision of the integration

      if (morder.eq.2) then
        precnlo=precdy/alphaspi
        crsnlo = sd2(precnlo,dynloi,dynlor)
        dydsig_dm_dy = crsLO  + (alphas/pi)*crsNLO
      end if

      if (morder.eq.3) then
        prectot=precdy
        dydsig_dm_dy = crsLO + sd2(prectot,dyi,dynlor)
      endif
           
      return
      end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C    Functions for the integrals over partonic cross section
C    for VEGAS
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     NLO
C     The NLO cross-section is a sum of three parts, because
C     of the integration boundaries; I treat those separately


      double precision function NLOCR(var)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 NLOCRA, NLOCRB, NLOCRC
      dimension var(9)


      xi1=var(1)
      xi2=var(2)

C     Region A
C     The integration limits are:
C      sqrt(u*x) < z < 1 and 0 < y < 1


      zminA=dsqrt(u*x)

      zz1=1.D0-dexp(xi1*dlog(1.D0-zminA)+dlog(buff)*(1.D0-xi1))
      zz2=1.D0-buff + buff*xi1
      zzm=dlog(1.D0-zminA)-dlog(buff)
      yym=dlog((1.D0-buff)/buff)

      yy1=buff*xi2
      yy2=1.D0/(1+dexp((1.D0-2.D0*xi2)*yym))
      yy3=1.D0-buff+buff*xi2

      NLOCRA =
     ,zzm*(dlog(buff)+buff/(1.D0-buff))*sigmaNLO(zz1,yy1)
     ,+zzm*2.D0*yym*sigmaNLO(zz1,yy2)
     ,+zzm*(dlog(buff)+buff/(1.D0-buff))*sigmaNLO(zz1,yy3)
     ,
     ,+dlog(buff)*(dlog(buff)+buff/(1.D0-buff))*sigmaNLO(zz2,yy1)
     ,+dlog(buff)*2.D0*yym*sigmaNLO(zz2,yy2)
     ,+dlog(buff)*(dlog(buff)+buff/(1.D0-buff))*sigmaNLO(zz2,yy3)


C     Region B
C     The integration limits are
C      sqrt(x/u) < z < sqrt(u*x) and yminB(z) < y < 1

      zmaxB = zminA
      zminB = dsqrt(x/u)
      zmB=dlog(1.D0-zminB)-dlog(1.D0-zmaxB)
      zB=1.D0-dexp(dlog(1.D0-zmaxB)+zmB*xi1)

      yminB=(u*x/zB-zB)/(1.D0+u*x/zB)/(1.D0-zB)


      if (yminB.lt.buff) then

      yymB=dlog((1.D0-buff)/buff)
      yy1B=yminB + (buff-yminB)*xi2
      yy2B=1.D0/(1+dexp((1.D0-2.D0*xi2)*yymB))
      yy3B=1.D0-buff+buff*xi2

      NLOCRB =
     , + zmB*(1-yminB/buff)
     ,   *(dlog(buff)+buff/(1.D0-buff))*sigmaNLO(zB,yy1B)
     , +zmB*2.D0*yymB*sigmaNLO(zB,yy2B)
     , + zmB*(dlog(buff)+buff/(1.D0-buff))*sigmaNLO(zB,yy3B)

       else

      yymB=(dlog((1.D0-buff)/buff)-dlog(yminB/(1-yminB)))
      yy2B=1.D0/(1+dexp(-dlog(yminB/(1-yminB))-yymB*xi2))
      yy3B=1.D0-buff+buff*xi2

      NLOCRB =
     ,   zmB*yymB*sigmaNLO(zB,yy2B)
     , + zmB*(dlog(buff)+buff/(1.D0-buff))*sigmaNLO(zB,yy3B)


      end if

C     Region C
C     The integration limits are
C     x < z < sqrt(x/u)   yminC < y < ymaxC

      zmaxC = zminB
      zminC = x
      zmC=dlog(1.D0-zminC)-dlog(1.D0-zmaxC)
      zC=1.D0-dexp(dlog(1.D0-zmaxC)+zmC*xi1)


      yminC = (u*x/zC-zC)/(1D0+u*x/zC)/(1D0-zC)
      ymaxC = (u*zC/x-zC)/(1D0+u*zC/x)/(1D0-zC)
      yC = yminC + (ymaxC - yminC)*xi2

      if (ymaxC.lt.1-buff) then

      yymC= dlog(ymaxC/(1-ymaxC))-dlog(yminC/(1-yminC))
      yy2C=1.D0/(1+dexp(-dlog(yminC/(1-yminC))-yymC*xi2))

      NLOCRC = zmC*yymC*sigmaNLO(zC,yy2C)

      else

      yymC= dlog((1-buff)/buff)-dlog(yminC/(1-yminC))
      yy2C=1.D0/(1+dexp(-dlog(yminC/(1-yminC))-yymC*xi2))
      yy3C=1-buff + (ymaxC - (1- buff))*xi2

      NLOCRC =
     ,  zmC*yymC*sigmaNLO(zC,yy2C)
     , +zmC*((ymaxC-1)/buff + 1)*(dlog(buff)+buff/(1-buff))
     ,  *sigmaNLO(zC,yy3C)

      end if

      NLOCR = NLOCRA + NLOCRB + NLOCRC

C      print *,  NLOCR

      end




CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     NNLO
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     The NNLO cross-section is a sum of three parts, because
C     of the integration boundaries; I treat those separately



      double precision function NNLOCR(var)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 NNLOCRA, NNLOCRB, NNLOCRC

      dimension var(9)

      xi1=var(1)
      xi2=var(2)

C     Region A
C     The integration limits are:
C      sqrt(u*x) < z < 1 and 0 < y < 1


      zminA=dsqrt(u*x)

      zz1=1D0-dexp(xi1*dlog(1D0-zminA)+dlog(buff)*(1D0-xi1))
      zz2=1D0-buff + buff*xi1
      zzm=dlog(1D0-zminA)-dlog(buff)
      yym=dlog((1D0-buff)/buff)

      yy1=buff*xi2
      yy2=1.D0/(1D0+dexp((1.D0-2.D0*xi2)*yym))
      yy3=1D0-buff+buff*xi2

      NNLOCRA =
     ,zzm*(dlog(buff)+buff/(1D0-buff))*sigmaNNLO(zz1,yy1)
     ,+zzm*2.D0*yym*sigmaNNLO(zz1,yy2)
     ,+zzm*(dlog(buff)+buff/(1D0-buff))*sigmaNNLO(zz1,yy3)
     ,
     ,+dlog(buff)*(dlog(buff)+buff/(1D0-buff))*sigmaNNLO(zz2,yy1)
     ,+dlog(buff)*2D0*yym*sigmaNNLO(zz2,yy2)
     ,+dlog(buff)*(dlog(buff)+buff/(1D0-buff))*sigmaNNLO(zz2,yy3)


C     Region B
C     The integration limits are
C      sqrt(x/u) < z < sqrt(u*x) and yminB(z) < y < 1

      zmaxB = zminA
      zminB = dsqrt(x/u)
      zmB=dlog(1D0-zminB)-dlog(1D0-zmaxB)
      zB=1.D0-dexp(dlog(1.D0-zmaxB)+zmB*xi1)

      yminB=(u*x/zB-zB)/(1D0+u*x/zB)/(1D0-zB)


      if (yminB.lt.buff) then

      yymB=dlog((1.D0-buff)/buff)
      yy1B=yminB + (buff-yminB)*xi2
      yy2B=1.D0/(1+dexp((1.D0-2.D0*xi2)*yymB))
      yy3B=1.D0-buff+buff*xi2

      NNLOCRB =
     , + zmB*(1-yminB/buff)
     ,   *(dlog(buff)+buff/(1.D0-buff))*sigmaNNLO(zB,yy1B)
     , + zmB*2.D0*yymB*sigmaNNLO(zB,yy2B)
     , + zmB*(dlog(buff)+buff/(1.D0-buff))*sigmaNNLO(zB,yy3B)

       else

      yymB=(dlog((1.D0-buff)/buff)-dlog(yminB/(1.D0-yminB)))
      yy2B=1.D0/(1+dexp(-dlog(yminB/(1-yminB))-yymB*xi2))
      yy3B=1.D0-buff+buff*xi2

      NNLOCRB =
     ,   zmB*yymB*sigmaNNLO(zB,yy2B)
     , + zmB*(dlog(buff)+buff/(1.D0-buff))*sigmaNNLO(zB,yy3B)


      end if

C     Region C
C     The integration limits are
C     x < z < sqrt(x/u)   yminC < y < ymaxC

      zmaxC = zminB
      zminC = x
      zmC=dlog(1D0-zminC)-dlog(1D0-zmaxC)
      zC=1.D0-dexp(dlog(1.D0-zmaxC)+zmC*xi1)


      yminC = (u*x/zC-zC)/(1D0+u*x/zC)/(1D0-zC)
      ymaxC = (u*zC/x-zC)/(1D0+u*zC/x)/(1D0-zC)
      yC = yminC + (ymaxC - yminC)*xi2

      if (ymaxC.lt.1D0-buff) then

      yymC= dlog(ymaxC/(1D0-ymaxC))-dlog(yminC/(1-yminC))
      yy2C=1.D0/(1+dexp(-dlog(yminC/(1-yminC))-yymC*xi2))

      NNLOCRC = zmC*yymC*sigmaNNLO(zC,yy2C)

      else

      yymC= dlog((1D0-buff)/buff)-dlog(yminC/(1-yminC))
      yy2C=1.D0/(1+dexp(-dlog(yminC/(1-yminC))-yymC*xi2))
      yy3C=1-buff + (ymaxC - (1- buff))*xi2

      NNLOCRC =
     ,  zmC*yymC*sigmaNNLO(zC,yy2C)
     , +zmC*((ymaxC-1)/buff + 1)*(dlog(buff)+buff/(1-buff))
     ,  *sigmaNNLO(zC,yy3C)

      end if

      NNLOCR = NNLOCRA + NNLOCRB + NNLOCRC

C      print *,  NNLOCR

      end




CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     Cross sections, LO,NLO,NNLO
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


C     LEADING ORDER CROSS SECTION

       double precision function sigmaLO(z,hh)
       implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 Lumqbarq,Lumqg, Lumgq, y, Ls

      dimension xpa(-6:6)

      xi1=dsqrt(x/u)
      xi2=dsqrt(x*u)
      q2=q**2

      select case(mode1)
       case(1)
         call pdfsMGDY(xi1,Q2,xpa)
       case(2)
         call pdfsDYn(xi1,Q2,xpa)
       case(3)
         call pdfsDYd(xi1,Q2,xpa)
       case(4)
         call pdfsDYcu(xi1,Q2,xpa)
       case default
         write(*,*) "SIGMALO: bad value for target,",mode1
      end select
      upv1=xpa(1)
      dnv1=xpa(2)
      usea1=xpa(-1)
      dsea1=xpa(-2)
      str1=xpa(-3)
      chm1=xpa(-4)
      bot1=xpa(-5)
      select case(mode2)
       case(1)
         call pdfsMGDY(xi2,Q2,xpa)
       case(2)
         call pdfsDYn(xi2,Q2,xpa)
       case(3)
         call pdfsDYd(xi2,Q2,xpa)
       case(4)
         call pdfsDYcu(xi2,Q2,xpa)
       case default
         write(*,*) "SIGMALO: bad value for target,",mode2
      end select
      upv2=xpa(1)
      dnv2=xpa(2)
      usea2=xpa(-1)
      dsea2=xpa(-2)
      str2=xpa(-3)
      chm2=xpa(-4)
      bot2=xpa(-5)

       sigmaLO = 2.D0*(
     ,    eu**2*(upv1*usea2+usea1*upv2 +2*usea1*usea2)
     ,   +ed**2*(dnv1*dsea2+dsea1*dnv2 +2*dsea1*dsea2)
     ,   +2*es**2*str1*str2
     ,   +2*ec**2*chm1*chm2
     ,   +2*eb**2*bot1*bot2
     , )

c       print *,sigmaLO

       end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   Next-to-leading order cross section
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


C      NEXT TO LEADING ORDER CROSS SECTION

       double precision function sigmaNLO(z,y)
       implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 Lumqbarq,Lumqg, Lumgq, y, Ls

      data z2 /1.644934068D0/

      dimension xpa(-6:6)

      Ls=dlog(mF**2/q**2)


      x1p=dsqrt(x/u/z*(z+y*(1.D0-z))/(1.D0-y*(1-z)))
      x2p=dsqrt(x*u/z*(1.D0-y*(1-z))/(z+y*(1.D0-z)))

C       print *, mode,x,u,q,buff

      if (x1p.gt.1d0) then
        print *, x,z,y,u,x1p,x2p
        stop
      endif

      if (x2p.gt.1d0) then
        print *, x,z,y,u,x1p,x2p
        stop
      endif


      q2=q**2

      select case(mode1)
       case(1)
         call pdfsMGDY(x1p,Q2,xpa)
       case(2)
         call pdfsDYn(x1p,Q2,xpa)
       case(3)
         call pdfsDYd(x1p,Q2,xpa)
       case(4)
         call pdfsDYcu(x1p,Q2,xpa)
       case default
         write(*,*) "SIGMANLO: bad value for target,",mode1
      end select
      upv1=xpa(1)
      dnv1=xpa(2)
      usea1=xpa(-1)
      dsea1=xpa(-2)
      str1=xpa(-3)
      chm1=xpa(-4)
      bot1=xpa(-5)
      glu1=xpa(0)
      select case(mode2)
       case(1)
         call pdfsMGDY(x2p,Q2,xpa)
       case(2)
         call pdfsDYn(x2p,Q2,xpa)
       case(3)
         call pdfsDYd(x2p,Q2,xpa)
       case(4)
         call pdfsDYcu(x2p,Q2,xpa)
       case default
         write(*,*) "SIGMANLO: bad value for target,",mode2
      end select
      upv2=xpa(1)
      dnv2=xpa(2)
      usea2=xpa(-1)
      dsea2=xpa(-2)
      str2=xpa(-3)
      chm2=xpa(-4)
      bot2=xpa(-5)
      glu2=xpa(0)

C      Parton luminosities

       Lumqbarq =
     ,    eu**2*(upv1*usea2+usea1*upv2 +2*usea1*usea2)
     ,   +ed**2*(dnv1*dsea2+dsea1*dnv2 +2*dsea1*dsea2)
     ,   +2*es**2*str1*str2
     ,   +2*ec**2*chm1*chm2
     ,   +2*eb**2*bot1*bot2


       Lumqg = (eu**2*(upv1 + 2*usea1) +ed**2*(dnv1+2*dsea1)
     , +2*(es**2*str1+ec**2*chm1+eb**2*bot1))*glu2

       Lumgq = glu1*(eu**2*(upv2 + 2*usea2) +ed**2*(dnv2+2*dsea2)
     ,     +2*(es**2*str2+ec**2*chm2+eb**2*bot2))

C     Scale dependence on the NLO qbarq part

      t1 = 1-z
      t2 = d1(t1)
      t4 = DD0(t1)
      t7 = z**2
      t11 = d1(y)
      t13 = d1(1-y)
      t18 = 4*(t2-2.D0/3.D0-2.D0/3.D0*z+4.D0/3.D0*t4)*t7/(z+1)*(t11+t13)
     #*Ls

      sigmaqbarqLs = t18


      sigmaqbarq = sigmaqbarqLs +
     , ((0.8D1 / 0.3D1 * z2 - 0.16D2 / 0.3D1) * d1(y) + (0.8D1 / 0.3D
     ,1 * z2 - 0.16D2 / 0.3D1) * d1(1 - y)) * d1(1 - z)
     , + (0.4D1 / 0.3D1
     ,*(z ** 2) * (-0.2D1 * (z ** 2) * dlog((z)) - 0.2D1
     ,*log((z)) - (4 * z) + (4 * z ** 2)) / (z + 1) * DD0(1
     ,- z) + 0.16D2 / 0.3D1 * (z ** 2) * (z ** 2 + 1) / (z +
     ,1) * (DD1(1 - z)) + 0.8D1 / 0.3D1 * (z ** 2)) * d1(y)
     , + (0.
     ,4D1 / 0.3D1 * (z ** 2) * (-0.2D1 * (z ** 2) * dlog((z))
     ,- 0.2D1 * dlog((z)) - (4 * z) + (4 * z ** 2)) / (z +
     , 1) * DD0(1 - z) + 0.16D2 / 0.3D1 * (z ** 2) * (z ** 2 + 1)
     , / (z + 1) * (DD1(1 - z)) + 0.8D1 / 0.3D1 * (z ** 2))
     , *d1(1 - y)
     , +8.D0/3.D0*DD0(1-z)*(DD0(y)+DD0(1-y))
     , -8.D0/3.D0*(z**3+z**2+2*z+1)/(z+1)*(DD0(y)+DD0(1-y))
     , -8.D0/3.D0*(-1+z)**3*(2*y**2+1-2*y)*z/(z+1)


      t1 = z**2
      t3 = (1-z)**2
      t8 = d1(y)
      t12 = (t1+t3)*t1/(z+1)*t8*Ls

      sigmaqgLs = t12

      sigmaqg = sigmaqgLs +
     ,(z ** 2) * (-1 + z) * (-2 * z + 2 * z ** 2 + 1) /
     , (z + 1) * DD0(1 - z) * d1(y) * dlog((z)) + (-(z ** 2) *
     , (-1 + z) / (z + 1) * DD0(1 - z) - (2 * z ** 2 * (-1 +
     ,z) * (-2 * z + 2 * z ** 2 + 1) / (z + 1) * DD1(1 - z)) - (z **
     ,2 * (-2 * z + 2 * z ** 2 + 1) / (z + 1))) * d1(y) + (-1 + y) *
     ,(-1 + z) * (1 - y + y * z) * (-z - y + y * z) * (z)
     , * (-y ** 2 * z ** 2 + 2 * y * z ** 2 - 2 * z ** 2 - 2 * y * z
     ,+ 2 * z + 2 * y ** 2 * z - 1 - y ** 2) / (z + 1) * DD0(1 - y) *
     , DD0(y) * DD0(1 - z)

      t1 = z**2
      t3 = (1-z)**2
      t8 = d1(1-y)
      t12 = (t1+t3)*t1/(z+1)*t8*Ls

      sigmagqLs = t12

      sigmagq =  sigmagqLs +
     , ((y * (-1 + z) * (-z - y + y * z) * (1 - y + y * z) * z *
     , (z ** 2 + y ** 2 * z**2-2*z + 2*y*z - 2*y**2*z + 2
     , - 2 * y + y ** 2) / (z + 1) * DD0(y) * DD0(1 - y)) + (z
     ,** 2) * (-1 + z) * (-0.2D1 * log((z)) * (z)
     , + 0.2D1 * dlog((z)) * (z ** 2) + log((z)) - 0.1D1) / (z + 1) *
     ,d1(1 - y)) * DD0(1 - z) - 0.2D1 * (z ** 2) * (-1 + z) *
     ,(-2 * z + 2 * z ** 2 + 1) / (z + 1) * d1(1 - y) * DD1(1
     ,- z) - (z ** 2) * (-2 * z + 2 * z ** 2 + 1) / (z + 1) *
     , d1(1 - y)


C      print *,  sigmaqbarq

       sigmaNLO=DD0inv(1-z)*DDinv(y)*
     , (1D0+z)/2/z**2/(1-y+y*z)/(z+y-y*z)*
     , (
     ,sigmaqbarq*Lumqbarq +
     ,  sigmaqg*Lumqg + sigmagq*Lumgq
     , )*z

C       print *, sigmaNLO

       end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   Next-to-leading order cross section
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


C      NEXT TO NEXT TO LEADING ORDER CROSS SECTION

       double precision function sigmaNNLO(z,y)
       implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

       real*8 Lumqbarq,Lumqg, Lumgq, y,t,r,NF,NFf, Lumgg
       real*8 u, Lumqq, Lumq1q2
       real*8 qquark1,qquark2,Lumq1q2a,Lumq1q2b, Lumq1q2c


      real*8 Ls

      data z2 /1.644934068D0/


       dimension var(9)

      dimension xpa(-6:6)

       external DD0inv, DDinv,DD3,DD2,DD1,hardqqnoNF
       external sigmaqbarq, sigmagg,sigmaqg, sigmaqq
       external sigmaq1q2a,sigmaq1q2b,sigmaq1q2c


       z3=1.202056903159594D0
       buff1=0.000001

       x1p=dsqrt(x/u/z*(z+y*(1.D0-z))/(1.D0-y*(1-z)))
       x2p=dsqrt(x*u/z*(1.D0-y*(1-z))/(z+y*(1.D0-z)))


        NF=5D0
        NFf = 3D0

C       print *, mode,x,u,q,buff

       if (x1p.gt.1) then
       print *, x,z,y,u,x1p,x2p
       stop
       else
       continue
       endif

       if (x2p.gt.1) then
       print *, x,z,y,u,x1p,x2p
       stop
       else
       continue
       endif



      q2=q**2

      select case(mode1)
       case(1)
         call pdfsMGDY(x1p,Q2,xpa)
       case(2)
         call pdfsDYn(x1p,Q2,xpa)
       case(3)
         call pdfsDYd(x1p,Q2,xpa)
       case(4)
         call pdfsDYcu(x1p,Q2,xpa)
       case default
         write(*,*) "SIGMANNLO: bad value for target,",mode1
      end select
      upv1=xpa(1)
      dnv1=xpa(2)
      usea1=xpa(-1)
      dsea1=xpa(-2)
      str1=xpa(-3)
      chm1=xpa(-4)
      bot1=xpa(-5)
      glu1=xpa(0)
      select case(mode2)
       case(1)
         call pdfsMGDY(x2p,Q2,xpa)
       case(2)
         call pdfsDYn(x2p,Q2,xpa)
       case(3)
         call pdfsDYd(x2p,Q2,xpa)
       case(4)
         call pdfsDYcu(x2p,Q2,xpa)
       case default
         write(*,*) "SIGMANNLO: bad value for target,",mode2
      end select
      upv2=xpa(1)
      dnv2=xpa(2)
      usea2=xpa(-1)
      dsea2=xpa(-2)
      str2=xpa(-3)
      chm2=xpa(-4)
      bot2=xpa(-5)
      glu2=xpa(0)

C      Parton luminosities


       Lumqbarq =
     ,    eu**2*(upv1*usea2+usea1*upv2 +2*usea1*usea2)
     ,   +ed**2*(dnv1*dsea2+dsea1*dnv2 +2*dsea1*dsea2)
     ,   +2*es**2*str1*str2
     ,   +2*ec**2*chm1*chm2
     ,   +2*eb**2*bot1*bot2


       Lumqg = (eu**2*(upv1 + 2*usea1) +ed**2*(dnv1+2*dsea1)
     , +2*(es**2*str1+ec**2*chm1+eb**2*bot1))*glu2

       Lumgq = glu1*(eu**2*(upv2 + 2*usea2) +ed**2*(dnv2+2*dsea2)
     ,     +2*(es**2*str2+ec**2*chm2+eb**2*bot2))


       Lumgg = glu1*glu2*(eu**2+ed**2+es**2+ec**2+eb**2)

C    qq luminosity for identical quarks

       Lumqq =  eu**2*( (upv1+usea1)*(upv2+usea2)+usea1*usea2)
     ,        +ed**2*( (dnv1+dsea1)*(dnv2+dsea2) +dsea1*dsea2)
     ,        +2*es**2*str1*str2
     ,        +2*ec**2*chm1*chm2
     ,        +2*eb**2*bot1*bot2


C    Luminosity for q1 and q2 being different

      qquark1 = upv1+2*usea1 + dnv1+2*dsea1+2*str1+2*chm1+2*bot1
      qquark2 = upv2+2*usea2 + dnv2+2*dsea2+2*str2+2*chm2+2*bot2

      Lumq1q2a =
     ,     (eu**2*(upv1+2*usea1)*(qquark2-upv2-2*usea2)
     ,     +ed**2*(dnv1+2*dsea1)*(qquark2-dnv2-2*dsea2)
     ,     +es**2*(2*str1)*(qquark2-2*str2)
     ,     +ec**2*(2*chm1)*(qquark2-2*chm2)
     ,     +eb**2*(2*bot1)*(qquark2-2*bot2) )

      Lumq1q2b =
     ,     (eu**2*(upv2+2*usea2)*(qquark1-upv1-2*usea1)
     ,     +ed**2*(dnv2+2*dsea2)*(qquark1-dnv1-2*dsea1)
     ,     +es**2*(2*str2)*(qquark1-2*str1)
     ,     +ec**2*(2*chm2)*(qquark1-2*chm1)
     ,     +eb**2*(2*bot2)*(qquark1-2*bot1) )


C    The Lumq1q2c is the part where charges enter linearly
C    since quark_sea = antiquark_sea by construction
C    the sea quark/antiquark contribution cancels away
C    The remainder is simple The minus sign is because
C    charges are defined this way

       Lumq1q2c = (-1D0)*eu*ed*(upv1*dnv2+upv2*dnv1)


       sigmaNNLO=DD0inv(1D0-z)*DDinv(y)*
     , (1.D0+z)/2/z/(1D0-y+y*z)/(z+y-y*z)*
     , (
     ,   sigmaqbarq(z,y)*Lumqbarq
     ,  + sigmagg(z,y)*Lumgg
     ,  +(sigmaqg(z,y)*Lumqg
     ,  +sigmaqg(z,1D0-y)*Lumgq)
     ,  +sigmaqq(z,y)*Lumqq
     ,  +sigmaq1q2a(z,y)*Lumq1q2a
     ,  +sigmaq1q2a(z,1D0-y)*Lumq1q2b
     ,  +sigmaq1q2c(z,y)*Lumq1q2c
     ,    )

       end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      double precision function sigmaqbarq(z,y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 u, NF, w, NFf, Ly, L1my, Lw, y
      real*8 Ldelta,Lxi,L1mxi,xi,r,delta,Ls,t, Lz
      external dilog, hardqqnoNF, d1, DD0,DD1,DD2,DD3

       z2=1.644934066848226D0
       z3=1.202056903159594D0
       buff1=0.00000001

       Ly=dlog(y+buff1)
       L1my=dlog(1D0-y+buff1)
       Lz=dlog(1D0-z)
       Ls=dlog(mF**2/q**2)
       w=1D0-z
       Lw=dlog(w)

        NF=5D0
        NFf = 3D0

C    These are the remnants of the qbarq part

C    Scale-dependent part through w^7

      t1 = 6*Ls
      t3 = Ls*(t1-34)
      t4 = 1D0-y
      t5 = d1(t4)
      t7 = d1(y)
      t10 = 1D0-z
      t11 = d1(t10)
      t16 = DD1(t10)
      t18 = DD0(t4)
      t20 = DD0(y)
      t24 = Ls*(t1-20)
      t30 = DD0(t10)
      t32 = (t3*t5+t3*t7)*t11/36+8.D0/9.D0*(Ls*t5+Ls*t7)*t16+(4.D0/9.D0*
     #t18*Ls*t20+t24*t5/27+t24*t7/27)*t30

      tsoftz = NF*t32



      t1 = 1D0-y
      t2 = DD0(t1)
      t3 = Ls*t2
      t4 = w**2
      t5 = t4**2
      t6 = t5*w
      t9 = t4*w
      t10 = t5*t9
      t11 = d1(t1)
      t20 = Ls**2
      t21 = t20*t11
      t24 = t5*t4
      t25 = d1(y)
      t32 = t20*t25
      t35 = DD0(y)
      t36 = Ls*t35
      t39 = Ls*t25
      t46 = Ls*t11
      t50 = t3*t6/144+127.D0/60480.D0*t10*t11*Ls+t3*t10/576-11.D0/3.D0*w
     #*t11*Ls+t21*t9/72-43.D0/10080.D0*t24*t25*Ls-43.D0/108.D0*t9*t25*Ls
     #+t32*t24/576-7.D0/18.D0*t36*t4+74.D0/27.D0*t39+t3*t9/36+t3*t5/72-1
     #0.D0/9.D0*t3+74.D0/27.D0*t46-5.D0/9.D0*t21-5.D0/9.D0*t32
      t52 = t9*Lw
      t55 = t5*Lw
      t58 = t24*Lw
      t61 = w*Lw
      t64 = t4*Lw
      t67 = t6*Lw
      t70 = t10*Lw
      t90 = -10.D0/9.D0*t36+t46*t52/18+t46*t55/36+t46*t58/144+2*t46*t61-
     #7.D0/9.D0*t46*t64+t46*t67/72+t46*t70/288+t39*t55/36+t39*t58/144+2*
     #t39*t61-7.D0/9.D0*t39*t64+t39*t67/72+t39*t70/288+t39*t52/18-103.D0
     #/1080.D0*t5*t25*Ls
      t128 = 13.D0/6.D0*t4*t25*Ls+t3*w+t36*t10/576+t21*t5/144+t36*t24/28
     #8-43.D0/10080.D0*t24*t11*Ls-103.D0/1080.D0*t5*t11*Ls+t21*t6/288-11
     #.D0/3.D0*w*t25*Ls+t36*t5/72+t32*w/2-7.D0/36.D0*t32*t4+127.D0/60480
     #.D0*t10*t25*Ls+t36*t9/36-7.D0/18.D0*t3*t4+t3*t24/288
      t164 = t21*w/2+t32*t10/1152-7.D0/36.D0*t21*t4-11.D0/432.D0*t6*t11*
     #Ls+t36*t6/144+t21*t24/576-11.D0/432.D0*t6*t25*Ls+t21*t10/1152+t32*
     #t5/144-20.D0/9.D0*t46*Lw+t32*t9/72-20.D0/9.D0*t39*Lw+t36*w+13.D0/6
     #.D0*t4*t11*Ls-43.D0/108.D0*t9*t11*Ls+t32*t6/288
      t166 = t50+t90+t128+t164

      tsofty=NF*t166

      t1 = w**2
      t2 = t1*w
      t3 = Ls*t2
      t5 = y**2
      t10 = t1**2
      t11 = t10*Ls
      t18 = t10*w*Ls
      t25 = t10*t1*Ls
      t32 = t10*t2*Ls
      t38 = 2.D0/9.D0*t3+4.D0/9.D0*t3*t5-4.D0/9.D0*t3*y-2.D0/9.D0*t11*t5
     #+2.D0/9.D0*t11*y-t11/9-t18/18-t18*t5/9+t18*y/9-t25*t5/18-t25/36+t2
     #5*y/18-t32/72-t32*t5/36+t32*y/36

      sigmaqbarqNFLs = t38*NF+tsoftz+tsofty


      t12 = 122.D0/9.D0*Ls*z3-8.D0/9.D0*Ls*(4*Ls-3)*z2+Ls*(-27*Ls+207)/3
     #6
      t13 = 1-y
      t14 = d1(t13)
      t16 = d1(y)
      t19 = 1-z
      t20 = d1(t19)
      t25 = DD2(t19)
      t27 = DD0(t13)
      t28 = t27*Ls
      t29 = DD0(y)
      t34 = Ls*(16*Ls-9)
      t40 = DD1(t19)
      t44 = DD1(t13)
      t49 = DD1(y)
      t58 = -4.D0/9.D0*Ls*z2+Ls*(45*Ls+18)/27
      t62 = DD0(t19)
      t64 = (t12*t14+t12*t16)*t20+64.D0/3.D0*(Ls*t14+Ls*t16)*t25+(64.D0/
     #3.D0*t28*t29+4.D0/9.D0*t34*t14+4.D0/9.D0*t34*t16)*t40+((2.D0/9.D0*
     #t34*t27+32.D0/3.D0*Ls*t44)*t29+32.D0/3.D0*t28*t49+t58*t14+t58*t16)
     #*t62

      tsoftz=t64

      t1 = Ls**2
      t2 = d1(y)
      t3 = t1*t2
      t5 = 1-y
      t6 = d1(t5)
      t7 = t1*t6
      t9 = Ls*t6
      t11 = Ls*t2
      t13 = DD0(t5)
      t14 = Ls*t13
      t15 = w**2
      t16 = t15**2
      t17 = t16*w
      t18 = t17*Lw
      t21 = t16*Lw
      t24 = DD1(y)
      t25 = Ls*t24
      t28 = t16*t2
      t31 = DD0(y)
      t32 = t1*t31
      t37 = t1*t13
      t40 = w*t6
      t43 = t15*w
      t44 = t16*t43
      t49 = DD1(t5)
      t50 = Ls*t49
      t57 = t16*t15
      t62 = Ls*t31
      t65 = -11.D0/18.D0*t3-11.D0/18.D0*t7-25.D0/3.D0*t9-25.D0/3.D0*t11+
     #t14*t18/3+2.D0/3.D0*t14*t21+t25*t17/6+4979.D0/2592.D0*t28*Ls+t32*t
     #16/9+1087.D0/216.D0*t3*t15-28.D0/9.D0*t37*t15+575.D0/18.D0*t40*Ls+
     #451.D0/80640.D0*t7*t44+8*t37*w-28.D0/3.D0*t50*t15-893.D0/4320.D0*t
     #14*t17+t50*t44/24+t25*t57/12+2.D0/3.D0*t50*t43-107.D0/60480.D0*t62
     #*t57
      t68 = t17*t6
      t71 = t57*t2
      t94 = t44*t6
      t97 = t44*t2
      t100 = w*t2
      t101 = Ls*Lw
      t104 = t43*t6
      t107 = t43*t2
      t112 = t43*z2
      t115 = 605.D0/36.D0*t62*t15-1187.D0/129600.D0*t68*Ls-432419.D0/181
     #4400.D0*t71*Ls+t37*t57/36+24*t50*w+10.D0/9.D0*t11*z2-160.D0/3.D0*t
     #14*Lw-481.D0/432.D0*t7*t43+t37*t16/9-475.D0/18.D0*t62*w-163.D0/36.
     #D0*t3*w-163.D0/36.D0*t7*w+t25*t16/3-723571.D0/3628800.D0*t94*Ls-72
     #3571.D0/3628800.D0*t97*Ls-539.D0/9.D0*t100*t101-841.D0/108.D0*t104
     #*t101-841.D0/108.D0*t107*t101-2189.D0/1080.D0*t28*t101-t9*t112/36
      t117 = Lw**2
      t118 = t15*t117
      t121 = w*z2
      t123 = t44*z2
      t126 = t17*z2
      t129 = t16*z2
      t132 = t57*z2
      t135 = t15*z2
      t138 = w*t117
      t141 = t16*t117
      t144 = t44*t117
      t147 = t57*t117
      t150 = t43*t117
      t153 = t17*t117
      t156 = t57*Lw
      t161 = t44*Lw
      t166 = w*Lw
      t169 = t15*Lw
      t172 = t43*Lw
      t175 = -56.D0/3.D0*t9*t118-t9*t121-t9*t123/576-t9*t126/144-t9*t129
     #/72-t9*t132/288+7.D0/18.D0*t9*t135+48*t9*t138+2.D0/3.D0*t9*t141+t9
     #*t144/12+t9*t147/6+4.D0/3.D0*t9*t150+t9*t153/3+t7*t156/18+2.D0/9.D
     #0*t7*t21+t7*t161/36+t7*t18/9+16*t7*t166-56.D0/9.D0*t7*t169+4.D0/9.
     #D0*t7*t172
      t215 = -t11*t112/36-56.D0/3.D0*t11*t118-t11*t121-t11*t123/576-t11*
     #t126/144-t11*t129/72-t11*t132/288+7.D0/18.D0*t11*t135+48*t11*t138+
     #2.D0/3.D0*t11*t141+t11*t144/12+t11*t147/6+4.D0/3.D0*t11*t150+t11*t
     #153/3+t3*t156/18+2.D0/9.D0*t3*t21+t3*t161/36+t3*t18/9+16*t3*t166-5
     #6.D0/9.D0*t3*t169
      t218 = t15*t6
      t221 = t16*t6
      t230 = t57*t6
      t239 = t17*t2
      t242 = t15*t2
      t262 = 2071.D0/54.D0*t218*t101+4979.D0/2592.D0*t221*Ls+t14*t161/12
     #+397.D0/2880.D0*t94*t101+397.D0/2880.D0*t97*t101+131.D0/4320.D0*t2
     #30*t101-2189.D0/1080.D0*t221*t101-893.D0/2160.D0*t68*t101+131.D0/4
     #320.D0*t71*t101-893.D0/2160.D0*t239*t101+2071.D0/54.D0*t242*t101-5
     #39.D0/9.D0*t40*t101-80.D0/9.D0*t37-713.D0/216.D0*t14*t43+2.D0/9.D0
     #*t32*t43-673.D0/18.D0*t242*Ls-673.D0/18.D0*t218*Ls-1187.D0/129600.
     #D0*t239*Ls-599.D0/24192.D0*t7*t57+314.D0/9.D0*t9*Lw
      t303 = 314.D0/9.D0*t11*Lw+t50*t57/12-160.D0/9.D0*t7*Lw-1013.D0/864
     #0.D0*t7*t17-160.D0/3.D0*t11*t117+10.D0/9.D0*t9*z2-160.D0/3.D0*t62*
     #Lw-1733.D0/4320.D0*t7*t16-713.D0/216.D0*t62*t43-160.D0/3.D0*t9*t11
     #7-481.D0/432.D0*t3*t43-599.D0/24192.D0*t3*t57+451.D0/80640.D0*t3*t
     #44+t37*t44/72-1013.D0/8640.D0*t3*t17-160.D0/9.D0*t3*Lw-1997.D0/216
     #0.D0*t14*t16+t37*t17/18+6289.D0/120960.D0*t14*t44+2.D0/9.D0*t37*t4
     #3
      t345 = -475.D0/18.D0*t14*w+t32*t44/72-1997.D0/2160.D0*t62*t16+t32*
     #t17/18-28.D0/9.D0*t32*t15+t32*t57/36-1733.D0/4320.D0*t3*t16+t50*t1
     #7/6+t50*t16/3+24*t25*w-28.D0/3.D0*t25*t15+2.D0/3.D0*t25*t43+3865.D
     #0/324.D0*t104*Ls+605.D0/36.D0*t14*t15+3865.D0/324.D0*t107*Ls-107.D
     #0/60480.D0*t14*t57+1087.D0/216.D0*t7*t15+6289.D0/120960.D0*t62*t44
     #+8*t32*w+t25*t44/24
      t381 = -893.D0/4320.D0*t62*t17+575.D0/18.D0*t100*Ls-432419.D0/1814
     #400.D0*t230*Ls-80.D0/3.D0*t50-80.D0/9.D0*t32+47.D0/3.D0*t62+48*t62
     #*t166-56.D0/3.D0*t14*t169+4.D0/9.D0*t3*t172+t14*t156/6-56.D0/3.D0*
     #t62*t169+48*t14*t166+4.D0/3.D0*t14*t172+t62*t156/6+2.D0/3.D0*t62*t
     #21+t62*t161/12+t62*t18/3+47.D0/3.D0*t14+4.D0/3.D0*t62*t172-80.D0/3
     #.D0*t25
      t384 = t65+t115+t175+t215+t262+t303+t345+t381

      tsofty = t384

      t1 = w**2
      t2 = t1**2
      t3 = t2*Ls
      t5 = Ls**2
      t8 = t1*w
      t13 = t2*w
      t16 = t2*t1
      t19 = t2*t8
      t20 = t19*t5
      t24 = t8*Ls
      t26 = y**2
      t27 = t19*t26
      t32 = Ls/(y+buff1)
      t35 = t19*y
      t38 = t16*y
      t43 = t26**2
      t44 = t43*y
      t53 = 421.D0/72.D0*t3+4.D0/3.D0*t1*t5+2.D0/9.D0*t8*t5-t2*t5/3-t13*
     #t5/6-t16*t5/12-t20/24-8.D0/9.D0*w*t5-170.D0/27.D0*t24+2971.D0/8640
     #.D0*t27*Ls-80.D0/3.D0*t32*L1my-t35*t5/72-7831.D0/4320.D0*t38*Ls-94
     #7.D0/1728.D0*t35*Ls+35.D0/24.D0*t19*t44*t5-41.D0/24.D0*t19*t43*t5-
     #17.D0/72.D0*t27*t5
      t54 = t1*Ls
      t57 = t26*y
      t58 = t19*t57
      t61 = t19*Ls
      t63 = t16*t26
      t68 = t43*t26
      t72 = t13*Ls
      t74 = Ls*t16
      t82 = t8*t26
      t96 = Ls/(-1+y-buff1)
      t105 = 16.D0/3.D0*t54*Lw+71.D0/72.D0*t58*t5-161.D0/2160.D0*t61+937
     #9.D0/4320.D0*t63*Ls-181.D0/2160.D0*t58*Ls+1063.D0/2160.D0*Ls*t68*t
     #19+8113.D0/4320.D0*t72+359.D0/864.D0*t74-1063.D0/720.D0*t61*t44+22
     #9.D0/180.D0*t61*t43-35.D0/72.D0*t20*t68-2.D0/3.D0*t82*t5-5.D0/9.D0
     #*t13*t43*t5-5.D0/18.D0*t16*t43*t5-t13*y*t5/18+80.D0/3.D0*t96*Ly+10
     #.D0/9.D0*t13*t57*t5-t13*t26*t5/2
      t113 = t8*y
      t121 = t16*t57
      t124 = w*Ls
      t143 = t2*y
      t146 = -7.D0/18.D0*t61*Lw-t38*t5/36-t63*t5/4+2.D0/3.D0*t113*t5-7.D
     #0/9.D0*t74*Lw+t2*t26*t5/3+5.D0/9.D0*t121*t5-32.D0/9.D0*t124*Lw-28.
     #D0/9.D0*t3*Lw-14.D0/9.D0*t72*Lw+2281.D0/720.D0*t72*t43-2281.D0/360
     #.D0*t72*t57+40.D0/9.D0*t24*Lw-10661.D0/2160.D0*t72*y-43.D0/60.D0*t
     #121*Ls+1094.D0/135.D0*t72*t26-t143*t5/3
      t157 = L1my*t13
      t160 = L1my*t16
      t166 = Ly*t19
      t169 = Ly*t1
      t172 = Ly*t8
      t175 = Ly*t2
      t184 = L1my*t8
      t191 = L1my*y
      t198 = 1699.D0/216.D0*t3*t26-605.D0/54.D0*t82*Ls+605.D0/54.D0*t113
     #*Ls-1699.D0/216.D0*t143*Ls+43.D0/120.D0*t74*t43+t32*t157/6+t32*t16
     #0/12+24*t32*L1my*w+11.D0/72.D0*t96*t166+20.D0/3.D0*t96*t169-26.D0/
     #9.D0*t96*t172+11.D0/9.D0*t96*t175-17.D0/9.D0*t32*t157*t57-7.D0/18.
     #D0*t32*t160*y-20.D0/9.D0*t32*t184*t26+20.D0/9.D0*t32*t184*t57-16.D
     #0/9.D0*t32*t191*w+10.D0/9.D0*t32*t160*t43
      t201 = Ly*t16
      t220 = t43*t57
      t228 = Ly*t13
      t232 = Ly*y
      t248 = L1my*t1
      t255 = L1my*t2
      t259 = 5.D0/3.D0*t96*t201*t43-5.D0/9.D0*t96*t201*t44+20.D0/9.D0*t3
     #2*t157*t43-7.D0/9.D0*t96*t201*y+8.D0/9.D0*t96*t166*t26-5.D0/9.D0*t
     #32*t160*t44-35.D0/36.D0*t32*L1my*t220*t19+7.D0/18.D0*t32*t160*t26+
     #10.D0/3.D0*t96*t228*t43-8.D0/3.D0*t96*t232*t2-14.D0/9.D0*t96*t228*
     #y+20.D0/9.D0*t96*t172*t57-7.D0/18.D0*t96*t166*y-37.D0/18.D0*t96*t2
     #01*t57+8.D0/3.D0*t32*t248*y+4.D0/3.D0*t96*t201*t26+10.D0/9.D0*t32*
     #t255*t26
      t275 = L1my*t19
      t314 = -14.D0/9.D0*t32*t191*t2+7.D0/9.D0*t32*t157*t26-10.D0/9.D0*t
     #32*t157*t44-7.D0/9.D0*t32*t157*y-19.D0/3.D0*t96*t166*t44+7.D0/36.D
     #0*t32*t275*t26-25.D0/36.D0*t32*t275*t57+71.D0/36.D0*t32*t275*t43+2
     #0.D0/9.D0*t32*t184*y-8.D0/3.D0*t96*t166*t57+35.D0/12.D0*t32*t275*t
     #68-7.D0/36.D0*t32*t275*y-10.D0/9.D0*t32*L1my*t57*t2-41.D0/12.D0*t3
     #2*t275*t44+97.D0/18.D0*t96*t166*t43+8.D0/3.D0*t96*t228*t26+11.D0/1
     #8.D0*t96*t228+11.D0/36.D0*t96*t201
      t319 = Lw*t26
      t322 = Lw*t57
      t325 = Lw*t43
      t334 = Lw*y
      t359 = -200.D0/9.D0*t96*Ly*w-25.D0/18.D0*t61*t319+71.D0/18.D0*t61*
     #t322-41.D0/6.D0*t61*t325+35.D0/6.D0*t61*Lw*t44-35.D0/18.D0*t61*Lw*
     #t68+7.D0/18.D0*t61*t334+40.D0/9.D0*t72*t322+20.D0/9.D0*t3*t334-20.
     #D0/9.D0*t3*t319+20.D0/9.D0*t74*t322-17.D0/9.D0*t74*t319+7.D0/9.D0*
     #t74*t334-10.D0/9.D0*t74*t325-34.D0/9.D0*t72*t319-20.D0/9.D0*t72*t3
     #25+14.D0/9.D0*t72*t334-40.D0/9.D0*t24*t334
      t407 = 40.D0/9.D0*t24*t319+t32*t275/24-28.D0/3.D0*t32*t248+2.D0/3.
     #D0*t32*t184+t32*t255/3-10.D0/9.D0*t96*Ly*t57*t2+35.D0/9.D0*t96*t16
     #6*t68-38.D0/9.D0*t124+40.D0/9.D0*t96*t172*y-40.D0/9.D0*t96*t172*t2
     #6+85.D0/18.D0*t54-35.D0/36.D0*t96*Ly*t220*t19-37.D0/9.D0*t96*t228*
     #t57-16.D0/9.D0*t96*t232*w+8.D0/3.D0*t96*t169*y-10.D0/9.D0*t96*t228
     #*t44+20.D0/9.D0*t96*t175*t26-17.D0/18.D0*t32*t160*t57
      t410 = t53+t105+t146+t198+t259+t314+t359+t407

      sigmaqbarqLs = t410 + tsofty + tsoftz + sigmaqbarqNFLs

C    Scale-independent part throught w^7

C    NFf-dependent part, it is very small and the precision of the
C    expansion is more than adequate


      t1 = w**2
      t3 = t1**2
      t4 = t3*t1*w
      t5 = y**2
      t6 = t5**2
      t7 = t6*t5
      t10 = t6*y
      t15 = t5*y
      t20 = t3**2
      t25 = t3*w
      t45 = 8.D0/135.D0*t4*t7-8.D0/45.D0*t10*t4+2.D0/9.D0*t6*t4-4.D0/27.
     #D0*t15*t4+t5*t4/18+5.D0/108.D0*t20*t15+37.D0/1440.D0*t20*t5+t6*t25
     #/9-43.D0/216.D0*t20*t7+11.D0/90.D0*t20*t6*t15-y*t20/72+7.D0/90.D0*
     #t20*t10-17.D0/288.D0*t20*t6-y*t4/90-2.D0/9.D0*t15*t25+y*t25/9


      sigmaqbarqNFf=NFf*t45


C     NF dependent part

      t3 = 2.D0/3.D0*z3-28.D0/27.D0*z2+127.D0/72.D0
      t4 = d1(y)
      t6 = 1D0-y
      t7 = d1(t6)
      t10 = 1D0-z
      t11 = d1(t10)
      t14 = DD2(t10)
      t18 = DD0(t6)
      t19 = DD0(y)
      t23 = DD1(t10)
      t26 = -4.D0/9.D0*z2+56.D0/81.D0
      t30 = DD1(t6)
      t34 = DD1(y)
      t38 = DD0(t10)
      t40 = (t3*t4+t3*t7)*t11
     # +8.D0/9.D0*(t4+t7)*t14+(-40.D0/27.D0*t4-40.
     #D0/27.D0*t7+8.D0/9.D0*t18*t19)*t23+(t26*t4+t26*t7+(-20.D0/27.D0*t1
     #8+4.D0/9.D0*t30)*t19+4.D0/9.D0*t34*t18)*t38

      t0softz = NF*t40

      t1 = d1(y)
      t2 = w**2
      t3 = t2**2
      t4 = t3*w
      t5 = t1*t4
      t9 = t1*w
      t11 = 1-y
      t12 = DD0(t11)
      t13 = t12*t2
      t15 = DD1(y)
      t16 = t2*w
      t19 = t1*t16
      t21 = DD1(t11)
      t26 = t12*t3
      t32 = t3*t16
      t33 = t1*t32
      t35 = d1(t11)
      t40 = -3781.D0/103680.D0*t5+10.D0/9.D0*t1*z2+185.D0/36.D0*t9+13.D0
     #/6.D0*t13+t15*t16/36+6193.D0/5184.D0*t19+t21*t4/144+t15*t4/144-103
     #.D0/1080.D0*t26+148.D0/27.D0*t1*Lw+t21*t16/36-10020887.D0/40642560
     #0.D0*t33+148.D0/27.D0*t35*Lw+10.D0/9.D0*t35*z2
      t41 = t12*w
      t43 = t3*t2
      t44 = t1*t43
      t46 = DD0(y)
      t47 = t46*t3
      t49 = t35*t4
      t51 = t35*t3
      t53 = t35*t2
      t60 = t12*t4
      t62 = t1*t3
      t66 = t46*t16
      t68 = t46*t2
      t70 = -11.D0/3.D0*t41-1864249.D0/50803200.D0*t44-103.D0/1080.D0*t4
     #7-3781.D0/103680.D0*t49+17831.D0/259200.D0*t51-845.D0/216.D0*t53-2
     #0.D0/9.D0*t12*Lw+t15*w+t15*t32/576-11.D0/432.D0*t60+17831.D0/25920
     #0.D0*t62-7.D0/18.D0*t15*t2-43.D0/108.D0*t66+13.D0/6.D0*t68
      t72 = Lw**2
      t75 = t12*t43
      t78 = t12*t16
      t80 = t35*t43
      t85 = t35*w
      t92 = t1*t2
      t101 = -20.D0/9.D0*t1*t72-43.D0/10080.D0*t75+t21*w-43.D0/108.D0*t7
     #8-1864249.D0/50803200.D0*t80-248.D0/81.D0*t1-103.D0/540.D0*t51*Lw+
     #2*t85*t72-t33*z2/576-22.D0/3.D0*t9*Lw+7.D0/18.D0*t92*z2+2*t9*t72+1
     #3.D0/3.D0*t92*Lw-43.D0/54.D0*t19*Lw
      t128 = t35*t16
      t131 = -t19*z2/36+t19*t72/18+t62*t72/36-103.D0/540.D0*t62*Lw-t62*z
     #2/72-t5*z2/144-11.D0/216.D0*t5*Lw+t5*t72/72-t44*z2/288-43.D0/5040.
     #D0*t80*Lw+t44*t72/144+t33*t72/288+127.D0/30240.D0*t33*Lw-43.D0/54.
     #D0*t128*Lw
      t138 = t46*w
      t159 = t60*Lw/72-t49*z2/144+2*t138*Lw+t66*Lw/18+t78*Lw/18+t128*t72
     #/18+t51*t72/36-t51*z2/72-11.D0/216.D0*t49*Lw-t9*z2-22.D0/3.D0*t85*
     #Lw-248.D0/81.D0*t35+74.D0/27.D0*t46-10.D0/9.D0*t15
      t161 = t46*t43
      t176 = t35*t32
      t184 = 74.D0/27.D0*t12-43.D0/10080.D0*t161-43.D0/5040.D0*t44*Lw-7.
     #D0/9.D0*t92*t72-t128*z2/36-t80*z2/288-7.D0/9.D0*t53*t72-10.D0/9.D0
     #*t21+t21*t43/288-10020887.D0/406425600.D0*t176+185.D0/36.D0*t85+61
     #93.D0/5184.D0*t128+t15*t43/288+t21*t3/72
      t188 = t12*t32
      t197 = t46*t4
      t200 = t46*t32
      t215 = t21*t32/576+127.D0/60480.D0*t188-11.D0/3.D0*t138-20.D0/9.D0
     #*t46*Lw+7.D0/18.D0*t53*z2+13.D0/3.D0*t53*Lw+t197*Lw/72+t200*Lw/288
     #+t161*Lw/144+t49*t72/72+t80*t72/144+t176*t72/288+127.D0/30240.D0*t
     #176*Lw-t176*z2/576
      t240 = -t85*z2-7.D0/9.D0*t13*Lw+t26*Lw/36+2*t41*Lw+t188*Lw/288+t75
     #*Lw/144-7.D0/9.D0*t68*Lw+t47*Lw/36+t15*t3/72-7.D0/18.D0*t21*t2-845
     #.D0/216.D0*t92-11.D0/432.D0*t197+127.D0/60480.D0*t200-20.D0/9.D0*t
     #35*t72
      t243 = t40+t70+t101+t131+t159+t184+t215+t240

      t0softy = NF*t243

      t1 = w**2
      t2 = t1*w
      t3 = t2*Lw
      t5 = y**2
      t6 = t2*t5
      t8 = t1**2
      t9 = t8*t5
      t11 = t8*w
      t12 = t5*y
      t13 = t11*t12
      t15 = t11*t5
      t17 = t8*t1
      t18 = t17*t12
      t20 = t17*y
      t22 = t17*t5
      t24 = t8*t2
      t25 = t5**2
      t28 = t24*t12
      t34 = t24*y
      t36 = t24*t5
      t38 = t8*y
      t42 = t11*y
      t46 = 1D0/(-1D0+y-buff1)*Ly
      t52 = t2*t12
      t55 = t2*y
      t58 = -15173.D0/161280.D0*t24*t25*y+1051.D0/69120.D0*t34+10621.D0/
     #483840.D0*t36-247.D0/160.D0*t38+73.D0/540.D0*t11*t25-3709.D0/8640.
     #D0*t42+10.D0/9.D0*t46-t46*t2/4-8.D0/9.D0*t46*t6+4.D0/9.D0*t46*t52+
     #2.D0/3.D0*t46*t55
      t62 = t8*Lw
      t67 = t11*Lw
      t72 = t17*Lw
      t79 = 1/(y+buff1)*L1my
      t88 = 4.D0/9.D0*t46*t9+4.D0/9.D0*t62*y-4.D0/9.D0*t62*t5-2.D0/9.D0*
     #t67*t5+7.D0/72.D0*t46*t8+t72*y/9-t46*t38/3-4.D0/9.D0*t79*t6+4.D0/9
     #.D0*t79*t52+2.D0/9.D0*t79*t55+2.D0/9.D0*t79*t9
      t98 = t24*Lw
      t108 = -t79*t13/9-t79*t42/18+t79*t15/9+373.D0/13440.D0*t17-2033.D0
     #/96768.D0*t24+1663.D0/8640.D0*t11-t98/36-t46*t13/9-t46*t42/6+2.D0/
     #9.D0*t46*t15-t46*t18/18
      t121 = t8*t12
      t151 = 13.D0/18.D0*w-251.D0/432.D0*t2-8.D0/9.D0*t3*y+t79*t36/36-2.
     #D0/9.D0*t79*t121-t79*t38/9-2.D0/9.D0*t62-t79*t18/18-t79*t20/36+t79
     #*t22/18-t79*t28/36
      t170 = -t79*t34/72-t72/18-t67/9-97.D0/108.D0*t1-t46*w-7.D0/18.D0*t
     #79*t1+t79*t8/72+t79*t11/144+t79*t17/288+t79*t24/576+t79*w
      t192 = 2.D0/9.D0*t67*y-t72*t5/9+t98*y/18-t98*t5/18+15173.D0/483840
     #.D0*t24*t25*t5-10.D0/9.D0*t79+11.D0/140.D0*t17*t25+7.D0/144.D0*t46
     #*t11+7.D0/288.D0*t46*t17+7.D0/576.D0*t46*t24+697.D0/432.D0*t55
      t195 = 4.D0/9.D0*t3-5681.D0/53760.D0*t28+9071.D0/69120.D0*t24*t25-
     #11.D0/70.D0*t18-11699.D0/120960.D0*t20+3029.D0/17280.D0*t22-73.D0/
     #270.D0*t13+4877.D0/8640.D0*t15-697.D0/432.D0*t6+383.D0/480.D0*t8+2
     #47.D0/160.D0*t9+t88+t58-t46*t28/36-t46*t34/24+t46*t36/18-2.D0/9.D0
     #*t46*t121+7.D0/18.D0*t46*t1+t79*t2/36+8.D0/9.D0*t3*t5+t108-t46*t20
     #/12+t46*t22/9+t151+t170+t192

      t0=NF*t195

      sigmaqbarqNF= t0softz + t0softy + t0


C     NF-independent part of the cross section

C     The soft part


      t2 = z2**2
      t5 = z3/3-19.D0/45.D0*t2+26.D0/3.D0*z2-2561.D0/144.D0
      t6 = d1(y)
      t8 = 1-y
      t9 = d1(t8)
      t12 = 1-z
      t13 = d1(t12)
      t16 = DD3(t12)
      t20 = DD0(t8)
      t21 = DD0(y)
      t25 = DD2(t12)
      t28 = -8.D0/9.D0*z2+4.D0/3.D0
      t32 = DD1(t8)
      t36 = DD1(y)
      t40 = DD1(t12)
      t44 = 190.D0/9.D0*z3+22.D0/3.D0*z2-404.D0/27.D0
      t51 = DD2(t8)
      t59 = DD2(y)
      t63 = DD0(t12)
      t65 = (t5*t6+t5*t9)*t13+128.D0/9.D0*(t6+t9)*t16+(-44.D0/3.D0*t6-44
     #.D0/3.D0*t9+64.D0/3.D0*t20*t21)*t25+(t28*t6+t28*t9+(-44.D0/3.D0*t2
     #0+64.D0/3.D0*t32)*t21+64.D0/3.D0*t36*t20)*t40+(t44*t6+t44*t9+((-4.
     #D0/9.D0*z2+2.D0/3.D0)*t20-22.D0/3.D0*t32+16.D0/3.D0*t51)*t21+(-22.
     #D0/3.D0*t20+32.D0/3.D0*t32)*t36+16.D0/3.D0*t59*t20)*t63

      t0softz=t65

      t1 = d1(y)
      t2 = w**2
      t3 = t1*t2
      t6 = 1-y
      t7 = DD1(t6)
      t8 = t2**2
      t9 = t7*t8
      t12 = d1(t6)
      t13 = t8*t2
      t14 = t12*t13
      t17 = t1*t13
      t20 = t12*w
      t21 = Lw**2
      t24 = t1*t8
      t27 = DD2(y)
      t32 = DD0(t6)
      t33 = t32*t13
      t36 = DD0(y)
      t37 = t36*t2
      t40 = t21*Lw
      t43 = t36*w
      t48 = -26119.D0/324.D0*t3*Lw+2.D0/3.D0*t9*Lw+18017.D0/60480.D0*t14
     #*z2+18017.D0/60480.D0*t17*z2-803.D0/9.D0*t20*t21+95.D0/144.D0*t24*
     #z3-40.D0/3.D0*t27-67.D0/3.D0*t12*z2+923.D0/27.D0*t1-t33*z2/288+7.D
     #0/18.D0*t37*z2+4.D0/9.D0*t24*t40+48*t43*t21+1387.D0/18.D0*t20*Lw
      t49 = t32*t2
      t52 = t2*w
      t53 = t1*t52
      t56 = t12*t2
      t59 = DD1(y)
      t60 = t59*t13
      t63 = t8*w
      t64 = t12*t63
      t67 = t8*t52
      t68 = t12*t67
      t73 = t32*t52
      t80 = t32*t8
      t82 = t32*t63
      t84 = 7.D0/18.D0*t49*z2+8.D0/9.D0*t53*t40-112.D0/9.D0*t56*t40+t60*
     #Lw/6-1253.D0/2160.D0*t64*t21+817.D0/6720.D0*t68*t21-41.D0/3.D0*t32
     #+923.D0/27.D0*t12+4225.D0/324.D0*t73-320.D0/9.D0*t12*t40+t27*t67/4
     #8-81.D0/2.D0*t49+24031.D0/12960.D0*t80-22787.D0/129600.D0*t82
      t87 = t32*t67
      t89 = t7*t67
      t91 = t7*t13
      t93 = t59*t63
      t95 = t59*t2
      t97 = t59*t52
      t99 = DD2(t6)
      t102 = t59*t67
      t105 = t1*t63
      t110 = t1*t67
      t117 = -675779.D0/1814400.D0*t33-1059091.D0/3628800.D0*t87+3769.D0
     #/120960.D0*t89-2627.D0/60480.D0*t91-1253.D0/4320.D0*t93+773.D0/36.
     #D0*t95-785.D0/216.D0*t97+t99*t8/6+t102*Lw/12+2.D0/9.D0*t105*t40-25
     #88849.D0/3175200.D0*t14*Lw+15461.D0/120960.D0*t110*z2+t14*t40/9+81
     #7.D0/6720.D0*t110*t21
      t144 = t36*t63
      t147 = -87.D0/4.D0*t3*z2+4.D0/3.D0*t73*t21+2767.D0/54.D0*t3*t21-26
     #93.D0/1080.D0*t24*t21+32*t20*t40+3287.D0/2160.D0*t24*z2+t87*t21/12
     #+95.D0/576.D0*t17*z3-167.D0/6048.D0*t17*t21-2588849.D0/3175200.D0*
     #t17*Lw+2903.D0/4320.D0*t64*z2+2.D0/9.D0*t64*t40+95.D0/1152.D0*t68*
     #z3+t144*t21/3
      t152 = t36*t67
      t159 = t1*w
      t174 = t32*w
      t177 = t36*t52
      t180 = t12*t8
      t183 = 15461.D0/120960.D0*t68*z2-t152*z2/576+t68*t40/18-665.D0/36.
     #D0*t56*z3+95.D0/2.D0*t159*z3+187.D0/6.D0*t159*z2-803.D0/9.D0*t159*
     #t21+32*t159*t40+2903.D0/4320.D0*t105*z2-1253.D0/2160.D0*t105*t21+t
     #91*Lw/6-755.D0/9.D0*t174*Lw-913.D0/108.D0*t177*Lw+3287.D0/2160.D0*
     #t180*z2
      t187 = t7*t63
      t190 = t7*w
      t193 = t7*t52
      t198 = t36*t13
      t214 = -t43*z2+2.D0/3.D0*t80*t21+t187*Lw/3+48*t190*Lw+4.D0/3.D0*t1
     #93*Lw+t93*Lw/3-675779.D0/1814400.D0*t198-t144*z2/144+2575.D0/54.D0
     #*t37*Lw+t89*Lw/12-56.D0/3.D0*t49*t21+277.D0/2880.D0*t87*Lw+277.D0/
     #2880.D0*t152*Lw+2767.D0/54.D0*t56*t21
      t218 = t12*Lw
      t221 = t12*t52
      t245 = -t73*z2/36+20.D0/9.D0*t218*z2+8.D0/9.D0*t221*t40-167.D0/604
     #8.D0*t14*t21-41.D0/3.D0*t36+12*t99*w+t99*t52/3+t99*t63/12+t99*t13/
     #24-14.D0/3.D0*t99*t2+t99*t67/48+t27*t8/6+t27*t52/3+12*t27*w
      t268 = -221.D0/9.D0*t218+10.D0/9.D0*t36*z2+10.D0/9.D0*t32*z2-160.D
     #0/3.D0*t7*Lw+554.D0/9.D0*t36*Lw-475.D0/9.D0*t12*z3-475.D0/9.D0*t1*
     #z3-67.D0/3.D0*t1*z2+578.D0/9.D0*t1*t21-22787.D0/129600.D0*t144+206
     #75561.D0/31104000.D0*t105-2627.D0/60480.D0*t60-81.D0/2.D0*t37+4225
     #.D0/324.D0*t177
      t274 = t7*t2
      t294 = -1253.D0/4320.D0*t187-2357.D0/2160.D0*t9+773.D0/36.D0*t274+
     #29*t59-40.D0/3.D0*t99+29*t7+719.D0/18.D0*t174-3710701.D0/6350400.D
     #0*t110*Lw-665.D0/36.D0*t3*z3+t110*t40/18+95.D0/1152.D0*t110*z3-112
     #.D0/9.D0*t3*t40+95.D0/288.D0*t105*z3+t152*t21/12
      t314 = Lw*z2
      t319 = -913.D0/108.D0*t73*Lw+95.D0/288.D0*t64*z3-229.D0/4320.D0*t3
     #3*Lw+1387.D0/18.D0*t159*Lw-56.D0/3.D0*t274*Lw+t27*t63/12+16939.D0/
     #324.D0*t3-13361549573.D0/341397504000.D0*t110+719.D0/18.D0*t43-14.
     #D0/3.D0*t27*t2-1410073.D0/62208.D0*t221-13266487.D0/15552000.D0*t2
     #4-t17*t314/144-t53*t314/18
      t340 = t59*t8
      t343 = t59*w
      t350 = -2*t159*t314-t105*t314/72-t110*t314/288+7.D0/9.D0*t3*t314-t
     #180*t314/36-t14*t314/144-t221*t314/18-2*t20*t314-13266487.D0/15552
     #000.D0*t180+95.D0/72.D0*t53*z3+2.D0/3.D0*t340*Lw+48*t343*Lw-56.D0/
     #3.D0*t95*Lw+4.D0/3.D0*t97*Lw
      t360 = t36*t8
      t379 = -t174*z2+48*t174*t21-t82*z2/144+t82*t21/3-t87*z2/576-2549.D
     #0/1080.D0*t360*Lw+t17*t40/9+635.D0/216.D0*t53*z2-1009.D0/108.D0*t5
     #3*t21+1895.D0/72.D0*t53*Lw-1186.D0/2025.D0*t64*Lw+10301.D0/3240.D0
     #*t24*Lw-1186.D0/2025.D0*t105*Lw+95.D0/2.D0*t20*z3
      t410 = -87.D0/4.D0*t56*z2+187.D0/6.D0*t20*z2+2575.D0/54.D0*t49*Lw-
     #t80*z2/72+t33*t21/6-2549.D0/1080.D0*t80*Lw-1253.D0/2160.D0*t82*Lw-
     #3710701.D0/6350400.D0*t68*Lw+10301.D0/3240.D0*t180*Lw-26119.D0/324
     #.D0*t56*Lw+95.D0/144.D0*t180*z3-2693.D0/1080.D0*t180*t21+4.D0/9.D0
     #*t180*t40+95.D0/576.D0*t14*z3
      t432 = 95.D0/72.D0*t221*z3+635.D0/216.D0*t221*z2-1253.D0/2160.D0*t
     #144*Lw-1009.D0/108.D0*t221*t21+1895.D0/72.D0*t221*Lw+4509686639.D0
     #/21337344000.D0*t17+24031.D0/12960.D0*t360+16939.D0/324.D0*t56+t27
     #*t13/24-785.D0/216.D0*t193-13361549573.D0/341397504000.D0*t68+4509
     #686639.D0/21337344000.D0*t14-1721.D0/36.D0*t20-320.D0/9.D0*t1*t40
      t459 = -2357.D0/2160.D0*t340-691.D0/18.D0*t343-t64*t314/72-t68*t31
     #4/288+7.D0/9.D0*t56*t314-691.D0/18.D0*t190+2.D0/3.D0*t360*t21-t198
     #*z2/288+t198*t21/6-t177*z2/36+4.D0/3.D0*t177*t21-229.D0/4320.D0*t1
     #98*Lw-755.D0/9.D0*t43*Lw-56.D0/3.D0*t37*t21
      t462 = t1*Lw
      t483 = -t360*z2/72+20.D0/9.D0*t462*z2-t24*t314/36-160.D0/3.D0*t59*
     #Lw-160.D0/3.D0*t36*t21-160.D0/3.D0*t32*t21+554.D0/9.D0*t32*Lw-221.
     #D0/9.D0*t462-1410073.D0/62208.D0*t53+3769.D0/120960.D0*t102+578.D0
     #/9.D0*t12*t21-1059091.D0/3628800.D0*t152+20675561.D0/31104000.D0*t
     #64-1721.D0/36.D0*t159
      t487 = t48+t84+t117+t147+t183+t214+t245+t268+t294+t319+t350+t379+t
     #410+t432+t459+t483

      t0softy = t487

      sigmaqbarqnoNFsoft = t0softz + t0softy

      sigmaqbarqnoNFhard = hardqqnoNF(z,y)

C     The total qbarq part of the cross section

      sigmaqbarq = sigmaqbarqLs
     , + (sigmaqbarqNFf + sigmaqbarqNF
     , + (sigmaqbarqnoNFsoft + sigmaqbarqnoNFhard) )

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     NF independent part of the qbarq cross section, hard part

      double precision function hardqqnoNF(z,y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 u,w,y
      real*8 Lw,Ly,L1my,xi,r,delta
      external dilog

      buff1=0.00000001

      z2=1.644934068D0


      Ly=dlog(y+buff1)
      L1my=dlog(1D0-y+buff1)
      w=1D0-z
      Lw=dlog(w)


      t2 = 1/(y+buff1)
      t3 = Lw*t2
      t7 = w**2
      t8 = t7**2
      t11 = 1D0/(-1D0+y-buff1)
      t12 = Lw*t11
      t17 = t7*w
      t18 = t17*z2
      t19 = y**2
      t22 = t8*t17
      t23 = Lw*t22
      t26 = t8*t7
      t27 = Lw**2
      t28 = t26*t27
      t31 = t19*y
      t34 = t8*w
      t35 = t34*z2
      t36 = t19**2
      t39 = Lw*t34
      t45 = Lw*t26
      t48 = Ly**2
      t49 = t11*t48
      t54 = t26*z2
      t57 = L1my**2
      t58 = t2*t57
      t63 = Lw*t8
      t70 = -8891.D0/1728.D0*t17+11082479.D0/5644800.D0*t26+133.D0/108.D
     #0*t45*t36-100.D0/9.D0*t49*w-1303.D0/432.D0*t45*y-10.D0/9.D0*t54*t3
     #1+t58*t17/3-175.D0/12.D0*t39*t31+689.D0/36.D0*t63*t19+8.D0/9.D0*t5
     #4*y-8*t18*y
      t74 = t22*t27
      t75 = t36*t19
      t84 = t34*t27
      t88 = t22*z2
      t91 = Lw*t17
      t96 = t2*L1my
      t99 = 11.D0/18.D0*t49*t8-35.D0/18.D0*t74*t75+160.D0/3.D0*t12*Ly-14
     #.D0/3.D0*t58*t7+t58*t8/6-20.D0/9.D0*t84*t36+3547.D0/108.D0*t7-71.D
     #0/36.D0*t88*t31+67.D0/3.D0*t91*y+28.D0/9.D0*Lw*w-1253.D0/4320.D0*t
     #96*t34
      t103 = t11*Ly
      t104 = t26*t75
      t107 = t8*y
      t110 = t34*t36
      t115 = t36*y
      t120 = t8*z2
      t123 = t17*t27
      t128 = -10.D0/9.D0*t28*t36+59.D0/4.D0*t63-977.D0/945.D0*t103*t104-
     #4.D0/3.D0*t49*t107+5.D0/3.D0*t49*t110+3025.D0/216.D0*t103*t17+35.D
     #0/6.D0*t74*t115-959.D0/36.D0*t103*t7-4*t120*t19+40.D0/9.D0*t123*t1
     #9+10.D0/3.D0*t49*t7
      t133 = t22*t75
      t168 = t8*t27
      t171 = -t54*t19/3+t58*t22/48+16.D0/9.D0*t35*y+20.D0/9.D0*t28*t31-1
     #100925529.D0/406425600.D0*t133-422387.D0/518400.D0*t34-1381.D0/504
     #0.D0*t23-83.D0/9.D0*t91+11.D0/72.D0*t49*t26+5.D0/9.D0*t54*t36+20.D
     #0/9.D0*t168*y
      t183 = Ly*t7
      t187 = Ly*t22
      t194 = Ly*t17
      t200 = 40.D0/9.D0*t84*t31-20.D0/9.D0*t35*t31+4.D0/9.D0*t88*y-20.D0
     #/9.D0*t168*t19-41.D0/6.D0*t74*t36+16.D0/3.D0*t12*t183*y+70.D0/9.D0
     #*t12*t187*t75+16.D0/3.D0*t7*t27+111203627.D0/58060800.D0*t22+80.D0
     #/9.D0*t12*t194*y-73.D0/9.D0*Lw*t7
      t219 = t34*t115
      t223 = -34.D0/9.D0*t84*t19+t58*t26/24+14.D0/9.D0*t84*y-11713.D0/21
     #60.D0*t103*t8+2993.D0/840.D0*t23*t75-67.D0/3.D0*t91*t19+12*t58*w+t
     #58*t34/12-35.D0/12.D0*t88*t115-5.D0/9.D0*t49*t219+1877.D0/2160.D0*
     #t45
      t234 = t26*t36
      t237 = t26*t31
      t240 = t26*t115
      t243 = t26*y
      t247 = t26*t19
      t257 = t36*t31
      t258 = t22*t257
      t261 = t7*t19
      t264 = L1my*t17
      t265 = t264*t19
      t270 = L1my*t34
      t271 = t270*t19
      t274 = t270*y
      t277 = L1my*t22
      t278 = t277*t19
      t281 = 7.D0/36.D0*t58*t247-400.D0/9.D0*t12*Ly*w+10.D0/9.D0*t58*t11
     #0-5.D0/9.D0*t58*t219-35.D0/72.D0*t58*t258+242.D0/27.D0*t96*t261-40
     #.D0/9.D0*t3*t265-7.D0/9.D0*t58*t107+14.D0/9.D0*t3*t271-14.D0/9.D0*
     #t3*t274+7.D0/18.D0*t3*t278
      t283 = t277*t115
      t286 = t277*t257
      t289 = t270*t115
      t292 = t270*t36
      t298 = L1my*t8
      t299 = t298*t31
      t302 = t298*y
      t305 = t277*t31
      t308 = t22*t36
      t312 = -41.D0/6.D0*t3*t283-35.D0/18.D0*t3*t286-20.D0/9.D0*t3*t289+
     #40.D0/9.D0*t3*t292+4334881.D0/806400.D0*t247-689.D0/36.D0*t63*y-20
     #.D0/9.D0*t3*t299-28.D0/9.D0*t3*t302-25.D0/18.D0*t3*t305-1713636389
     #.D0/406425600.D0*t308-13.D0/9.D0*t49*t17
      t319 = t277*y
      t325 = L1my*t26
      t326 = t325*t36
      t329 = t325*t31
      t332 = t325*y
      t335 = t325*t115
      t338 = t277*t36
      t341 = 11.D0/36.D0*t49*t34+11.D0/144.D0*t49*t22-40.D0/9.D0*t123*y-
     #7.D0/18.D0*t3*t319-35.D0/72.D0*t49*t258-14.D0/9.D0*t84+20.D0/9.D0*
     #t3*t326-17.D0/9.D0*t3*t329-7.D0/9.D0*t3*t332-10.D0/9.D0*t3*t335+71
     #.D0/18.D0*t3*t338
      t344 = t325*t19
      t347 = Ly*t34
      t352 = L1my*y*w
      t355 = t277*t75
      t358 = t270*t31
      t361 = t298*t19
      t364 = t264*y
      t367 = t264*t31
      t373 = Ly*t8
      t387 = L1my*t7
      t388 = t387*y
      t394 = Ly*t26
      t401 = t34*t31
      t413 = 16.D0/9.D0*t12*t187*t19-16.D0/3.D0*t12*t373*y-7.D0/9.D0*t12
     #*t187*y+16.D0/3.D0*t3*t388-16.D0/3.D0*t12*t187*t31-14.D0/9.D0*t12*
     #t394*y+97.D0/9.D0*t12*t187*t36-17.D0/18.D0*t58*t401+40.D0/9.D0*t12
     #*t373*t19-10.D0/9.D0*t12*t394*t115-37.D0/9.D0*t12*t394*t31
      t430 = t22*t31
      t432 = t34*y
      t435 = t22*t115
      t438 = t22*y
      t441 = t22*t19
      t444 = Ly*t2
      t445 = t444*t11
      t448 = 10.D0/3.D0*t12*t394*t36+40.D0/9.D0*t12*t194*t31-80.D0/9.D0*
     #t12*t194*t19-28.D0/9.D0*t12*t347*y-74.D0/9.D0*t12*t347*t31-2967649
     #81.D0/58060800.D0*t430-7.D0/18.D0*t58*t432-41.D0/24.D0*t58*t435-7.
     #D0/72.D0*t58*t438+7.D0/72.D0*t58*t441-8.D0/3.D0*t445*t388
      t449 = t34*t19
      t452 = t17*y
      t455 = t17*t31
      t458 = y*w
      t465 = t11*L1my
      t480 = 7.D0/18.D0*t58*t449+10.D0/9.D0*t58*t452+10.D0/9.D0*t58*t455
     #-8.D0/9.D0*t58*t458+71.D0/72.D0*t58*t308-25.D0/72.D0*t58*t430-24*t
     #444*t465*w+16.D0/3.D0*t12*t347*t19+8.D0/3.D0*t12*t394*t19+40.D0/9.
     #D0*t445*t265-2.D0/3.D0*t444*t465*t17
      t497 = L1my*w
      t521 = t8*t31
      t526 = t7*y
      t529 = t17*t19
      t534 = t8*t36
      t539 = 8.D0/3.D0*t445*t387*t19-20.D0/9.D0*t445*t364-t444*t465*t34/
     #6-t444*t465*t26/12-5.D0/9.D0*t58*t521+35.D0/24.D0*t58*t133+4.D0/3.
     #D0*t58*t526-10.D0/9.D0*t58*t529-242.D0/27.D0*t103*t261+179.D0/135.
     #D0*t103*t534+80.D0/3.D0*t444*t465
      t549 = t8*t19
      t564 = -59.D0/2.D0*t103*t452+169.D0/18.D0*t103*t526-30893.D0/3780.
     #D0*t103*t133+14.D0/9.D0*t103*t458+5.D0/9.D0*t58*t549-t444*t465*t22
     #/24+5.D0/3.D0*t445*t335-3120769.D0/635040.D0*t237+8.D0/3.D0*t445*t
     #358+7.D0/9.D0*t445*t274-14.D0/9.D0*t445*t271
      t578 = t36**2
      t584 = -40.D0/3.D0*t58-37.D0/18.D0*t445*t326-7.D0/18.D0*t445*t278+
     #7.D0/36.D0*t445*t319+20.D0/9.D0*t445*t299+14.D0/9.D0*t445*t302+8.D
     #0/9.D0*t445*t305-35.D0/36.D0*t445*t277*t578+293.D0/9.D0*t96-t54/4-
     #t120
      t629 = -37.D0/9.D0*t445*t292+10.D0/3.D0*t445*t289-8.D0/3.D0*t7*z2-
     #100169.D0/5184.D0*t452+6191.D0/864.D0*t103*t432+3433.D0/216.D0*t10
     #3*t401-1633.D0/108.D0*t103*t549+4573.D0/1080.D0*t103*t240+5651.D0/
     #864.D0*t103*t237-7279.D0/1080.D0*t103*t234+100169.D0/5184.D0*t529
      t652 = 5267.D0/8640.D0*t103*t438+1537.D0/108.D0*t103*t107-23689.D0
     #/2160.D0*t103*t110+2993.D0/1680.D0*t103*t258+175.D0/48.D0*t103*t21
     #9+6295.D0/432.D0*t103*t435+1351.D0/216.D0*t103*t521-8539.D0/4320.D
     #0*t103*t441+7093553.D0/259200.D0*t107-67.D0/6.D0*t103*t455+977.D0/
     #945.D0*t96*t104
      t676 = -179.D0/135.D0*t96*t534+850.D0/27.D0*t103*t529+32321.D0/180
     #0.D0*t401-35.D0/18.D0*t12*t187*t257-20.D0/9.D0*t12*t347*t115+16.D0
     #/9.D0*w*z2+7.D0/18.D0*t74*y+71.D0/18.D0*t74*t31+791.D0/18.D0*t103*
     #w-6241.D0/120960.D0*t103*t22-33479.D0/60480.D0*t103*t26
      t721 = -67.D0/6.D0*t96*t455+109.D0/54.D0*t96*t529-461.D0/54.D0*t96
     #*t526-10421.D0/2160.D0*t96*t449+4069.D0/3780.D0*t96*t234+37697.D0/
     #30240.D0*t96*t237+3083.D0/10080.D0*t96*t243-2977.D0/1512.D0*t96*t2
     #40+325.D0/3024.D0*t96*t308-148266743.D0/50803200.D0*t243-177367561
     #3.D0/406425600.D0*t438
      t743 = -34193.D0/30240.D0*t96*t247-4811.D0/12096.D0*t96*t430-197.D
     #0/756.D0*t96*t438+4457.D0/1512.D0*t96*t435+2141.D0/20160.D0*t96*t4
     #41+2993.D0/1680.D0*t96*t258+175.D0/48.D0*t96*t219-7843.D0/1080.D0*
     #t96*t110+967.D0/120.D0*t96*t107+3362815811.D0/406425600.D0*t441+11
     #00925529.D0/135475200.D0*t435
      t761 = 9433.D0/4320.D0*t96*t432-7842317.D0/518400.D0*t449+3188093.
     #D0/518400.D0*t432+3120769.D0/1270080.D0*t234-7093553.D0/259200.D0*
     #t549-32321.D0/3600.D0*t110-32.D0/9.D0*w*t27-5.D0/18.D0*t49*t240+5.
     #D0/6.D0*t49*t234+10.D0/9.D0*t49*t455-20.D0/9.D0*t49*t529
      t807 = t3*t325/6+t3*t277/12+48*t3*t497+1387.D0/120.D0*t96*t521+40.
     #D0/3.D0*t12*t183-52.D0/9.D0*t12*t194+22.D0/9.D0*t12*t373+2.D0/3.D0
     #*t3*t298-7.D0/18.D0*t49*t243-7.D0/36.D0*t49*t438-697.D0/360.D0*t10
     #3*t34
      t830 = -91.D0/2.D0*t96*w+1573.D0/24192.D0*t96*t22-193.D0/20160.D0*
     #t96*t26+1835.D0/432.D0*t45*t19-133.D0/54.D0*t45*t31+8413.D0/756.D0
     #*t23*t36-2993.D0/280.D0*t23*t115-15629.D0/30240.D0*t23*y-2741.D0/2
     #160.D0*t96*t8-347.D0/72.D0*t96*t17-293.D0/9.D0*t103
      t853 = 4.D0/3.D0*t49*t449-19.D0/6.D0*t49*t435-4.D0/3.D0*t49*t430+4
     #.D0/9.D0*t49*t441+97.D0/36.D0*t49*t308+2.D0/3.D0*t49*t247-5.D0/9.D
     #0*t49*t521+2831.D0/108.D0*t96*t7+t3*t270/3+4.D0/3.D0*t3*t264+11.D0
     #/18.D0*t12*t394
      s1 = t223+11.D0/36.D0*t12*t187-32.D0/9.D0*t12*Ly*y*w+5.D0/9.D0*t58
     #*t234-17.D0/36.D0*t58*t237+4.D0/3.D0*t49*t526+t281-7.D0/36.D0*t58*
     #t243-5.D0/18.D0*t58*t240+t312+t341+7.D0/9.D0*t3*t344-32.D0/9.D0*t3
     #*t352+35.D0/6.D0*t3*t355-34.D0/9.D0*t3*t358+20.D0/9.D0*t3*t361+40.
     #D0/9.D0*t3*t364+40.D0/9.D0*t3*t367+t413+t480+7.D0/18.D0*t445*t332+
     #t448-7.D0/9.D0*t445*t344-8.D0/3.D0*t445*t338+16.D0/9.D0*t445*t352-
     #8.D0/3.D0*t445*t361-19.D0/3.D0*t445*t355+t539-40.D0/9.D0*t445*t367
     #+t564+t629+35.D0/9.D0*t445*t286+4.D0/3.D0*t445*t329+97.D0/18.D0*t4
     #45*t283+t584+t652+t676-611.D0/48.D0*t103*t308+14.D0/9.D0*t96*t458-
     #64987.D0/15120.D0*t96*t133+509.D0/60.D0*t96*t401-12527.D0/1080.D0*
     #t96*t549-t96*t452/27-5837.D0/432.D0*t103*t449+t721-2111.D0/432.D0*
     #t103*t247+1763.D0/720.D0*t103*t243+10199.D0/1728.D0*t103*t430+40.D
     #0/9.D0*t123+t743+t761-281.D0/18.D0*w
      t858 = s1+20.D0/9.D0*t49*t452+10.D0/9.D0*t49*t549-37.D0/18.D0*t49*
     #t401-7.D0/9.D0*t49*t432-37.D0/36.D0*t49*t237-8.D0/9.D0*t49*t458+35
     #.D0/18.D0*t49*t133+11.D0/9.D0*t12*t347-56.D0/3.D0*t3*t387+t807+t83
     #0-160.D0/3.D0*t3*L1my+t853-5.D0/9.D0*t445*t325*t75+20.D0/9.D0*t445
     #*t264*t36+20.D0/3.D0*t12*t347*t36-20.D0/9.D0*t12*t373*t31-38.D0/3.
     #D0*t12*t187*t115-t444*t465*t8/3-16.D0/9.D0*t445*t497*t19-10.D0/9.D
     #0*t445*t270*t75+28.D0/3.D0*t444*t465*t7+10.D0/9.D0*t35*t36-6715.D0
     #/1512.D0*t23*t31+1927.D0/2016.D0*t23*t19+8*t18*t19+26.D0/9.D0*t18-
     #4303849.D0/259200.D0*t8-t35/2-17.D0/9.D0*t28*t19-7.D0/9.D0*t28+40.
     #D0/3.D0*t49+635.D0/144.D0*t39-703.D0/72.D0*t39*y+t70-t88/8-7.D0/18
     #.D0*t74+t99+t128+t88*t19/18+41.D0/12.D0*t88*t36-2.D0/3.D0*t35*t19+
     #175.D0/24.D0*t39*t36+307.D0/18.D0*t39*t19-25.D0/18.D0*t74*t19+t171
     #-10.D0/9.D0*t445*t298*t36-28.D0/9.D0*t168+7.D0/9.D0*t28*y+t200+4*t
     #120*y+35.D0/36.D0*t133*z2


      hardqqnoNF=t858

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      double precision function sigmagg(z,y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 u, Ls, y, w
      real*8 Ldelta,Lxi,L1mxi,xi,r,delta, Ly, L1my, Lw
      external dilog

      buff1=0.000000001

      z2=1.644934068D0

C      r=dsqrt((z+y*(1D0-z))/(1D0-y*(1D0-z)))
C      t=dsqrt(z)
C      delta=1D0-t
C      xi=y*(1D0+r**2)*t/(r+t)
C      Ldelta=dlog(delta+buff1)
C      Lxi=dlog(xi+buff1)
C      L1mxi=dlog(1D0-xi+buff1)

      Ls=dlog(mF**2/q**2)
      w=1D0-z
      Lw=dlog(w)
      Ly=dlog(y+buff1)
      L1my=dlog(1D0-y+buff1)


C     Scale dependence

C     Expansion through w^7

      t1 = 2*L1my
      t2 = 2*Ly
      t3 = 4*Lw
      t10 = w**2
      t13 = y**2
      t14 = t13*Lw
      t16 = L1my*y
      t18 = L1my*t13
      t20 = Ly*y
      t22 = Ly*t13
      t27 = Ls*t13
      t29 = Ls*y
      t32 = y*Lw
      t34 = 4*t14-2*t16+2*t18-2*t20+2*t22-7+4*Ly+4*L1my+8*Lw+t27+2*Ls-t2
     #9-2*t13+2*y-4*t32
      t36 = t10*w
      t53 = 24*t14-12*t16+12*t18-12*t20+12*t22-65+24*Ly+24*L1my+48*Lw+6*
     #t27+12*Ls-6*t29-21*t13+21*y-24*t32
      t55 = t10**2
      t58 = t13*y
      t59 = L1my*t58
      t61 = t13**2
      t62 = L1my*t61
      t64 = t58*Lw
      t66 = t61*Lw
      t68 = Ly*t58
      t70 = Ly*t61
      t72 = Ls*t61
      t74 = Ls*t58
      t84 = -336*t59+168*t62-672*t64+336*t66-336*t68+168*t70+84*t72-168*
     #t74+336*t14+168*t18+168*t22-35+84*t27-282*t61-252*t13+564*t58-30*y
      t105 = 180*t27+180*t72-360*t74+360*t22-720*t68+360*t70+360*t18-720
     #*t59+360*t62+720*t14-1440*t64+720*t66-111+995*t61+1090*t13-1990*t5
     #8-95*y
      t121 = t61*y
      t124 = -112+13440*t70+1200*t22+600*t27+1200*t18+510*t13+2400*t14-1
     #57*y-12960*t64+26880*t66-6480*t68+6720*t72-12240*Ly*t121
      t125 = t61*t13
      t147 = 4080*Ly*t125+2040*Ls*t125+4080*L1my*t125-6120*Ls*t121+6170*
     #t58-20275*t61-3240*t74-12240*L1my*t121+20628*t121-6876*t125+8160*t
     #125*Lw-6480*t59+13440*t62-24480*t121*Lw
      t153 = Ls*(t1+t2+Ls+t3)*w/4-3.D0/4.D0*Ls*(-2+t2+t1+t3+Ls)*t10+Ls*t
     #34*t36/2-Ls*t53*t55/24+Ls*t84*t55*w/96+Ls*t105*t55*t10/960+Ls*(t12
     #4+t147)*t55*t36/1920

      sigmaggLs=t153

C     Ls independent piece, accurate through w^7

      t2 = L1my*Lw
      t3 = Ly**2
      t5 = L1my**2
      t7 = Ly*Lw
      t8 = Ly*L1my
      t10 = Lw**2
      t23 = 3.D0/2.D0*z2-3*t2-3*t7-3*t10-3.D0/4.D0-3.D0/4.D0*t5-3.D0/4.D
     #0*t3+3*Lw+3.D0/2.D0*Ly+3.D0/2.D0*L1my-3.D0/2.D0*t8
      t24 = w**2
      t34 = y*t10
      t36 = y**2
      t39 = z2*t36
      t40 = Ly*y
      t41 = 105.D0/32.D0-7.D0/2.D0*L1my+2*t8+4*t10+4*t7+t5+t3-7*Lw-7.D0/
     #2.D0*Ly-2*z2+4*t2-2*t34+7.D0/4.D0*t36-7.D0/4.D0*y-t39+t40
      t42 = t3*y
      t44 = Ly*t36
      t45 = L1my*t36
      t46 = t5*y
      t48 = t36*Lw
      t50 = y*Lw
      t52 = t36*t10
      t54 = L1my*y
      t55 = t3*t36
      t57 = z2*y
      t58 = t8*t36
      t59 = t45*Lw
      t61 = t54*Lw
      t63 = t8*y
      t64 = t44*Lw
      t66 = t40*Lw
      t68 = t5*t36
      t70 = -t42/2-t44-t45-t46/2-2*t48+2*t50+2*t52+t54+t55/2+t57+t58+2*t
     #59-2*t61-t63+2*t64-2*t66+t68/2
      t72 = t24*w
      t87 = 65.D0/24.D0*L1my-t8-2*t10-2*t7-t5/2-t3/2+65.D0/12.D0*Lw+65.D
     #0/24.D0*Ly+z2-2*t2+t34-119.D0/96.D0*t36+119.D0/96.D0*y+t39/2-7.D0/
     #8.D0*t40+t42/4
      t99 = -1933.D0/576.D0+7.D0/8.D0*t44+7.D0/8.D0*t45+t46/4+7.D0/4.D0*
     #t48-7.D0/4.D0*t50-t52-7.D0/8.D0*t54-t55/4-t57/2-t58/2-t59+t61+t63/
     #2-t64+t66-t68/4
      t101 = t24**2
      t108 = t36*y
      t109 = L1my*t108
      t110 = t109*Lw
      t112 = t36**2
      t113 = t112*L1my
      t114 = t113*Lw
      t129 = -35.D0/96.D0*L1my-35.D0/48.D0*Lw-35.D0/96.D0*Ly+3619.D0/768
     #.D0*t36-23.D0/384.D0*y-7*t110+7.D0/2.D0*t114-7.D0/4.D0*t39-5.D0/16
     #.D0*t40-21.D0/8.D0*t44-21.D0/8.D0*t45-21.D0/4.D0*t48-5.D0/8.D0*t50
     #+7.D0/2.D0*t52-5.D0/16.D0*t54+7.D0/8.D0*t55+7.D0/4.D0*t58+7.D0/2.D
     #0*t59+7.D0/2.D0*t64+7.D0/8.D0*t68
      t130 = z2*t112
      t132 = t3*t108
      t134 = t5*t108
      t138 = t5*t112
      t140 = t112*Lw
      t142 = t112*t10
      t144 = t108*Lw
      t147 = t3*t112
      t149 = t8*t112
      t151 = Ly*t112
      t154 = z2*t108
      t157 = Ly*t108
      t159 = t8*t108
      t161 = t157*Lw
      t163 = t151*Lw
      t165 = t108*t10
      t167 = 7.D0/8.D0*t147+7.D0/4.D0*t149-47.D0/16.D0*t151+1191.D0/256.
     #D0*t112+7.D0/2.D0*t154-1191.D0/128.D0*t108+47.D0/8.D0*t157-7.D0/2.
     #D0*t159-7*t161+7.D0/2.D0*t163-7*t165
      t192 = -37.D0/320.D0*L1my-37.D0/160.D0*Lw-37.D0/320.D0*Ly-6925.D0/
     #4608.D0*t36-211.D0/4608.D0*y-3.D0/2.D0*t110+3.D0/4.D0*t114-3.D0/8.
     #D0*t39-19.D0/192.D0*t40+109.D0/96.D0*t44+109.D0/96.D0*t45+109.D0/4
     #8.D0*t48-19.D0/96.D0*t50+3.D0/4.D0*t52-19.D0/192.D0*t54+3.D0/16.D0
     #*t55+3.D0/8.D0*t58+3.D0/4.D0*t59+3.D0/4.D0*t64+3.D0/16.D0*t68
      t214 = 3.D0/16.D0*t147+3.D0/8.D0*t149+199.D0/192.D0*t151-223.D0/14
     #4.D0*t112+3.D0/4.D0*t154+223.D0/72.D0*t108-199.D0/96.D0*t157-3.D0/
     #4.D0*t159-3.D0/2.D0*t161+3.D0/4.D0*t163-3.D0/2.D0*t165
      t234 = -7.D0/120.D0*L1my-7.D0/60.D0*Lw-7.D0/120.D0*Ly+47.D0/60.D0*
     #t36-8537.D0/230400.D0*y-27.D0/4.D0*t110+14*t114-5.D0/8.D0*t39-157.
     #D0/1920.D0*t40+17.D0/64.D0*t44+17.D0/64.D0*t45+17.D0/32.D0*t48-157
     #.D0/960.D0*t50+5.D0/4.D0*t52-157.D0/1920.D0*t54
      t235 = t112*t36
      t241 = t112*y
      t255 = 17.D0/8.D0*t8*t235+5.D0/16.D0*t55+5.D0/8.D0*t58+5.D0/4.D0*t
     #59-51.D0/8.D0*t8*t241+5.D0/4.D0*t64+5.D0/16.D0*t68-7*t130-27.D0/16
     #.D0*t132-27.D0/16.D0*t134+617.D0/192.D0*t109-4055.D0/384.D0*t113+7
     #.D0/2.D0*t138-4055.D0/192.D0*t140+14*t142+617.D0/96.D0*t144
      t268 = t235*L1my
      t271 = Ly*t241
      t276 = 4121.D0/115200.D0+7.D0/2.D0*t147+7*t149-4055.D0/384.D0*t151
     #+892751.D0/46080.D0*t112+27.D0/8.D0*t154-177449.D0/23040.D0*t108+6
     #17.D0/192.D0*t157-27.D0/8.D0*t159-27.D0/4.D0*t161+14*t163-27.D0/4.
     #D0*t165+17.D0/4.D0*t268*Lw-51.D0/4.D0*t271*Lw+39739.D0/6400.D0*t23
     #5-119217.D0/6400.D0*t241
      t289 = Ly*t235
      t297 = L1my*t241
      t307 = 17.D0/16.D0*t3*t235+1719.D0/160.D0*t271-17.D0/8.D0*z2*t235-
     #51.D0/16.D0*t3*t241-51.D0/16.D0*t5*t241-573.D0/160.D0*t268+17.D0/1
     #6.D0*t5*t235-573.D0/160.D0*t289+17.D0/4.D0*t235*t10+1719.D0/80.D0*
     #t241*Lw+51.D0/8.D0*z2*t241+1719.D0/160.D0*t297-51.D0/4.D0*t297*Lw+
     #17.D0/4.D0*t289*Lw-51.D0/4.D0*t241*t10-573.D0/80.D0*t235*Lw
      t312 = (-z2/2+t2+t3/4+t5/4+t7+t8/2+t10)*w+t23*t24+(t41+t70)*t72+(t
     #87+t99)*t101+(t129+3965.D0/4608.D0-7.D0/4.D0*t130-7.D0/4.D0*t132-7
     #.D0/4.D0*t134+47.D0/8.D0*t109-47.D0/16.D0*t113+7.D0/8.D0*t138-47.D
     #0/8.D0*t140+7.D0/2.D0*t142+47.D0/4.D0*t144+t167)*t101*w+(t192+733.
     #D0/5120.D0-3.D0/8.D0*t130-3.D0/8.D0*t132-3.D0/8.D0*t134-199.D0/96.
     #D0*t109+199.D0/192.D0*t113+3.D0/16.D0*t138+199.D0/96.D0*t140+3.D0/
     #4.D0*t142-199.D0/48.D0*t144+t214)*t101*t24+(t234+t255+t276+t307)*t
     #101*t72


      sigmagg=t312 + sigmaggLs

      end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      double precision function sigmaqg(z,y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 u,w
      real*8 Ldelta,Lxi,L1mxi,xi,r,delta,Lz,Ls,Lw,Ly,L1my
      external dilog,d1,DD0,DD1,DD2

      buff1=0.00000000001D0

      z2=1.644934068D0
      z3=1.20205690315959D0

      r=dsqrt((z+y*(1.D0-z))/(1.D0-y*(1.D0-z)))
      t=dsqrt(z)

      delta=1D0-t
      xi=y*(1D0+r**2)*t/(r+t)
      Ldelta=dlog(delta+buff1)
      Lxi=dlog(xi+buff1)
      L1mxi=dlog(1D0-xi+buff1)
      Lz=dlog(1D0-z+buff1)
      Ls=dlog(mF**2/q**2)
      w=1D0-z
      Ly=dlog(y+buff1)
      L1my = dlog(1D0-y+buff1)
      Lw=dlog(w)


      t1 = Ls**2
      t2 = DD0(y)
      t3 = t1*t2
      t5 = d1(y)
      t6 = t5*t1
      t8 = Ls*t2
      t9 = w**2
      t10 = t9**2
      t11 = t10*w
      t14 = t5*Ls
      t17 = t9*w
      t18 = t10*t17
      t21 = Lz**2
      t25 = t10*t9
      t35 = DD1(y)
      t36 = Ls*t35
      t39 = 2.D0/3.D0*t3+3.D0/4.D0*t6-4201.D0/2880.D0*t8*t11+1103729.D0/
     #691200.D0*t14*t11-10891.D0/80640.D0*t8*t18+105.D0/64.D0*t14*t21*t1
     #0+105.D0/256.D0*t14*t21*t25+105.D0/128.D0*t14*t21*t11-147.D0/8.D0*
     #t14*t21*w+2*t36-13.D0/12.D0*t14
      t50 = t10**2
      t63 = -947.D0/576.D0*t6*t10-9.D0/4.D0*t3*t17+5.D0/16.D0*t36*t11-10
     #67.D0/23040.D0*t6*t18-587.D0/1152.D0*t6*t11+5.D0/384.D0*t3*t50-641
     #.D0/3840.D0*t6*t25-7.D0/3.D0*t3*w+5.D0/24.D0*t3*t10+5.D0/96.D0*t3*
     #t25-t6*w/8
      t86 = 21.D0/4.D0*t14*t21+t14*Lz-t14*z2/12+203645.D0/13824.D0*t14*t
     #10-492649.D0/1382400.D0*t14*t25+17.D0/24.D0*t14*w-429691.D0/774144
     #.D0*t14*t18-47123.D0/1728.D0*t14*t17+1307.D0/96.D0*t14*t9-15690576
     #1.D0/361267200.D0*t14*t50-1405.D0/288.D0*t8*t10
      t107 = Lz*Ls
      t112 = -2741.D0/5760.D0*t8*t25+21.D0/4.D0*t8*w+2749.D0/144.D0*t8*t
     #17-433.D0/24.D0*t8*t9-27.D0/4.D0*t36*t17-649.D0/40320.D0*t8*t50+15
     #59.D0/288.D0*t6*t17-15.D0/3584.D0*t6*t50-179.D0/48.D0*t6*t9+7.D0/2
     #.D0*t3*t9+29.D0/6.D0*t107*t2+5.D0/8.D0*t36*t10
      t129 = Lz*t5
      t141 = 5.D0/32.D0*t36*t25-7*t36*w+5.D0/64.D0*t36*t18+21.D0/2.D0*t3
     #6*t9+5.D0/128.D0*t36*t50+5.D0/48.D0*t3*t11+5.D0/192.D0*t3*t18+7.D0
     #/4.D0*t129*t1+105.D0/512.D0*t14*t21*t18-567.D0/32.D0*t14*t21*t17+4
     #41.D0/16.D0*t14*t21*t9
      t173 = 105.D0/1024.D0*t14*t21*t50+35.D0/512.D0*t129*t1*t18+35.D0/2
     #56.D0*t129*t1*t25-1765.D0/144.D0*t129*Ls*t10+t8/2-577.D0/576.D0*t1
     #29*Ls*t25-4849.D0/1440.D0*t129*Ls*t11+12*t129*Ls*w-9307.D0/40320.D
     #0*t129*Ls*t18+3163.D0/72.D0*t129*Ls*t17-487.D0/12.D0*t129*Ls*t9
      t208 = 359.D0/20160.D0*t129*Ls*t50+145.D0/96.D0*t107*t2*t10+145.D0
     #/384.D0*t107*t2*t25+145.D0/192.D0*t107*t2*t11-203.D0/12.D0*t107*t2
     #*w+145.D0/768.D0*t107*t2*t18-261.D0/16.D0*t107*t2*t17+203.D0/8.D0*
     #t107*t2*t9+145.D0/1536.D0*t107*t2*t50+35.D0/128.D0*t129*t1*t11-189
     #.D0/32.D0*t129*t1*t17
      t245 = 35.D0/1024.D0*t129*t1*t50+147.D0/16.D0*t129*t1*t9-49.D0/8.D
     #0*t129*t1*w+35.D0/64.D0*t129*t1*t10+9.D0/32.D0*t14*t17*z2-5.D0/307
     #2.D0*t14*t50*z2-7.D0/16.D0*t14*t9*z2-5.D0/768.D0*t14*t25*z2-5.D0/1
     #92.D0*t14*t10*z2-5.D0/384.D0*t14*t11*z2+7.D0/24.D0*t14*w*z2-5.D0/1
     #536.D0*t14*t18*z2
      t248 = t39+t63+t86+t112+t141+t173+t208+t245


      tsoftyLs = t248


      t2 = 1/(y+buff1)
      t3 = Ls*t2
      t6 = w**2
      t7 = t6**2
      t8 = t7*t6
      t9 = t8*L1my
      t12 = t6*w
      t13 = t7*t12
      t14 = y**2
      t15 = t14**2
      t16 = t13*t15
      t19 = t14*y
      t20 = t8*t19
      t24 = t8*t15
      t28 = t7*w
      t29 = t28*L1my
      t33 = t28*Ly
      t49 = 17.D0/6.D0*t3*L1my+85.D0/384.D0*t3*t9+24305.D0/3072.D0*t3*t1
     #6-19.D0/4.D0*t3*t20*Lw+57.D0/4.D0*t3*t24*Lw-37.D0/24.D0*t3*t29*y+1
     #1.D0/2.D0*t3*t33*t19-51.D0/8.D0*t3*t33*t15+14.D0/3.D0*t3*t29*t19+5
     #.D0/24.D0*t3*t29*t14-143.D0/24.D0*t3*t29*t15
      t50 = Ls**2
      t51 = t50*t2
      t52 = t7**2
      t53 = t15*t14
      t54 = t52*t53
      t57 = t15**2
      t58 = t52*t57
      t61 = t52*t15
      t66 = t13*t53
      t69 = t52*y
      t72 = t15*t19
      t73 = t52*t72
      t76 = t7*Ly
      t80 = t7*y
      t84 = t7*t14
      t88 = t7*L1my
      t92 = t15*y
      t96 = 385.D0/96.D0*t51*t54+547.D0/384.D0*t51*t58+80779.D0/18432.D0
     #*t3*t61-7161.D0/512.D0*t3*t24-301.D0/96.D0*t51*t66+49261.D0/322560
     #.D0*t3*t69-385.D0/96.D0*t51*t73-t3*t76*y/2+13.D0/12.D0*t3*t80*Lw+3
     #.D0/4.D0*t3*t84*Lw+17.D0/12.D0*t3*t88*t15+29.D0/12.D0*t3*t29*t92
      t107 = t8*t14
      t111 = t8*t53
      t115 = t7*t19
      t119 = t8*t92
      t129 = t28*t92
      t137 = 29.D0/12.D0*t3*t33*t92-5.D0/6.D0*t3*t33*t14-11.D0/12.D0*t3*
     #t33*y+65.D0/48.D0*t3*t107*Lw+23.D0/4.D0*t3*t111*Lw-19.D0/3.D0*t3*t
     #115*Lw-187.D0/12.D0*t3*t119*Lw-3.D0/2.D0*t3*t88*t19-29.D0/6.D0*t3*
     #t76*t19+29.D0/6.D0*t3*t129*Lw+10.D0/3.D0*t3*w*Ly*y
      t141 = t7*t15
      t151 = t12*L1my
      t155 = t12*Ly
      t162 = t12*t14
      t166 = t12*t19
      t175 = t28*t19
      t179 = 9.D0/4.D0*t3*t76*t15+11.D0/3.D0*t3*t141*Lw+8.D0/3.D0*t3*t76
     #*t14-23.D0/12.D0*t3*t88*t14-3*t3*t151*t14-11.D0/2.D0*t3*t155*t14+2
     #1.D0/2.D0*t3*t151*y-17.D0/2.D0*t3*t162*Lw+t3*t166*Lw/3-2.D0/3.D0*t
     #3*t151*t19+t3*t155*t19+61.D0/6.D0*t3*t175*Lw
      t185 = t13*t72
      t188 = t28*t15
      t192 = t28*t14
      t196 = t28*y
      t200 = t52*Ly
      t213 = t6*t14
      t217 = t6*Ly
      t221 = 19.D0/12.D0*t3*t88*y+59.D0/48.D0*t51*t185-37.D0/3.D0*t3*t18
     #8*Lw-5.D0/8.D0*t3*t192*Lw-59.D0/24.D0*t3*t196*Lw+47.D0/48.D0*t3*t2
     #00*t15+5.D0/48.D0*t3*t200*t14-11.D0/96.D0*t3*t200*y-385.D0/48.D0*t
     #3*t200*t72+6*t3*t213*Lw-32.D0/3.D0*t3*t217*y
      t225 = t6*L1my
      t229 = t6*y
      t239 = t12*y
      t243 = w*L1my
      t247 = w*y
      t251 = t52*L1my
      t264 = 3*t3*t217*t14-14*t3*t225*y-74.D0/3.D0*t3*t229*Lw+3*t3*t225*
     #t14+29.D0/3.D0*t3*t155*y+121.D0/6.D0*t3*t239*Lw+5*t3*t243*y+25.D0/
     #3.D0*t3*t247*Lw+547.D0/192.D0*t3*t251*t57-59.D0/192.D0*t3*t69*Lw-3
     #85.D0/48.D0*t3*t251*t72+547.D0/96.D0*t3*t58*Lw
      t269 = t52*t92
      t273 = t52*t19
      t280 = t52*t14
      t296 = t13*y
      t301 = 385.D0/24.D0*t3*t54*Lw-115.D0/16.D0*t3*t269*Lw-29.D0/48.D0*
     #t3*t273*Lw-385.D0/24.D0*t3*t73*Lw+65.D0/192.D0*t3*t280*Lw-115.D0/3
     #2.D0*t3*t251*t92+385.D0/48.D0*t3*t251*t53-115.D0/64.D0*t51*t269+15
     #.D0/32.D0*t51*t61+85.D0/1536.D0*t3*t251+1783.D0/2560.D0*t3*t296-5.
     #D0/192.D0*t51*t69
      t302 = t13*t92
      t305 = t13*t14
      t308 = t13*t19
      t332 = 235.D0/96.D0*t51*t302-4541.D0/2560.D0*t3*t305+1609.D0/2304.
     #D0*t3*t308-80993.D0/4608.D0*t3*t302+1085447.D0/161280.D0*t3*t73-30
     #449.D0/12288.D0*t3*t58-t51*t273/12-32813.D0/46080.D0*t3*t280-t3*t2
     #00*t19/4+193.D0/96.D0*t3*t61*Lw+33.D0/32.D0*t3*t251*t15-69709.D0/1
     #5360.D0*t3*t269
      t363 = -51947.D0/15360.D0*t3*t54-901.D0/7680.D0*t3*t273+23.D0/16.D
     #0*t51*t111+54617.D0/3840.D0*t3*t66-23381.D0/5760.D0*t3*t185-t51*t3
     #08/12+547.D0/192.D0*t3*t200*t57-37.D0/192.D0*t3*t251*y+15.D0/64.D0
     #*t3*t251*t14-115.D0/32.D0*t3*t200*t92+385.D0/48.D0*t3*t200*t53
      t382 = t13*L1my
      t389 = -83.D0/192.D0*t51*t16-5.D0/96.D0*t51*t296-43.D0/12.D0*t3*t2
     #47+119.D0/8.D0*t3*t225-47.D0/96.D0*t3*t162+487.D0/144.D0*t3*t166-1
     #53.D0/16.D0*t3*t151+9.D0/2.D0*t51*t239-85.D0/48.D0*t3*t213+85.D0/7
     #68.D0*t3*t382+t51*t247-13.D0/24.D0*t3*t382*t19
      t397 = t13*Ly
      t425 = -65.D0/96.D0*t3*t382*t15+15.D0/32.D0*t3*t382*t14+235.D0/48.
     #D0*t3*t397*t92-301.D0/48.D0*t3*t382*t53-301.D0/48.D0*t3*t397*t53+5
     #9.D0/24.D0*t3*t397*t72-37.D0/96.D0*t3*t382*y-t3*t397*t19/3-25.D0/3
     #2.D0*t3*t397*t15-17.D0/48.D0*t3*t251*t19+59.D0/24.D0*t3*t382*t72
      t432 = t8*Ly
      t463 = 235.D0/48.D0*t3*t382*t92-301.D0/24.D0*t3*t66*Lw-11.D0/24.D0
     #*t3*t432*y+235.D0/24.D0*t3*t302*Lw+59.D0/12.D0*t3*t185*Lw+23.D0/8.
     #D0*t3*t432*t53-37.D0/48.D0*t3*t9*y-13.D0/6.D0*t3*t432*t19+337.D0/4
     #8.D0*t3*t432*t15-187.D0/24.D0*t3*t9*t92+5.D0/12.D0*t3*t432*t14+23.
     #D0/8.D0*t3*t9*t53
      t484 = t8*y
      t500 = 5.D0/24.D0*t3*t397*t14-11.D0/48.D0*t3*t397*y-7.D0/8.D0*t3*t
     #308*Lw-35.D0/24.D0*t3*t16*Lw+65.D0/96.D0*t3*t305*Lw-59.D0/96.D0*t3
     #*t296*Lw-59.D0/48.D0*t3*t484*Lw-31.D0/12.D0*t3*t9*t19+347.D0/48.D0
     #*t3*t9*t15+15.D0/16.D0*t3*t9*t14-187.D0/24.D0*t3*t432*t92
      t525 = 407.D0/24.D0*t3*t229+3.D0/2.D0*t51*t213-4*t51*t229+24709.D0
     #/11520.D0*t3*t484+7.D0/6.D0*t51*t166+675.D0/64.D0*t3*t84-469.D0/72
     #.D0*t3*t115+19.D0/6.D0*t51*t84-119.D0/12.D0*t3*t243-5.D0/24.D0*t51
     #*t196-161.D0/48.D0*t51*t188+29521.D0/2304.D0*t3*t188
      t551 = -469.D0/24.D0*t3*t239+29.D0/24.D0*t51*t129+85.D0/192.D0*t3*
     #t29-4189.D0/1152.D0*t3*t192-5239.D0/576.D0*t3*t175-29549.D0/5760.D
     #0*t3*t129-5.D0/6.D0*t51*t192+37.D0/12.D0*t51*t175-15.D0/4.D0*t51*t
     #162+85.D0/96.D0*t3*t88-43.D0/36.D0*t3*t80+35.D0/24.D0*t51*t141
      t576 = 3535.D0/384.D0*t3*t20+126569.D0/11520.D0*t3*t119-19571.D0/5
     #760.D0*t3*t111-11.D0/12.D0*t51*t20+329.D0/96.D0*t51*t24-5.D0/48.D0
     #*t51*t484+805.D0/1152.D0*t3*t141-187.D0/48.D0*t51*t119+2335.D0/384
     #.D0*t3*t196-3535.D0/768.D0*t3*t107-15.D0/4.D0*t51*t115-13.D0/12.D0
     #*t51*t80
      t580 = t49+t96+t137+t179+t221+t264+t301+t332+t363+t389+t425+t463+t
     #500+t525+t551+t576

      sigmaqgLs=tsoftyLs + t580

C     Ls-independent part
C     expansion through w^8

      t1 = w**2
      t2 = t1**2
      t3 = t2*w
      t6 = t2*t1
      t8 = t1*w
      t10 = t2*t8
      t14 = t2**2
      t17 = d1(y)
      t19 = Lz**2
      t31 = DD0(y)
      t64 = DD1(y)
      t66 = t14*z2
      t70 = t8*z2
      t73 = t3*z2
      t76 = w*z2
      t79 = t6*z2
      t83 = t1*z2
      t86 = t2*z2
      t88 = t10*z2
      t91 = -5.D0/1536.D0*t66-564291323.D0/541900800.D0*t14-42413.D0/576
     #.D0*t8+9.D0/16.D0*t70+1251179.D0/345600.D0*t3-5.D0/192.D0*t73-175.
     #D0/48.D0*w+7.D0/12.D0*t76-z2/6-5.D0/384.D0*t79-685919.D0/691200.D0
     #*t6+4213.D0/96.D0*t1-7.D0/8.D0*t83+9637.D0/256.D0*t2-5.D0/96.D0*t8
     #6-5.D0/768.D0*t88-18614191.D0/13547520.D0*t10-19.D0/8.D0
      t112 = -5.D0/3072.D0*t66-163303201.D0/361267200.D0*t14-5.D0/768.D0
     #*t79-360169.D0/1382400.D0*t6-5.D0/1536.D0*t88-2128871.D0/3870720.D
     #0*t10-z2/12-13.D0/12.D0+7.D0/24.D0*t76-55.D0/24.D0*w-5.D0/384.D0*t
     #73+1397489.D0/691200.D0*t3+9.D0/32.D0*t70-59795.D0/1728.D0*t8-7.D0
     #/16.D0*t83+2075.D0/96.D0*t1-5.D0/192.D0*t86+230141.D0/13824.D0*t2
      t123 = DD2(y)
      t147 = -19.D0/12.D0+63.D0/8.D0*w+6*t83+137582737.D0/82944000.D0*t3
     #-3679.D0/128.D0*t1+25362781.D0/11059200.D0*t6+1022275.D0/20736.D0*
     #t8+175.D0/72.D0*t86+z2/2+5641.D0/161280.D0*t66-295.D0/36.D0*t70+64
     #9.D0/720.D0*t73-25.D0/12.D0*t76
      t170 = 247.D0/720.D0*t79+11.D0/90.D0*t88+15177343361.D0/1264435200
     #0.D0*t10-10383599.D0/331776.D0*t2+865321158179.D0/1820786688000.D0
     #*t14+25.D0/24.D0*z3+125.D0/1536.D0*t6*z3+125.D0/3072.D0*t10*z3-175
     #.D0/48.D0*w*z3-225.D0/64.D0*t8*z3+125.D0/6144.D0*t14*z3+125.D0/384
     #.D0*t2*z3+175.D0/32.D0*t1*z3+125.D0/768.D0*t3*z3
      t173 = (1285.D0/2304.D0*t3+1799.D0/96.D0*t1+1285.D0/4608.D0*t6-771
     #.D0/64.D0*t8+1285.D0/9216.D0*t10+1285.D0/1152.D0*t2-1799.D0/144.D0
     #*w+257.D0/72.D0+1285.D0/18432.D0*t14)*t17*t19*Lz+((-147.D0/8.D0*w+
     #105.D0/64.D0*t2+105.D0/128.D0*t3+21.D0/4.D0+105.D0/256.D0*t6+441.D
     #0/16.D0*t1+105.D0/1024.D0*t14-567.D0/32.D0*t8+105.D0/512.D0*t10)*t
     #31+(-1267.D0/24.D0*t1+45.D0/3584.D0*t14+7699.D0/144.D0*t8-22453.D0
     #/80640.D0*t10-4135.D0/288.D0*t2-2263.D0/576.D0*t3+39.D0/2.D0*w-1-2
     #257.D0/1920.D0*t6)*t17)*t19+((3649.D0/72.D0*t8-1855.D0/144.D0*t2-1
     #-857.D0/40320.D0*t14-5299.D0/1440.D0*t3+19*w-667.D0/576.D0*t6-613.
     #D0/12.D0*t1-12457.D0/40320.D0*t10)*t31+(145.D0/192.D0*t3+145.D0/38
     #4.D0*t6+145.D0/768.D0*t10-203.D0/12.D0*w+145.D0/1536.D0*t14-261.D0
     #/16.D0*t8+29.D0/6.D0+203.D0/8.D0*t1+145.D0/96.D0*t2)*t64+t91*t17)*
     #Lz+t112*t31+(5.D0/32.D0*t3+5.D0/16.D0*t2-27.D0/8.D0*t8+5.D0/64.D0*
     #t6+5.D0/256.D0*t14+21.D0/4.D0*t1-7.D0/2.D0*w+1+5.D0/128.D0*t10)*t1
     #23+(-1.D0/2.D0+3235.D0/144.D0*t8-4651.D0/2880.D0*t3-559.D0/24.D0*t
     #1-3191.D0/5760.D0*t6-2873.D0/80640.D0*t14-1495.D0/288.D0*t2-14041.
     #D0/80640.D0*t10+35.D0/4.D0*w)*t64+(t147+t170)*t17

      softyqg = t173

C     expansion through w^8

      t2 = 1/(y+buff1)
      t3 = w**2
      t4 = t3**2
      t6 = t2*t4*t3
      t7 = y**2
      t8 = z2*t7
      t11 = t7**2
      t12 = t11*y
      t13 = z2*t12
      t16 = t11*t7
      t17 = z2*t16
      t20 = Lw**2
      t21 = t11*t20
      t24 = Ly*t7
      t27 = Ly**2
      t28 = t27*y
      t31 = Ly*y
      t34 = t16*t20
      t37 = y*t20
      t40 = y*Lw
      t43 = t12*t20
      t46 = t12*Lw
      t49 = Lw*t16
      t52 = 19.D0/32.D0*t6*t8+187.D0/24.D0*t6*t13-23.D0/8.D0*t6*t17+115.
     #D0/8.D0*t6*t21-3619.D0/768.D0*t6*t24-11.D0/48.D0*t6*t28+26869.D0/1
     #1520.D0*t6*t31+23.D0/4.D0*t6*t34-67.D0/48.D0*t6*t37+377.D0/72.D0*t
     #6*t40-187.D0/12.D0*t6*t43+26773.D0/1152.D0*t6*t46-2303.D0/320.D0*t
     #6*t49
      t53 = t7*y
      t54 = t27*t53
      t57 = t27*t7
      t60 = t11*Lw
      t63 = t53*Lw
      t66 = t7*Lw
      t69 = t7*t20
      t72 = t53*t20
      t76 = t2*t4*w
      t79 = z2*t11
      t82 = z2*t53
      t85 = z2*y
      t88 = t3*w
      t89 = t2*t88
      t90 = L1my**2
      t91 = t90*t7
      t94 = t2*t4
      t95 = L1my*Lw
      t100 = -13.D0/12.D0*t6*t54+5.D0/24.D0*t6*t57-7557.D0/256.D0*t6*t60
     #+11495.D0/576.D0*t6*t63-1039.D0/96.D0*t6*t66+319.D0/192.D0*t6*t69-
     #241.D0/48.D0*t6*t72+77.D0/8.D0*t76*t72+43.D0/6.D0*t76*t79-83.D0/12
     #.D0*t76*t82-7.D0/16.D0*t76*t85-3.D0/2.D0*t89*t91+85.D0/48.D0*t94*t
     #95+137.D0/48.D0*t76*t8
      t108 = Ly*t16
      t111 = t27*t12
      t114 = Ly*t12
      t117 = t27*t11
      t132 = -29.D0/12.D0*t76*t13-145.D0/12.D0*t76*t21+8359.D0/384.D0*t7
     #6*t60-4655.D0/1152.D0*t6*t108-187.D0/48.D0*t6*t111+149033.D0/11520
     #.D0*t6*t114+337.D0/96.D0*t6*t117-153.D0/8.D0*t89*t95-23.D0/6.D0*t9
     #4*t79+23.D0/2.D0*t94*t82+131.D0/24.D0*t94*t85-101.D0/8.D0*t94*t8-7
     #7.D0/18.D0*t94*t31
      t135 = Ly*t11
      t138 = Ly*t53
      t141 = L1my*t7
      t144 = L1my*y
      t151 = L1my*t11
      t152 = t151*Lw
      t155 = t135*Lw
      t158 = L1my*t53
      t159 = t158*Lw
      t162 = t138*Lw
      t165 = t141*Lw
      t168 = t144*Lw
      t171 = L1my*Ly
      t172 = t171*t53
      t175 = 9.D0/8.D0*t94*t117+7.D0/384.D0*t94*t135-191.D0/24.D0*t94*t1
     #38+1441.D0/64.D0*t94*t141-1457.D0/288.D0*t94*t144+19.D0/6.D0*t94*t
     #21+1177.D0/576.D0*t94*t60+11.D0/4.D0*t94*t152+43.D0/12.D0*t94*t155
     #-11.D0/4.D0*t94*t159+t89*t162/3-49.D0/8.D0*t89*t165+85.D0/4.D0*t89
     #*t168-2.D0/3.D0*t89*t172
      t178 = t31*Lw
      t183 = t24*Lw
      t191 = t2*t4*t88
      t192 = t171*t16
      t203 = t171*y
      t210 = 25.D0/24.D0*t94*t178+25.D0/8.D0*t94*t168+29.D0/48.D0*t94*t1
     #83-73.D0/12.D0*t94*t162-191.D0/48.D0*t94*t165-301.D0/48.D0*t191*t1
     #92-4.D0/3.D0*t89*t159+109.D0/24.D0*t76*t172-69.D0/8.D0*t89*t183+24
     #5.D0/12.D0*t89*t178+37.D0/24.D0*t94*t203-5.D0/4.D0*t94*t172+43.D0/
     #4.D0*t89*t203
      t211 = t171*t11
      t214 = t171*t7
      t239 = t90*y
      t242 = 4.D0/3.D0*t94*t211-25.D0/8.D0*t89*t214-25.D0/16.D0*t76*t203
     #-33.D0/16.D0*t94*t214-53.D0/12.D0*t94*t72+303.D0/8.D0*t94*t66-27.D
     #0/16.D0*t94*t69+25.D0/12.D0*t94*t37-2689.D0/288.D0*t94*t40-29.D0/1
     #2.D0*t94*t54+4.D0/3.D0*t94*t57+983.D0/64.D0*t94*t24-t94*t28/4+19.D
     #0/24.D0*t94*t239
      t248 = t90*t11
      t251 = t90*t53
      t258 = t4**2
      t259 = t2*t258
      t266 = L1my*t12
      t267 = t266*Lw
      t270 = t114*Lw
      t275 = t90*t16
      t278 = 85.D0/96.D0*t94*t171+2333.D0/1152.D0*t94*t151+17.D0/24.D0*t
     #94*t248-3.D0/4.D0*t94*t251-541.D0/36.D0*t94*t158-1655.D0/72.D0*t94
     #*t63-139.D0/192.D0*t259*t159+129.D0/64.D0*t259*t155-119.D0/192.D0*
     #t259*t162-115.D0/16.D0*t259*t267-115.D0/16.D0*t259*t270+397.D0/192
     #.D0*t259*t152+385.D0/96.D0*t259*t275
      t279 = L1my*t16
      t280 = t279*Lw
      t283 = t108*Lw
      t288 = t2*w
      t309 = t2*t3
      t312 = 385.D0/24.D0*t259*t280+385.D0/24.D0*t259*t283+17.D0/12.D0*t
     #90*t2+10*t288*t168+5*t288*t203-133.D0/12.D0*t89*t66-59.D0/8.D0*t89
     #*t69+125.D0/6.D0*t89*t37-693.D0/16.D0*t89*t40+t89*t54/2-11.D0/4.D0
     #*t89*t57-539.D0/96.D0*t89*t24+29.D0/6.D0*t89*t28+37.D0/12.D0*t309*
     #t214
      t332 = t27*t16
      t343 = -469.D0/24.D0*t89*t31-7*t309*t239+119.D0/8.D0*t309*t171+3.D
     #0/2.D0*t309*t91+85.D0/1536.D0*t259*t171-74221.D0/15360.D0*t259*t26
     #6-12847.D0/1152.D0*t76*t141-2533.D0/1024.D0*t259*t279+385.D0/96.D0
     #*t259*t332+87.D0/16.D0*t89*t138-149.D0/6.D0*t309*t178+73.D0/12.D0*
     #t309*t183-149.D0/96.D0*t6*t168
      t372 = -169.D0/6.D0*t309*t168+73.D0/12.D0*t309*t165-23.D0/24.D0*t9
     #4*t91-175.D0/32.D0*t89*t141+21.D0/4.D0*t89*t239-35.D0/4.D0*t89*t85
     #+47.D0/4.D0*t89*t8-t89*t72/2+835.D0/72.D0*t89*t63-153.D0/16.D0*t89
     #*t171+887.D0/144.D0*t89*t158-1141.D0/48.D0*t89*t144+119.D0/4.D0*t3
     #09*t95-29.D0/6.D0*t89*t82
      t378 = t11*t53
      t379 = t171*t378
      t382 = t171*t12
      t397 = t90*t12
      t402 = t11**2
      t406 = -t89*t251/3-85.D0/6.D0*t309*t203+59.D0/24.D0*t191*t379+235.
     #D0/48.D0*t191*t382-119.D0/384.D0*t259*t178+269.D0/384.D0*t191*t183
     #-119.D0/192.D0*t191*t178+269.D0/768.D0*t259*t183+123.D0/256.D0*t25
     #9*t165-149.D0/192.D0*t191*t168-115.D0/64.D0*t259*t397-6533.D0/9216
     #.D0*t24*t259+547.D0/384.D0*t259*t90*t402
      t407 = L1my*t402
      t410 = t90*t378
      t413 = L1my*t378
      t432 = Ly*t378
      t439 = -2810713.D0/1290240.D0*t259*t407-385.D0/96.D0*t259*t410+939
     #847.D0/161280.D0*t259*t413+15.D0/128.D0*t259*t91+85.D0/768.D0*t259
     #*t95+83587.D0/18432.D0*t259*t151+33.D0/64.D0*t259*t248-17.D0/96.D0
     #*t259*t251-6667.D0/23040.D0*t259*t158-11.D0/192.D0*t259*t28+20863.
     #D0/107520.D0*t259*t31+1623271.D0/161280.D0*t259*t432-461041.D0/460
     #80.D0*t259*t108-115.D0/64.D0*t259*t111
      t453 = Ly*t402
      t456 = t27*t378
      t470 = t413*Lw
      t473 = 9873.D0/5120.D0*t259*t114+47.D0/96.D0*t259*t117+26683.D0/18
     #432.D0*t259*t135+7217.D0/23040.D0*t259*t138+547.D0/384.D0*t259*t27
     #*t402-4057049.D0/1290240.D0*t259*t453-385.D0/96.D0*t259*t456-149.D
     #0/384.D0*t259*t168+547.D0/192.D0*t259*t171*t402-385.D0/48.D0*t259*
     #t379+63.D0/256.D0*t259*t214-71.D0/192.D0*t259*t172-385.D0/24.D0*t2
     #59*t470
      t474 = t432*Lw
      t505 = -385.D0/24.D0*t259*t474-115.D0/32.D0*t259*t382-25.D0/128.D0
     #*t259*t203+29.D0/6.D0*t76*t267+547.D0/96.D0*t259*t407*Lw+547.D0/96
     #.D0*t259*t453*Lw+25.D0/3.D0*t288*t178+199.D0/192.D0*t259*t211+385.
     #D0/48.D0*t259*t192+59.D0/12.D0*t191*t474-301.D0/24.D0*t191*t280+23
     #5.D0/24.D0*t191*t267+235.D0/24.D0*t191*t270-29.D0/32.D0*t191*t162
      t533 = -301.D0/24.D0*t191*t283+59.D0/12.D0*t191*t470-23111.D0/4608
     #0.D0*t259*t141-5321.D0/322560.D0*t259*t144-37.D0/384.D0*t259*t239-
     #13.D0/48.D0*t191*t251+1853.D0/2304.D0*t191*t158+15.D0/64.D0*t191*t
     #91+85.D0/384.D0*t191*t95+59.D0/48.D0*t191*t410-152323.D0/40320.D0*
     #t191*t413-301.D0/96.D0*t191*t275+10277.D0/768.D0*t191*t279
      t551 = z2*t378
      t564 = -301.D0/96.D0*t191*t332-169.D0/192.D0*t259*t79+7.D0/96.D0*t
     #259*t82-7.D0/128.D0*t259*t85+19.D0/128.D0*t259*t8+115.D0/32.D0*t13
     #*t259-385.D0/48.D0*t17*t259-547.D0/192.D0*t259*z2*t402+385.D0/48.D
     #0*t259*t551+49.D0/24.D0*t259*t21+55135.D0/9216.D0*t259*t60-43.D0/6
     #4.D0*t259*t72+55.D0/2304.D0*t259*t63-581.D0/480.D0*t259*t66
      t585 = t378*t20
      t588 = t378*Lw
      t599 = 319.D0/768.D0*t259*t69-67.D0/192.D0*t259*t37+14317.D0/80640
     #.D0*t259*t40-115.D0/16.D0*t259*t43-22301.D0/7680.D0*t259*t46-11446
     #27.D0/215040.D0*t259*t402*Lw+547.D0/96.D0*t259*t402*t20-385.D0/24.
     #D0*t259*t585+1281559.D0/80640.D0*t259*t588+385.D0/24.D0*t259*t34-2
     #87513.D0/23040.D0*t259*t49-t259*t54/8+5.D0/96.D0*t259*t57
      t628 = -5329.D0/1440.D0*t191*t66+319.D0/384.D0*t191*t69-67.D0/96.D
     #0*t191*t37-119.D0/12.D0*t288*t171+8557.D0/5760.D0*t191*t40+235.D0/
     #24.D0*t191*t43-349909.D0/11520.D0*t191*t46+59.D0/12.D0*t191*t585-4
     #8217.D0/6720.D0*t191*t588+123.D0/128.D0*t191*t165-139.D0/96.D0*t19
     #1*t155-25.D0/64.D0*t191*t203-107.D0/96.D0*t191*t159-43.D0/32.D0*t1
     #91*t152
      t656 = -2.D0/3.D0*t191*t211+63.D0/128.D0*t191*t214-55.D0/96.D0*t19
     #1*t172+29.D0/4.D0*t6*t211+23.D0/8.D0*t6*t192+63.D0/64.D0*t6*t214+1
     #43743.D0/5760.D0*t191*t49-t191*t54/6+5.D0/48.D0*t191*t57-8285.D0/4
     #608.D0*t24*t191+t288*t85/2-125.D0/12.D0*t288*t40-11.D0/96.D0*t191*
     #t28
      t685 = 251.D0/128.D0*t191*t63+18079.D0/23040.D0*t191*t31-136979.D0
     #/40320.D0*t191*t432+133331.D0/11520.D0*t191*t108+235.D0/96.D0*t191
     #*t111-314597.D0/23040.D0*t191*t114-119.D0/6.D0*t288*t95+5.D0/2.D0*
     #t288*t239+55.D0/6.D0*t288*t37-11.D0/2.D0*t288*t144-25.D0/64.D0*t19
     #1*t117+50947.D0/9216.D0*t191*t135+2665.D0/2304.D0*t191*t138+59.D0/
     #48.D0*t191*t456
      t714 = -4871.D0/2560.D0*t191*t141+5383.D0/7680.D0*t191*t144-37.D0/
     #192.D0*t191*t239+85.D0/768.D0*t191*t171+235.D0/96.D0*t191*t397-128
     #407.D0/7680.D0*t191*t266-301.D0/24.D0*t191*t34+23.D0/4.D0*t6*t283+
     #23.D0/4.D0*t6*t280+685.D0/48.D0*t6*t155-187.D0/12.D0*t6*t270-251.D
     #0/48.D0*t6*t159-77.D0/16.D0*t6*t162
      t731 = L1my*t2
      t743 = 695.D0/48.D0*t6*t152-187.D0/12.D0*t6*t267-25.D0/32.D0*t6*t2
     #03+123.D0/64.D0*t6*t165+269.D0/192.D0*t6*t183-187.D0/24.D0*t6*t382
     #-119.D0/96.D0*t6*t178-127.D0/48.D0*t6*t172-t731/2+33451.D0/11520.D
     #0*t6*t144-37.D0/96.D0*t6*t239+85.D0/384.D0*t6*t171-187.D0/48.D0*t6
     #*t397-1079.D0/96.D0*t76*t63
      t771 = -t76*t69/96-67.D0/24.D0*t76*t37+2395.D0/144.D0*t76*t40+29.D
     #0/6.D0*t76*t43-9239.D0/960.D0*t76*t46+11.D0/4.D0*t76*t54-5.D0/12.D
     #0*t76*t57+11245.D0/1152.D0*t6*t138-4693.D0/768.D0*t6*t141-4649.D0/
     #1152.D0*t76*t24-11.D0/24.D0*t76*t28+7469.D0/1152.D0*t76*t31+29.D0/
     #24.D0*t76*t111
      t800 = -2949.D0/640.D0*t76*t114-51.D0/16.D0*t76*t117+26305.D0/2304
     #.D0*t76*t135-4711.D0/576.D0*t76*t138-59.D0/48.D0*t309*t141+70.D0/3
     #.D0*t309*t144+1299.D0/128.D0*t76*t144-243.D0/16.D0*t76*t66-37.D0/4
     #8.D0*t76*t239+29.D0/24.D0*t76*t397+85.D0/192.D0*t76*t171+7.D0/3.D0
     #*t76*t251-143.D0/48.D0*t76*t248+23849.D0/2304.D0*t76*t151
      t830 = 29.D0/6.D0*t76*t270-295.D0/24.D0*t76*t155+221.D0/24.D0*t76*
     #t159-95.D0/8.D0*t76*t152-119.D0/48.D0*t76*t178+49.D0/96.D0*t76*t16
     #5-17.D0/32.D0*t76*t183+241.D0/24.D0*t76*t162+29.D0/96.D0*t76*t214-
     #71.D0/12.D0*t76*t211+29.D0/12.D0*t76*t382-t191*t82/48+19.D0/64.D0*
     #t191*t8
      t859 = -235.D0/48.D0*t13*t191+301.D0/48.D0*t17*t191-59.D0/24.D0*t1
     #91*t551-67.D0/48.D0*t191*t21+6767.D0/512.D0*t191*t60-97.D0/96.D0*t
     #191*t72-72977.D0/4608.D0*t6*t135+1055.D0/24.D0*t309*t40+3.D0/2.D0*
     #t309*t57-49.D0/48.D0*t309*t24-16.D0/3.D0*t309*t28+165.D0/8.D0*t309
     #*t31-7.D0/64.D0*t191*t85-59.D0/12.D0*t288*t31
      t887 = 5.D0/3.D0*t288*t28+11.D0/3.D0*t309*t85-10.D0/3.D0*t309*t8+7
     #0859.D0/9216.D0*t191*t151-65.D0/192.D0*t191*t248+118697.D0/11520.D
     #0*t6*t266-63049.D0/4608.D0*t6*t151+347.D0/96.D0*t6*t248-31.D0/24.D
     #0*t6*t251+1305.D0/128.D0*t6*t158+15.D0/32.D0*t6*t91-9.D0/4.D0*t309
     #*t66+73.D0/12.D0*t309*t69
      t916 = -53.D0/2.D0*t309*t37+85.D0/192.D0*t6*t95+23.D0/16.D0*t6*t27
     #5-18179.D0/5760.D0*t6*t279+23.D0/16.D0*t6*t332+47.D0/48.D0*t191*t7
     #9-9631.D0/1920.D0*t76*t266-1763.D0/576.D0*t76*t158+5.D0/48.D0*t76*
     #t91+85.D0/96.D0*t76*t95-53.D0/8.D0*t6*t79+35.D0/24.D0*t6*t82-7.D0/
     #32.D0*t6*t85+427951.D0/483840.D0*t191*t53
      t945 = 4063.D0/144.D0*t89*L1my+85.D0/384.D0*t76*t90+24617959.D0/55
     #2960.D0*t6*t11+85.D0/768.D0*t6*t90+3449477.D0/138240.D0*t76*t7-324
     #1513.D0/92160.D0*t6*t12+711079.D0/69120.D0*t76*t53-3971.D0/1728.D0
     #*t89*t53+2952624901.D0/433520640.D0*t259*t402-10896503.D0/9676800.
     #D0*t191*t7-119.D0/24.D0*t288*t90-233249.D0/153600.D0*t6*y+379.D0/6
     #4.D0*t309*t7
      t974 = -5947.D0/2880.D0*t76*L1my-141833.D0/4608.D0*t94*t7-3479.D0/
     #5760.D0*t6*L1my-667.D0/24.D0*t309*L1my-84809227.D0/25804800.D0*t25
     #9*t7+85.D0/1536.D0*t191*t90+705217069.D0/19353600.D0*t191*t12+17.D
     #0/6.D0*t731*Ly+6385.D0/3072.D0*t94*t11+17.D0/3.D0*t731*Lw+21779.D0
     #/576.D0*t89*y-153.D0/32.D0*t89*t90+3534391.D0/230400.D0*t76*t12-22
     #15.D0/288.D0*t94*L1my
      t1002 = 33414329.D0/19353600.D0*t191*y+1159.D0/80640.D0*t259*L1my+
     #85.D0/192.D0*t94*t90-10873.D0/80640.D0*t191*L1my-117644371.D0/7741
     #440.D0*t191*t11-147861.D0/5120.D0*t6*t53+101.D0/24.D0*t288*y-54368
     #0941.D0/30105600.D0*t259*t378+136093.D0/13824.D0*t6*t7+156776887.D
     #0/38707200.D0*t259*t53+85.D0/3072.D0*t259*t90+41.D0/4.D0*t288*L1my
     #+7386067.D0/691200.D0*t6*t16
      t1031 = 134223899.D0/11059200.D0*t259*t16+32910727.D0/9676800.D0*t
     #259*t12-315.D0/128.D0*t89*t7-564523.D0/27648.D0*t76*y-71713867.D0/
     #10321920.D0*t259*t11-319915417.D0/9676800.D0*t191*t16+119.D0/16.D0
     #*t309*t90+13177.D0/6912.D0*t94*y-2585.D0/96.D0*t309*y+261251113.D0
     #/135475200.D0*t259*y+347682043.D0/33868800.D0*t191*t378+91249.D0/6
     #912.D0*t94*t53-9031663.D0/276480.D0*t76*t11-149.D0/48.D0*t76*t168
      t1036 = t859+t887+t52+t175+t210+t1031+t312+t278+t343+t372+t439+t40
     #6+t473+t533+t505+t564+t628+t599+t685+t656+t916+t714+t771+t830+t800
     #+t100+t945+t743+t1002+t974+t132+t242


      sigmaqg=softyqg + t1036 + sigmaqgLs

      end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     Identical quarks

      double precision function sigmaqq(z,y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 u,w,Ls,Lw,y
      real*8 Ldelta,Lxi,L1mxi,xi,r,delta,Lz,Ly,L1my
      external dilog,d1,DD0,DD1,DD2

      buff1=0.0000001D0

      z2=1.644934068D0

      Ls=dlog(mF**2/q**2)
      w=1D0-z
      Ly=dlog(y+buff1)
      L1my = dlog(1D0-y+buff1)
      Lw=dlog(w)


C     scale dependense
C     through w^8

      t1 = w**2
      t2 = t1**2
      t3 = t2*t1
      t4 = Ls*t3
      t5 = 1-y
      t6 = d1(t5)
      t7 = t6*Lw
      t10 = t2*w
      t11 = Ls*t10
      t14 = Ls*t2
      t17 = t1*w
      t18 = Ls*t17
      t21 = Ls*t1
      t23 = Ls*w
      t26 = t2**2
      t27 = Ls*t26
      t28 = d1(y)
      t29 = t28*Lw
      t32 = t2*t17
      t33 = Ls*t32
      t51 = -37.D0/720.D0*t4*t7-37.D0/360.D0*t11*t7-t14*t7/4+5.D0/6.D0*t
     #18*t7-t21*t7+2.D0/3.D0*t23*t7-71.D0/4032.D0*t27*t29-97.D0/3360.D0*
     #t33*t29-37.D0/720.D0*t4*t29-37.D0/360.D0*t11*t29-t14*t29/4+5.D0/6.
     #D0*t18*t29-t21*t29+2.D0/3.D0*t23*t29-71.D0/4032.D0*t27*t7-97.D0/33
     #60.D0*t33*t7
      t52 = DD0(y)
      t55 = DD0(t5)
      t58 = Ls**2
      t59 = t58*t17
      t70 = t58*t3
      t73 = t58*t2
      t88 = t58*t10
      t91 = -t21*t52/2-37.D0/720.D0*t11*t55+5.D0/24.D0*t59*t6-2071.D0/43
     #200.D0*t11*t28-439661.D0/8467200.D0*t33*t6+137.D0/288.D0*t14*t28-t
     #21*t55/2-37.D0/2880.D0*t70*t6-t73*t6/16-37.D0/1440.D0*t4*t52+5.D0/
     #12.D0*t18*t52-5245.D0/150528.D0*t27*t6-47.D0/72.D0*t18*t28+11.D0/1
     #2.D0*t21*t28+t23*t55/3-37.D0/1440.D0*t88*t28
      t107 = t58*t32
      t118 = t58*t1
      t121 = t58*w
      t128 = -t23*t28/3-t14*t52/8-2071.D0/43200.D0*t11*t6+5.D0/24.D0*t59
     #*t28-71.D0/8064.D0*t27*t55-5245.D0/150528.D0*t27*t28-t14*t55/8-97.
     #D0/13440.D0*t107*t28-6221.D0/86400.D0*t4*t6-439661.D0/8467200.D0*t
     #33*t28-37.D0/2880.D0*t70*t28-t73*t28/16-t118*t28/4+t121*t28/6-97.D
     #0/13440.D0*t107*t6-37.D0/1440.D0*t88*t6
      t151 = t58*t26
      t162 = -47.D0/72.D0*t18*t6+11.D0/12.D0*t21*t6+t23*t52/3-97.D0/6720
     #.D0*t33*t52-71.D0/8064.D0*t27*t52-37.D0/1440.D0*t4*t55-t118*t6/4+5
     #.D0/12.D0*t18*t55-37.D0/720.D0*t11*t52+t121*t6/6+137.D0/288.D0*t14
     #*t6-71.D0/16128.D0*t151*t28-71.D0/16128.D0*t151*t6-6221.D0/86400.D
     #0*t4*t28-t23*t6/3-97.D0/6720.D0*t33*t55
      t164 = t51+t91+t128+t162

      softyLs=t164

      t1 = w**2
      t2 = t1**2
      t4 = t2*t1*Ls
      t6 = t2**2
      t8 = t6*w*Ls
      t10 = t6*Ls
      t12 = t1*w
      t14 = t2*t12*Ls
      t16 = t12*Ls
      t17 = t2*Ls
      t20 = t2*w*Ls
      t26 = y**2
      t27 = t26**2
      t28 = t27*t26
      t33 = t27*y
      t40 = t27**2
      t51 = t26*y
      t55 = 181.D0/1440.D0*t4+5641.D0/193536.D0*t8+49.D0/1152.D0*t10+43.
     #D0/630.D0*t14-t16+17.D0/72.D0*t17+419.D0/1440.D0*t20-2.D0/3.D0*Ls*
     #w+7.D0/6.D0*Ls*t1-3127.D0/5040.D0*t14*t28-3349.D0/896.D0*t10*t27+3
     #127.D0/1680.D0*t14*t33-75223.D0/241920.D0*t8*t26+t16*y/6-49381.D0/
     #69120.D0*t8*t40-t16*t26/6+1923977.D0/241920.D0*t8*t33+t17*t26/8-59
     #9689.D0/96768.D0*t8*t27+49381.D0/17280.D0*t8*t27*t51
      t99 = 6329.D0/5040.D0*t14*t51-5491.D0/2520.D0*t14*t27-101.D0/1344.
     #D0*t14*y+14785.D0/8064.D0*t10*t51-11209.D0/40320.D0*t10*t26-601.D0
     #/8064.D0*t10*y+45497.D0/13440.D0*t10*t33-45497.D0/40320.D0*t10*t28
     #-18523.D0/241920.D0*t8*y-723941.D0/120960.D0*t8*t28+598373.D0/2419
     #20.D0*t8*t51
      t101 = t55-t17*y/8-223.D0/720.D0*t20*y-11.D0/90.D0*t20*t26+311.D0/
     #360.D0*t20*t51-311.D0/720.D0*t20*t27-343.D0/1440.D0*t4*t26-133.D0/
     #1440.D0*t4*y+119.D0/180.D0*t4*t51-119.D0/360.D0*t4*t27-4889.D0/201
     #60.D0*t14*t26+t99

      sigmaqqLs = softyLs +t101

C     Scale independent piece

C     Expansion accurate through w^8

      t1 = w**2
      t2 = t1*w
      t3 = t1**2
      t4 = t3*t2
      t5 = DD0(y)
      t6 = t4*t5
      t8 = 1D0-y
      t9 = d1(t8)
      t10 = t4*t9
      t12 = t3*t1
      t13 = DD0(t8)
      t14 = t12*t13
      t16 = t4*t13
      t18 = w*t13
      t20 = DD1(y)
      t25 = t12*t5
      t27 = t3*w
      t28 = t27*t5
      t30 = DD1(t8)
      t33 = d1(y)
      t34 = t3*t33
      t38 = t12*t33
      t40 = t1*t9
      t42 = -439661.D0/8467200.D0*t6+2030471.D0/148176000.D0*t10-6221.D0
     #/86400.D0*t14-439661.D0/8467200.D0*t16-t18/3-t1*t20/2-97.D0/6720.D
     #0*t4*t20-6221.D0/86400.D0*t25-2071.D0/43200.D0*t28-97.D0/6720.D0*t
     #4*t30-1393.D0/3456.D0*t34-t3*t30/8+360841.D0/2592000.D0*t38-25.D0/
     #24.D0*t40
      t43 = w*t33
      t45 = t3*t13
      t47 = t1*t13
      t49 = t3**2
      t50 = t49*t5
      t52 = t49*t13
      t68 = w*t9
      t70 = t12*t9
      t72 = 4.D0/9.D0*t43+137.D0/288.D0*t45+11.D0/12.D0*t47-5245.D0/1505
     #28.D0*t50-5245.D0/150528.D0*t52-71.D0/8064.D0*t49*t30+5.D0/12.D0*t
     #2*t20-t3*t20/8-37.D0/720.D0*t27*t20-37.D0/1440.D0*t12*t20-71.D0/80
     #64.D0*t49*t20+w*t20/3+4.D0/9.D0*t68+360841.D0/2592000.D0*t70
      t76 = t2*t33
      t80 = t1*t33
      t82 = t27*t13
      t84 = t2*t5
      t86 = t3*t5
      t88 = t2*t9
      t90 = Lw**2
      t93 = t49*t9
      t98 = t4*t33
      t103 = t3*t9
      t106 = -37.D0/1440.D0*t12*t30
     #+65.D0/72.D0*t76+w*t30/3-25.D0/24.D0*t80
     #-2071.D0/43200.D0*t82-47.D0/72.D0*t84+137.D0/288.D0*t86
     #+65.D0/72.D0*t88
     #+2.D0/3.D0*t43*t90
     #-5245.D0/75264.D0*t93*Lw
     #-37.D0/720.D0*t14*Lw+97.D0/6720.D0*t98*z2-t43*z2/3
     #+137.D0/144.D0*t103*Lw
      t109 = t27*t33
      t132 = t49*t33
      t135 = -t103*t90/4-37.D0/360.D0*t109*t90-t40*t90-47.D0/36.D0*t88*L
     #w-97.D0/3360.D0*t98*t90-5.D0/12.D0*t76*z2+5.D0/6.D0*t88*t90-37.D0/
     #720.D0*t38*t90-37.D0/720.D0*t70*t90-t34*t90/4+5.D0/6.D0*t84*Lw-t47
     #*Lw-2.D0/3.D0*t43*Lw+71.D0/8064.D0*t132*z2
      t144 = t27*t9
      t155 = t2*t13
      t159 = w*t5
      t165 = -37.D0/720.D0*t25*Lw-47.D0/36.D0*t76*Lw-2.D0/3.D0*t68*Lw-20
     #71.D0/21600.D0*t144*Lw+37.D0/1440.D0*t38*z2+37.D0/720.D0*t144*z2-9
     #7.D0/3360.D0*t6*Lw-6221.D0/43200.D0*t70*Lw-47.D0/72.D0*t155-310321
     #93.D0/1137991680.D0*t93-31032193.D0/1137991680.D0*t132-t159/3+97.D
     #0/6720.D0*t10*z2-97.D0/3360.D0*t10*t90
      t194 = 71.D0/8064.D0*t93*z2-71.D0/4032.D0*t93*t90+11.D0/6.D0*t40*L
     #w+t40*z2/2-2071.D0/21600.D0*t109*Lw+37.D0/720.D0*t109*z2-6221.D0/4
     #3200.D0*t38*Lw-439661.D0/4233600.D0*t98*Lw-5245.D0/75264.D0*t132*L
     #w-71.D0/4032.D0*t132*t90+2.D0/3.D0*t68*t90-37.D0/360.D0*t28*Lw+5.D
     #0/6.D0*t76*t90+137.D0/144.D0*t34*Lw
      t200 = t1*t5
      t223 = t34*z2/8-71.D0/4032.D0*t50*Lw-t200*Lw+2.D0/3.D0*t159*Lw-t86
     #*Lw/4-37.D0/360.D0*t82*Lw-97.D0/3360.D0*t16*Lw-71.D0/4032.D0*t52*L
     #w+5.D0/6.D0*t155*Lw+2.D0/3.D0*t18*Lw-t45*Lw/4+11.D0/6.D0*t80*Lw+t8
     #0*z2/2-t80*t90
      t247 = -t68*z2/3-5.D0/12.D0*t88*z2+t103*z2/8-37.D0/360.D0*t144*t90
     #+37.D0/1440.D0*t70*z2-439661.D0/4233600.D0*t10*Lw+5.D0/12.D0*t2*t3
     #0+518251.D0/1296000.D0*t144-1393.D0/3456.D0*t103+11.D0/12.D0*t200-
     #t1*t30/2-37.D0/720.D0*t27*t30+518251.D0/1296000.D0*t109+2030471.D0
     #/148176000.D0*t98
      t250 = t42+t72+t106+t135+t165+t194+t223+t247

      softyqq=t250


      t1 = w**2
      t2 = t1**2
      t3 = t2*w
      t4 = t3*Lw
      t6 = y**2
      t7 = t6**2
      t8 = t7*t6
      t9 = t2**2
      t12 = t1*w
      t13 = t12*Lw
      t15 = t9*w
      t18 = t2*t1
      t19 = t18*Lw
      t21 = t7*y
      t24 = t2*Lw
      t28 = t6*y
      t31 = t9*Lw
      t33 = t2*t12
      t34 = t33*Lw
      t36 = 419.D0/720.D0*t4+9561943.D0/10160640.D0*t8*t9-2*t13-19396193
     #23.D0/203212800.D0*t8*t15+181.D0/720.D0*t19+77401577.D0/29030400.D
     #0*t21*t15+17.D0/36.D0*t24+2139359507.D0/1625702400.D0*t6*t15-11499
     #06073.D0/406425600.D0*t28*t15+49.D0/576.D0*t31+43.D0/315.D0*t34
      t40 = 1/(y+buff1)
      t41 = t9*t40
      t42 = L1my*y
      t45 = t18*t40
      t46 = L1my*t21
      t49 = L1my*t7
      t52 = t2*t40
      t56 = 1/(-1+y-buff1)
      t57 = t3*t56
      t58 = Ly*t21
      t61 = t12*t56
      t62 = Ly*t28
      t65 = t33*t56
      t66 = t7*t28
      t67 = Ly*t66
      t70 = t1*t56
      t71 = Ly*y
      t74 = t15*t56
      t75 = t7**2
      t76 = t75*y
      t80 = -73.D0/576.D0*t2+49649.D0/112896.D0*t33+49.D0/1152.D0*t41*t4
     #2-119.D0/360.D0*t45*t46+119.D0/180.D0*t45*t49+17.D0/72.D0*t52*t42-
     #311.D0/720.D0*t57*t58-t61*t62/6-3127.D0/5040.D0*t65*t67+7.D0/6.D0*
     #t70*t71+33959.D0/26880.D0*t74*Ly*t76
      t82 = w*t56
      t99 = -2.D0/3.D0*t82*t71+181.D0/1440.D0*t45*t42+153511.D0/259200.D
     #0*t18+13773497.D0/45158400.D0*t9+46450771.D0/5644800.D0*t66*t15+2.
     #D0/3.D0*w-3983131.D0/1693440.D0*t75*t15+2767695097.D0/13005619200.
     #D0*t15+107.D0/216.D0*t12+60569.D0/259200.D0*t3+7.D0/3.D0*t1*Lw
      t103 = t15*Lw
      t107 = Ly*t7
      t120 = t9*t56
      t125 = -5.D0/2.D0*t1+487269287.D0/162570240.D0*t7*t15+10721.D0/184
     #320.D0*t103-863.D0/576.D0*t65*t62+146261.D0/92160.D0*t74*t107-2694
     #61.D0/184320.D0*t74*t62-2909.D0/720.D0*t65*t58+2473.D0/720.D0*t65*
     #t107-85549.D0/129600.D0*y*t3+175.D0/1296.D0*t7*t3-45497.D0/40320.D
     #0*t120*t67+258563.D0/2580480.D0*t74*t71
      t128 = t2*t56
      t131 = t18*t56
      t134 = Ly*t6
      t141 = Ly*t8
      t154 = t128*t62/8-259.D0/288.D0*t131*t62+7.D0/48.D0*t131*t134+413.
     #D0/2880.D0*t65*t71+13.D0/36.D0*t128*t71+3127.D0/1260.D0*t65*t141+1
     #19.D0/120.D0*t131*t107-t128*Ly/9+781.D0/432.D0*t6*t2-44.D0/27.D0*t
     #6*t12+293.D0/1440.D0*t120*t134
      t180 = -t128*t134/4+3209.D0/576.D0*t120*t107-92363.D0/13440.D0*t74
     #*Ly*t75-2027.D0/960.D0*t120*t62+241.D0/1440.D0*t65*t134-39645643.D
     #0/67737600.D0*y*t9+12369859.D0/4233600.D0*t7*t33-783804221.D0/1625
     #702400.D0*y*t15+59773.D0/64800.D0*t6*t18-4.D0/3.D0*w*Lw+3457.D0/20
     #480.D0*t74*t134+173.D0/288.D0*t57*t71
      t202 = w*t40
      t205 = t61*t134/3-7.D0/6.D0*t61*t71+108971.D0/7680.D0*t74*t67-3.D0
     #/16.D0*t57*t134-71.D0/72.D0*t57*t62+311.D0/240.D0*t57*t107-3419.D0
     #/480.D0*t120*t58+45497.D0/10080.D0*t120*t141+162773.D0/36864.D0*t7
     #4*t58-4309579.D0/322560.D0*t74*t141-2.D0/3.D0*t202*t42
      t206 = t33*t40
      t209 = L1my*t28
      t212 = t15*t40
      t217 = t3*t40
      t220 = L1my*t6
      t235 = -5491.D0/2520.D0*t206*t46-4889.D0/20160.D0*t206*t209+170487
     #.D0/143360.D0*t212*t49-97553.D0/430080.D0*t212*t209+419.D0/1440.D0
     #*t217*t42-47233.D0/645120.D0*t212*t220+6329.D0/5040.D0*t206*t49+45
     #497.D0/6720.D0*t31*t21+108369767.D0/101606400.D0*t6*t9-16099.D0/12
     #9600.D0*t7*t18-119.D0/360.D0*t131*t58+59.D0/504.D0*t120*t71
      t241 = t12*t40
      t244 = L1my*t66
      t247 = t1*t40
      t265 = -311.D0/720.D0*t217*t46-t241*t209/6-3127.D0/5040.D0*t206*t2
     #44+7.D0/6.D0*t247*t42+33959.D0/26880.D0*t212*L1my*t76-9561943.D0/3
     #386880.D0*t21*t9+157.D0/720.D0*t131*t71+134303837.D0/40642560.D0*t
     #7*t9+16099.D0/64800.D0*t28*t18-38684407.D0/20321280.D0*t28*t9-781.
     #D0/432.D0*y*t2
      t268 = L1my*t8
      t288 = 461753.D0/53760.D0*t212*t244-1550191.D0/322560.D0*t212*t268
     #+311.D0/360.D0*t217*t49-223.D0/720.D0*t217*t220+t241*t220/6-t241*t
     #42-9043.D0/8640.D0*y*t18+103049.D0/129600.D0*t6*t3-175.D0/648.D0*t
     #28*t3-926753.D0/705600.D0*t28*t33-11987.D0/4410.D0*t21*t33
      t313 = 11987.D0/13230.D0*t8*t33+44.D0/27.D0*y*t12+488659.D0/508032
     #.D0*t6*t33-139483.D0/430080.D0*t212*t46+45497.D0/13440.D0*t41*t268
     #-t52*t220/8-45497.D0/40320.D0*t41*t244-101.D0/1344.D0*t206*t220-50
     #279.D0/8960.D0*t212*L1my*t75+t52*t209/8+15031.D0/516096.D0*t212*t4
     #2
      t338 = -343.D0/1440.D0*t45*t209+43.D0/630.D0*t206*t42-133.D0/1440.
     #D0*t45*t220-11.D0/90.D0*t217*t209+14785.D0/8064.D0*t41*t49-11209.D
     #0/40320.D0*t41*t209-601.D0/8064.D0*t41*t220-3349.D0/896.D0*t41*t46
     #-4814729.D0/6350400.D0*y*t33+3127.D0/1680.D0*t206*t268-17.D0/504.D
     #0*t120*Ly-t13*t6/3
      t363 = -3349.D0/448.D0*t31*t7+311.D0/180.D0*t4*t28+461333.D0/26880
     #.D0*t103*t8-11.D0/45.D0*t4*t6+3127.D0/840.D0*t34*t21-771473.D0/806
     #40.D0*t103*t21-11209.D0/20160.D0*t31*t6-45497.D0/20160.D0*t31*t8-5
     #491.D0/1260.D0*t34*t7-15293.D0/107520.D0*t103*y+14785.D0/4032.D0*t
     #31*t28
      t388 = -12073.D0/516096.D0*t74*Ly-119.D0/180.D0*t19*t7-101.D0/672.
     #D0*t34*y-71.D0/8064.D0*t41*L1my-307.D0/53760.D0*t212*L1my-t52*L1my
     #/8-2.D0/3.D0*t70*Ly+5.D0/12.D0*t241*L1my+t202*L1my/3-133.D0/720.D0
     #*t19*y-311.D0/360.D0*t4*t7+t13*y/3
      t412 = -601.D0/4032.D0*t31*y+7.D0/12.D0*t61*Ly-t247*L1my/2-97.D0/6
     #720.D0*t206*L1my-37.D0/1440.D0*t45*L1my-31.D0/576.D0*t65*Ly-3127.D
     #0/2520.D0*t34*t8+394871.D0/161280.D0*t103*t28-312301.D0/645120.D0*
     #t103*t6-t131*Ly/10+119.D0/90.D0*t19*t28
      t437 = -23.D0/96.D0*t57*Ly+6329.D0/2520.D0*t34*t28-t24*y/4-343.D0/
     #720.D0*t19*t6+t82*Ly/3-37.D0/720.D0*t217*L1my-4889.D0/10080.D0*t34
     #*t6+t24*t6/4-75401.D0/6720.D0*t103*t66-223.D0/360.D0*t4*y-155723.D
     #0/215040.D0*t103*t7+33959.D0/13440.D0*t103*t75
      t441 = t36+t80+t99+t125+t154+t180+t205+t235+t265+t288+t313+t338+t3
     #63+t388+t412+t437

C      print *, t441

      sigmaqq=softyqq + t441 + sigmaqqLs

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     Different quarks // three different functions

      double precision function sigmaq1q2a(z,y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 u,w,Ls
      real*8 Ldelta,Lxi,L1mxi,xi,r,delta,Lw, Ly, L1my
      external dilog,d1,DD0,DD1,DD2

      buff1=0.000000001

      z2=1.644934068D0

C      r=dsqrt((z+y*(1.D0-z))/(1.D0-y*(1.D0-z)))
C      t=dsqrt(z)

      Ls=dlog(mF**2/q**2)
      w=1D0-z
      Ly=dlog(y+buff1)
      L1my = dlog(1D0-y+buff1)
      Lw=dlog(w)


C     scale dependence through w^8

      t1 = w**2
      t2 = Ls*t1
      t3 = DD0(y)
      t6 = Ls*w
      t7 = d1(y)
      t10 = Ls**2
      t14 = t1**2
      t15 = t14*w
      t16 = Ls*t15
      t19 = t14*t1
      t20 = Ls*t19
      t23 = t14**2
      t24 = Ls*t23
      t29 = t1*w
      t30 = Ls*t29
      t46 = t14*t29
      t47 = Ls*t46
      t53 = -t2*t3/2-t6*t7/3+t10*w*t7/6-1591.D0/43200.D0*t16*t7-5741.D0/
     #86400.D0*t20*t7-71.D0/8064.D0*t24*t3+t6*t3/3-47.D0/72.D0*t30*t7-37
     #.D0/1440.D0*t20*t3-86291.D0/2257920.D0*t24*t7+11.D0/12.D0*t2*t7-37
     #.D0/720.D0*t16*t3-37.D0/1440.D0*t10*t15*t7+5.D0/12.D0*t30*t3-97.D0
     #/6720.D0*t47*t3-t10*t1*t7/4
      t57 = Ls*t14
      t76 = t7*Lw
      t92 = -71.D0/16128.D0*t10*t23*t7-t57*t3/8-37.D0/2880.D0*t10*t19*t7
     #-443021.D0/8467200.D0*t47*t7-t10*t14*t7/16-97.D0/13440.D0*t10*t46*
     #t7+5.D0/24.D0*t10*t29*t7+137.D0/288.D0*t57*t7-t2*t76-t57*t76/4-37.
     #D0/360.D0*t16*t76+5.D0/6.D0*t30*t76-37.D0/720.D0*t20*t76-97.D0/336
     #0.D0*t47*t76-71.D0/4032.D0*t24*t76+2.D0/3.D0*t6*t76
      t93 = t53+t92

      softyLs=t93

      t1 = w**2
      t2 = t1*w
      t3 = t2*Ls
      t4 = y**2
      t7 = t1**2
      t8 = t7*Ls
      t12 = t7*w*Ls
      t13 = t4**2
      t18 = t4*y
      t22 = t7*t1*Ls
      t23 = t13*y
      t31 = t7*t2*Ls
      t36 = t7**2
      t38 = t36*w*Ls
      t41 = -t3*t4/12-t8*t4/3-311.D0/1440.D0*t12*t13+t3*y/4+19.D0/48.D0*
     #t12*t18+11.D0/40.D0*t22*t23-307.D0/360.D0*t22*t13+85.D0/96.D0*t22*
     #t18-1879.D0/960.D0*t31*t13-17.D0/144.D0*t22*y-28535.D0/4608.D0*t38
     #*t13
      t44 = t1*Ls
      t51 = t13*t4
      t56 = t36*Ls
      t63 = t13**2
      t68 = 19.D0/72.D0*t8*t18-t44*y/6+83.D0/64.D0*t31*t18-37.D0/144.D0*
     #t31*t4-3127.D0/10080.D0*t31*t51-19.D0/72.D0*t22*t4-3407.D0/2016.D0
     #*t56*t51+23.D0/18.D0*t31*t23-413.D0/1440.D0*t56*t4-49381.D0/138240
     #.D0*t38*t63-t56*y/12
      t89 = t13*t18
      t92 = -43.D0/480.D0*t31*y+361.D0/192.D0*t56*t18+t22/10-179.D0/48.D
     #0*t56*t13+57.D0/16.D0*t56*t23-79.D0/960.D0*t38*y-19.D0/60.D0*t38*t
     #4+t8*y/12-13.D0/36.D0*t12*y+7267.D0/2880.D0*t38*t18+176291.D0/8064
     #0.D0*t38*t89
      t110 = 755.D0/96.D0*t38*t23-227527.D0/40320.D0*t38*t51+1441.D0/448
     #0.D0*t56*t89+23.D0/96.D0*t12+31.D0/576.D0*t31-Ls*w/3+1511.D0/64512
     #.D0*t38+17.D0/504.D0*t56+t8/9+2.D0/3.D0*t44-t12*t4/144-7.D0/12.D0*
     #t3
      t112 = t41+t68+t92+t110

      sigmaq1q2aLs = softyLs +t112


      t1 = w**2
      t2 = t1**2
      t3 = t2*w
      t4 = d1(y)
      t5 = t3*t4
      t7 = t2*t4
      t9 = DD1(y)
      t14 = t2*t1
      t15 = DD0(y)
      t16 = t14*t15
      t18 = t1*w
      t19 = t2*t18
      t20 = t19*t15
      t22 = t19*t4
      t25 = t2**2
      t28 = t18*t4
      t32 = t25*t15
      t34 = t2*t15
      t36 = w*t4
      t37 = Lw**2
      t42 = 495871.D0/1296000.D0*t5-1385.D0/3456.D0*t7-t2*t9/8+w*t9/3-57
     #41.D0/86400.D0*t16-443021.D0/8467200.D0*t20-443021.D0/4233600.D0*t
     #22*Lw-71.D0/8064.D0*t25*t9+25.D0/27.D0*t28-37.D0/1440.D0*t14*t9-86
     #291.D0/2257920.D0*t32+137.D0/288.D0*t34+2.D0/3.D0*t36*t37-97.D0/67
     #20.D0*t19*t9
      t47 = t14*t4
      t56 = t1*t15
      t58 = t3*t15
      t63 = t1*t4
      t66 = t25*t4
      t75 = t7*z2/8-37.D0/720.D0*t3*t9-5741.D0/43200.D0*t47*Lw+5.D0/12.D
     #0*t18*t9-t7*t37/4-71.D0/4032.D0*t32*Lw-t56*Lw-37.D0/360.D0*t58*Lw-
     #1591.D0/21600.D0*t5*Lw+t63*z2/2+71.D0/8064.D0*t66*z2-71.D0/4032.D0
     #*t66*t37+97.D0/6720.D0*t22*z2-97.D0/3360.D0*t20*Lw
      t82 = t18*t15
      t99 = w*t15
      t102 = -37.D0/720.D0*t16*Lw-9.D0/8.D0*t63-t36*z2/3+5.D0/6.D0*t82*L
     #w-t63*t37+t36/2-t1*t9/2-71082341.D0/5689958400.D0*t66+377461.D0/25
     #92000.D0*t47+11.D0/6.D0*t63*Lw-86291.D0/1128960.D0*t66*Lw-47.D0/36
     #.D0*t28*Lw+137.D0/144.D0*t7*Lw+2.D0/3.D0*t99*Lw
      t126 = -t99/3-5.D0/12.D0*t28*z2-2.D0/3.D0*t36*Lw-t34*Lw/4+5.D0/6.D
     #0*t28*t37-37.D0/360.D0*t5*t37+37.D0/720.D0*t5*z2+37.D0/1440.D0*t47
     #*z2-37.D0/720.D0*t47*t37-97.D0/3360.D0*t22*t37-47.D0/72.D0*t82+747
     #451.D0/24696000.D0*t22-1591.D0/43200.D0*t58+11.D0/12.D0*t56
      t128 = t42+t75+t102+t126

      softy=t128

      t1 = w**2
      t2 = t1*w
      t3 = t1**2
      t4 = t3*t2
      t5 = y**2
      t6 = t5**2
      t7 = t6*t5
      t12 = t3*t1
      t15 = t12*Ly
      t17 = t5*y
      t20 = t3**2
      t25 = t3*w
      t34 = 906061.D0/1411200.D0*t4*t7-2.D0/3.D0*w*Lw+86833.D0/43200.D0*
     #t12*t6+t15/10-1351.D0/864.D0*t12*t17-114677.D0/241920.D0*t20*y+750
     #1.D0/9450.D0*t20*t5-6913.D0/8640.D0*t25*t5+2219.D0/4320.D0*t12*t5+
     #1434241.D0/241920.D0*t20*t6+445127.D0/134400.D0*t4*t6
      t39 = t25*Ly
      t41 = t20*w
      t42 = t41*Lw
      t43 = t6*t17
      t46 = t6*y
      t49 = t25*Lw
      t56 = t41*Ly
      t62 = -64397.D0/134400.D0*t4*y-715.D0/864.D0*t3*t17+23.D0/96.D0*t3
     #9-63041.D0/5760.D0*t42*t43-46447.D0/5760.D0*t42*t46-t49*t5/72-t39*
     #t5/144-241897.D0/184320.D0*t42*t6-46447.D0/11520.D0*t56*t46-17.D0/
     #216.D0*t3+25057.D0/86400.D0*t25*t6-t1
      t65 = t1*Lw
      t70 = t2*Lw
      t72 = t20*Lw
      t75 = t3*Ly
      t78 = t4*Lw
      t81 = t2*Ly
      t87 = 1/(y+buff1)
      t88 = t41*t87
      t91 = t4*Ly
      t94 = 33779.D0/120960.D0*t4+4.D0/3.D0*t65+2719.D0/4320.D0*t25*y+t2
     #/2-7.D0/6.D0*t70+1441.D0/2240.D0*t72*t43+t75*y/12+23.D0/9.D0*t78*t
     #46-t81*t5/12-43.D0/240.D0*t78*y-307.D0/53760.D0*t88*L1my-1879.D0/9
     #60.D0*t91*t6
      t105 = t1*Ly
      t112 = t2*t87
      t117 = 83.D0/64.D0*t91*t17+13361.D0/43200.D0*t12-851.D0/5760.D0*t2
     #5+2801.D0/12544.D0*t20+w/3+23.D0/18.D0*t91*t46+t70*y/2-t105*y/6-t7
     #5*t5/3-3127.D0/5040.D0*t78*t7+5.D0/12.D0*t112*L1my-749.D0/1536.D0*
     #t42*t5
      t124 = t25*t87
      t127 = t1*t87
      t134 = t12*Lw
      t141 = t20*Ly
      t144 = t3*Lw
      t147 = 13427.D0/5120.D0*t42*t17+312551.D0/20160.D0*t42*t7-37.D0/72
     #0.D0*t124*L1my-t127*L1my/2-t70*t5/6+19.D0/48.D0*t39*t17+85.D0/48.D
     #0*t134*t17-593.D0/7680.D0*t56*y-t65*y/3+57.D0/16.D0*t141*t46-2.D0/
     #3.D0*t144*t5
      t154 = t6**2
      t167 = w*t87
      t174 = -593.D0/3840.D0*t42*y+312551.D0/40320.D0*t56*t7-63041.D0/11
     #520.D0*t56*t43+75301.D0/53760.D0*t56*t154+19.D0/72.D0*t75*t17-307.
     #D0/360.D0*t15*t6+85.D0/96.D0*t15*t17-179.D0/48.D0*t141*t6+83.D0/32
     #.D0*t78*t17+t167*L1my/3-w*Ly/3-8861947.D0/627200.D0*t41*t7
      t188 = t4*t87
      t191 = t20*t87
      t202 = -14502179.D0/6451200.D0*t41*t17-112721173.D0/33868800.D0*t4
     #1*t154+19.D0/36.D0*t144*t17+t144*y/6+t81*y/4-19.D0/36.D0*t134*t5-9
     #7.D0/6720.D0*t188*L1my-71.D0/8064.D0*t191*L1my-307.D0/180.D0*t134*
     #t6-37.D0/72.D0*t78*t5-t72*y/6-19.D0/72.D0*t15*t5
      t213 = t3*t87
      t228 = 361.D0/192.D0*t141*t17-17.D0/72.D0*t134*y-311.D0/720.D0*t49
     #*t6+19.D0/24.D0*t49*t17-13.D0/18.D0*t49*y-t213*L1my/8-413.D0/720.D
     #0*t72*t5-311.D0/1440.D0*t39*t6-179.D0/24.D0*t72*t6+11.D0/40.D0*t15
     #*t46+57.D0/8.D0*t72*t46+13427.D0/10240.D0*t56*t17
      t252 = t12*t87
      t255 = 1441.D0/4480.D0*t141*t43-t141*y/12-37.D0/144.D0*t91*t5+2669
     #0753.D0/4838400.D0*t41*t46-749.D0/3072.D0*t56*t5-3407.D0/1008.D0*t
     #72*t7+11.D0/20.D0*t134*t46-413.D0/1440.D0*t141*t5+361.D0/96.D0*t72
     #*t17-241897.D0/368640.D0*t56*t6-37.D0/1440.D0*t252*L1my
      t274 = -31126219.D0/33868800.D0*t20*t43+12073.D0/258048.D0*t42-165
     #83.D0/21600.D0*t12*t46+17.D0/252.D0*t72+t75/9+t1*y/12+7033741.D0/6
     #04800.D0*t41*t43+17.D0/504.D0*t141-5911907.D0/12902400.D0*t41*y+31
     #.D0/288.D0*t78-7.D0/12.D0*t81+47.D0/720.D0*t25*t17
      t297 = 2.D0/9.D0*t144+23.D0/48.D0*t49+310464491.D0/1734082560.D0*t
     #41+13.D0/9.D0*t3*t5+75301.D0/26880.D0*t42*t154-13.D0/36.D0*t39*y-1
     #879.D0/480.D0*t78*t6-17.D0/144.D0*t15*y-43.D0/480.D0*t91*y-3127.D0
     #/10080.D0*t91*t7-3407.D0/2016.D0*t141*t7-73.D0/72.D0*t3*y
      t299 = L1my*t7
      t302 = L1my*t43
      t305 = L1my*t154
      t323 = L1my*y
      t326 = t134/5+11.D0/40.D0*t252*t299-3127.D0/10080.D0*t188*t302+144
     #1.D0/4480.D0*t191*t305+75301.D0/53760.D0*t88*L1my*t154*y+12073.D0/
     #516096.D0*t56+43094719.D0/22118400.D0*t41*t6+23685947.D0/25804800.
     #D0*t41*t5-234527.D0/120960.D0*t4*t17-337307.D0/48384.D0*t20*t46-75
     #19.D0/17280.D0*t12*y-t167*t323/3
      t333 = L1my*t5
      t336 = L1my*t6
      t339 = L1my*t17
      t342 = L1my*t46
      t357 = -3407.D0/2016.D0*t191*t302+17.D0/504.D0*t191*t323-43.D0/480
     #.D0*t188*t333+83.D0/64.D0*t188*t336-37.D0/144.D0*t188*t339-1879.D0
     #/960.D0*t188*t342+23.D0/18.D0*t188*t299+31.D0/576.D0*t188*t323-255
     #83.D0/322560.D0*t88*t333+235243.D0/184320.D0*t88*t336-21067.D0/921
     #60.D0*t88*t339-227977.D0/368640.D0*t88*t342
      t381 = -186823.D0/46080.D0*t88*t299+156433.D0/20160.D0*t88*t302-39
     #41.D0/720.D0*t88*t305+8639.D0/368640.D0*t88*t323-7.D0/12.D0*t112*t
     #323+t112*t333/4+t213*t333/12-t213*t339/3+t213*t323/9-17.D0/144.D0*
     #t252*t333+2.D0/3.D0*t127*t323+2.D0/3.D0*t105
      t406 = 5.D0/12.D0*t2*y-19.D0/72.D0*t2*t5+156959.D0/241920.D0*t4*t5
     #-145861.D0/60480.D0*t4*t46+31.D0/576.D0*t91+85.D0/96.D0*t252*t336-
     #19.D0/72.D0*t252*t339-307.D0/360.D0*t252*t342+t252*t323/10-13.D0/3
     #6.D0*t124*t333+19.D0/48.D0*t124*t336-t124*t339/144
      t431 = 23.D0/96.D0*t124*t323-t191*t333/12+361.D0/192.D0*t191*t336-
     #413.D0/1440.D0*t191*t339-179.D0/48.D0*t191*t342+57.D0/16.D0*t191*t
     #299-t127*t333/6-t112*t339/12+19.D0/72.D0*t213*t336-311.D0/1440.D0*
     #t124*t342-22303.D0/8640.D0*t20*t17+341945.D0/84672.D0*t20*t7
      t435 = t34+t62+t94+t117+t147+t174+t202+t228+t255+t274+t297+t326+t3
     #57+t381+t406+t431

      sigmaq1q2a=softy + t435 + sigmaq1q2aLs

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      double precision function sigmaq1q2b(z,y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 u,w, Ls
      real*8 Ldelta,Lxi,L1mxi,xi,r,delta,Lz
      external dilog,d1,DD0,DD1,DD2

      buff1=0.00000001

      z2=1.644934068D0

      r=dsqrt((z+y*(1.D0-z))/(1.D0-y*(1.D0-z)))
      t=dsqrt(z)

      delta=1.D0-t
      xi=y*(1.D0+r**2)*t/(r+t)
      Ldelta=dlog(delta+buff1)
      Lxi=dlog(xi+buff1)
      L1mxi=dlog(1-xi+buff1)
      Lz=dlog(1-z+buff1)
      Ls=dlog(mF**2/q**2)
      w=1-z

C     scale dependence

      t1 = Ls*w
      t3 = w**2
      t5 = t3*w
      t7 = t3**2
      t9 = t7*w
      t11 = -480+720*w-600*t3+180*t5+74*t7+37*t9
      t30 = 1110*Ls*t9-699*t9-7138*t7+2220*Ls*t7-38700*t5+5400*Ls*t5+804
     #00*t3-18000*Ls*t3-79200*w+21600*t1-14400*Ls+28800
      t34 = 1-y
      t35 = d1(t34)
      t37 = DD0(t34)
      t41 = (-t1*t11*Lz/720-t1*t30/86400)*t35-t1*t11*t37/1440

      tsoftyLs = t41


      t3 = 10080*xi
      t6 = delta**2
      t9 = xi**2
      t22 = t6**2
      t25 = -2.D0/3.D0*Ls*delta-Ls*(-35280-t3)*t6/15120-Ls*(20160*t9+806
     #40+t3)*t6*delta/15120-Ls*(47880*xi+43680*t9*xi-113400-120960*t9)*t
     #22/15120

      sigmaq1q2bLs = t25 + tsoftyLs

      t1 = w**2
      t2 = t1*w
      t5 = t1**2
      t8 = t5*w
      t11 = w*(180*t2+720*w+74*t5-480-600*t1+37*t8)
      t12 = Lz**2
      t21 = w*(28800+56400*t1-79200*w+3182*t5-41100*t2+5741*t8)
      t40 = 377461*t8+66600*t8*z2+991742*t5+133200*t5*z2+324000*t2*z2-10
     #38750*t2-1080000*t1*z2+2400000*t1-2916000*w+1296000*w*z2+1296000-8
     #64000*z2
      t44 = 1-y
      t45 = d1(t44)
      t47 = DD1(t44)
      t54 = DD0(t44)
      t56 = (-t11*t12/720-t21*Lz/43200+w*t40/2592000)*t45-t11*t47/1440+(
     #-t11*Lz/720-t21/86400)*t54

      softy = t56

      t1 = Lxi*xi
      t2 = dlog(2.D0)
      t3 = t2*xi
      t5 = xi*Ldelta
      t7 = L1mxi*xi
      t12 = 1/(-1+xi-buff1)
      t16 = xi**2
      t17 = L1mxi*t16
      t20 = t2*t16
      t22 = Lxi*t16
      t24 = Ldelta*t16
      t34 = 2*t17+3*t16+4*t20+2*t22+4*t24+3*t1+10*t5+5*t7-13*xi+10*t3-14
     #*t2+10-14*Ldelta-7*L1mxi
      t36 = delta**2
      t39 = t16*xi
      t40 = t2*t39
      t43 = Ldelta*t39
      t45 = t39*L1mxi
      t47 = Lxi*t39
      t61 = 48*t40+44*t39+48*t43+24*t45+24*t47+6*t16-24*t20-24*t24-12*t1
     #7+84*t7+48*t1+168*t3-171*xi+168*t5+121-96*L1mxi-192*Ldelta-192*t2
      t66 = t16**2
      t83 = -1862*t66+624*Ldelta*t66+624*t2*t66+312*t66*L1mxi+312*Lxi*t6
     #6-1176*t45-2352*t40-2352*t43-1248*t47+3572*t39+2412*t20+1206*t17
      t95 = -1248-3501*t16+2412*t24+1170*t22-2304*t5+3039*xi-2304*t3-115
     #2*t7-702*t1+810*L1mxi+1620*Ldelta+1620*t2
      t98 = t36**2
      t101 = -2.D0/3.D0*(t1+2*t3-xi+2*t5+t7-2*Ldelta-2*t2-L1mxi+1)*t12*d
     #elta+t34*t12*t36/3-t61*t12*t36*delta/18-(t83+t95)*t12*t98/108

      sigmaq1q2b = softy + t101 +  sigmaq1q2bLs

      end


      double precision function sigmaq1q2c(z,y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 u,w, Ls
      real*8 Ldelta,Ly,L1my,xi,r,delta,Lz
      external dilog,d1,DD0,DD1,DD2

      buff1=0.0000001

      z2=1.644934068D0


      Ly=dlog(y+buff1)
      L1my=dlog(1-y+buff1)
      Ls=dlog(mF**2/q**2)
      w=1D0-z
      Lw=dlog(w)

      t1 = w**2
      t2 = t1*w
      t3 = t1**2
      t4 = t3*t2
      t6 = t3*t1
      t8 = t3**2
      t10 = t8*w
      t12 = y**2
      t13 = t12**2
      t18 = t13**2
      t21 = t13*t12
      t26 = t12*y
      t43 = t13*y
      t50 = -145291.D0/1058400.D0*t4-383.D0/1800.D0*t6-3664909.D0/451584
     #00.D0*t8-23397019.D0/541900800.D0*t10-228829393.D0/108380160.D0*t1
     #0*t13+950917.D0/12902400.D0*t10*y-10649353.D0/33868800.D0*t10*t18-
     #4143053.D0/967680.D0*t10*t21-205454521.D0/541900800.D0*t10*t12+114
     #706187.D0/135475200.D0*t10*t26+175729.D0/529200.D0*t4*y-121.D0/300
     #.D0*t6*t12+8249.D0/43200.D0*t6*t13-8249.D0/21600.D0*t6*t26+25673.D
     #0/43200.D0*t6*y-1975661.D0/8467200.D0*t8*t12-4766203.D0/16934400.D
     #0*t8*t26-946559.D0/752640.D0*t8*t43+946559.D0/2257920.D0*t8*t21+41
     #2567.D0/2508800.D0*t8*y
      t55 = t3*w
      t89 = 47.D0/108.D0*t2*t12-47.D0/108.D0*t2*y+5917.D0/43200.D0*t55*t
     #13+39019.D0/43200.D0*t55*y-731.D0/864.D0*t3*t12+731.D0/864.D0*t3*y
     #-1133263.D0/4233600.D0*t4*t12+2560363.D0/4233600.D0*t4*t13-381299.
     #D0/705600.D0*t4*t43+381299.D0/2116800.D0*t4*t21-613.D0/800.D0*t55*
     #t12+66307963.D0/33868800.D0*t10*t13*t26+71262817.D0/16934400.D0*t1
     #0*t43-18163.D0/58800.D0*t4*t26+7.D0/12.D0*t1+47.D0/108.D0*t2-12073
     #.D0/43200.D0*t55-371.D0/1728.D0*t3-5917.D0/21600.D0*t55*t26+805243
     #31.D0/67737600.D0*t8*t13
      t90 = t50+t89

      sigmaq1q2c = t90

      end




CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Functions for parton luminosities
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     Measure factors

      double precision function DDinv(y)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      xxx=buff
      xxx1=1.D0-buff

      if (y.lt.xxx) then
      DDinv=buff*(1.D0-buff)/((1.D0-buff)*dlog(buff)+buff)
      else
      if (y.lt.xxx1) then
      DDinv=y*(1.D0-y)
      else
      DDinv=buff*(1.D0-buff)/((1.D0-buff)*dlog(buff)+buff)
      end if
      end if

      end


      double precision function DD0inv(z)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      xxx=buff

      if (z.gt.xxx) then
      DD0inv=z
      else
      DD0inv=buff*1.D0/dlog(buff)

      end if

      end



C****************************************************************************
C   Plus distributions
C****************************************************************************

C delta-function

      double precision function d1(z)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf


      xxx=buff

      if (z.gt.xxx) then
      d1=0.D0
      else
      d1=1.D0/buff

      end if


      end
************************************************************


      double precision function DD0(z)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      xxx=buff

      if (z.gt.xxx) then
      DD0=1.D0/(z)
      else
      DD0=1.D0/buff*dlog(buff)

      end if

      end


      double precision function DD1(z)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      xxx=buff

      if (z.gt.xxx) then
      DD1=1.D0/(z)*dlog(z)
      else
      DD1=1.D0/2.D0/buff*dlog(buff)**2

      end if

      end

      double precision function DD2(z)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf


      xxx=buff

      if (z.gt.xxx) then
      DD2=1.D0/(z)*dlog(z)**2
      else
      DD2=1.D0/3.D0/buff*dlog(buff)**3

      end if

      end

      double precision function DD3(z)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf


      xxx=buff

      if (z.gt.xxx) then
      DD3=1.D0/(z)*dlog(z)**3
      else
      DD3=1.D0/4.D0/buff*dlog(buff)**4

      end if

      end


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   DILOGARITHMS
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      double precision function dilog(x)
      implicit real*8(a-h,o-z)

      z2=1.644934068

      if(x.lt.-0.5D0) then

      z=-x/(1.D0-x)
      res1 = z+.2500000000*z**2+.1111111111*z**3+0.06250000000*z**4
     , +0.04000000000*z**5+0.02777777778*z**6+0.02040816327*z**7
     , +0.01562500000*z**8+0.01234567901*z**9+0.01000000000*z**10
     , +0.008264462810*z**11+0.006944444444*z**12+0.005917159763*z**13

      res=-res1-1.D0/2.D0*dlog(1-x)**2

      else

      if(x.gt.0.5D0) then
      z=1D0-x

      res1 = z+.2500000000*z**2+.1111111111*z**3+0.06250000000*z**4
     , +0.04000000000*z**5+0.02777777778*z**6+0.02040816327*z**7
     , +0.01562500000*z**8+0.01234567901*z**9+0.01000000000*z**10
     , +0.008264462810*z**11+0.006944444444*z**12+0.005917159763*z**13

      res = -res1-dlog(x)*dlog(1D0-x)+z2

      else

      z=x
      res = z+.2500000000*z**2+.1111111111*z**3+0.06250000000*z**4
     , +0.04000000000*z**5+0.02777777778*z**6+0.02040816327*z**7
     , +0.01562500000*z**8+0.01234567901*z**9+0.01000000000*z**10
     , +0.008264462810*z**11+0.006944444444*z**12+0.005917159763*z**13

      end if
      end if

      dilog=res

      return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     TRILOGS
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      double precision function trilog(x)
      implicit real*8(a-h,o-z)
      external dilog,dS12

      z2=1.644934068
      z3=1.202056903

      if(x.lt.-0.5D0) then
      z=-x/(1D0-x)

      res1 = z+0.1250000000*z**2+0.03703703704*z**3
     , +0.01562500000*z**4+0.008000000000*z**5
     , +0.004629629630*z**6+0.002915451895*z**7
     , +0.001953125000*z**8+0.001371742112*z**9
     , +0.001000000000*z**10+0.0007513148009*z**11
     , +0.0005787037037*z**12+0.0004551661356*z**13

      res = -res1 +dS12(x)+dlog(1-x)*dilog(x)+1D0/6D0*dlog(1-x)**3

      else

      if(x.gt.0.5D0) then
      z=1D0-x

      res = -dS12(z)+dlog(x)*dilog(x)+1D0/2D0*dlog(z)*dlog(x)**2+z3

      else

      z=x
      res = z+0.1250000000*z**2+0.03703703704*z**3
     , +0.01562500000*z**4+0.008000000000*z**5
     , +0.004629629630*z**6+0.002915451895*z**7
     , +0.001953125000*z**8+0.001371742112*z**9
     , +0.001000000000*z**10+0.0007513148009*z**11
     , +0.0005787037037*z**12+0.0004551661356*z**13
      end if
      end if

      trilog=res

      return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      double precision function dS12(x)
      implicit real*8(a-h,o-z)
      external dilog,trilog

      z2=1.644934068
      z3=1.202056903

      if(x.lt.-0.5D0) then
      z=-x/(1.D0-x)

      res1 = 0.2500000000*z**2+0.1666666667*z**3+0.1145833333*z**4
     , +0.08333333333*z**5+0.06342592593*z**6+0.05000000000*z**7
     , +0.04051339286*z**8+0.03355379189*z**9+0.02828968254*z**10
     , +0.02420634921*z**11+0.02097137045*z**12

      res = res1 + 1.D0/6.D0*dlog(1-x)**3

      else

      if(x.gt.0.5D0) then
      z=1.D0-x

      res = -trilog(z)-dlog(z)*dilog(x)
     , -1.D0/2.D0*dlog(x)*dlog(z)**2+z2*dlog(z)+z3

      else

      z=x
      res = 0.2500000000*z**2+0.1666666667*z**3+0.1145833333*z**4
     , +0.08333333333*z**5+0.06342592593*z**6+0.05000000000*z**7
     , +0.04051339286*z**8+0.03355379189*z**9+0.02828968254*z**10
     , +0.02420634921*z**11+0.02097137045*z**12

      end if
      end if

      dS12=res

      return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  The running strong coupling constant devided by pi
C  in dependence of scale, 5 flavors
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



      double precision function SSC(xx)
      implicit real*8(a-h,o-z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 Mz, q2

      q2=q**2

      as=alphasDY(q2)/3.1415924

      SSC=as

      return
      end
c--------------
      real*8 function dyi(xx)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      COMMON /fordyi/ eu,ed,es,ec,eb,vu,vd,vs,vc,vb,as,ac,ab
     , ,x,u,q,buff,mF,wg,wz,wzg,alphaspi
     , ,mode,mode1,mode2,morder
      COMMON /fordyren/ fscale_dy,rscale_dy
      real*8 mf

      real*8 xx(2),nlocr,nnlocr

      dyi=alphaspi*nlocr(xx)+alphaspi**2*nnlocr(xx)

      return
      end

c--------------
      real*8 function dynloi(xx)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      real*8 xx(2),nlocr

      dynloi=nlocr(xx)

      return
      end

c--------------
      real*8 function dynnloi(xx)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      real*8 xx(2),nnlocr

      dynnloi=nnlocr(xx)

      return
      end
c--------------
      subroutine dynlor(ll,xx,aa,bb)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      real*8 aa(2),bb(2),xx(2)

      del=1d-4

      aa(1)=del
      bb(1)=1-del
      aa(2)=del
      bb(2)=1-del

      return
      end

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C     FUNCTION SD2(P,F,REGION)   COMPUTES INTEGRAL
C
C                                      B        2
C                                SD2 = $ F(X) DX
C                                      A

C  WITH ABSOLUTE PRECISION  P.
C  FUNCTION F  AND SUBROUTINE REGION   SHOULD BE DECLARED AS EXTERNAL.
C  LIMITS A(L),B(L), L<=2, AS FUNCTIONS OF X(L-1),...,X(1) SHOULD BE COMPUTED
C  IN THE SUBPOUTINE REGION(L,X,A,B)

C  OPTIONS --------------------------------------------------------------

C  OUTPUT: LEVEL OF PROTOCOL OUTPUT, ESTIMATE OF ROOT-MEAN-SQUARE ERROR,
C          INSTANT APPER LIMITS AND VALUES OF INTEGRALS, NUMBRS OF QUADRATURES,
C          TOTAL INSTANT VALUES OF INSTNTEGRALS, NUMBER OF LINES OUTPUT, NUMBER
C          OF POINTS USED:
C /SD2OUT/LETOUT,NLINES,NQUAD(2),NPOINT(3)
C       ,RMSERR(2),CLIMIT(2),SUMTOC(2),TOT(2),EXTRA1(2),EXTRA2(2)
C         IF LETOUT>0, PROGRESS REPORT OF LEVEL 'LETOUT' IS OUTPUT ON CHANNEL 1
C  INPUT: LIMITS ON SEGMEMT AND GAP SIZES:/SD2GAP/ STEPMX(2),GAPMAX(2),GAPMIN(2)
C       : 10 HIT POINTS BETWEEN A AND B, WHERE INTEGRAL FROM A TO HIT SHOULD
C         BE COMPUTED: /SD2HIT/NHITS(2),HIT(2,10),YHIT(2,10)

C ONE-STEP MODE: IF P=0, 1 STEP IN EVERY DITECTION IS MADE, USING 8-INTERVAL
C                       QUADRATURE FORMULA

      real*8 FUNCTION SD2(P,F,REGION)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      COMMON /SD2GAP/STEPMX(2),GAPMAX(2),GAPMIN(2)
     *       /SD2HIT/NHITS(2),HIT(2,10),YHIT(2,10)
     *       /SD2OUT/ RMSERR(2),CLIMIT(2),SUMTOC(2),TOT(2),RQA(2),RQB(2)
     *              ,LETOUT,NLINES,NQUAD(2),NPOINT(3)

C --------------------- ARRAYS OF INTERNAL USE -------------------------------
      COMMON /SD214V/
     * X1(2),G(2),PA(2),ESTER,DS, GA(2,2),GB(2,2),Y(2,21)
     *   ,YI(2,16),YT(2,19),errold,L,N(2),INS(2,16),MADE(2,5)
     *                 ,NOLD,MOLD,MODY

      DIMENSION Z(2,200),DW(2),YW(2),
     *          XACCUM(5),YACCUM(5),JQ(2)

      LOGICAL NOMEM(2)

      DIMENSION SUM(3),IN(2),LB(2),LINS(2),M(2),LOST(2),IHIT(2)
     *,SQUERR(2),KIND(2),A(2),B(2),D(2),P03(2),P200(2),X2(2)
      DIMENSION  LEAP(2),LEAP4(2),E1(2),E11(2),PR(2),PR1(2)
     *,LOT(2),LOT0(2),LOT1(2)    ,IODD(2),DS0(2)
      DIMENSION X(2),SGMAX(2),GMIN(2),GMAX(2)
      DIMENSION  W(81),W1(20),W2(29),W3(32)
      EQUIVALENCE (W(1),W1(1)),(W(21),W2(1)),(W(50),W3(1))
      DATA ACCURA/1.E-8/
C     ACCURA: RELATIVE ACCURACY OF FLOATING POINT NUMBERS MULTIPLIED BY 10
      DATA W1/
     *  1.0,
     1  0.5,
     2  0.1666666666666667,  0.3333333333333333,
     3  0.1250000000000000,  0.3750000000000000,
     4  0.0777777777777778,  0.3555555555555556,  0.0666666666666666,
     5  0.0659722222222222,  0.2604166666666667,  0.1736111111111111,
     6  0.0488095238095238,  0.2571428571428571,  0.0321428571428571,
     *  0.1619047619047619,
     7  0.0434606481481481,  0.2070023148148148,  0.0765625000000000,
     *  0.1729745370370370/
       DATA W2/
     8  0.0348853615520282,  0.2076895943562610, -0.0327336860670194,
     *  0.3702292768959436, -0.0800705467372134,
     9  0.0318861607142857,  0.1756808035714286,  0.0120535714285714,
     *  0.2158928571428571,  0.0644866071428571,
     1  0.0268341483619261,  0.1775359414248303, -0.0810435706269040,
     *  0.4549462882796216, -0.4351551226551227,  0.3568823152156486,
     1  0.0249332309119636,  0.1548553585207231, -0.0371692317937978,
     1  0.2896582547949735, -0.1101780891754850,  0.1779004767416226,
     1  0.0216394874966304,  0.1570361067503925, -0.1203219637505352,
     2  0.5664988979274694, -0.8165056372199229,  1.3877596689025260,
     * -0.6961065601065601/
       DATA W3/
     1  0.0203347191051236,  0.1398760852657854, -0.0777118702884142,
     3  0.3878961542438828, -0.3769238163321176,  0.5136761795561554,
     * -0.1071474515504153,
     1  0.0180344712157984,  0.1420877946927330, -0.1540253470523532,
     4  0.6997489104402685, -1.3239976056465254,  2.5240777544357791,
     * -3.3578644895056932,  1.9519385114199930,
     1  0.0170872997716259,  0.1285073786774052, -0.1127229050595052,
     5  0.5070427082102236, -0.7562931148441402,  1.1913603495067990,
     * -0.9680052114962480,  0.4930234952338398,
     1  0.0153989471166495,  0.1306411914401402, -0.1839764466493380,
     6  0.8518689889045043, -1.9750740358891965,  4.2762649967201477,
     * -6.9673071185989400,  9.5901711055393664, -5.2379876285833335/

C ENTRY
      ND=2
      SD2=0d0

      DO 2 I=1,ND
    2 NPOINT(I)=0
      NACCUM=0
      NLINES=0
      PA(1)=ABS(P)*1.5
      L=0
          GO TO 30
C               30: RECURSIVE CALL OF SDX-INS
C---------------------------------------------------------------------------
C FUNCTION INSERT (L,IN,F): INSERTS POINT INTO THE GAP 'IN'

   10 IF(INS(L,IN(L)).EQ.1) GO TO 14
      INS(L,IN(L))=1
      X(L)=X1(L)+(IN(L)-0.5)*G(L)
          LB(L)=1
             GO TO 30
   12 YI(L,IN(L))=SUM(L+1)
   14 LI=LINS(L)
      GO TO(1001,1002,1003,1004,1005,1006,1007,1008,1009,1010
     *     ,1011,1012),LI
C--------------------------------------------------------------------------

C RECURSIVE CALL OF SDX-INS

   30 L=L+1

      IF(L.LE.ND)GO TO 60
C                      60: START OF INTEGRATION OF L-LEVEL
C-------------- L=ND+1 ------------
      SUM(L)=F(X)
C L-TH RETURN --------------------
   40 NPOINT(L)=NPOINT(L)+1
      IF(L-1.GT.LETOUT)GO TO 49
      IF(L.LE.LETOUT)GO TO 44
      NACCUM=NACCUM+1
      IF(L.GT.1)                XACCUM(NACCUM)=X(L-1)
      YACCUM(NACCUM)=SUM(L)
      IF(NACCUM.LT.5)GO TO 49
      WRITE(1,5040)(XACCUM(I),YACCUM(I),I=1,5)
 5040 FORMAT(5(E16.6,E15.6))
      NLINES=NLINES+1
      NACCUM=0
      GO TO 49
   44 IF(L.EQ.LETOUT.AND.NACCUM.GT.0)WRITE(1,5041)
     *                            (XACCUM(I),YACCUM(I),I=1,NACCUM)
      NACCUM=0
 5041 FORMAT(5(E16.6,E15.6))
      LO=L-1
      XL1=0d0
      IF(L.GT.1)XL1=X(LO)
      WRITE(1,5042)L,LO,XL1,L,SUM(L),RMSERR(L),NQUAD(L),
     *                           (NPOINT(I),I=1,ND+1)
 5042 FORMAT(' L=',I2,'  X(',I1,')=',E17.9,'  INTEGRAL(',I2,')=',E17.9
     *,'  ER=',E10.2,' NQ',I8,' N',9I9)
      NLINES=NLINES+1
   49 L=L-1
      IF(L.EQ.0)GO TO 50
      LBL=LB(L)
      GO TO     (12,90,116),LBL
C EXIT    -----------------------


   50 SD2=SUM(1)
      RETURN
C========================================================================

C START
C      CHECKS  CONSISTENCY OF STEP AND GAP LIMITS
C      MAKES  B-A>=SGMAX>=GMAX>=GMIN
C      NULLIFIES  SDX ETC.
C      SETS   X1=A, Y(3)=F(A)
C ZERO
   60 SUM(L)=0d0
      TOT(L)=0d0
      M(L)=0
      LOST(L)=0
      SQUERR(L)=0d0
      IHIT(L)=0
      NQUAD(L)=0
      KIND(L)=0
         JQ(L)=0
      CALL REGION(L,X,A,B)
      IF(L.GT.LETOUT)GO TO 70
      NLINES=NLINES+4
      IF(L.GT.1)GO TO 61
      WRITE(1,5060)A(1),B(1)
 5060 FORMAT(' 1  ++++++++++++++++      A1 B1',2E20.9/)
      GO TO 70
   61 LO=L-1
      IF(JQ(LO).EQ.0)WRITE(1,5061)LO,NQUAD(LO),X1(LO),SUM(LO),TOT(LO)
     *,RMSERR(LO)
      IF(JQ(LO).EQ.1)WRITE(1,5061)LO,NQUAD(LO),X1(LO),SUM(LO),TOT(LO)
     *,RMSERR(LO),RQA(LO),RQB(LO)
         JQ(LO)=0
 5061 FORMAT(' STATE OF L=',I1,' : NQ',I3
     *,' INTEGRAL (FROM A TO C=',E18.9,') =',E18.9
     *,'  TOTAL',E17.9,' ERR',E10.2,' EXTRA',2E14.6)
      GO TO(62,63,64,65,66,67,68),LO
   62 WRITE(1,5062)
 5062 FORMAT(/20X,' 2  +++++++++++++++++')
      GO TO 69
   63 WRITE(1,5063)
 5063 FORMAT(/40X,' 3  +++++++++++++++++')
      GO TO 69
   64 WRITE(1,5064)
 5064 FORMAT(/60X,' 4  +++++++++++++++++')
      GO TO 69
   65 WRITE(1,5065)
 5065 FORMAT(/80X,' 5  ++++++++++++++++')
      GO TO 69
   66 WRITE(1,5066)
 5066 FORMAT(/100X,' 6  ++++++++++++++++')
      GO TO 69
   67 WRITE(1,5067)
 5067 FORMAT(/120X,' 7  ++++++++++++++++')
      GO TO 69
   68 WRITE(1,5068)
 5068 FORMAT(/140X,' 8  ++++++++++++++++')

   69 WRITE(1,5069)LO,X(LO),L,A(L),L,B(L)
 5069 FORMAT('  X(',I1,')=',E17.9,'     A(',I1,')=',E17.9,
     *                              '  B(',I1,')=',E17.9)

   70       IF(B(L).EQ.A(L))GO TO 40
C                                 40: L-TH RETURN
C LIMITS
      D(L)=B(L)-A(L)
      AD=ABS(D(L))
           R=ABS(STEPMX(L)/D(L))
      SGMAX(L)=AD*R
              IF(R.LT.ACCURA.OR.R.GT.1.)SGMAX(L)=AD
           R=ABS(GAPMAX(L)/AD)
      GMAX(L)=SGMAX(L)*R
              IF(R.LT.ACCURA.OR.R.GT.1.)GMAX(L)=SGMAX(L)
           R=ABS(GAPMIN(L)/GMAX(L))
      GMIN(L)=GMAX(L)*R
              IF(R.LT.ACCURA.OR.R.GT.1.)GMIN(L)=GMAX(L)*ACCURA
          D1=ABS(A(L))+ABS(B(L))
          GMIN4=D(L)*GMIN(L)/(4.*AD)
      IF(A(L)+GMIN4.EQ.A(L).OR.B(L)-GMIN4.EQ.B(L))GMIN(L)=D1*ACCURA
      IF(P.GT.0d0)GO TO 71


C P=0 CASE
      GMIN(L)=SGMAX(L)/8

C PRECISION
   71 IF(L.GT.1) PA(L)= PA(L-1)/ABS( D(L-1 ) )
      P03(L)=PA(L)*0.3d0
      P200(L)=PA(L)*200d0

C ERASE
      DO 72 I=1,2
      GA(L,I)=0d0
   72 GB(L,I)=0d0
C PUT
      X1(L)=A(L)
C----------------------Y(3)=F(A)
      X(L)=A(L)
      LB(L)=2
             GO TO 30
   90 Y(L,3)=SUM(L+1)
C111111111111111111111111111111111111111111111111111111111

C SET                   GO   TO 140:OLD, 120:RECALL, 130:JUMP
  100 CLIMIT(L)=X1(L)
      NOLD=N(L)
      IF(KIND(L).NE.0)GO TO 140
C --------------------------------------------------------------------
C                             STEP WITH NEW BEGINNING X1

      IF(X1(L).EQ.A(L))GO TO 102
C                           PUT LEFT WING BY SUBROUTINE WING(KIND,LOT,...)
      CALL WING2(2,0,DW,YW)
      Y(L,2)=YW(1)
      Y(L,1)=YW(2)
      GA(L,1)=DW(1)
      GB(L,1)=DW(2)-DW(1)
      Y(L,3)=Y(L,3+N(L))
  102 IF(LOST(L).EQ.0.AND.M(L).NE.0)GOTO 120
C                                  120: RECALL
C ---------------------------------------------------------------------


C NEW  .......                        (CASES M=0, LOST=0 AND M>0, LOST>0)

C GAP SIZE AND RIGHT WING SETTING
  110 IF(LOST(L).GT.0)GOTO 118
C          NOTHING IS LOST. GAP SIZE -   IN MOVE TOWARD B (LOST=0,M=0)
          G(L)=SGMAX(L)
          G1=B(L)-X1(L)
            IF(G1/G(L).LT.1.)G(L)=G1

C>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>    RETURN TO 40

          IF(((X1(L)+0.5*G(L))-X1(L))/D(L).EQ.0d0) GO TO 40

          IF(IHIT(L).GE.NHITS(L))GO TO 114
             HI=HIT(L,IHIT(L)+1)
                IF(HI.EQ.X1(L))GO TO 112
             IF((HI-X1(L))*(X1(L)+G(L)-HI).LT.0d0)GO TO 114
          G(L)=HI-X1(L)
  112 YHIT(L,IHIT(L))=SUM(L)
          IHIT(L)=IHIT(L)+1
C                                             WING-2
  114     X2(L)=X1(L)+G(L)
C---------------------- Y(4)=F(X2)
          X(L)=X2(L)
          LB(L)=3
                 GO TO 30
  116 Y(L,4)=SUM(L+1)
          GA(L,2)=0d0
            GOTO 119
C                      GAP SIZE -  IN MOVE TOWARD X2. CASE LOST>0 (HENCE M>0)
  118     X2(L)=Z(L,M(L)-2)
          G(L) =X2(L)-X1(L)
          IF(((X1(L)+0.5*G(L))-X1(L))/D(L).EQ.0d0)GO TO 120
C                        120: RECALL             WING-2
          N0=Z(L,M(L)-1)+8.1
                       IF(Z(L,M(L)).GT.0d0)N0=7
          Y(L,4)=Z(L,M(L)-N0)
          Y(L,5)=Z(L,M(L)-N0+1)
          Y(L,6)=Z(L,M(L)-N0+2)

          GA(L,2)=Z(L,M(L)-4)
          GB(L,2)=Z(L,M(L)-3)
C                       SET N
  119     N(L)=1
          INS(L,1)=0
           GO TO 190
C                190: PUT YT FOR TRY

C RECALL                FROM MEMORY Z(200)
  120 LOST(L)=0
      IF(Z(L,M(L)).GT.0.5) GO TO 130
C                           130: JUMP
C --------------------- TAKE FROM MEMORY:

      INSN=-Z(L,M(L))+0.1
      N(L)=Z(L,M(L)-1)+0.1
      X1(L)=Z(L,M(L)-2)
      GB(L,2)=Z(L,M(L)-3)
      GA(L,2)=Z(L,M(L)-4)
      G(L)=Z(L,M(L)-5)
C ---------------------- NODES
      NL=N(L)
      DO 121 I=1,NL+2
  121 Y(L,3+I)=Z(L,M(L)-NL-8+I)
C                        ERASE INS
      DO 122 I=1,NL
  122 INS(L,I)=0

      M(L)=M(L)-NL-9-2*INSN
C                        INSERTED
      IF(INSN.EQ.0)GOTO 190
          DO 124  I=1,INSN
             II=Z(L,M(L)+I)+0.1
          YI(L,II)=Z(L,M(L)+INSN+I)
  124  INS(L,II)=1
      GO TO 190
C           190: MAKE YT FOR TRY

C JUMP ============================ JUMP  OVER THE HOLE
  130 X1(L)=Z(L,M(L)-1)
C                         SUM SDX WITH INTEGRAL OVER THE HOLE
      SUM(L)=SUM(L)+Z(L,M(L)-13)
      GB(L,1)=Z(L,M(L)-12)
      GA(L,1)=Z(L,M(L)-11)
       Y(L,1)=Z(L,M(L)-10)
       Y(L,2)=Z(L,M(L)-9)
       Y(L,3)=Z(L,M(L)-8)
        M(L)  =M(L)-14
C                         GO TO 110:NEW, 120:RECALL
      IF(M(L).EQ.0)GO TO 110
                     GO TO 120
C------------------------------------------------------------------------
C          PROCEDURE      OLD(LOT,KIND)
C
C    RECOINS OLD SEGMENT, OR ITS PART, INTO NEW SEGMENT

C   INPUT :
C           N: OF OLD SEGMENT
C         LOT: NUMBER OF GAPS OF OLD SEGMENT THAT WAS USED (SUMMED OR STORED)
C        KIND: FIXED END OF THE PART THAT WAS USED

C       USES  ARRAYS Y,YI,YNAIL  AND INDICATORS AND GAPS RELATED
C             YT(19): WORKING ARRAY OF MERGE
C   VARIABLES:

C               NOUT: NUMB. OF GAPS TAKEN OUT
C               NEW : N OF NEW SEGMENT
C               KO  : OPPOSIT KIND (KIND OF PART TAKEN OUT)

C              FIND KIND,LOT OF RECOINED PART
  140 KO=KIND(L)
      KIND(L)=3-KO
      NOUT=LOT(L)
      LOT(L)=N(L)-NOUT
      IF(NOUT.GT.0)CALL WING2(KIND(L),LOT(L),DW,YW)
      IF(KIND(L).EQ.1)GO TO 144
C SHIFT                      OF  RIGHT PART
      LOTC=LOT(L)
      DO 141 I=1,LOTC+3
  141 Y(L,2+I)=Y(L,2+NOUT+I)
      DO 142 I=1,LOTC
      YI(L,I)=YI(L,I+NOUT)
  142 INS(L,I)=INS(L,I+NOUT)
C                       PLACE WING
      Y(L,2)=YW(1)
      Y(L,1)=YW(2)
      GA(L,1)=DW(1)
      GB(L,1)=DW(2)-DW(1)
      N(L)=LOTC
           GO TO 190
C                190: MAKE YT FOR TRY

C MERGE ----------------  ARRAYS Y AND YI INTO YT
  144 NEW=LOT(L)+LOT(L)
      YT(L,1)=Y(L,3)
      LOTC=LOT(L)
      DO 145 I=1,LOTC
                   YT(L,I+I)=YI(L,I)
  145              YT(L,1+I+I)=Y(L,3+I)
C                                   ERASE  INS
      DO 146 I=1,NEW
  146 INS(L,I)=0

C -------------------------------------------
C             FIXED END
  150 IF (NOUT.EQ.0) GO TO 176

C     ----------------------------------------------
C          PLACE WINGS
C                                PLACE GAPS AND ERASE NAIL
      GA(L,2)=DW(1)
      GB(L,2)=DW(2)-DW(1)
C     ----------------------------------------
C                        PLACE (FREE) RIGHT WING
      YT(L,2+NEW)=YW(1)
      Y(L,5+NEW)=YW(2)
      GO TO 180
C           180:MIDDLE
C     ----------------------------------------
C                        PLACE (FIXED) RIGHT WING
  176 YT(L,2+NEW)=Y(L,4+N(L))
      Y(L,5+NEW)=Y(L,5+N(L))
C     ----------------------------------------

C MIDDLE                 PLACE MIDDLE OF SEGMENT
  180 DO 182 I=1,NEW+2
  182 Y(L,2+I)=YT(L,I)
C             SET NEW N AND G
      G(L)=G(L)*0.5
      N(L)=NEW
            GO TO 200
C           PUT YT FOR TRY
  190 NL=N(L)
      DO 192 I=1,NL+1
  192 YT(L,I)=Y(L,2+I)


C22222222222222222222222222222222222222222222222222222222222222222222

C TRY :
C       SETS (LOT,KIND) AND TRANSFERS CONTROL TO SUM, OR SET, OR HOLE, OR STORE
C       CHANGES ARRAYS  NAIL AND  INS.   USES YT FOR FILTERING

C33333333333333333333333333333333333333333333333333333333333333333333
C22222222222222222222222222222222222222222222222222222222222222222222222

C     PROCEDURE TRY(LOT,KIND,JOB,NOMEM)


C    TRIES SEGMENT AND ITS PARTS, WHETHER THEY ARE OKEY FOR QUADRATURE
C OUTPUTS: (LOT,KIND): OF SECTION

C TRY ========= ERASE E-FUNCTION MEMORY
  200 DS=0d0
      DO 201 I=1,2
      MADE(L,I)=0
  201 MADE(L,I+2)=0

C CONTROL GAP SIZE.1:  IF G IS TOO BIG, MAKE INSERTIONS WITHOUT TRIES
      LOT(L)=0
           IF(ABS(G(L)).GT.GMAX(L))GO TO 250
C                                  N=1 WITH INS-POINT: N=2 WITH G/2
      KIND(L)=2
      IF(N(L).EQ.1.AND.INS(L,1).EQ.1)GO TO 100
C                 2: IF G IS TOO SMALL, SUM WITHOUT CHECKS
      KIND(L)=1
      LOT(L)=N(L)
C ALTERNATION OF YT
      DO 202 I=1,N(L),2
  202 YT(L,1+I)=-YT(L,1+I)
      E1(L)=E2(1,N(L),1)
           IF(ABS(G(L)).LT.2*GMIN(L))GO TO 500
C --------------------------            500: SUM
C                                   INSERT POINT NEAR BARE ENDS
      IF(GA(L,1).NE.0d0)GO TO 3000
        IF(N(L).EQ.2.AND.10d0*E1(L).LE.PA(L))GO TO 500
C====                                CALL INSERT(1,F)
      IN(L)=1
      LINS(L)=1
               GO TO 10

 1001 IF(N(L).GT.1)GO TO 3000
      GO TO 3001
 3000 IF(GA(L,2).NE.0d0)GO TO 3002
C====                             CALL INSERT(LOT,F)
      IN(L)=LOT(L)
      LINS(L)=2
              GO TO 10
 1002 IF(N(L).GT.1)GO TO 3002
 3001 KIND(L)=2
      LOT(L)=0
      GO TO 100

C                      LEAP: LEAP=1 IS A SIGNAL TO SKIP SOME CHECKS
 3002 LEAP(L)=1

C  TOTAL  ------------------- IS TOTAL SEGMENT OKEY?

C FILTERING
            IF(E1(L).GT.P200(L)) GO TO 230
C                                230: PARTS (WITH LEAPS)
C---------------------------------------------------
      EP=E1(L)+P03(L)
      PR(L)=PA(L)
      IF(EP.LT.PA(L))PR(L)=EP
      IF(INS(L,1).EQ.1)PR(L)=PA(L)
      IF (N(L).NE.2*(N(L)/2)) GO TO 210
C EVEN N
C LEFT END CHECK
C                                                 CHECK LEFT END
      IF(E2(3,N(L),1).LE.PR(L) )GO TO 203
C MAYBE 2ND WING POINT IS BAD: MAKE STRINGENT CHECKS WITH 1ST W-POINTS INSTEAD
       IF(E2(2,N(L),1).GT.P03(L))  GO TO 206
        IF(E2(2,N(L),2).GT.P03(L))  GO TO 206
C ================                                 TRY TOTAL
  203 IF(E2(4,N(L),1).LE.PR(L) )GO TO 500
C                                               500: SUM

C==============================

      IF(INS(L,N(L)).EQ.1)GO TO 229
C                          229: PARTS (WITHOUT  LEAPS)
C                                TRY TO MEND RIGHT END BY INSERTION
      IN(L)=N(L)
      LINS(L)=3
               GO TO 10
 1003 GO TO 208
C=====================================================================
  206 IF(INS(L,N(L)).EQ.1)PR(L)=PA(L)
        IF(E2(3,N(L),2).LE.PR(L) )GO TO 207
        GO TO 230
C             230: PARTS WITH LEAPS
C-------------------------------
  207 IF(INS(L,1).EQ.1)GO TO 229
        IN(L)=1
        LINS(L)=4
                 GO TO 10
 1004 CONTINUE
C    ..........................   TRY TOTAL AGAIN
  208 IF(E2(4,N(L),1).LE.PA(L) )GO TO 500
C                                              500: SUM
      GO TO 229

C===========================================================================
C ODD N                                        L-END  CHECK
  210 IF (E2(2,N(L),1).LE.PR(L) )GO TO 211
      GO TO 214
C--------------------------                    R-END CHECK
  211 DS0(L)=DS
      IF(INS(L,N(L)).EQ.1)PR(L)=PA(L)
      IF(E2(2,N(L),2).LE.PR(L) )GO TO 500
      IF (INS(L,N(L)).EQ.1) GO TO 229
C---------------------            TRY TO MAKE R-END GOOD BY INSERTING
  212 IN(L)=N(L)
      LINS(L)=5
               GO TO 10
 1005 IF(E2(2,N(L),2).LE.PA(L)) GO TO 500
      GO TO 229
C=========================================================================

  214 IF(INS(L,N(L)).EQ.1)PR(L)=PA(L)
      IF(E2(2,N(L),2).GT.PR(L) )GO TO 230
C-------------------------                TRY TO SAVE L-END BY INSERTION
  215 DS0(L)=DS
      IN(L)=1
      LINS(L)=6
                GO TO 10
 1006 IF(E2(2,N(L),1).LE.PA(L) ) GO TO 500

C============================================================================
C PARTS OF SEGMENT: SECTION WITH 'LOT' GAPS
  229 LEAP(L)=0
  230 LOT(L)=N(L)
      LOT1(L)=0
C L-CYCLE ------------- L-END FIXED
  240 LOT0(L)=LOT(L)
      LEAP4(L)=LEAP(L)*(int(LOT(L)/4))
      LOT(L)=LOT(L)-1-LEAP4(L)
      IF(LOT(L).EQ.0) GO TO 250
C                        250: R-CYCLE.    LEAP4 : HOW MUCH TO SKIP
C ----------------------  CHECK FIXED END
      IF(LOT(L).EQ.LOT1(L))GO TO 241
C                              MAKE ROUGH CHECK
      E1(L)=E2(1,LOT(L),1)
C                              FILTERING
      IF(E1(L).GT.P200(L))GO TO 240
C ---------------------------   CORRECTED PRECISION
      EP=E1(L)+P03(L)
      PR(L)=PA(L)
      IF(EP.LT.PA(L))PR(L)=EP
      IF(INS(L,1).EQ.1)PR(L)=PA(L)
      IF(LEAP4(L).EQ.0)GO TO 242
C                               STORE E1, PR1
      LEAP(L)=0
      LOT1(L)=LOT(L)
      LOT(L)=LOT0(L)
      E11(L)=E1(L)
      PR1(L)=PR(L)
C             IF E IS NOT BIG, MAKE OMITTED CHECKS
      GO TO 240
  241 E1(L)=E11(L)
      PR(L)=PR1(L)
  242 IODD(L)=LOT(L)-2*(LOT(L)/2)
C-----------------------                          CHECK FIXED L-END
      IF(E2(3-IODD(L) ,LOT(L),1).LE.PR(L))GO TO 244

      IF(INS(L,1).EQ.1)GO TO 240
      IF(E1(L).GT.PR(L))   GO TO 240
C-----------------------------  TRY TO SAVE FIXED L-END
      IN(L)=1
      LINS(L)=7
                GO TO 10
 1007 IF(E2(3-IODD(L),LOT(L),1).GT.PR(L))GO TO 240
C==============================================
  244 DS0(L)=DS
C --------------------    CHECK FREE END, USING OUTSIDE INSERTED POINT
      IN(L)=LOT(L)+1
      LINS(L)=8
               GO TO 10
 1008 IF(E2(6+IODD(L),LOT(L),1).LE.PR(L) )GO TO 500
      IF(LOT(L).GT.1)GO TO 246
C---                     CARE OF DOUBLE-CHECK AT LOT=1
      IF(INS(L,1).EQ.1)GO TO 240
C -----                   TRY TO SAVE FREE END: USE INSIDE INSERTED POINT
  246 IN(L)=LOT(L)
      LINS(L)=9
               GO TO 10
 1009 IF(E2(4+IODD(L),LOT(L),1).LE.PA(L))GO TO 500
      GO TO 240
 1010 CONTINUE

C==========================================================================
C RIGHT PARTS TO TRY                       R-END IS FIXED
  250 KIND(L)=2
      LOT(L)=N(L)-LOT(L)
C----------- CAN POSSIBLE JUMP BE STORED? ------------------

                JUMP0=0
      IF(M(L).NE.0)JUMP0=Z(L,M(L))
             NOMEM(L)=LOST(L).NE.0.OR.(JUMP0.LT.1.AND.M(L).GT.186)
C            NOMEM: TRUE, IF NO MEMORY FOR STORING JUMP IS AVAILABLE
C------------------------------------------------------------------


C RIGHT CYCLE     IF: KEEP 16 GAPS FOR NEW SEGMENT, STORE WHAT RESTS

  260 IF(N(L)-LOT(L).GE.8)GO TO 300
C                         300: STORE
      LOT(L)=LOT(L)-1
C                                          COMPLETE INSERTIONS
      IN(L)=N(L)-LOT(L)
      LINS(L)=11
                GO TO 10
 1011 IF(LOT(L).EQ.0)GO TO 100
C                       100: SET (MAKE NEW SEGMENT OF FULL OLD ONE)
                   IF(NOMEM(L).OR.ABS(G(L)).GT.GMAX(L))GO TO 260
C-----------                   TENTATIVE   CHECK FIXED END
      E1(L)=E2(1,LOT(L),2)
C                             FILTERING
      IF(E1(L).GT.P200(L))GO TO 260
C                CORRECTED PRECISION
      EP=E1(L)+P03(L)
      PR(L)=PA(L)
      IF(EP.LT.PA(L))PR(L)=EP
      IF(INS(L,N(L)).EQ.1)PR(L)=PA(L)
      IODD(L)=LOT(L)-2*(LOT(L)/2)
C================================            CHECK OF FIXED R-END
 1260 IF(E2(3-IODD(L),LOT(L),2).LE.PR(L)  )GO TO 262
      IF(INS(L,N(L)).EQ.1 )GO TO 260
      IF(E1(L).GT.PR(L))GO TO 260
C--------------------------------------- TRY TO SAVE FIXED R-END
  261 IN(L)=N(L)
      LINS(L)=12
                GO TO 10
 1012 IF(E2(3-IODD(L),LOT(L),2).GT.PA(L))GO TO 260
C------------                              CHECK FREE END WITH OUTSIDE POINT
C           IF OKEY, STORE JUMP.IF NOT: NOT TRY TO SAVE, JUST STORE FOR FUTURE
  262 DS0(L)=DS
      IF(E2(6+IODD(L),LOT(L),2).LE.PA(L) )GO TO 400
C                                              400: HOLE
      GO TO 300
C           300: STORE
C33333333333333333333333333333333333333333333333333333333333333333333

C STORE
C              MEMORY LAYOUT
C      N= LOT                        JUMP FROM X1 TO X2
C      M: -INSN                          M: 1
C     -1: N                             -1: X2
C     -2: X1 OF LEFT END                -2: X1       USE AS R-WING
C     -3: GB(2)                         -3: GB(2)
C     -4: GA(2)                         -4: GA(2)
C     -5: G                             -5: Y(N+5)
C     -6: Y(N+5)                        -6: Y(N+4)
C     -7: Y(N+4)                        -7: Y(N+3)
C     -8: Y(N+3) OF R-END               -8: Y(3)     USE AS L-WING
C     ...                               -9: Y(2)
C   -N-8: Y(3)   OF L-END              -10: Y(1)
C                                      -11: GA(1)
C                                      -12: GB(1)
C  -N- 9: YI(IN(INSN))                 -13: S OF JUMP
C     ...
C  -N- 8-INSN: YI(IN(1))
C  -N- 9-INSN: IN(INSN)
C     ...
C  -N- 8-2*INSN: IN(1)
C
C   ----------------                 ----------------
C   M(-1)=M-N-2*INSN-9                M(-1)=M-14
C--------------------------------------------------------

C STORE
  300   IF(LOST(L).NE.0)GO TO 304
        INPUT=LOT(L)+9
        INSN=0
C                 INSN: NUMBER OF INSERTIONS TO STORE
      DO 301 I=N(L)-LOT(L)+1, N(L)
  301 INSN=INSN+INS(L,I)
                      INPUT=INPUT+2*INSN
C                  IS IT POSSIBLE TO STORE?
      LOST(L)=LOT(L)
      IF(M(L)+INPUT.GT.200)GO TO 304
      LOST(L)=0
C                                STORE INSERTIONS
      J=0
      DO 302 I=1,LOT(L)
          IF(INS(L,N(L)-LOT(L)+I).EQ.0)GO TO 302
        J=J+1
        Z(L,M(L)+J)=I
        Z(L,M(L)+J+INSN)=YI(L,N(L)-LOT(L)+I)
  302 CONTINUE
      M(L)=M(L)+INPUT
C          M: NEW M: MEMORY FILLING
C                                STORE NODES
      DO 303  I=1,LOT(L)+3
  303 Z(L,M(L)-LOT(L)-9+I)=Y(L,2+N(L)-LOT(L)+I)
C                               STORE GAPS ETC.
      Z(L,M(L)-5)=G(L)
      Z(L,M(L)-4)=GA(L,2)
      Z(L,M(L)-3)=GB(L,2)
      Z(L,M(L)-2)=X1(L)+G(L)*(N(L)-LOT(L))
      Z(L,M(L)-1)=LOT(L)
      Z(L,M(L))=-INSN
  304    GO TO 100
C              100: SET

C4444444444444444444444444444444444444444444444444444444444444444444444

C HOLE   STORING HOLES IN THE INTERVAL (A,B), WHERE INTEGRATION IS DONE
  400 IF(M(L).EQ.0) GO TO 402
      IF(Z(L,M(L)).GT.0.5)GO TO 410

C FIRST                    410: APPEND
C           SET  INTEGRAL OVER HOLE TO ZERO
  402 Z(L,M(L)+1)=0d0
      M(L)=M(L)+14
C           SET FLAG Z(M) OF THE RECORD
      Z(L,M(L))=1.
      Z(L,M(L)-8)=Y(L,N(L)+3)
C                             MAKE L-WING DATA
      CALL WING2(2,0,DW,YW)
               Z(L,M(L)-11)=DW(1)
               Z(L,M(L)-9)=YW(1)
      Z(L,M(L)-12)=DW(2)-DW(1)
      Z(L,M(L)-10)=YW(2)
      Z(L,M(L)-1)= X1(L)+G(L)*N(L)
C APPEND
  410 LEFT=N(L)-LOT(L)
      Z(L,M(L)-2)=X1(L)+LEFT*G(L)
      Z(L,M(L)-7)=Y(L,LEFT+3)
      CALL WING2(1,LEFT,DW,YW)
      Z(L,M(L)-6)=YW(1)
      Z(L,M(L)-5)=YW(2)
      Z(L,M(L)-4)=DW(1)
      Z(L,M(L)-3)=DW(2)-DW(1)
C555555555555555555555555555555555555555555555555555555555555555555555555

C SUM:
C    SUMS POINTS  Y  WITH WEIGHTS  W  AND ACCUMULATES THE INTEGRALS SDT AND TOT

  500 L2=LOT(L)/2
C              NW: NUMBER OF DIFFERENT WEIGHTS IN THE QUADRATURE FORMULA
      NW=L2+1
C              JJ: POINTER TO SET OF WEIGHTS
      JJ=(LOT(L)-L2)*NW
C                           LEFT: LEFT PART LEFT ASIDE (NUMBER OF GAPS)
      LEFT=(N(L)-LOT(L))*(KIND(L)-1)
C----------------------------------------------------------------------------

C        QUADRATURE           QU: AVERAGE VALUE OF THE FUNCTION IN THE SECTION
      QU=DS
      IF(L2+L2.NE.LOT(L))QU=(DS+DS0(L))*0.5
      L2=LEFT+2
      LL2=L2+LOT(L)+2
      DO 502 I=1,NW
  502 QU=QU+(Y(L,L2+I)+Y(L,LL2-I))*W(JJ+I)
C                             H: LENGTH OF THE INTEGRATED  SECTION
      H=G(L)*LOT(L)
C                  'IF': FIGHTS ACCUMULATION OF ROUNDING-OFF ERRORS
      IF(M(L).GT.0.AND.LOT(L).EQ.N(L).AND.LOST(L).EQ.0)
     *         H=Z(L,M(L)-2)-X1(L)
      S=QU*H
C                S: INTEGRAL OVER THE SECTION WITH  LOT GAPS
C----------------------------------------------------------------------------

C     INTEGRAL ACCUMULATION     SDT: INTEGRAL FROM A TO X1+H*(2-KIND)
C                               TOT: TOTAL INTEGRAL INCLUDING HOLES
      IF(LEFT.EQ.0)GO TO 505
      RQA(L)=X1(L)+LEFT*G(L)
      RQB(L)=X1(L)+N(L)*G(L)
      JQ(L)=1
C                                 PUT S  IN THE HOLE
               Z(L,M(L)-13)=Z(L,M(L)-13)+S
      GOTO 506

C                                 ADD S  TO SDT
  505           SUM(L)=SUM(L)+S
  506 TOT(L)=TOT(L)+S
      SUMTOC(L)=SUM(L)
C------------------------------------------------
C     ERROR ACCUMULATION
C                       NUMBER OF QUADRATURES
      NQUAD(L)=NQUAD(L)+1
C                       ROUNDING-OFF ERROR
      ROUND=S*ACCURA
C                       SQARED ERROR AND ROOT-MEAN-SQUARED ERROR
      SQUERR(L)=SQUERR(L) +ESTER*ESTER +ROUND*ROUND
      RMSERR(L)=SQRT(SQUERR(L))
C------------------------------------------------
             IF(KIND(L).EQ.2)GO TO 100
C                               100: SET
C           STEP FORWARD                           IS DONE, IF KIND=1
      X1(L)=X1(L)+H
              IF(LOT(L).EQ.N(L))KIND(L)=0


      GO TO 100
C              100: SET
      END




      SUBROUTINE WING2(KIND,LOT,DW,YW)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      COMMON /SD214V/
     * X1(2),G(2),PA(2),ESTER,DS, GA(2,2),GB(2,2),Y(2,21)
     *   ,YI(2,16),YT(2,19),errold,L,N(2),INS(2,16),MADE(2,5)
     *                 ,NOLD,MOLD,MODY

      DIMENSION DW(2),YW(2)
C--------------------------------------------------------------
C   WING SETTING
C              VARIABLES:
C                        I: NUMB. OF GAPS PASSED
C                        J: NUMB. OF WING POINTS FOUND
C                       NI: INDEX OF A CONSIDERED GAP
C              YW: VALUES OF POINTS FOUND
C              DW: (-) COORDINATES (FROM FREE END) OF POINTS
C---------------------------------------------------------------
      NOUT=N(L)-LOT
      KO=3-KIND
      I=0
      J=0
    1 IF(KIND-1)  3,3,4

    3   NI=LOT+1+I
      GOTO 5
    4   NI=NOUT-I
C--------------------------------------------------------
C------------------------------------------------------
C                                                INSERTED POINT INTO WING
C     SKIP, IF INSERTED POINT MAY NOT BE (NI<=0) OR
C                                       MAY BE (NI>0) BUT IS NOT (INS=0)
    5 IF (NI.LE.0) GO TO 2
      IF(INS(L,NI).EQ.0) GO TO 2
        J=J+1
        YW(J)=YI(L,NI)
        DW(J)=(I+0.5)*G(L)
    2 IF(J.GT.1) RETURN
C-------------------------------------------------
C--------------------------------------------------
C                                             NODE INTO WING
      I=I+1
      J=J+1
      YW(J)=Y(L,3+NOUT-I)
      IF(KIND.EQ.1) YW(J)=Y(L,3+LOT+I)
      IF(NOUT.GE.I) GOTO 10
                    GOTO 11
   10   DW(J)=I*G(L)
        GOTO 12
   11   DW(J)=G(L)+GA(L,KO)

   12   IF(J.LT.2) GO TO 1
      RETURN
      END



      real*8 FUNCTION E2(MODE,LOT,KIND)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

C                                E: ERROR ESTIMATE


C   MODE (OF USING CONTROL POINTS):
C                         POSITION OF CONTROL POINT AT FREE END
C
C                         INSIDE  0  OUTSIDE
C   NUMBER OF     $   2           3
C   CONTROL POINTS$   1       4   2   6
C   AT FIXED END  $   0       5   1   7
C
C   LOT: NUMBER OF GAPS TO KEEP IN THE CHECKED PART OF SEGMENT
C
C   KIND (OF FIXED END):    1= LEFT END
C                           2= RIGHT END
C
C   COMMONS:
C           N: NUMBER OF GAPS IN THE (TOTAL) SEGMENT
C       Y(21): FUNCTION VALUES AT NODES
C           G: (GAP) DIFFERENCE OF COORDINATES OF ADJACENT NODS
C       GA(2): CLOSEST GAP ADJACENT TO SEGMENT
C       GB(2): NEXT GAP AFTER THE CLOSEST
C
C              NUMERATION:

C NODS=  1     2     3     4     5   ...  N+3   N+4   N+5    INDEX IN Y
C        .GB(1).GA(1).  G  .  G  .   ...   .GA(2).GB(2).     GAP VALUES
C GAPS=                 1     2      ...                     INDEX IN YI,INS
C
C     INS(16): (INSERTIONS) INDICATORS OF INSERTED POINTS
C                            INS(I)=0: NO INSERTED POINT IN THE GAP I
C                            INS(I)=1: INSERTED POINT IS IN THE GAP I
C      YI(16): (Y INSERTED) FUNCTION VALUES AT INSERTED POINTS

C    MADE(5): INDICATORS OF COMPUTATIONS MADE. MADE(KM)=N   MEANS THAT FOR
C              LOT <=N AND FOR THE CASE KM=(KIND,MODE) INTERMEDIATE VALUES
C              ARE COMPUTED. FOR MODE=2 NEGATIVE SIGN OF MADE(KM) MEANS THAT
C              ONLY NORMALISATION OF POINTS IS MADE.

C OUTPUT PARAMETER DS:

C       DS: CORRECTION TO THE QUADRATURE FORMULA FOR THE CURRENT MODE AND LOT

C INTERNAL VARIABLES:

C     V(80)=   V(5,16): INTERMEDIAT VALUES DEPENDING ON Y
C     Q(80)=   Q(5,16): COEFFICIENTS DEPENDING ON DISTANCES
C     T(42)=   T(2,21): NORMALIZED AND REORDERED FUNCTION VALUES
C     RA     : RELATIVE DISTANCE FROM THE END OF SPAN TO THE
C              WING POINT (IF RA>0), OR NAIL POINT (IF RA<0), OR INSERTED
C              POINT (IF RA=-0.5)
C     CB     : DISTANCE TO ADDITIONAL WING POINT
C     TA,TB  : FUNCTION VALUES IN THE ADDITIONAL (WING OR NAIL) POINTS
      DIMENSION JR(16),RC(64),CF(16)
      DATA JR/0,0,1,2,4,6,9,12,16,20,25,30,36,42,49,56/
      DATA RC/
     *1.d0,3.d0,
     *4.d0,3.d0,
     *5.d0,10.d0,
     *6.d0,15.d0,10.d0,
     *7.d0,21.d0,35.d0,
     *8.d0,28.d0,56.d0,35.d0,
     *9.d0,36.d0,84.d0,126.d0,
     *10.d0,45.d0,120.d0,210.d0,126.d0,
     *11.d0,55.d0,165.d0,330.d0,462.d0,
     *12.d0,66.d0,220.d0,495.d0,792.d0,462.d0,
     *13.d0,78.d0,286.d0,715.d0,1287.d0,1716.d0,
     *14.d0,91.d0,364.d0,1001.d0,2002.d0,3003.d0,1716.d0,
     *15.d0,105.d0,455.d0,1365.d0,3003.d0,5005.d0,6435.d0,
     *16.d0,120.d0,560.d0,1820.d0,4368.d0,8008.d0,11440.d0,6435.d0/
      DATA CF     /0.16666666667d0,-0.06666666667d0,
     3             0.05000000000d0,-0.06349206349d0,
     5             0.02728174603d0,-0.06000000000d0,
     7             0.01804012346d0,-0.05695045695d0,
     9             0.01316456980d0,-0.05436160694d0,
     1             0.01020545107d0,-0.05215584416d0,
     3             0.00824255288d0,-0.05025673744d0,
     5             0.00685759239d0,-0.04860299139d0/
C     JR(I): POINTER TO THE ITH PART OF THE TABLE RC
C     RC : TABLE OF BINOMIAL COEFFICIENTS
C     CF : WEIGHTS USED FOR COMPUTATION  OF CORRECTIONS DS
C                SKETCH OF STRUCTURE
C     'E' REMEMBERS AND REUSES INTERMEDIATE VALUES V,Q,
C     WHILE THEY ARE VALID. IF NAIL OR INSERTED POINT IS
C     AVAILABLE, 'E' PUTS IT ON PLACE OF A WING
C     POINT, AND ERASES ON OBSOLETE PART OF ITS MEMORY.

C     CHAIN OF BLOCKS:
C     FAST MODE 1 - INDEX PACKING - NORMALIZATION AND ORDERING -
C     WING POINTS LISTING -  INSERTED POINT LISTING
C     AND PARTIAL ERASURE-
C     V,Q FOR MODE=2,5,7
C     V,Q FOR MODES 3,4,6 - ATERNATION OF V -
C     DEVIATION AND E - CORRECTIONS DS.
C
      COMMON /SD214V/
     * X1(2),G(2),PA(2),ESTER,DS, GA(2,2),GB(2,2),Y(2,21)
     *   ,YI(2,16),YT(2,19),errold,L,N(2),INS(2,16),MADE(2,5)
     *                 ,NOLD,MOLD,MODY

      DIMENSION V(2,80),Q(2,80),T(2,42),RA(2,2),CB(2,2)
     *   ,TA(2,2),TB(2,2),Y0(2,2)

      save V,Q,T,RA,CB,TA,TB,Y0

C--------------------------------------------------------------
      N2=LOT/2
      JLOT=JR(LOT)
      IF(MODE-2)10,20,20
C-----------------------------------------------------------------
C FAST MODE 1 ++++++++++++  I1,LAST: FIRST AND LAST POINT IN YT TO USE
   10 I1=1+(N(L)-LOT)*(KIND-1)
      LAST=I1+LOT
      DEV=YT(L,I1)+YT(L,LAST)
      IF(N2)14,14,12
   12 DO 13 I=1,N2
   13 DEV=DEV+(YT(L,I1+I)+YT(L,LAST-I))*RC(JLOT+I)
   14 E2=ABS(DEV*G(L))*4.
      GO TO 1000
C------------------------------------------------------------------
C           GENERAL PROCEDURE FOR WING-MODES.  IOUT:INDICATOR OF MODES 6,7
   20 E2=PA(L)*10d0
      NL=N(L)
      IOUT=0
           IF(MODE.GE.6) IOUT=1
C            PACKING INDECES
      IF(KIND.EQ.2) GO TO 22
      KN=1
      KK=LOT+IOUT
           GO TO 24
   22 KN=NL
      KK=NL+1-LOT-IOUT
C            (KIND,MODE) PACKING
   24 KM=KIND+2*(MODE-2)
      KMQ=KM
           IF(MODE.GT.3) KMQ=5
      KAM=16*(KMQ-1)
      KIN=21*(KIND-1)
      KO=3-KIND
        IF(GA(L,KIND).NE.0d0)GO TO 25
        IF(INS(L,KN).NE.0)GO TO 25
        IF(MODE.EQ.2)GO TO 1000
        IF(MODE.EQ.4.OR.MODE.EQ.6)GO TO 1000
   25 CONTINUE
C----------------------------------------------------------------
C         KM=(KIND,MODE)
C         KN:INDEX OF GAP AT FIXED END
C         KK:INDEX OF GAP AT FREE END
C     ONE-DIMENSIONAL CONVERSION OF ARRAYS V,Q,T:
C         T(KIN+I)=T(KIND,I)
C         V(KAM+I)=V(KMQ,I)=V(KIND,MODE,I)
C         KMQ: INDEX KM OF SHORTENED RANGE FOR ARRAYS Q,V
C         KO: KIND OPPOSITE
C--------------------------------------------------------------

C                 NORMALIZE AND ORDER
      IF(ABS(MADE(L,KIND)).GE.LOT) GO TO 42
C         SKIP NORMALIZATION AND WING POINTS HANDLING

      GO TO(31,33),KIND

C                 NORMALIZE TO LEFT END
   31 Y0(L,1)=Y(L,3)
          DO 32 I=1,LOT+5
   32     T(L,I)=Y(L,I)-Y0(L,1)
      GOTO 35
C------
C                 NORMALIZE TO RIGHT END AND ORDER
   33 Y0(L,2)=Y(L,NL+3)
          DO 34 I=1,LOT+5
   34      T(L,21+I)=Y(L,NL+6-I)-Y0(L,2)
C---------------------------------------------------------------
   35 MADE(L,KIND)=-LOT
C                   BOOK-KEEPING: HOW MUCH IS MADE
C---------------------------------------------------------------
C          WING POINTS
      RA(L,KIND)=GA(L,KIND)/G(L)
      TA(L,KIND)=T(L,KIN+2)
      TB(L,KIND)=T(L,KIN+1)
      CB(L,KIND)=GA(L,KIND)+GB(L,KIND)
C---------
C          INS-POINT USE

   42 IF(INS(L,KN).EQ.0)GO TO 47
      IF(RA(L,KIND).EQ.-0.5d0)GO TO 47
C         IF INS-POINT APPEARED
      RA(L,KIND)=-0.5
      TA(L,KIND)=YI(L,KN)-Y0(L,KIND)
      TB(L,KIND)=T(L,KIN+2)
      CB(L,KIND)=GA(L,KIND)
C         ERASE WING - DEPENDENT PART OF STORE
      MADE(L,KIND)=0
      MADE(L,KIND+2)=0
C-----------------------------------------------------------
   47 IF(MODE.GT.3)GO TO 48
      IF(MADE(L,KMQ).GE.LOT) GO TO 600
C                                600: SKIP Q,V CALCUL.

   48 RK=RA(L,KIND)
      MOD=MODE-1
      GO TO(200,300,400,201,412,201),MOD
C         Q AND V BLOCKS
C-----------------------------------------------------------
C           MODES 2,5,7: LINE SUBTRACTION
  200 TAK=TA(L,KIND)
      GO TO 221
C           MODES 5,7
  201 RK=-LOT-IOUT+0.5
      YA=YI(L,KK)
C---------------------------------------------------
C           JOINT PARTS IF MODES 2,5,7: LINE SUBTRACTION
  220 TAK=YA-Y0(L,KIND)
C                      TAR: LINE COEFFICIENT
  221 TAR=TAK/RK
      KIN3=KIN+3
      DO 224 I=1,LOT
      Q(L,KAM+I)=I+RK
  224 V(L,KAM+I)=(T(L,KIN3+I)+TAR*I)/Q(L,KAM+I)
      GO TO 500
C---------------------------------------------------
C           MODE 3: PARABOLA SUBTRACTION. ONE WING USED
  300 R=CB(L,KIND)/G(L)
      IF(GA(L,KIND).EQ.0d0)GO TO 1000
      TBK=TB(L,KIND)
      GO TO 422
C---------------------------------------------------
C            MODE 4: TWO WINGS USED
  400 IF(LOT.LT.NL)GOTO 412
        IF(INS(L,KK).NE.0)GO TO 414

C---------------------------------------------------
  410 IF(GA(L,KO).EQ.0d0)GO TO 412
C                             USE FREE WING POINT, IF AVAILABLE
      YB=Y(L,KK+4)
      R =-NL-GA(L,KO)/G(L)
      GO TO 420
C ---------------------------------------------------
  412 IF(INS(L,KK).EQ.0) STOP ' USE OF NONEXISTENT INS-POINT'
C            MODES 4,6: USE INSERTED POINT
  414 YB=YI(L,KK)
      R=-LOT-IOUT+0.5
C---------------------------------------------------
C            JOINT PART OF MODES 3,4,6: PARABOLA SUBTRACTION
C---------------------------------------------------
C            RK=RA(KIND): FIXED END NAIL
  420 TBK=YB-Y0(L,KIND)
  422 RB=R-RK
      R1=TBK/(RB*R)
      R2=TA(L,KIND)/(RB*RK)
         A1=R1*RK-R2*R
         A2=R1-R2
C            A1,A2: PARABOLA COEFF.
      KIN3=KIN+3
      DO 424 I=1,LOT
      Q(L,KAM+I)=(I+RK)*(I+R)
  424 V(L,KAM+I)=(T(L,KIN3+I)-I*(A1+I*A2))/Q(L,KAM+I)
C---------------------------------------------------
C            ALTERNATING V'S
  500 DO 502 I=1,LOT,2
  502 V(L,KAM+I)=-V(L,KAM+I)
C---------------------------------------------------
C            STORE KEEPING
      MADE(L,KMQ)=LOT
C---------------------------------------------------
C            DEVIATION AND E
  600 KAMLOT=KAM+LOT
      DEV=V(L,KAMLOT)
      IF(N2.EQ.0) GOTO 603
C                 DO SUM
      DO 602 I=1,N2
  602 DEV=DEV+(V(L,KAM+I)+V(L,KAMLOT-I))*RC(JLOT+I)

C----------------  DEV: WEIGHTED DEVIATION
  603  DEVIAT=DEV*Q(L,KAMLOT)
C---------------   DEVIAT: NATURAL DEVIATION OF EXTRAPOLANT FROM THE END POINT
      E2=ABS(DEVIAT*G(L))
C                  CORRECTION TO CLOSENESS OF NAIL
      IF(MODE.GE.6)E2=E2+E2+E2

      IF(MODE.EQ.4.AND.R+LOT.LT.0d0.AND.LOT.EQ.2)E2=E2*3.0

  605 ESTER=E2
C     CORRECTION DS
      DS=0d0
C                NO DS, IF LOT IS EVEN AND ONLY ONE ADDITIONAL POINT USED
      IF(MODE.EQ.2.AND.LOT.EQ.2*N2) GO TO 1000
      DS=DEV*CF(LOT)
C---------------------------------------------------------
 1000 RETURN

      END

